#!/usr/bin/php
<?php
$DEFAULT_TIMEOUT = 40; //seconds
$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);
$PRIORITIES_STATES=array("CRITICAL","WARNING","UNKNOWN","OK");
$PING_PATH="/usr/local/gesa/plugins/check_icmp";
$NMAP_PATH="/usr/bin/nmap";
$SHORT_OPTS="M:w:c:I:R:D";
$LONG_OPTS=array("mac-address:","warning:","critical:","packet:","ttl:","timeout:","help","ip:","range:","debug");

$PATH_MUTEX="/tmp/nmap.mutex.txt";

$options = getopt($SHORT_OPTS, $LONG_OPTS);

if ( isset($options["help"]) || isset($options["h"]) ){
	print_help();
	exit($INT_STATUS["UNKNOWN"]);
}

$mac 		= getParam($options,"mac-address","M");
$warning 	= getParam($options,"warning","w","200,40,100");
$critical 	= getParam($options,"critical","c","500,80,200");
$packet 	= getParam($options,"packet","n",5);
$ttl 	    = getParam($options,"ttl","l",64);
$timeout 	= getParam($options,"timeout","t",$DEFAULT_TIMEOUT);
$byte 	    = getParam($options,"byte","b",64);
$verifie_IP = getParam($options,"ip","I",NULL);
$debug		= getParam($options,"debug","D",NULL);
$range 		= getParam($options,"range","R","default");
if(empty($mac))
{
    exit_state("UNKNOWN","Mac address not specified!!!",TRUE);
}
$arr_warning = explode(",", $warning);
if(count($arr_warning) ==2)
{
    $arr_warning[2]=100;
}
elseif(count($arr_warning) !=3)
{
    exit_state("UNKNOWN","Bad warning paramèter",TRUE);
}
$arr_critical = explode(",", $critical);
if(count($arr_critical) ==2)
{
    $arr_critical[2]=150;
}
elseif(count($arr_critical) !=3)
{
    exit_state("UNKNOWN","Bad critical paramèter",TRUE);
}
//Armement timeout
pcntl_alarm($timeout+2);
pcntl_signal(SIGALRM, "sig_handler");
$mac = strtolower($mac);
if(isset($debug))
{
	echo "DEBUG: recherche de la mac\n"; 
}
$ip = getIPAddressByMac("$NMAP_PATH",$range,$mac);
if(isset($debug))
{
	echo "DEBUG: Correspondance $mac <=> $ip\n"; 
}
if(!empty($ip))
{
    if(!empty($verifie_IP))
    {
        if($ip!=$verifie_IP)
        {
            exit_state("CRITICAL"," Bad IP for $mac != $verifie_IP. IP IN ARP TABLE $ip.");
        }
    }
    $command_line = "$PING_PATH -H $ip -w $arr_warning[0],$arr_warning[1] -c $arr_critical[0],$arr_critical[1] -n $packet -l $ttl -b $byte";
    exec($command_line,$outputs,$retvar);
    //echo $retvar." = ".$command_line."\n";
    $out="";
    foreach($outputs as $line)
    {
        $out.="$line";
    }
    //echo "$out";
    //si le ping est déjà en critique on ne va pas plus loin
    if($retvar==2)
    {
        echo "$out rtdelta=0,000ms;;;;\n";
        exit($retvar);
    }
    //on décortique le message
    if(preg_match("#([A-Z]*)\s+\-.*rtmax=([0-9,]*).*rtmin=([0-9,]*)#",$out,$regs))
    {
        $state = $regs[1];
        $rtmax = (float)preg_replace('#\,#','.',$regs[2]);
        $rtmin = (float)preg_replace('#\,#','.',$regs[3]);
        $ret = 'OK';
        $rtdelta = $rtmax - $rtmin;
        $deltamsg = ', delta '.$rtdelta.'ms ';
        if($rtdelta > $arr_critical[2])
        {
            $ret = 'CRITICAL';
            $deltamsg.='> '.$arr_critical[2].'ms';
        }
        elseif($rtdelta > $arr_warning[2])
        {
            $deltamsg.='> '.$arr_warning[2].'ms';
            if($state!='CRITICAL')
            {
                $ret = 'WARNING';
            }
        }   

        $out = preg_replace('#^[A-Z]*\s+-#','',$out);
        $out = preg_replace('#\|rta#', $deltamsg.'|rta',$out);
        exit_state($ret,"$out rtdelta=".$rtdelta."ms;;;;");
    }
    else
    {
        exit_state("UNKNOWN","BAD OUTPUT no rtdelta\n$out\n");
    }
}
else
{
    exit_state("UNKNOWN"," Mac address not in ARP TABLE");
}

//
//Fonction
//

function print_usage(){
	echo "USAGE :  check_icmp_gigue -H <IP> -w 200,40,100 -c 500,80,200\n";
    print_help();
}

function print_help(){

	echo "\n";
	echo "\t-m, ".str_pad("--mac-address=target",20)			."Mac address\n";
	echo "\t-w, ".str_pad("--warning=WARNING",20)	."warning threshold (default: 200ms,40%,100ms)\n";
	echo "\t-c, ".str_pad("--critical=CRITICAL",20)	."critical threshold (default: 500ms,80%,200ms)\n";
    echo "\t-n, ".str_pad("--packet=NUMBER",20)	."number of packets to send (default: 5)\n";
    echo "\t-l, ".str_pad("--ttl=TTL",20)	." TTL on outgoing packets\n";
    echo "\t-t, ".str_pad("--timeout=TIMEOUT",20)	."timeout value (default:  10 seconds)\n";
    echo "\t-b, ".str_pad("--byte=TIMEOUT",20)	."Number of icmp data bytes to send. Packet size will be data bytes + icmp header (currently 68 + 8)\n";
	echo "\n";
	echo "\t-h, ".str_pad("--help",20)				."Print this help\n";
    echo "Notes:\n";
    echo "\t The threshold alert and critical format means this: 200,40%,100\n";
    echo "\t    200 is the latency\n";
    echo "\t    40 is the lost packet percentage\n";
    echo "\t    100 is the maximum delta between the different ping latencies\n";
	echo "\n";
}

function getParam($options,$name1,$name2,$default=null){
	if( isset( $options[$name1] ) ) return $options[$name1];
	if( isset( $options[$name2] ) ) return $options[$name2];
	return $default;
}

function isNull($value){
	return ( empty($value) || strtolower($value) == "null" || strtolower($value) == "none" || $value == null );
}

function exit_state($state,$msg=null,$usage=false){
	$STR_STATUS=$GLOBALS["STR_STATUS"];
	$INT_STATUS=$GLOBALS["INT_STATUS"];

	$ret=$state;
	if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
	else $state=$STR_STATUS[$ret];

	if( $msg != null ) echo "$state - $msg\n";
	if( $usage ) print_usage();
	exit($ret);
}
/*function getIPAddressByMac($mac)
{
    exec('arp -n',$output);
    $arp =  array();
    foreach($output as $line)
    {
	    if(preg_match('#^([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*)\s+ether\s+([0-9\:A-Za-z]*)#',$line,$regs))
	    {
            if($regs[2] == $mac)
            {
                return $regs[1];
            }
	    } 
    }
    return null;
}*/
function getIPAddressByMac($NMAP_PATH="/usr/bin/nmap",$range="default",$macsearch='')
{
    $flag = false;
    $filename = '/tmp/nmapscan'.preg_replace('#/#','',$range);
    $macsearch = strtoupper($macsearch);
	global $debug;
    $mactable= array();
    if(file_exists($filename))
    {
		if(isset($debug))
		{
			echo "DEBUG: $filename exist\n";
		}
		$handle = fopen($filename, "r");
		if(flock($handle, LOCK_SH|LOCK_NB))
		{
			$contents = fread($handle, filesize($filename));
			if(isset($debug))
			{
				echo "DEBUG: CONTENT ".time()."\n $contents\n";
			}
			flock($handle, LOCK_UN);
			fclose($handle); 
        	$arr = json_decode($contents,true);
			if(isset($debug))
			{
				echo "DEBUG: TIME ".$arr['time']." > ". (time()-420)."\n DEBUG: JSON ".json_last_error_msg()."\n";
			}
            $mactable = $arr['HOST'];
		    if($arr['time'] > (time()-420))
		    {
		        if(array_key_exists($macsearch,$mactable))
		        {
					if(isset($debug))
					{
						echo "DEBUG: MAC FOUND ".$mactable[$macsearch]."\n";
					}
		            return $arr['HOST'][$macsearch];
		        }   
		        else
		        {
		            $flag=false;
		        }
		    }
		    else
		    {
		        $flag=true;
		    }  
		}
		else
		{
			fclose($handle); 
			exit_state("UNKNOWN"," unable to obtain read lock");
			return null;
		}
    }
    else
    {
        $flag=true;
    }
	if(isset($debug))
	{
		echo "DEBUG: FLAG VALUE: $flag\n";
	}
    if($flag == true)
    {
        $outnmap = array();
        if($range == "default")
        {
            exec('ifconfig',$output);
            foreach($output as $line)
            {
	            if(preg_match('#(inet\s+|addr:)([0-9\.]*)\s+.*[netmask\s+|Mask:]([0-9\.]*)#',$line,$regs))
	            {
                    $ip = $regs[2];
                    $netmask= $regs[3];
                    if(!preg_match('#^127#', $ip))
                    {
						$long = ip2long($netmask);
                        $base = ip2long('255.255.255.255');
                        $net = 32-log(($long ^ $base)+1,2);
						$range = "$ip/$net";
						break;
                    }
	            } 
            }
        }
		if(isset($debug))
		{
			echo "DEBUG: SCAN ouverture $filename en ecriture\n";
		}
		$handle = fopen($filename, 'w+');
		if (flock($handle, LOCK_EX| LOCK_NB)) 
		{
			if(isset($debug))
			{
				echo "DEBUG: SCAN EXEC $NMAP_PATH -sn $range\n";
			}
			exec("$NMAP_PATH --max-rate 85 -sn $range",$outnmap);
		    $i=0;
		    foreach($outnmap as $line)
		    {
		        if(preg_match('#Address:\s+([0-9\:A-Za-z]*)#',$line,$regs))
		        {
		            $tmac = $regs[1];
		            if(preg_match('#([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*)#',$outnmap[$i-2],$ip))
		            {
                        $mactable[$tmac]= $ip[1];
                        //return $ip[1];
		            }
		        }
		        $i++;
		    }
        	$bufmac = array('time'=>time(),'HOST' =>$mactable );
			if(isset($debug))
			{
				echo "DEBUG: WRITE CONTENT\n";
			}
			rewind($handle);
			fwrite($handle, json_encode($bufmac));
			flock($handle, LOCK_UN);
			fclose($handle); 
		}
		else
		{
			fclose($handle); 
			exit_state("UNKNOWN"," unable to obtain write lock");
			return null;
		}
  		if(isset($debug))
		{
			echo "DEBUG: WRITE SUCCESS\n";
		}
       // echo "PUT CONTENT\n";
        if(array_key_exists($macsearch,$mactable))
        {
            return $mactable[$macsearch];
        }
    }
    return null;
}
declare(ticks = 1);
function sig_handler($signo){
	global $timeout;		
	exit_state("UNKNOWN","Timeout after $timeout seconds");
}
?>
