#!/usr/bin/perl -w
############################## check_snmp_dram_flash_alliedTelesis #################
my $Version='1.0';
# Date : April 11 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################
use strict;
use Net::SNMP;
use POSIX;
use Getopt::Long;
my $PROGNAME='check_snmp_DRAM_flash_alliedTelesis.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $warning='70';
my $port='161';
my $critical='80';
my $target="";
my $usedKb;
my $totalKb;
my $result;
my $pourcent;
my $type;
my $TYPE;
my ($total, $unityMax, $used, $unityUsed, $dispo);
my @unity = ("B","KB", "MB", "GB", "TB");
my @tab_value_read;
my $allied_descr = "1.3.6.1.2.1.1.1.0";

my $allied_dram = ".1.3.6.1.4.1.207.8.4.4.3.21.2.1.1"; # This object indicates the host DRAM information
my $allied_flash = ".1.3.6.1.4.1.207.8.4.4.3.21.2.1.2"; # This object indicates the host Flash information

sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname) 	Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -t  target of check (flash or dram) (default: flash and dram) \n";
	print " -w	warning value in % (default: 60)\n";
	print " -c	critical value in % (default: 65)\n";
	print " -V (--version)	Plugin version\n";
	print " -h (--help)	usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print " $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] [-w warning] [-c critical] [-t target ]\n";
	print " $PROGNAME [-h | --help]\n";
	print " $PROGNAME [-V | --version]\n";
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,      "warning=i"      => \$warning,
	"c=i" => \$critical,     "critical=i"     => \$critical,
	"t=s" => \$target,		 "target=s"		  => \$target
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

my ($snmp, $snmperror);
if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname  => $hostname,
			-version   => 'snmpv1',
			-domain    => $domain,
			-port      => $port,
			-community => $community,
			-timeout   => 2
		);
}
my $return=$ERRORS{OK};
if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
	exit $ERRORS{"UNKNOWN"};
}
else
{
	my $cmpt = 0;
	my @oid_type;
	my @type;
	if($target eq "flash")
	{
		$oid_type[0] = $allied_flash;
		$type[0] = "FLASH";
	}
	else
	{
		if($target eq "dram")
		{
			$oid_type[0] = $allied_dram;
			$type[0] = "DRAM";
		}
		else
		{
			@oid_type = ($allied_flash,$allied_dram);
			@type = ("FLASH","DRAM");
		}
	}	
	my $result = $snmp->get_table(Baseoid =>$allied_flash);
	if(!defined($result))
	{
	    if($snmp_version == 2)
	    {
		    ($snmp, $snmperror) = Net::SNMP->session(
			    -hostname => $hostname,
			    -version => 'snmpv1',
			    -community => $community
		    );
		    $result = $snmp->get_table(Baseoid =>$allied_flash);
		    if (!defined($result)) 
		    {
	       		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	       		$snmp->close;
	      		 exit $ERRORS{"UNKNOWN"};
		    }
	    }
	    else
	    {
		    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		    $snmp->close;
		     exit $ERRORS{"UNKNOWN"};
	    }
	}
    my $graph_data="";
    foreach my $key (keys %$result)
    {
        if($key =~ /\.([0-9]*)$/)
        {
            my $descr =$$result{"$key"};
            my $used=0;
            my $free=0;
            if($descr=~/Used\:\s+([0-9\.]*)/)
            {
                $used = $1;
            }
            if($descr=~/Available\:\s+([0-9\.]*)/)
            {
                $free = $1;
            }
            if($used != 0 && $free != 0)
            {
                my $total = $used + $free;
                my $percent_used = int(($used / $total)*100);
                my $state ='OK';
                my $message ="< $warning%";
                if($percent_used >= $critical)
                {
                    $state ='CRITICAL';
                    $message ="> $critical%";
                }
                elsif($percent_used >= $warning)
                {
                    $state ='WARNING';
                    $message ="> $warning%";
                }
                print "$state : FLASH used in $percent_used% $message\n";
                $return = priority_state($return,$ERRORS{$state});
                $graph_data.="flash=$percent_used.00";
            }
            else
            {
		        printf("UNKNOWN ERROR: BAD FLASH FORMAT :\n");
            }
        }
    }
    $result = $snmp->get_table(Baseoid =>$allied_dram);
    foreach my $key (keys %$result)
    {
        if($key =~ /\.([0-9]*)$/)
        {
            my $descr =$$result{"$key"};
            my $total=0;
            my $free=0;
            if($descr=~/Total\:\s+([0-9]*)/)
            {
                $total = $1;
            }
            if($descr=~/Free\:\s+([0-9]*)/)
            {
                $free = $1;
            }
            if($total != 0 && $free != 0)
            {
                my $used = $total - $free;
                my $percent_used = int(($used / $total)*100);
                my $state ='OK';
                my $message ="< $warning%";
                if($percent_used >= $critical)
                {
                    $state ='CRITICAL';
                    $message ="> $critical%";
                }
                elsif($percent_used >= $warning)
                {
                    $state ='WARNING';
                    $message ="> $warning%";
                }
                print "$state : DRAM used in $percent_used% $message\n";
                $return = priority_state($return,$ERRORS{$state});
                $graph_data.="dram=$percent_used.00";
            }
            else
            {
		        printf("UNKNOWN ERROR: BAD DRAM FORMAT :\n");
            }
        }
    }
    print "|$graph_data";
}
exit $return;
