#!/usr/bin/perl -w
############################## check_snmp_dram_flash_alliedTelesis #################
my $Version='1.0';
# Date : April 11 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################
use strict;
use Net::SNMP;
use POSIX;
use Getopt::Long;
my $PROGNAME='check_snmp_aliedTelesis_env.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $warning='70';
my $port='161';
my $critical='80';

my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";

my $allied_fan_descr  = "1.3.6.1.4.1.207.8.4.4.3.12.1.1.4"; 
my $allied_fan_speed  = "1.3.6.1.4.1.207.8.4.4.3.12.1.1.5";
my $allied_fan_state  = "1.3.6.1.4.1.207.8.4.4.3.12.1.1.7"; 

my $allied_temp_descr    = "1.3.6.1.4.1.207.8.4.4.3.12.3.1.4"; 
my $allied_temp_current  = "1.3.6.1.4.1.207.8.4.4.3.12.3.1.5"; 
my $allied_temp_state    = "1.3.6.1.4.1.207.8.4.4.3.12.3.1.7"; 

my $allied_power_descr  = "1.3.6.1.4.1.207.8.4.4.3.12.4.2.1.5"; 
my $allied_power_state  = "1.3.6.1.4.1.207.8.4.4.3.12.4.2.1.6"; 
	
my $allied_voltage_descr  = "1.3.6.1.4.1.207.8.4.4.3.12.2.1.4"; 
my $allied_voltage_state  = "1.3.6.1.4.1.207.8.4.4.3.12.2.1.8"; 

sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname) 	Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -t  target of check (flash or dram) (default: flash and dram) \n";
	print " -w	warning value in % (default: 60)\n";
	print " -c	critical value in % (default: 65)\n";
	print " -V (--version)	Plugin version\n";
	print " -h (--help)	usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print " $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] [-w warning] [-c critical] [-t target ]\n";
	print " $PROGNAME [-h | --help]\n";
	print " $PROGNAME [-V | --version]\n";
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 
sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}
########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,      "warning=i"      => \$warning,
	"c=i" => \$critical,     "critical=i"     => \$critical,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

my ($snmp, $snmperror);
if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname  => $hostname,
			-version   => 'snmpv1',
			-domain    => $domain,
			-port      => $port,
			-community => $community,
			-timeout   => 2
		);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysObjectOID");


#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysObjectOID");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $Sysoid_value = $$result{"$oid_sysObjectOID"};
if($Sysoid_value =~/3\.6\.1\.4\.1\.([0-9]*)/)
{
    $Sysoid_value = $1;
}
my $return=$ERRORS{OK};
my $graph_data ="";
$result = $snmp->get_table(Baseoid =>$allied_power_descr);
foreach my $key (keys %$result)
{

    if($key =~ /([0-9]*\.[0-9]*\.[0-9]*)$/)
    {
        my $index =$1;
        my $descr =$$result{"$key"};
        my $powerstate=getsnmpOID($snmp,"$allied_power_state.$index");
        my $state = "OK";
        my $message= "normal";
        if($powerstate == 2)
        {
            $state = "OK";
            $message= "normal";
        }
        elsif($powerstate == 1)
        {
            $state = "CRITICAL";
            $message= "failed";
        }
        elsif($powerstate == 3)
        {
            $state = "OK";
            $message= "Not powered";
        }
        else
        {
            $state = "WARNING";
            $message= "other error";
        }
        print "$state : $descr is $message\n";
        $return = priority_state($return,$ERRORS{$state});
    }
}
$result = $snmp->get_table(Baseoid =>$allied_voltage_descr);
foreach my $key (keys %$result)
{

    if($key =~ /([0-9]*\.[0-9]*\.[0-9]*)$/)
    {
        my $index =$1;
        my $descr =$$result{"$key"};
        my $voltagestate=getsnmpOID($snmp,"$allied_voltage_state.$index");
        my $state = "OK";
        my $message= "normal";
        if($voltagestate != 2)
        {
            $state = "CRITICAL";
            $message= "out of range";
        }
        print "$state : $descr is $message\n";
        $return = priority_state($return,$ERRORS{$state});
    }
}
$result = $snmp->get_table(Baseoid =>$allied_fan_descr);
foreach my $key (keys %$result)
{

    if($key =~ /([0-9]*\.[0-9]*\.[0-9]*)$/)
    {
        my $index =$1;
        my $descr =$$result{"$key"};
        my $fanstate=getsnmpOID($snmp,"$allied_fan_state.$index");
        my $state = "OK";
        my $message= "normal";
        if($fanstate != 2)
        {
            $state = "CRITICAL";
             $message= "failed";
        }
        my $fanspeed=getsnmpOID($snmp,"$allied_fan_speed.$index");
        print "$state : $descr is $message (current rpm= $fanspeed)\n";
        $graph_data.="speedcurrent=$fanspeed.00,";
        $return = priority_state($return,$ERRORS{$state});
    }
}
$result = $snmp->get_table(Baseoid =>$allied_temp_descr);
foreach my $key (keys %$result)
{

    if($key =~ /([0-9]*\.[0-9]*\.[0-9]*)$/)
    {
        my $index =$1;
        my $descr =$$result{"$key"};
        my $tempstate=getsnmpOID($snmp,"$allied_temp_state.$index");
        my $currentTemp=getsnmpOID($snmp,"$allied_temp_current.$index");
        my $state = "OK";
        my $message= "normal";
        if($tempstate != 2)
        {
            $state = "CRITICAL";
            $message= "out of range";
        }
        if($currentTemp >=$critical)
        {
            $state = "CRITICAL";
            $message= "very high";
        }
        elsif($currentTemp >=$warning)
        {
            $state = "WARNING";
            $message= "high";
        }
        print "$state : $descr is $message (current temperature=$currentTemp°C )\n";
        $return = priority_state($return,$ERRORS{$state});
        if($descr =~ /System/)
        {
            $graph_data.="temsys=$currentTemp.00,";
        }
        elsif($descr =~ /CPU/)
        {
           $graph_data.="temcpu=$currentTemp.00,";
        }        
    }
}
print "|$graph_data";
exit $return;
