#!/usr/bin/perl -w
############################## check_snmp_hp5400_temp.pl ########
my $Version='1.0';
# Date : Oct 28 2016
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_ap_ubiquity.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 1;
# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password
my $snmp_domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $type = 'cpu';
my $warning = 80;
my $critical =90;
my $ssid_name='';
my $oid_sysobject='.1.3.6.1.2.1.1.2.0';
my $oid_cpu_name  ='.1.3.6.1.4.1.10002.1.1.1.4.2.1.2';
my $oid_cpu_load  ='.1.3.6.1.4.1.10002.1.1.1.4.2.1.3';
my $oid_mem_total ='1.3.6.1.4.1.10002.1.1.1.1.1.0';
my $oid_mem_free  ='1.3.6.1.4.1.10002.1.1.1.1.2.0';
my $oid_ssid_name ='1.3.6.1.4.1.41112.1.4.5.1.2';
my $oid_ssid_station_count='1.3.6.1.4.1.41112.1.4.5.1.15';
#UNIFY
my $oid_default_cpu = "1.3.6.1.2.1.25.3.3.1.2";
my $oid_default_mem_total ='1.3.6.1.4.1.2021.4.5.0';
my $oid_default_mem_free  ='1.3.6.1.4.1.2021.4.6.0';
my $oid_default_mem_cache ='1.3.6.1.4.1.2021.4.15.0'; # Real memory cached
my $oid_unify_ssid_name ='1.3.6.1.4.1.41112.1.6.1.2.1.6';
my $oid_unify_ssid_station_count='1.3.6.1.4.1.41112.1.6.1.2.1.8';
my $oid_unify_ssid_station_type='1.3.6.1.4.1.41112.1.6.1.2.1.9';

sub print_help() {
	printf "check SNMP ubiquiti Access point \n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -T                  test type default: cpu [cpu|mem|client]\n";
	print " -w                  warning threshold (default: 80)\n";
	print " -c                  critical threshold (default: 90)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}
sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}
#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"T=s" => \$type,"Type=s" => \$type,
	"n=s" => \$ssid_name,"name=s" => \$ssid_name,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	'l=s'	=> \$o_login,		'login=s'	=> \$o_login,
	'x=s'	=> \$o_passwd,		'passwd=s'	=> \$o_passwd,
	'X=s'	=> \$o_privpass,		'privpass=s'	=> \$o_privpass,
	'L=s'	=> \$v3protocols,		'protocols=s'	=> \$v3protocols,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if (defined($o_login) || defined($o_passwd))
{
    $snmp_version =3;
}
if($hostname =~ /(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))/ )
{
    $snmp_domain ="UDP/IPV6";
}
if ($snmp_version == 3)
{
    my @v3proto=split(/,/,$v3protocols);
    if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
    if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
    if (!defined ($o_privpass))
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -domain     => $snmp_domain,
          -timeout          => 2
        );
    }
    else
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -privpassword	=> $o_privpass,
	      -privprotocol => $o_privproto,
          -domain     => $snmp_domain,
          -timeout          => 5
        );
    }
}
elsif ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
        -domain     => $snmp_domain,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
        -domain     => $snmp_domain,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations
my $result = $snmp->get_request("$oid_sysobject");

#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
            -domain     => $snmp_domain,
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysobject");
		if (!defined($result))
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $sysobject = $$result{$oid_sysobject};
if ($sysobject =~/4\.1\.41112/)
{
    if($type eq 'cpu')
    {
        $result = $snmp->get_table(Baseoid => $oid_cpu_name);
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $name =$$result{"$key"};
                my $message ="< $warning%";
                if($name =~/5\s+Minute/)
                {
                    my $cpu = getsnmpOID($snmp,"$oid_cpu_load.$index");
                    my $state='OK';
                    if($cpu > $critical)
                    {
                        $state='CRITICAL';
                        $message ="< $critical%";
                    }
                    elsif($cpu > $warning)
                    {
                        $state='WARNING';
                        $message ="> $warning%";
                    }
                    print "$state : $name is used $cpu % $message\n";
                    $return = priority_state($return,$ERRORS{$state});
                    last;
                }
            }
        }
    }
    elsif($type eq 'mem')
    {
        my $total = getsnmpOID($snmp,"$oid_mem_total");
        my $free = getsnmpOID($snmp,"$oid_mem_free");
        my $used = $total - $free;
        $used = sprintf("%.0f", (($used/$total)*100));
        my $message ="< $warning%";
        my $state='OK';
        if($used > $critical)
        {
            $state='CRITICAL';
            $message ="> $critical%";
        }
        elsif($used > $warning)
        {
            $state='WARNING';
            $message ="> $warning%";
        }
        print "$state : memory is used $used % $message\n";
        $return = priority_state($return,$ERRORS{$state});
    }
    elsif($type eq 'client')
    {
        $result = $snmp->get_table(Baseoid => $oid_unify_ssid_name);
        my $total_client=0;
        my $found=0;
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $name =$$result{"$key"};
                if($ssid_name eq '' || $ssid_name=~/$name/)
                {
                    my $client = getsnmpOID($snmp,"$oid_unify_ssid_station_count.$index");
                    $total_client = $total_client + $client;
                    my $message ="< $warning";
                    my $state='OK';
                    if($client > $critical)
                    {
                        $state='CRITICAL';
                        $message ="< $critical";
                    }
                    elsif($client > $warning)
                    {
                        $state='WARNING';
                        $message ="> $warning";
                    }
                    print "$state : SSID $name: number of client: $client $message\n";
                    $return = priority_state($return,$ERRORS{$state});
                    $found=1;
                }
            }
        }
        if($ssid_name eq '')
        {
            my $message ="< $warning";
            my $state='OK';
            if($total_client > $critical)
            {
                $state='CRITICAL';
                $message ="< $critical";
            }
            elsif($total_client > $warning)
            {
                $state='WARNING';
                $message ="> $warning";
            }
            print "$state : total of client: $total_client $message\n";
            $return = priority_state($return,$ERRORS{$state});
        }
        else
        {
            if($found==0)
            {
                print "UNKNOWN : SSID $ssid_name not found\n";
                $return = priority_state($return,$ERRORS{UNKNOWN});
            }
        }
        print"|totalclient=$total_client";
    }
}
else
{
    if($type eq 'cpu')
    {
        $result = $snmp->get_table(Baseoid => $oid_default_cpu);
        my $nbr=0;
        my $totalcpu=0;
        foreach my $key ( keys %$result)
        {
            $totalcpu=$totalcpu+$$result{"$key"};
            $nbr++;
        }
        my $cpu = int($totalcpu/$nbr);
        my $state='OK';
        my $message ="< $warning";
        if($cpu > $critical)
        {
            $state='CRITICAL';
            $message ="< $critical%";
        }
        elsif($cpu > $warning)
        {
            $state='WARNING';
            $message ="> $warning%";
        }
        print "$state : CPU is used $cpu % $message\n";
        $return = priority_state($return,$ERRORS{$state});
    }
    elsif($type eq 'mem')
    {
        my $total = getsnmpOID($snmp,"$oid_default_mem_total");
        my $free = getsnmpOID($snmp,"$oid_default_mem_free");
        my $cache = getsnmpOID($snmp,"$oid_default_mem_cache");
        my $used = $total - $free-$cache;
        $used = sprintf("%.0f", (($used/$total)*100));
        my $message ="< $warning%";
        my $state='OK';
        if($used > $critical)
        {
            $state='CRITICAL';
            $message ="> $critical%";
        }
        elsif($used > $warning)
        {
            $state='WARNING';
            $message ="> $warning%";
        }
        print "$state : memory is used $used % $message\n";
        $return = priority_state($return,$ERRORS{$state});
    }
    elsif($type eq 'client')
    {
        $result = $snmp->get_table(Baseoid => $oid_unify_ssid_name);
        my $total_client=0;
        my $found=0;
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $name =$$result{"$key"};
                if($ssid_name eq '' || $ssid_name=~/$name/)
                {
                    my $client = getsnmpOID($snmp,"$oid_unify_ssid_station_count.$index");
                    my $stationtype = getsnmpOID($snmp,"$oid_unify_ssid_station_type.$index");
                    $total_client = $total_client + $client;
                    my $message ="< $warning";
                    my $state='OK';
                    if($client > $critical)
                    {
                        $state='CRITICAL';
                        $message ="< $critical";
                    }
                    elsif($client > $warning)
                    {
                        $state='WARNING';
                        $message ="> $warning";
                    }
                    print "$state : SSID $name ($stationtype): number of client: $client $message\n";
                    $return = priority_state($return,$ERRORS{$state});
                    $found=1;
                }
            }
        }
        if($ssid_name eq '')
        {
            my $message ="< $warning";
            my $state='OK';
            if($total_client > $critical)
            {
                $state='CRITICAL';
                $message ="< $critical";
            }
            elsif($total_client > $warning)
            {
                $state='WARNING';
                $message ="> $warning";
            }
            print "$state : total of client: $total_client $message\n";
            $return = priority_state($return,$ERRORS{$state});
        }
        else
        {
            if($found==0)
            {
                print "UNKNOWN : SSID $ssid_name not found\n";
                $return = priority_state($return,$ERRORS{UNKNOWN});
            }
        }
        print"|totalclient=$total_client";
    }
}

exit($return);
