#!/usr/bin/perl
#
use strict;
use warnings;
use lib "/usr/local/esia/plugins"  ;
use vars qw($PROGNAME);
use strict;
use Net::SNMP;
use Getopt::Long;
use Data::Dumper;

my $TIMEOUT = 50;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);

# Just in case of problems, let's not hang Nagios
$SIG{'ALRM'} = sub {
	print ("ERROR: Plugin took too long to complete (alarm)\n");
	exit $ERRORS{"UNKNOWN"};
};
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 
sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}
alarm($TIMEOUT);

$PROGNAME = "check_snmp_ups.pl";
sub print_help ();
sub print_usage ();
use POSIX qw(floor);

my ($opt_h,$opt_V);
my $community = "public";
my $snmp_version = 1;
my ($hostname);
my $lineLoadWarning=70;
my $lineLoadCritical=80;
my $timecritical=60;
my $return =0;
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
my $oid_ats_input_name = "1.3.6.1.4.1.318.1.1.8.5.3.2.1.6";
my $oid_ats_input_type = "1.3.6.1.4.1.318.1.1.8.5.3.2.1.5";
my %oid_ats_input_type_message = (
		"1" => "unknown(1)",
		"2" => "main(2)",
		"3" => "bypass(3)"
	);
my $oid_ats_statuscomm = "1.3.6.1.4.1.318.1.1.8.5.1.1.0";
my %oid_ats_statuscomm_message = (
		"1" => "atsNeverDiscovered(1)",
		"2" => "atsCommEstablished(2)",
		"3" => "atsCommLost(3)"
	);
my $oid_ats_status_source = "1.3.6.1.4.1.318.1.1.8.5.1.2.0";
my $oid_ats_status_redundancy = "1.3.6.1.4.1.318.1.1.8.5.1.3.0";	
my %oid_ats_status_redundancy = (
		"1" => "atsRedundancyLost(1)",
		"2" => "atsFullyRedundant(2)",
	);
my $configuredsource;
use Getopt::Long;
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"s=s" => \$configuredsource,	"selected=s" => \$configuredsource,
	"w=i" => \$lineLoadWarning,	"warning=i" => \$lineLoadWarning,
	"c=i" => \$lineLoadCritical,	"critical=i" => \$lineLoadCritical,
	"tc=i" => \$timecritical,	"timecritical=i" => \$timecritical,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
# -V & --version print version
if ($opt_V) { print_help(); exit $ERRORS{'OK'}; }
# Invalid hostname print usage
#if (!utils::is_hostname($hostname)) { print_usage(); exit $ERRORS{'UNKNOWN'}; }


my ($snmp, $snmperror);
if ($snmp_version == 2) {
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
        -timeout => 3,
	);
} elsif ($snmp_version == 3) {
	my ($v3_username,$v3_password,$v3_protocol,$v3_priv_passphrase,$v3_priv_protocol) = split(":",$community);
	my @auth = ();
	if (defined($v3_password)) { push(@auth,($v3_password =~ /^0x/) ? 'authkey' : 'authpassword',$v3_password); }
	if (defined($v3_protocol)) { push(@auth,'authprotocol',$v3_protocol); }
	if (defined($v3_priv_passphrase)) { push(@auth,($v3_priv_passphrase =~ /^0x/) ? 'privkey' : 'privpassword',$v3_priv_passphrase); }
	if (defined($v3_priv_protocol)) { push(@auth,'privprotocol',$v3_priv_protocol); }

	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv3',
		-username => $v3_username,
		@auth
	);
} else {
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community
	);
}

if (!defined($snmp)) {
	print ("UNKNOWN - SNMP error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

my $state = 'UNKNOWN';
my $output = "UPS status retrieval failed.";

#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysObjectOID");
#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysObjectOID");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $Sysoid_value = $$result{"$oid_sysObjectOID"};
if($Sysoid_value =~/3\.6\.1\.4\.1\.([0-9]*)/)
{
    $Sysoid_value = $1;
}
my $statuscom =  getsnmpOID($snmp,$oid_ats_statuscomm);
if($statuscom != 2)
{
    print "CRITICAL : ats communication is ".$oid_ats_statuscomm_message{$statuscom}."\n";
    $return = priority_state($return,$ERRORS{CRITICAL}); 
}
else
{
    print "OK : ats communication is ".$oid_ats_statuscomm_message{$statuscom}."\n";
    $return = priority_state($return,$ERRORS{OK}); 
}
my  $statusredundancy =  getsnmpOID($snmp,$oid_ats_status_redundancy);
if($statusredundancy != 2)
{
    print "CRITICAL : redundancy status is ".$oid_ats_status_redundancy{$statusredundancy}."\n";
    $return = priority_state($return,$ERRORS{CRITICAL}); 
}
else
{
    print "OK : redundancy status is ".$oid_ats_status_redundancy{$statusredundancy}."\n";
    $return = priority_state($return,$ERRORS{OK}); 
}
my $selectedsource =  getsnmpOID($snmp,$oid_ats_status_source);
$result = $snmp->get_table(Baseoid => $oid_ats_input_name);
foreach my $key ( keys %$result) 
{
    $state='OK';
    if($key=~/3.6.1.4.1.318.1.1.8.5.3.2.1.6.([0-9]*)/)
    {
        my $descr =$$result{"$key"};
        my $id = $1;
        my $selected="";
        if($id == $selectedsource)
        {
            $selected="(selected) ";
            if($configuredsource ne '')
            {
                if($configuredsource ne $descr )
                {
                    $state='CRITICAL';
                    $selected="( BAD source selected) ";
                    $return = priority_state($return,$ERRORS{CRITICAL}); 
                }
            }
        }
        my $type = getsnmpOID($snmp,$oid_ats_input_type.$id);    
        print "$state : Input $descr ".$selected."type is ".$oid_ats_input_type_message{$statuscom}."\n";
    }	
}

exit $return;

sub print_help() {
	print "Copyright (c) 2019 Esia SA\n";
	print "\n";
	printf "Check UPS status via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community or v3 auth (defaults to public)\n";
	print "                     (v3 specified as username:authpassword:... )\n";
	print "                       username = SNMPv3 security name\n";
	print "                       authpassword = SNMPv3 authentication pass phrase (or hexidecimal key)\n";
	print "                       authprotocol = SNMPv3 authentication protocol (md5 (default) or sha)\n";
	print "                       privpassword = SNMPv3 privacy pass phrase (or hexidecmal key)\n";
	print "                       privprotocol = SNMPv3 privacy protocol (des (default) or aes)\n";
	print " -v (--snmp_version) 1 for SNMP v1 default)\n";
	print "                     2 for SNMP v2c\n";
	print " -w (--warning)  value of warning load. Default: 70 percent\n";
	print " -c (--critical) value of critical load. Default: 80 percent\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

