#!/usr/bin/perl
my $version='1.0';

use strict;
use warnings;
use lib "/usr/local/esia/plugins";
use strict;
use Net::SNMP;
use Getopt::Long;
use Data::Dumper;

my $PROGNAME = "check_snmp_apc_pdu.pl";

my $TIMEOUT = 50;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return = 3;
my ($opt_h,$opt_V);
my $community = "public";
my $snmp_version = 2;
my $voltage;
my ($hostname);
my $type = 'health';
my $warning = '';
my $critical = '';
my $ignore = '';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
my $name;


# Health
# MIB OIDs rPDU2
my $oid_rpdu2_load_status="1.3.6.1.4.1.318.1.1.26.4.3.1.4.1";
my $oid_rpdu2_hundredthskw="1.3.6.1.4.1.318.1.1.26.4.3.1.5.1"; #Watts (replace oid_rPDU2PhaseStatusPower - historical reason/first dev)
my $oid_rpdu2_tenthskwh="1.3.6.1.4.1.318.1.1.26.4.3.1.9.1";    #KWH

#affichage pour les status
my %rpdu2LoadStatuses = (
	1 => "lowLoad(1)",
	2 => "normal(2)",
	3 => "nearOverload(3)",
	4 => "overload(4)",
    5 => "not supported(5)"
);

#Phase_table
my $oid_rPDU2PhaseStatusCurrent         = '1.3.6.1.4.1.318.1.1.26.6.3.1.5.1'; # see oid_rpdu_load_amp
my $oid_rPDU2PhaseStatusVoltage         = '1.3.6.1.4.1.318.1.1.26.6.3.1.6.1';
my $oid_rPDU2PhaseStatusPower           = '1.3.6.1.4.1.318.1.1.26.6.3.1.7.1'; # see oid_rpdu2_hundredthskw
my $oid_rPDU2PhaseStatusApparentPower   = '1.3.6.1.4.1.318.1.1.26.6.3.1.8.1';
my $oid_rPDU2PhaseStatusPowerFactor     = '1.3.6.1.4.1.318.1.1.26.6.3.1.9.1';

#Bank
my $oid_rPDU2BankStatusCurrent          = '1.3.6.1.4.1.318.1.1.26.8.3.1.5';

# MIB OIDs rPDU
my $oid_rpdu_load_status = "1.3.6.1.4.1.318.1.1.12.2.3.1.1.3.1";
my $oid_rpdu_load_amp = "1.3.6.1.4.1.318.1.1.12.2.3.1.1.2.1"; # Tenths of Amps (replace oid_rPDU2PhaseStatusCurrent - historical reason/first dev)
my $oid_rpdu_device_voltage = "1.3.6.1.4.1.318.1.1.12.1.15.0";
my $oid_rpdu_powersupply_alarm = "1.3.6.1.4.1.318.1.1.12.4.1.3.0"; # PSU Alarm



# Status
my %rpduLoadStatus = (
	1 => "phaseLoadNormal(1)",
	2 => "phaseLoadLow(2)",
	3 => "phaseLoadNearOverload(3)",
	4 => "phaseLoadOverload(4)"
);
my %rpduPSUAlarmStatus = (
	1 => "allAvailablePowerSuppliesOK(1)",
	2 => "powerSupplyOneFailed(2)",
	3 => "powerSupplyTwoFailed(3)",
	4 => "powerSupplyOneandTwoFailed(4)"
);

# Outlet
my $oid_rpdu2_outlet_index = "1.3.6.1.4.1.318.1.1.26.9.2.3.1.1";
my $oid_rpdu2_outlet_name = "1.3.6.1.4.1.318.1.1.26.9.2.3.1.3";
my $oid_rpdu2_outlet_state = "1.3.6.1.4.1.318.1.1.26.9.2.3.1.5";
# Status
my %rpdu2OutletStatus = (
	1 => "OFF",
	2 => "ON"
);

# Sensors
my $oid_rpdu2_sensors_idx           = "1.3.6.1.4.1.318.1.1.26.10.2.2.1.1";
my $oid_rpdu2_sensors_name          = "1.3.6.1.4.1.318.1.1.26.10.2.2.1.3";
my $oid_rpdu2_sensors_type          = "1.3.6.1.4.1.318.1.1.26.10.2.2.1.5";
my $oid_rpdu2_sensors_temp          = "1.3.6.1.4.1.318.1.1.26.10.2.2.1.8";
my $oid_rpdu2_sensors_temp_status   = "1.3.6.1.4.1.318.1.1.26.10.2.2.1.9";
my $oid_rpdu2_sensors_humi          = "1.3.6.1.4.1.318.1.1.26.10.2.2.1.10";
my $oid_rpdu2_sensors_humi_status   = "1.3.6.1.4.1.318.1.1.26.10.2.2.1.11";


my %rpduSensorType = (
    1 => 'temperatureOnly(1)',
    2 => 'temperatureHumidity(2)',
    3 => 'commsLost(3)',
    4 => 'notInstalled(4)'
);

my %rpduSensorStatus = (
    1 => 'noPresent(1)',
    2 => 'belowMin(2)',
    3 => 'belowlow(3)',
    4 => 'normal(4)',
    5 => 'aboveHigh(5)',
    6 => 'aboveMax(6)'
);


sub print_help() {
	print "Check PDU status via SNMP.\n";
	print "Version : ".$version."\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community or v3 auth (defaults to public)\n";
	print "                     (v3 specified as username:authpassword:... )\n";
	print "                       username = SNMPv3 security name\n";
	print "                       authpassword = SNMPv3 authentication pass phrase (or hexidecimal key)\n";
	print "                       authprotocol = SNMPv3 authentication protocol (md5 (default) or sha)\n";
	print "                       privpassword = SNMPv3 privacy pass phrase (or hexidecmal key)\n";
	print "                       privprotocol = SNMPv3 privacy protocol (des (default) or aes)\n";
	print " -T                  Test type (health,bank,outlet,temperature,humidity)\n";
    print " -n                  For sensor, Sensor name or 'all' for all\n";
    print " -w                  For sensor to specify min,max treshold warning state\n";
    print " -c                  For sensor to specify min,max treshold critical state\n";
	print " -w                  For outlets, slot IDs to pass in warning state instead of critical if outlet state is OFF. Ex: 1,2,5\n";
	print " -i                  For outlets, slot IDs to ignore. Ex: 1,2,5\n";
	print " -U (voltage)		Specify the voltage (U) (default : 220V)\n";
	print " -v (--snmp_version) 1 for SNMP v1 (default)\n";
	print "                     2 for SNMP v2c\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

my @priorities = (
	$ERRORS{CRITICAL},
	$ERRORS{WARNING},
	$ERRORS{UNKNOWN},
	$ERRORS{OK}
);

sub priority_state {
	my($a,$b) = @_;
	if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

	foreach my $v (@priorities) {
		if ( $v == $a || $v == $b ) { return $v; }
	}
}

sub getsnmpOID {
	my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"T=s" => \$type,"type=s" => \$type,
    "n:s" => \$name,"name:s" => \$name,
	"w:s" => \$warning,"warning:s" => \$warning,
    "c:s" => \$critical,"critical:s" => \$critical,
	"i:s" => \$ignore,"ignore:s" => \$ignore,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"U=i" => \$voltage,"voltage=i" => \$voltage,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
# -V & --version print version
if ($opt_V) { print_revision($PROGNAME,'$Revision: 0.1 $ '); exit $ERRORS{'OK'}; }

if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }

my ($snmp, $snmperror);
if ($snmp_version == 2) {
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community
	);
} elsif ($snmp_version == 3) {
	my ($v3_username,$v3_password,$v3_protocol,$v3_priv_passphrase,$v3_priv_protocol) = split(":",$community);
	my @auth = ();
	if (defined($v3_password)) { push(@auth,($v3_password =~ /^0x/) ? 'authkey' : 'authpassword',$v3_password); }
	if (defined($v3_protocol)) { push(@auth,'authprotocol',$v3_protocol); }
	if (defined($v3_priv_passphrase)) { push(@auth,($v3_priv_passphrase =~ /^0x/) ? 'privkey' : 'privpassword',$v3_priv_passphrase); }
	if (defined($v3_priv_protocol)) { push(@auth,'privprotocol',$v3_priv_protocol); }

	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv3',
		-username => $v3_username,
		@auth
	);
} else {
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community
	);
}
#on interroge en SNMP pour récupérer les informations
my $result = $snmp->get_request("$oid_sysObjectOID");

#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysObjectOID");
		if (!defined($result))
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
			exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: V1 Description table : %s.\n", $snmp->error);
		$snmp->close;
		exit $ERRORS{"UNKNOWN"};
	}
}
my $Sysoid_value = $$result{"$oid_sysObjectOID"};
if($Sysoid_value =~/3\.6\.1\.4\.1\.([0-9]*)/)
{
    $Sysoid_value = $1;
}

my $state = 'UNKNOWN';
my $output = "";
my $graph = "";

if($type eq 'health')
{
	my $rPDU2LoadStatus = getsnmpOID($snmp, $oid_rpdu2_load_status);
	my $rPDULoadStatus = getsnmpOID($snmp, $oid_rpdu_load_status);
	if ($rPDU2LoadStatus != 5) # Data for rPDU2 OIDs (All OIDs are present for all devices but get a state 5 notSupported if the device doesn't use those OIDs but another part of the MIB)
	{
		my $rPDU2HundredKW = getsnmpOID($snmp, $oid_rpdu2_hundredthskw);
		my $rPDU2TenthKwh = getsnmpOID($snmp, $oid_rpdu2_tenthskwh);

        my $current       = getsnmpOID($snmp, $oid_rPDU2PhaseStatusCurrent);
        my $voltage       = getsnmpOID($snmp, $oid_rPDU2PhaseStatusVoltage);
        #my $outputPower   = getsnmpOID($snmp, $oid_rPDU2PhaseStatusPower);
        my $apparentPower = getsnmpOID($snmp, $oid_rPDU2PhaseStatusApparentPower);
        my $powerFactor   = getsnmpOID($snmp, $oid_rPDU2PhaseStatusPowerFactor);

        $current /= 10;
        $apparentPower *= 10;

		$snmp->close;

		$output = "PDU status is ".$rpdu2LoadStatuses{$rPDU2LoadStatus}."\n";
		if($rPDU2LoadStatus == 4)
		{
			$state = 'CRITICAL';
			$return = $ERRORS{CRITICAL};
		}
		elsif($rPDU2LoadStatus == 1 || $rPDU2LoadStatus == 3)
		{
			$state = 'WARNING';
			$return = $ERRORS{WARNING};
		}
		else
		{
			$state = 'OK';
			$return = $ERRORS{OK};
		}

		#Consommation KW
		if (($rPDU2HundredKW !~ /^noSuch/))
		{
			my $watts = ($rPDU2HundredKW*0.01)*1000;                   
			$output .= "The power consumption of the Rack PDU load is $current A for $voltage V with in : $apparentPower VA and out : $watts Watts (Power factor : $powerFactor %)\n";
			$graph = "watts=$watts,va=$apparentPower,powerfactor=$powerFactor,amperes=$current,voltage=$voltage";
		}
		#Consommation KW/H
		if (($rPDU2TenthKwh !~ /^noSuch/))
		{
			my $KWH = $rPDU2TenthKwh*0.1;
			$output .= "The Rack PDU total load energy consumption sinds last value reset is $KWH kWh\n";
			$graph .= ",KWH=$KWH";
		}

	}
	elsif(($rPDULoadStatus !~ /^noSuch/)) # Data for rPDU OIDs
	{
		my $rPDULoadAmp = getsnmpOID($snmp, $oid_rpdu_load_amp);
		my $rPDUDeviceVoltage = getsnmpOID($snmp, $oid_rpdu_device_voltage);
		my $rPduPsuAlarm = getsnmpOID($snmp, $oid_rpdu_powersupply_alarm);
		$snmp->close;

		$output = "PDU status is ".$rpduLoadStatus{$rPDULoadStatus}."\n";
		if($rPDULoadStatus == 4)
		{
			$state = 'CRITICAL';
			$return = $ERRORS{CRITICAL};
		}
		elsif($rPDULoadStatus == 2 || $rPDULoadStatus == 3)
		{
			$state = 'WARNING';
			$return = $ERRORS{WARNING};
		}
		else
		{
			$state = 'OK';
			$return = $ERRORS{OK};
		}

		if (($rPDUDeviceVoltage !~ /^noSuch/))
		{
			$voltage = $rPDUDeviceVoltage; # If available use device real voltage else use plugin voltage parameter
		}
		# Consumption
		if (($rPDULoadAmp !~ /^noSuch/))
		{
			my $amperes = $rPDULoadAmp / 10;
			my $watts = $amperes * $voltage;
			$output .= "The power consumption of the Rack PDU load is $amperes A (for $voltage V = $watts Watts)\n";
			$graph = "watts=$watts,amperes=$amperes,voltage=$voltage";
		}

		# PSU
		if (($rPduPsuAlarm !~ /^noSuch/))
		{
			$output .= "Power supplies status is ".$rpduPSUAlarmStatus{$rPduPsuAlarm}."\n";
			if($rPDULoadStatus == 4)
			{
				$state = 'CRITICAL';
				$return = priority_state($return,$ERRORS{CRITICAL});
			}
			elsif($rPDULoadStatus == 2 || $rPDULoadStatus == 3)
			{
				$state = 'WARNING';
				$return = priority_state($return,$ERRORS{WARNING});
			}
		}
	}
	else
	{
		$state = "UNKNOWN";
		$output = "PDU status not available\n";
	}

	print "$state : $output|$graph\n";
}
elsif($type eq 'bank')
{
    my $count = 0;
    my $total_current = getsnmpOID($snmp, $oid_rPDU2PhaseStatusCurrent);
    $total_current /= 10;

    my $result = $snmp->get_table(Baseoid => $oid_rPDU2BankStatusCurrent);

   	$state = "OK";
	$return = 0;
    $output = '';
    foreach my $key (sort keys %$result)
    {
        if($key=~/\.([0-9]*)$/)
		{
			my $id = $1;
			if( !defined($name) || $id eq $name )
			{
                $count++;
                my $bankStatusCurrent = $$result{"$key"};
                $bankStatusCurrent/=10;
                
                my $percent = sprintf("%.2f",($bankStatusCurrent / $total_current )* 100);
                $output .= "$state : Bank $id current conso $bankStatusCurrent A/$total_current A Total Bank conso ($percent %)\n";
                $graph = "current=$bankStatusCurrent,total=$total_current,percent=$percent\n";
            }
       }
    }

	$snmp->close;

	if($count == 0)
	{
		printf("UNKNOWN ERROR: No banks found.\n");
		exit $ERRORS{"UNKNOWN"};
	}
    elsif($count == 1)
    {
        print "$output|$graph";
    }
    else
    {
	    print "$output";
    }
}
elsif($type eq 'outlet')
{
	my $rPDUDeviceVoltage = getsnmpOID($snmp, $oid_rpdu_device_voltage);
	if (($rPDUDeviceVoltage !~ /^noSuch/))
	{
		$voltage = $rPDUDeviceVoltage; # If available use device real voltage else use plugin voltage parameter
	}

	$result = $snmp->get_table(Baseoid => $oid_rpdu2_outlet_index);
	my %warnings = map { $_ => 1 } split(",", $warning);
	my %ignores = map { $_ => 1 } split(",", $ignore);
	my $count = 0;
	$state = "OK";
	$return = 0;
	foreach my $key (sort keys %$result)
    {
        # Get the index
        if($key =~ /([0-9]*)$/ )
        {
			my $index = $1;
			if(exists($ignores{$index}))
			{
				next;
			}
			$count++;

			my $rPDUOutletName = getsnmpOID($snmp,"$oid_rpdu2_outlet_name.$index");
			my $rPDUOutletState = getsnmpOID($snmp,"$oid_rpdu2_outlet_state.$index");
			my $loadMessage = "";

			if($rPDUOutletState == 2)
			{
				$state = 'OK';
				$return = priority_state($return,$ERRORS{OK});
			}
			elsif(exists($warnings{$index}))
			{
				$state = 'WARNING';
				$return = priority_state($return,$ERRORS{WARNING});
			}
			else
			{
				$state = 'CRITICAL';
				$return = priority_state($return,$ERRORS{CRITICAL});
			}

			$output .= "$state: slot $index => $rPDUOutletName is ".$rpdu2OutletStatus{$rPDUOutletState}."$loadMessage\n";
        }
    }
	$snmp->close;

	if($count == 0)
	{
		printf("UNKNOWN ERROR: No outlets found.\n");
		exit $ERRORS{"UNKNOWN"};
	}

	print "$output";
}
elsif($type eq 'temperature' or $type eq 'humidity')
{
    $result = $snmp->get_table(Baseoid => $oid_rpdu2_sensors_idx);

    $state = "OK";
	$return = 0;
    my $count = 0;
	foreach my $key (sort keys %$result)
    {
        if($key =~ /([0-9]*)$/ )
        {
            my $current_state = 'OK';
            my $current_return = 0;
            my $index = $1;

            # On filtre si possible sur l'index ou le name
            my $sensor_name = getsnmpOID($snmp,"$oid_rpdu2_sensors_name.$index");
            if( $name ne '' && $name ne $sensor_name && $name ne $index)
            {
                next;
            }

            $count++;

            # On récupére le type et on vérifie si le capteur est bon
            my $sensor_type = getsnmpOID($snmp,"$oid_rpdu2_sensors_type.$index");
    
            my $str = "Sensor $index. name : $sensor_name type : ".$rpduSensorType{$sensor_type};
            if( $type eq 'humidity' && $sensor_type != 2 )
            {
	            $current_state = 'UNKNOWN';
                $current_return = 3;
                $return = priority_state($return,$current_state);
                $str .= " doesn't have humidity capabilities";
            }
            elsif ($sensor_type > 2 )
            {
                $current_state = 'UNKNOWN';
                $current_return = 3;
                $return = priority_state($return,$current_state);
                $str .= " hasn't correct data";
            }
            
            if( $current_return == 3 )
            {
                if( $return == 0 )
                {
                    $state = "UNKNOWN";
	                $return = 3;
                }
                print $current_state . $str . "\n";
                next;        
            }

            # On récupére le reste des données
            my $unit;
            my $sensor_val;
            my $sensor_status;
            if( $type eq 'temperature' )
            {
                $unit = ' degrees(C)';
                $sensor_val     = getsnmpOID($snmp,"$oid_rpdu2_sensors_temp.$index");
                $sensor_val = $sensor_val/10;
                $sensor_status  = getsnmpOID($snmp,"$oid_rpdu2_sensors_temp_status.$index");
            }
            else
            {
                $unit = '%';
                $sensor_val     = getsnmpOID($snmp,"$oid_rpdu2_sensors_humi.$index");
                $sensor_status  = getsnmpOID($snmp,"$oid_rpdu2_sensors_humi_status.$index");
            }

            # Configuration du str
            my $sensor_status_str = $rpduSensorStatus{$sensor_status};
            $str .= " value : $sensor_val$unit status : $sensor_status_str";            

            # Analyse du resultat
            if( $sensor_status == '4' )
            {
                $current_state = 'OK';
                $current_return = 0;
            }
            elsif( $sensor_status == '3' || $sensor_status == '5' )
            {
                $current_state = 'WARNING';
                $current_return = 1;
            }
            elsif( $sensor_status == '2' || $sensor_status == '6' )
            {
                $current_state = 'CRITICAL';
                $current_return = 2;
            }
            else{
                $current_state = 'UNKNOWN';
                $current_return = 3; 
            }

            $str = "$current_state - ".$str;
            print "$str\n";

            $return = priority_state($return,$current_return);
        }
    }   
   if( $count == 0 )
    {
        if( $name ne '' )
        {
            print "UNKNOWN - Sensor $name not found\n";
        }
        else
        {
            print "UNKNOWN - NO sensors found\n";
        }
        $return = 3;
    }
}

exit($return);
