#!/usr/bin/perl
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
#use lib "/root/libperl/"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Math::Round;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;

#nom du programme & description
my $PROGNAME = "check_snmp_cisco_switch.pl";
my $PROGDESC = "Cisco switches by SNMP";
my $PROGVERSION = "1.0";
#Chargement du timer par défaut et des erreurs
# a installer apt install libmath-round-perl
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

ESIA::PLUGIN::arm_end_signal($TIMEOUT);

Getopt::Long::Configure( "bundling" );
my ($opt_h,$opt_V);
my $community = "public";
my $snmp_version = 1;
my ($hostname);
my $warning=80;
my $critical=90;
my $return =0;
my $type;
my $showIgnored;

my $oid_sysObjectOID                = "1.3.6.1.2.1.1.2.0";

# CISCO-ENVMON-MIB
my $oid_ciscoFanDesc 		= "1.3.6.1.4.1.9.9.13.1.4.1.2"; 		#Description
my $oid_ciscoFanStatut 		= "1.3.6.1.4.1.9.9.13.1.4.1.3"; 		#Status normal(1), warning(2), critical(3), shutdown(4), notPresent(5), notFunctioning(6)
my $oid_ciscoPowerDesc 		= "1.3.6.1.4.1.9.9.13.1.5.1.2"; 		#Description
my $oid_ciscoPowerStatut    = "1.3.6.1.4.1.9.9.13.1.5.1.3"; 		#Status normal(1), warning(2), critical(3), shutdown(4), notPresent(5), notFunctioning(6)
my $oid_ciscoTempDesc 		= "1.3.6.1.4.1.9.9.13.1.3.1.2"; 		#Description
my $oid_ciscoTempStatut     = "1.3.6.1.4.1.9.9.13.1.3.1.6";         #Status normal(1), warning(2), critical(3), shutdown(4), notPresent(5), notFunctioning(6)
my $oid_ciscoTempValue      = "1.3.6.1.4.1.9.9.13.1.3.1.3";         #Value


# Cisco CISCO-ENTITY-SENSOR-MIB
my $oid_cisco_ios_xe_physicaldescr      = "1.3.6.1.2.1.47.1.1.1.1.7";
my $oid_cisco_ios_xe_type     	    	= "1.3.6.1.4.1.9.9.91.1.1.1.1.1";
my $oid_cisco_ios_xe_precision	  	    = "1.3.6.1.4.1.9.9.91.1.1.1.1.3";
my $oid_cisco_ios_xe_value     	    	= "1.3.6.1.4.1.9.9.91.1.1.1.1.4";
my $oid_cisco_ios_xe_status     	 	= "1.3.6.1.4.1.9.9.91.1.1.1.1.5";   #Status normal(1), unavailable(2), nonoperational(3)
my $oid_cisco_ios_xe_ThresholdEval     	= "1.3.6.1.4.1.9.9.91.1.2.1.1.5";   # 1 evaluation true, 2 faux

# Cisco OLD-CISCO-CHASSIS-MIB
my $oid_cisco_chassis_card_descr		= "1.3.6.1.4.1.9.3.6.11.1.3"; 		# Chassis card description
my $oid_cisco_chassis_card_slot		    = "1.3.6.1.4.1.9.3.6.11.1.7"; 		# Chassis card slot number
my $oid_cisco_chassis_card_state		= "1.3.6.1.4.1.9.3.6.11.1.9";       # operating status of card - 1 : Not specified, 2 : Up, 3: Down, 4 : standby

## FAN
my $oid_ciscoEnvMonFanStatusDesc   = "1.3.6.1.4.1.9.6.1.101.83.1.1.1.2";
my $oid_ciscoEnvMonFanState        = "1.3.6.1.4.1.9.6.1.101.83.1.1.1.3";

## PSU
my $oid_ciscoEnvMonSupplyStatusDescr    = "1.3.6.1.4.1.9.6.1.101.83.1.2.1.2";
my $oid_ciscoEnvMonSupplyState          = "1.3.6.1.4.1.9.6.1.101.83.1.2.1.3";

## TEMP
my $oid_rlEnvFanDataTemp                = "1.3.6.1.4.1.9.6.1.101.83.5.1.1.2";

## LOAD
my $oid_cisco_cpu_5m = "1.3.6.1.4.1.9.2.1.58.0"; # Cisco CPU load (5min %)
my $oid_cisco_cpu_1m = "1.3.6.1.4.1.9.2.1.57.0"; # Cisco CPU load (1min %)
#my $oid_cisco_cpu_5s = "1.3.6.1.4.1.9.2.1.56.0"; # Cisco CPU load (5sec %)

my $oid_cisco_sb_cpu_5m = "1.3.6.1.4.1.9.6.1.101.1.9.0"; # Cisco Small Business CPU load (5min %)
my $oid_cisco_sb_cpu_1m = "1.3.6.1.4.1.9.6.1.101.1.8.0"; # Cisco Small Business CPU load (1min %)
#my $oid_cisco_sb_cpu_5s = "1.3.6.1.4.1.9.6.1.101.1.7.0"; # Cisco Small Business CPU load (5sec %)

## MEM
my $cisco_mem_pool      = "1.3.6.1.4.1.9.9.48.1.1.1"; # Cisco memory pool
my $cisco_index         = "1.3.6.1.4.1.9.9.48.1.1.1.2"; # memory pool name and index
my $cisco_valid         = "1.3.6.1.4.1.9.9.48.1.1.1.4"; # Valid memory if 1
my $cisco_used          = "1.3.6.1.4.1.9.9.48.1.1.1.5"; # Used memory
my $cisco_free          = "1.3.6.1.4.1.9.9.48.1.1.1.6"; # Free memory
#FLASH

my $oid_cisco_sb_FileTotalSizeOfFlash = '1.3.6.1.4.1.9.6.1.101.96.4.0'; # in bytes.
my $oid_cisco_sb_FileFreeSizeOfFlash = '1.3.6.1.4.1.9.6.1.101.96.5.0';  # in bytes.
my $oid_cisco_flash_name_partition = "1.3.6.1.4.1.9.9.10.1.1.4.1.1.10";
my $oid_cisco_flash_size = "1.3.6.1.4.1.9.9.10.1.1.4.1.1.4"; # in bytes.
my $oid_cisco_flash_free = "1.3.6.1.4.1.9.9.10.1.1.4.1.1.5"; # in bytes.
my $oid_cisco_flash_extended_size = "1.3.6.1.4.1.9.9.10.1.1.4.1.1.13"; # in bytes.
my $oid_cisco_flash_extended_free = "1.3.6.1.4.1.9.9.10.1.1.4.1.1.14"; # in bytes.

my %ciscoEnvMonState = (
    "1" => "normal(1)",
    "2" => "warning(2)",
    "3" => "critical(3)",
    "4" => "shutdown(4)",
    "5" => "notPresent(5)",
    "6" => "notFunctioning(6)",
);

my %ciscoEnvMonStateToEsia = (
    "1" => "OK",
    "2" => "WARNING",
    "3" => "CRITICAL",
    "4" => "CRITICAL",
    "5" => "IGNORE",
    "6" => "CRITICAL",
);
my %envmon_status_message  = ( "1"  => "normal","2" => "warning","3"  => "critical","4"  => "shutdown","5"  => "notPresent","6"  => "notFunctioning");
my %envmon_status = ( "1"  => "OK","2" => "WARNING","3"  => "CRITICAL","4"  => "UNKNOWN","5"  => "OK","6"  => "CRITICAL");

my @cisco_ios_xe_type_text = ("not_specified","other","unknown","voltsAC","voltsDC","amperes","watts","hertz","celsius","percent","rpm","cmm","truthvalue","specialEnum","dBm"); # correspondance de la valeur type

my %chassis_status_message  = ( "1"  => "Not specified","2" => "Up","3"  => "Down","4"  => "standby");
my %chassis_status = ( "1"  => "UNKNOW","2" => "OK","3"  => "CRITICAL","4"  => "OK");

# CISCO-ENTITY-SENSOR-MIB
sub getCiscoEntitySensorMib
{
	my($snmp,$type) = @_;
	my $ret = 0;
	my $out = "";
	my $found = 0;
	my $result = $snmp->get_table(Baseoid => $oid_cisco_ios_xe_physicaldescr);
    if (defined($result))
    {
    	$found = 1;
		foreach my $key ( keys %$result)
		{
			#on récupère l'index
			if($key=~ /([0-9]*)$/ )
			{
				my $index =$1;
				my $descr =$$result{"$key"};
				if($descr=~/$type/)
				{
					my $Sensorstate = ESIA::SNMP::get($snmp,"$oid_cisco_ios_xe_status.$index");
					my $state = "OK";
					my $message ="operational";
					if($Sensorstate ==2)
					{
						$message ="unavailable";
						$state = "WARNING";
						$ret = ESIA::PLUGIN::priority_state($ret,$ERRORS{$state});
					}
					elsif($Sensorstate==3)
					{
						$message ="nonoperational";
						$state = "CRITICAL";
						$ret = ESIA::PLUGIN::priority_state($ret,$ERRORS{$state});
					}
					else
					{
						my $value = ESIA::SNMP::get($snmp,"$oid_cisco_ios_xe_value.$index");
						my $type =ESIA::SNMP::get($snmp,"$oid_cisco_ios_xe_type.$index");
						$message.=" $value ".$cisco_ios_xe_type_text[$type];
						my $snmp_thresh= $snmp->get_table(Baseoid => "$oid_cisco_ios_xe_ThresholdEval.$index");
						my $sensorstreshstate = 0;
						#on évalue les données des capteurs
						foreach my $index ( keys %$snmp_thresh)
						{
							my $treshState =$$snmp_thresh{"$index"};
							if($treshState != 2)
							{
								$state = "CRITICAL";
								$ret = ESIA::PLUGIN::priority_state($ret,$ERRORS{$state});
								$message.= " ==> Threshold activate!";
								last;
							}

						}
					}
					$out .= "$state : $descr is $message \n";
				}
			}
		}
	}
	return $found,$ret,$out;
}
# CISCO-ENTITY-SENSOR-MIB
sub getCiscoOldChassis
{
	my($snmp,$type) = @_;
	my $ret = 0;
	my $out = "";
	my $found = 0;
    my $result = $snmp->get_table(Baseoid => $oid_cisco_chassis_card_descr);
    if (defined($result))
    {
    	$found = 1;
        #Test des variable de chassis
        foreach my $key ( keys %$result)
        {
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
		        my $descr =$$result{"$key"};
		        if($descr=~/$type/)
				{
					my $resultcard = $snmp->get_request("$oid_cisco_chassis_card_state.$index");
		        	my $chassisStatus =   ESIA::SNMP::get($snmp,"$oid_cisco_chassis_card_state.$index");
                	my $state = $chassis_status{$chassisStatus};
                	$out .= "$state : $descr is ".$chassis_status_message{$chassisStatus}."\n";
                	$ret = priority_state($ret,$ERRORS{$state});
                }
            }
        }
    }
    return $found,$ret,$out;
}
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::Session(@ARGV);
my $match;
GetOptions(
	"V"   => \$opt_V,	 "version"    => \$opt_V,
	"h"   => \$opt_h,	 "help"       => \$opt_h,
	"T=s" => \$type,	 "Type=s" => \$type,
	"m=s" => \$match,	 "match=s" => \$match,
	"w=i" => \$warning,	 "warning=i" => \$warning,
	"c=i" => \$critical, "critical=i" => \$critical,
    "show-ignore" => \$showIgnored,
);

# -h & --help print help
if ($opt_h)
{
    #affiche l'aide par défaut
    ESIA::SNMP::print_snmp_usage($PROGNAME,'-T [fan|psu|temp] [options]');
    my @arr;
    push(@arr,["-T","--type","type of test [fan,cpu,unit]" ]);
    push(@arr,["-w","--warning","warning threshold max in (degrees for temp, percent for cpu)" ]);
    push(@arr,["-c","--critical","critical threshold max in (degrees for temp, percent for cpu)"]);
    push(@arr,["","--show-ignore","Print also components/values ignored"]);
    ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
    exit $ERRORS{'OK'};
}

# -V & --version print version
if ($opt_V)
{
    print "$PROGNAME version $PROGVERSION\n";
    exit $ERRORS{'OK'};
}

#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

my $state = 'UNKNOWN';
my $output = "";

my $sysOid_value = ESIA::SNMP::get($snmp,$oid_sysObjectOID);
if( !defined( $sysOid_value ) )
{
	print ("UNKNOWN: SNMP no response. $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

#FOR Cisco>otherEnterprises>cisco_sb
if( $sysOid_value =~ /1\.3\.6\.1\.4\.1\.9\.6\.1/ )
{
    # WORK WITH :.3.6.1.4.1.9.9.10.1.1.4.1.1.10
    # - SG300-28PP 28-Port Gigabit PoE+ Managed Switch
    # - SG350X-48P 48-Port Gigabit PoE Stackable Managed Switch
    if( $type eq 'cpu' )
    {
        if( $warning > $critical )
        {
            print "UNKNOWN : You must specify correct warning,critical tresholds.\n";
            exit $ERRORS{UNKNOWN};
        }
        my $load_time;
        my $load_value;

        if( !defined($match) || $match eq '5' || $match eq '5m' || $match eq '300s' )
        {
            $load_time = 5;
            $load_value = ESIA::SNMP::get($snmp,"$oid_cisco_sb_cpu_5m");

        }
        elsif( $match eq '1' || $match eq '1m' || $match eq '60s' )
        {
            $load_time = 1;
            $load_value = ESIA::SNMP::get($snmp,"$oid_cisco_sb_cpu_1m");
        }
        else
        {
            print "UNKNOWN : You must specify correct -m, --match for load ( 1,5 | 1m,5m | 60s,300s )\n";
            exit $ERRORS{UNKNOWN};
        }

        if( $load_value < $warning )
        {
            $state = 'OK';
            $output = "$state: CPU used $load_value% (load average $load_time minutes) (< $warning%)";
        }
        elsif( $load_value < $critical )
        {
            $state = 'WARNING';
            $output = "$state: CPU used $load_value% (load average $load_time minutes) (>= $warning%)";
        }
        else
        {
            $state = 'CRITICAL';
            $output = "$state: CPU used $load_value% (load average $load_time minutes) (>= $critical%)";
        }
		$output.="|cpu=$load_time;warning=$warning;critical=$critical;\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
    # WORK WITH :
    # - SG300-28PP 28-Port Gigabit PoE+ Managed Switch
    # - SG350X-48P 48-Port Gigabit PoE Stackable Managed Switch
    elsif($type eq 'mem')
    {
 		print "OK : Cisco switch $type not availaible on this cisco switch model\n";
		exit $ERRORS{OK};
    }
    elsif($type eq 'fan')
    {
        my $state = 'OK';
        my $result = $snmp->get_table( $oid_ciscoEnvMonFanStatusDesc);

        foreach my $key ( sort keys %$result)
        {
            if($key=~/\.([0-9]*)$/)
            {
                my $id = $1;
                my $description = $$result{"$key"};
                if( !defined($match) || $description =~ /$match/)
                {
                    my $state_int = ESIA::SNMP::get($snmp,"$oid_ciscoEnvMonFanState.$id");
                    my $state_desc = $ciscoEnvMonState{$state_int};
                    $state = $ciscoEnvMonStateToEsia{$state_int};

                    if( !$showIgnored && $state eq 'IGNORE')
                    {
                        last;
                    }

                    $output .= "$state: FAN $description state is $state_desc\n";
                    if( $state eq 'IGNORE' )
                    {
                        $state = 'OK';
                    }
                }
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
        }
    }
    # WORK WITH :
    # - SG350X-48P 48-Port Gigabit PoE Stackable Managed Switch
    # NOT WORKING WITH :
    # - SG300-28PP 28-Port Gigabit PoE+ Managed Switch
    elsif($type eq 'psu')
    {
        my $state = 'OK';
        my $result = $snmp->get_table( $oid_ciscoEnvMonSupplyStatusDescr);

        foreach my $key ( sort keys %$result)
        {
            if($key=~/\.([0-9]*)$/)
            {
                my $id = $1;
                my $description = $$result{"$key"};
                if( !defined($match) || $description =~ /$match/)
                {
                    my $state_int = ESIA::SNMP::get($snmp,"$oid_ciscoEnvMonSupplyState.$id");
                    my $state_desc = $ciscoEnvMonState{$state_int};
                    $state = $ciscoEnvMonStateToEsia{$state_int};

                    if( !$showIgnored && $state eq 'IGNORE')
                    {
                        last;
                    }

                    $output .= "$state: PSU $description state is $state_desc\n";
                    if( $state eq 'IGNORE' )
                    {
                        $state = 'OK';
                    }
                }
                $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
            }
        }
		#OID no present on this model
		if($output eq "")
		{
			print "OK : Cisco switch $type not availaible on this cisco switch model\n";
			exit $ERRORS{OK};
		}
    }
    # WORK WITH :
    # - SG350X-48P 48-Port Gigabit PoE Stackable Managed Switch
    # NOT WORKING WITH :
    # - SG300-28PP 28-Port Gigabit PoE+ Managed Switch
    elsif($type eq 'temp')
    {
		if( $warning == 80 && $critical == 90)
		{
			$warning=60;
			$critical=70;
		}
        if( $warning > $critical )
        {
            print "UNKNOWN : You must specify correct warning,critical tresholds.\n";
            exit $ERRORS{UNKNOWN};
        }

        my $state = 'OK';
        my $result = $snmp->get_table( $oid_rlEnvFanDataTemp);

		my $nb = 0;
		my $min = 0;
		my $sum = 0;
		my $max = 0;

        foreach my $key ( sort keys %$result)
        {
            if($key=~/\.([0-9]*)$/)
            {
                my $id = $1;
                my $temp = $$result{"$key"};
                if( !defined($match) || "$id" eq $match )
                {
                    if( $temp == 0 )
                    {
                        if( !$showIgnored )
                        {
                            last;
                        }
                        else
                        {
                            $state = 'OK';
                            $output .= "$state: TEMP $id value $temp degrees";
                        }
                    }
                    else
                    {
                        if( $temp < $warning )
                        {
                            $state = 'OK';
                            $output .= "$state: TEMP $id value $temp degrees (< $warning)";
                        }
                        elsif( $temp < $critical )
                        {
                            $state = 'WARNING';
                            $output .= "$state: TEMP $id value $temp degrees (> $warning)";
                        }
                        else
                        {
                            $state = 'CRITICAL';
                            $output .= "$state: TEMP $id value $temp degrees (> $critical)";
                        }

						if( $nb == 0 )
						{
							$min = $sum = $max = $temp;
						}
						else{
							if( $min > $temp )
							{
								$min = $temp;
							}
							if( $max < $temp )
							{
								$max = $temp;
							}
							$sum+= $temp;
						}
						$nb++;
                    }
                    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                }
				if( $nb != 0 )
				{
					my $avg = round($sum/$nb);
					$output.="|min_value=$min;temp=$avg;max_value=$max;warning=$warning;critical=$critical\n";
				}
            }
        }
		#OID no present on this model
		if($output eq "")
		{
			print "OK : Cisco switch $type not availaible on this cisco switch model\n";
			exit $ERRORS{OK};
		}
    }
   # WORK WITH :
	# - SG300-28PP 28-Port Gigabit PoE+ Managed Switch
    # - SG350X-48P 48-Port Gigabit PoE Stackable Managed Switch
    elsif($type eq 'flash')
    {
        if( $warning > $critical )
        {
            print "UNKNOWN : You must specify correct warning,critical tresholds.\n";
            exit $ERRORS{UNKNOWN};
        }
		if( defined($match) && $match ne "flash" )
		{
			print "UNKNOWN : Cisco switch $type status retrieval failed. don't use match or use -m, --match falsh for this model\n";
			exit $ERRORS{UNKNOWN};
		}

		my $mem_total = ESIA::SNMP::get($snmp,"$oid_cisco_sb_FileTotalSizeOfFlash");
		my $mem_free = ESIA::SNMP::get($snmp,"$oid_cisco_sb_FileFreeSizeOfFlash");


		if( $mem_total <= 0 || $mem_free < 0)
		{
			print "UNKNOWN : Cisco switch $type status retrieval failed.\n";
			exit $ERRORS{UNKNOWN};
		}

		my $mem_usage = $mem_total - $mem_free;

		my $mem_usage_per = ($mem_usage / $mem_total) * 10000;
		$mem_usage_per = round($mem_usage_per);
		$mem_usage_per/=100;


		my $mem_total_h = round($mem_total/1024);
		my $mem_usage_h = round($mem_usage/1024);
		my $mem_unit = 'KB';

		if( $mem_total_h > 10000 )
		{
			$mem_total_h = round($mem_total/1024);
			$mem_usage_h = round($mem_usage/1024);
			$mem_unit = 'MB';
		}

   		if( $mem_usage_per < $warning )
        {
            $state = 'OK';
            $output = "$state: FLASH usage $mem_usage_per% ($mem_usage_h/$mem_total_h $mem_unit) (< $warning%)";
        }
        elsif( $mem_usage_per < $critical )
        {
            $state = 'WARNING';
            $output = "$state: FLASH usage $mem_usage_per% ($mem_usage_h/$mem_total_h $mem_unit) (>= $warning%)";
        }
        else
        {
            $state = 'CRITICAL';
            $output = "$state: FLASH usage $mem_usage_per% ($mem_usage_h/$mem_total_h $mem_unit) (>= $critical%)";
        }
		$output.="|percent=$mem_usage_per;usage=$mem_usage;total=$mem_total;warning=$warning;critical=$critical;\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
    else
    {
        print "UNKNOWN : Incorrect parameter -T, --type $type.\n";
        ESIA::SNMP::print_snmp_usage($PROGNAME,'-T [fan|psu|temp]');
        exit $ERRORS{UNKNOWN};
    }
}
#FOR CiscoProducts
elsif( $sysOid_value =~ /1\.3\.6\.1\.4\.1\.9\.1/ )
{
	# WORK WITH :
	# - C2950 Software (C2950-I6Q4L2-M)
    # - C3750 Software (C3750-IPBASEK9-M),
    if( $type eq 'cpu' )
    {
        if($warning > $critical )
        {
            print "UNKNOWN : You must specify correct warning,critical tresholds.\n";
            exit $ERRORS{UNKNOWN};
        }
        my $load_time;
        my $load_value;

        if( !defined($match) || $match eq '5' || $match eq '5m' || $match eq '300s' )
        {
            $load_time = 5;
            $load_value = ESIA::SNMP::get($snmp,"$oid_cisco_cpu_5m");

        }
        elsif( $match eq '1' || $match eq '1m' || $match eq '60s' )
        {
            $load_time = 1;
            $load_value = ESIA::SNMP::get($snmp,"$oid_cisco_cpu_1m");
        }
        else
        {
            print "UNKNOWN : You must specify correct -m, --match for load ( 1,5 | 1m,5m | 60s,300s )\n";
            exit $ERRORS{UNKNOWN};
        }

        if( $load_value < $warning )
        {
            $state = 'OK';
            $output = "$state: CPU used $load_value% (load average $load_time minutes) (< $warning%)";
        }
        elsif( $load_value < $critical )
        {
            $state = 'WARNING';
            $output = "$state: CPU used $load_value% (load average $load_time minutes) (>= $warning%)";
        }
        else
        {
            $state = 'CRITICAL';
            $output = "$state: CPU used $load_value% (load average $load_time minutes) (>= $critical%)";
        }
		$output.="|cpu=$load_time;warning=$warning;critical=$critical;\n";
        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
    }
    elsif( $type eq 'mem' )
    {
        if( !defined($match))
        {
        	$match='.*';
        }
        my $result = $snmp->get_table( $cisco_index);
        my $graphdata="";
        foreach my $key ( sort keys %$result)
        {
            if($key=~/\.([0-9]*)$/)
            {
                my $state = 'OK';
                my $id = $1;
                my $description = $$result{"$key"};
                if($description=~/$match/)
                {
					my $used = ESIA::SNMP::get($snmp,"$cisco_used.$id");
					my $free = ESIA::SNMP::get($snmp,"$cisco_free.$id");
					my $total = $used + $free;
					my $used_percent = ($used/$total)*100;
					$used_percent = sprintf("%.0f", $used_percent);
					if( $used_percent < $warning )
					{
						$state = 'OK';
						$output .= "$state: $description memory used $used_percent% (< $warning%)\n";
					}
					elsif( $used_percent < $critical )
					{
						$state = 'WARNING';
						$output .= "$state: $description memory used $used_percent% (>= $warning%)\n";
					}
					else
					{
						$state = 'CRITICAL';
						$output .= "$state: $description memory used $used_percent% (>= $critical%)\n";
					}
					$graphdata .="$description=$used_percent;";
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
                }
            }
        }
       	$output.="|$graphdata;warning=$warning;critical=$critical;\n";
    }
    elsif($type eq 'fan')
    {
        my $state = 'OK';
        my $found = 0;
        my $result = $snmp->get_table( $oid_ciscoFanDesc);
		if (defined($result))
		{
			#
			# CISCO-ENVMON-MIB
			#
			$found = 1;
			foreach my $key ( keys %$result)
			{
				#on récupère l'index
				if($key=~ /([0-9]*)$/ )
				{
				    my $index =$1;
					my $descr =$$result{"$key"};
					my $fanstatus =  ESIA::SNMP::get($snmp,"$oid_ciscoFanStatut.$index");
				    my $state = $envmon_status{$fanstatus};
				    $output .= "$state : FAN $descr is ".$envmon_status_message{$fanstatus}."\n";
				    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});

				}
			}
		}
		else
		{
			#
			# Cisco CISCO-ENTITY-SENSOR-MIB
			#
			my ($f,$ret,$out) = getCiscoEntitySensorMib($snmp,'Fan|FAN|fan');
			if($f == 1)
			{
				$return = ESIA::PLUGIN::priority_state($return,$ret);
				$output=$out;
			}
			else
			{
				my ($f,$ret,$out) = getCiscoOldChassis($snmp,'Fan|FAN|fan');
				if($f == 1)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ret);
					$output=$out;
				}
				else
				{
				    print "OK : model not support fans status\n";
					$snmp->close;
					exit $ERRORS{"OK"};
				}
			}
		}
    }
    elsif($type eq 'psu')
    {
        my $state = 'OK';
        my $found = 0;
        my $result = $snmp->get_table( $oid_ciscoPowerDesc);
		if (defined($result))
		{
			#
			# CISCO-ENVMON-MIB
			#
			$found = 1;
			foreach my $key ( keys %$result)
			{
				#on récupère l'index
				if($key=~ /([0-9]*)$/ )
				{
				    my $index =$1;
					my $descr =$$result{"$key"};
					my $powerstatus =  ESIA::SNMP::get($snmp,"$oid_ciscoPowerStatut.$index");
            		my $state = $envmon_status{$powerstatus};
				    $output .= "$state : PSU $descr is ".$envmon_status_message{$powerstatus}."\n";
				    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});

				}
			}
		}
		else
		{
			#
			# Cisco CISCO-ENTITY-SENSOR-MIB
			#
			my ($f,$ret,$out) = getCiscoEntitySensorMib($snmp,'PSU|psu');
			if($f == 1)
			{
				$return = ESIA::PLUGIN::priority_state($return,$ret);
				$output=$out;
			}
			else
			{
				my ($f,$ret,$out) = getCiscoOldChassis($snmp,'PSU|psu');
				if($f == 1 && $out ne "")
				{
					$return = ESIA::PLUGIN::priority_state($return,$ret);
					$output=$out;
				}
				else
				{
				    print "OK : model not support PSU status\n";
					$snmp->close;
					exit $ERRORS{"OK"};
				}
			}
		}
    }
    elsif($type eq 'temp')
    {
		if( $warning == 80 && $critical == 90)
		{
			$warning=60;
			$critical=70;
		}
        if( $warning > $critical )
        {
            print "UNKNOWN : You must specify correct warning,critical tresholds.\n";
            exit $ERRORS{UNKNOWN};
        }
        my $state = 'OK';
        my $found = 0;
        my $result = $snmp->get_table( $oid_ciscoTempDesc);
		if (defined($result))
		{
			#
			# CISCO-ENVMON-MIB
			#
			$found = 1;
			foreach my $key ( keys %$result)
			{
				#on récupère l'index
				if($key=~ /([0-9]*)$/ )
				{
				    my $index =$1;
					my $descr =$$result{"$key"};
					my $tempstatus =  ESIA::SNMP::get($snmp,"$oid_ciscoTempStatut.$index");
				    if($tempstatus ne 'noSuchObject')
				    {
						my $tempval =  ESIA::SNMP::get($snmp,"$oid_ciscoTempValue.$index");
				        my $state = $envmon_status{$tempstatus};
				        $output .="$state : $descr is ".$envmon_status_message{$tempstatus}.", $tempval\n";
				    	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
				    }

				}
			}
		}
		else
		{
			#
			# Cisco CISCO-ENTITY-SENSOR-MIB
			#
			my ($f,$ret,$out) = getCiscoEntitySensorMib($snmp,'Temp|temp');
			if($f == 1 && $out ne "")
			{
				$return = ESIA::PLUGIN::priority_state($return,$ret);
				$output=$out;
			}
			else
			{
				my ($f,$ret,$out) = getCiscoOldChassis($snmp,'Temp|temp');
				if($f == 1)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ret);
					$output=$out;
				}
				else
				{
				    print "OK : model not support temperature status\n";
					$snmp->close;
					exit $ERRORS{"OK"};
				}
			}
		}
    }
    elsif($type eq 'flash')
    {
        if( !defined($match))
        {
        	$match='.*';
        }
		if( $warning == 80 && $critical == 90)
		{
			$warning=95;
			$critical=98;
		}
        my $result = $snmp->get_table( $oid_cisco_flash_name_partition);
        my $graphdata="";
        foreach my $key ( keys %$result)
        {
        	$state="OK";
			if($key=~ /3.6.1.4.1.9.9.10.1.1.4.1.1.10.([0-9\.]*)$/ )
			{
				my $index =$1;
				my $descr =$$result{"$key"};
				if($descr=~/$match/)
                {
					my $size = ESIA::SNMP::get($snmp,"$oid_cisco_flash_extended_size.$index");
					my $free =0;
					if($size eq 'noSuchObject')
					{
						$size =  ESIA::SNMP::get($snmp,"$oid_cisco_flash_size.$index");
						$free =  ESIA::SNMP::get($snmp,"$oid_cisco_flash_free.$index");
					}
					else
					{
						$free =  ESIA::SNMP::get($snmp,"$oid_cisco_flash_extended_free.$index");
					}
					my $used = $size - $free;
					my $percent = ($used/$size)*100;
					$percent = sprintf("%.0f", $percent);
					my $message = " < $warning%";
					if($percent < $warning)
					{
						$state="OK";
					}
					elsif($percent < $critical)
					{
						$state="WARNING";
						$message = " > $warning%";
					}
					else
					{
						$state="CRITICAL";
						$message = " > $critical%";
					}
					$output .="$state : $descr use $percent% $message\n";
					$graphdata="percent=$percent;usage=$used;total=$size";
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
				}
			}
        }
        if($output eq "")
        {
        	$output .="UNKNOWN : flash $match not found\n";
        	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
        }
        $output.="|$graphdata;warning=$warning;critical=$critical;\n";
    }
}
else
{
    print "UNKNOWN : Device type not implemented for sysobject id $sysOid_value\n";
    exit $ERRORS{UNKNOWN};
}

if($output eq "")
{
    print "UNKNOWN : Cisco switch $type status retrieval failed.\n";
    exit $ERRORS{UNKNOWN};
}
else
{
    print $output;
    exit $return;
}
