#!/usr/bin/perl -w
############################## check_snmp_ciscoxr_fru #################
my $Version='1.0';
# Date : Sept 18 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use POSIX qw(strftime);
use Switch;
use strict;
use Net::SNMP;
use Getopt::Long;
my $PROGNAME='check_snmp_ciscoxr_fru.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $warning='70';
my $port='161';
my $critical='80';
my $result;
my @tab_value_read;

my $cisco_descr         = "1.3.6.1.2.1.1.1.0";
my $cisco_sensor_descr =  "1.3.6.1.2.1.47.1.1.1.1.2"; #STRING: "ASR 9000 8-port 100GE Consumption Model Line Card"  
my $cefcModuleAdminStatus = "1.3.6.1.4.1.9.9.117.1.2.1.1.1"; # (cefcModuleAdminStatus) => enabled(1), disabled(2), reset(3), outOfServiceAdmin(4)
my $cefcModuleOperStatus = "1.3.6.1.4.1.9.9.117.1.2.1.1.2"; # => unknown(1), powerUp(2), parityError(3), clearConfigReset(4), manualReset(5), watchDogTimeoutReset(6), 
#resourceOverflowReset(7), missingTaskReset(8), lowVoltageReset(9), controllerReset(10), systemReset(11), switchoverReset(12), upgradeReset(13), downgradeReset(14), 
#cacheErrorReset(15), deviceDriverReset(16), softwareExceptionReset(17), restoreConfigReset(18), abortRevReset(19), burnBootReset(20), standbyCdHealthierReset(21), 
#nonNativeConfigClearReset(22), memoryProtectionErrorReset(23)


sub print_help() {
  printf "check SNMP interface via SNMP.\n";
  print "\n";
  print_usage();
  print "\n";
  print " -H (--hostname)   Hostname to query - (required)\n";
  print " -D (--domain)   UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
  print " -C (--community)  SNMP read community (defaults to public)\n";
  print " -v (--snmp_version) 1 for SNMP v1\n";
  print "        2 for SNMP v2c (default)\n";
  print " -w  warning value in % (default: 70)\n";
  print " -c  critical value in % (default: 80)\n";
  print " -V (--version)  Plugin version\n";
  print " -h (--help) usage help\n";
  print "\n";
}

sub print_usage() {
  print "Usage: \n";
  print " $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] [-w warning] [-c critical]\n";
  print " $PROGNAME [-h | --help]\n";
  print " $PROGNAME [-V | --version]\n";
}

sub readOID {
    my($snmp,$oid) = @_;
  my $result = $snmp->get_request("$oid");
  return $$result{"$oid"};
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}
########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
  "V"   => \$opt_V,        "version"        => \$opt_V,
  "D=s" => \$domain,       "domain=s"       => \$domain,
  "h"   => \$opt_h,        "help"           => \$opt_h,
  "C=s" => \$community,    "community=s"    => \$community,
  "H=s" => \$hostname,     "hostname=s"     => \$hostname,
  "v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
);

# -h & --help print help
if($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

my ($snmp, $snmperror);
if ($snmp_version == 2)
{
  ($snmp, $snmperror) = Net::SNMP->session(
      -hostname   => $hostname,
      -version    => 'snmpv2c',
      -domain     => $domain,
      -port       => $port,
      -community  => $community,
      -timeout    => 2
    );
}
else
{
  ($snmp, $snmperror) = Net::SNMP->session(
      -hostname  => $hostname,
      -version   => 'snmpv1',
      -domain    => $domain,
      -port      => $port,
      -community => $community,
      -timeout   => 2
    );
}

my $return=$ERRORS{OK};
if(!defined($snmp))
{
  print " Error SNMP not defined !!! \n";
  exit $ERRORS{"UNKNOWN"};
}
else
{
  my $resultat = $snmp->get_table(Baseoid => $cefcModuleAdminStatus);
  if (defined($resultat))
  {
    foreach my $key ( keys %$resultat)
    {
      my $id = ($key =~ /1.3.6.1.4.1.9.9.117.1.2.1.1.1.(\S+)/);
      $id = $1; 
      if(defined($id))
      {
        my $result_cisco_sensor_descr = readOID($snmp,"$cisco_sensor_descr.$id");
        my $result_cefcModuleAdminStatus = readOID($snmp,"$cefcModuleAdminStatus.$id");
        my $result_cefcModuleOperStatus = readOID($snmp,"$cefcModuleOperStatus.$id");   

        if($result_cefcModuleAdminStatus == 1)
        {
          $result_cefcModuleAdminStatus = "enabled";

          my $state = 0;
          switch ($result_cefcModuleOperStatus) 
          {
            case 1  { $state = "UNKNOWN"; $result_cefcModuleOperStatus = "unknown";  $return = priority_state($return,$ERRORS{UNKNOWN}) }
            case 2  { $state = "OK"; $result_cefcModuleOperStatus = "powerUp";  }
            case 3  { $state = "CRITICAL"; $result_cefcModuleOperStatus = "parityError";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 4  { $state = "CRITICAL"; $result_cefcModuleOperStatus = "clearConfigReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 5  { $state = "CRITICAL"; $result_cefcModuleOperStatus = "manualReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 6  { $state = "CRITICAL"; $result_cefcModuleOperStatus = "watchDogTimeoutReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 7  { $state = "CRITICAL"; $result_cefcModuleOperStatus = "resourceOverflowReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 8  { $state = "CRITICAL"; $result_cefcModuleOperStatus = "missingTaskReset"; $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 9  { $state = "CRITICAL"; $result_cefcModuleOperStatus = "lowVoltageReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 10 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "controllerReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 11 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "systemReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 12 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "switchoverReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 13 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "upgradeReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 14 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "downgradeReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 15 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "cacheErrorReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 16 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "deviceDriverReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 17 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "softwareExceptionReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 18 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "restoreConfigReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 19 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "abortRevReset"; $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 20 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "burnBootReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 21 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "standbyCdHealthierReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 22 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "nonNativeConfigClearReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            case 23 { $state = "CRITICAL"; $result_cefcModuleOperStatus = "memoryProtectionErrorReset";  $return = priority_state($return,$ERRORS{CRITICAL}) }
            else    { $state = "UNKNOWN"; $result_cefcModuleOperStatus = "unknown";  $return = priority_state($return,$ERRORS{UNKNOWN})}            
          }
          print "$state - $result_cisco_sensor_descr is $result_cefcModuleAdminStatus and $result_cefcModuleOperStatus \n";
        } 
        else
        {
          switch ($result_cefcModuleAdminStatus) 
          {
            case 2  { $result_cefcModuleAdminStatus= "disabled"; }
            case 3  { $result_cefcModuleAdminStatus= "reset"; }
            case 4  { $result_cefcModuleAdminStatus= "outOfServiceAdmin"; }
            else { $result_cefcModuleAdminStatus= "unknown"; }
          }

          print "$result_cisco_sensor_descr is $result_cefcModuleAdminStatus \n";
        }      
        
      }
    }
  }
  else
  {
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    #si l'host ne répond pas on blocke le script
    if (!defined($result))
    {
      if($snmp_version == 2)
      {
        ($snmp, $snmperror) = Net::SNMP->session(
            -hostname  => $hostname,
            -domain    => $domain,
            -version   => 'snmpv2c',
            -port      => $port,
            -community => $community
          );

        $result = $snmp->get_request("$cisco_descr");
        if (!defined($result))
        {
          printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
          $snmp->close;
          exit $ERRORS{"UNKNOWN"};
        }
      }
      else
      {
        printf("UNKNOWN ERROR: Description table : %s.\n",$snmp->error);
        $snmp->close;
        exit $ERRORS{"UNKNOWN"};
      }
    }
    exit $ERRORS{"UNKNOWN"};
  }
}
exit $return;
