#!/usr/bin/php
<?php
$DEFAULT_TIMEOUT = 50;
$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);

$short_opts="hH:C:w:c:";
$long_opts=array("help","host:","community:","warning:","critical:");

$options=getopt($short_opts,$long_opts);

function print_usage($bad_usage){
        echo 'USAGE : '.basename($_SERVER['PHP_SELF'])." -H <HOST> -C <COM> -w <WARN> -c <CRITIC> [-t <TIMEOUT>]\n";
        if($bad_usage) echo 'HELP  : '.basename($_SERVER['PHP_SELF'])." -h\n";
}

function print_help(){
        $col=20;
        print_usage(false);
        echo "\n";
        echo "Check the current number of established TCP connections\n\n";
        echo "\t-H, ".str_pad("--host=HOST",$col)               ."Address or domain name\n";
        echo "\t-C, ".str_pad("--community=COM",$col)           ."SNMP community\n";
//        echo "\t-t, ".str_pad("--timeout=SECONDES",$col)        ."Timeout in seconds\n";
        echo "\t-w, ".str_pad("--warning=INTEGER",$col)         ."Max tolerated temperature for warning\n";
        echo "\t-c, ".str_pad("--critical=INTEGER",$col)        ."Max tolerated temperature for critical\n";
        echo "\n";
        echo "\t-h, ".str_pad("--help",$col)                    ."Print this help\n";
        echo "\n";
}


function exit_state($state,$msg=null,$usage=false){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];

        $ret=$state;
        if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
        else $state=$STR_STATUS[$ret];

        if( $msg != null ) echo "$state - $msg\n";
        if( $usage ) print_usage(true);
        exit($ret);
}

function getParam($name1,$name2,$default=null){
        global $options;

        if( isset( $options[$name1] ) ) return $options[$name1];
        if( isset( $options[$name2] ) ) return $options[$name2];
        return $default;
}

function getTableIdxValues($host,$community,$oid){
	$data = @snmprealwalk($host,$community,$oid);
	if( $data === FALSE )  {
        	$message = error_get_last();
        	$message = $message["message"];
        	$message = trim(explode(":",$message)[1]);
        	exit_state(3,$message,false);
	}
	
	$regex = "(\w+): (.*)";

	$values = array();
	foreach($data as $key => $row){
		if(preg_match("/$regex/",$row,$matches)){
			$id = array_pop(explode('.',$key));
			$values[$id] = trim($matches[2],'"');
		}
	}
	return $values;	
}

function getSNMPValue($host,$com,$oid){
	$ret = @snmpget($host,$com,$oid);
	if( $ret === FALSE )  {
                $message = error_get_last();
                $message = $message["message"];
                $message = trim(explode(":",$message)[1]);
                exit_state(3,$message,false);
        }

	if(preg_match("/(\w+): (.*)/",$ret,$matches)){
                        return trim($matches[2],'"');
        }

	exit_state(3,"Unable to get SNMP data",false);
}

function getSensorValue($host,$com,$key){
	$val = getSNMPValue($host,$com,"1.3.6.1.4.1.9.9.91.1.1.1.1.4.$key");
	$mult = getSNMPValue($host,$com,"1.3.6.1.4.1.9.9.91.1.1.1.1.2.$key");
	$prec = getSNMPValue($host,$com,"1.3.6.1.4.1.9.9.91.1.1.1.1.3.$key");

	if($mult != 9 ){
		$exp = ($mult-9)*3;
		$digit = pow(10,$exp);
		$val = $val*$digit;
	}

	if( $prec > 0 ){
		$val /= (10*$prec);
	}

	return $val;
}

if ( isset($options["help"]) || isset($options["h"]) ){
        print_help();
        exit($INT_STATUS["UNKNOWN"]);
}

$host = getParam("H","host");
$community = getParam("C","community");
$timeout = getParam("t","timeout",$DEFAULT_TIMEOUT);
$warning = getParam("w","warning",60);
$critical = getParam("c","critical",70);

if( $host == null ) exit_state(3,"You forget to set host address -H.",false);
if( $community == null ) exit_state(3,"You forget to set community -C.",false);
if( !is_numeric($warning) || !is_numeric($critical) || $warning < 0) exit_state(3,"warning and critical values must be integers.",false);
if( $timeout < 0 ) exit_state(3,"The timeout must be > 0.",false);
if( $warning >= $critical ) exit_state(3,"warning must be lower than critical.",false);

//On récupére tout les sensors
$sensors_type_arr = getTableIdxValues($host,$community,'1.3.6.1.4.1.9.9.91.1.1.1.1.1');

//On ne garde que les sensors type température
$temp_sensors = array();
foreach($sensors_type_arr as $key => $sensor_type){
	if( $sensor_type == 8 ){
		$temp_sensors[]= $key;
	}
}

//On vérifie lesquels sont de type hot
$hot_sensors = array();
foreach($temp_sensors as $key){
	$descr = getSNMPValue($host,$community,"iso.3.6.1.2.1.47.1.1.1.1.2.$key");
	if( preg_match('/^Hot/',$descr)){
			$hot_sensors[$key] = $descr; 
	}
}

//On récupére les valeurs des sensors
$sensors_values = array();
foreach($hot_sensors as $key => $descr){
	$sensors_values[] = getSensorValue($host,$community,$key);
}

$nbre = count ($sensors_values);
$vmin = min($sensors_values);
$vmax = max($sensors_values);
$vavg = round(array_sum($sensors_values) / $nbre,1);

if( $vmax < $warning ){
	echo "OK - Temperature min={$vmin}° avg={$vavg}° max={$vmax}° (w:$warning,c:$critical) / $nbre sensors. \n";
	exit($INT_STATUS["OK"]);
}
else if( $vmax < $critical ){
	echo "WARN - Temperature min={$vmin}° avg={$vavg}° max={$vmax}° (w:$warning,c:$critical) / $nbre sensors. \n";
	exit($INT_STATUS["WARNING"]);
}
else{
	echo "CRIT - Temperature min={$vmin}° avg={$vavg}° max={$vmax}° (w:$warning,c:$critical) / $nbre sensors. \n";
	exit($INT_STATUS["CRITICAL"]);
}
?>
