#!/usr/bin/perl -w
################## check_snmp_clavister_env #################
my $Version='1.0';
# Date : Sept 12 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use POSIX qw(strftime);
use Switch;
use strict;
use Net::SNMP;
use Getopt::Long;
my $PROGNAME='check_snmp_clavister_env.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "esia_read";
my $snmp_version = 2;
my ($hostname);
my $warning='70';
my $port='161';
my $critical='80';
my $output="";
my $status="";
my $state="";
my $service="";
my $result;
my ($snmp, $snmperror);
my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst);
my @tab_value_read;


my $clavister_descr   = "1.3.6.1.2.1.1.1.0";
#tempCpu
my $clavister_clvHWSensorIndex = "1.3.6.1.4.1.5089.1.2.1.11.1.1"; # INTEGER: 1
my $clavister_clvHWSensorName = "1.3.6.1.4.1.5089.1.2.1.11.1.2"; # STRING: "CpuTemp"
my $clavister_clvHWSensorValue = "1.3.6.1.4.1.5089.1.2.1.11.1.3"; # Gauge32: 47
my $clavister_clvHWSensorUnit = "1.3.6.1.4.1.5089.1.2.1.11.1.4"; # STRING: "C"
# rule
my $clavister_clvRuleName = "1.3.6.1.4.1.5089.1.2.3.2.1.2";
my $clavister_clvRuleUse  = "1.3.6.1.4.1.5089.1.2.3.2.1.3";

sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname) 	Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -w	warning value in % (default: 60)\n";
	print " -c	critical value in % (default: 65)\n";
	print " -V (--version)	Plugin version\n";
	print " -h (--help)	usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print " $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] [-w warning] [-c critical] \n";
	print " $PROGNAME [-h | --help]\n";
	print " $PROGNAME [-V | --version]\n";
}

sub error() {
	print (" ERROR SNMP ENV\n");
	#si l'host ne répond pas on blocke le script
	if (!defined($result))
	{
		if($snmp_version == 2)
		{
			($snmp, $snmperror) = Net::SNMP->session(
					-hostname  => $hostname,
					-domain    => $domain,
					-version   => 'snmpv1',
					-port      => $port,
					-community => $community
				);

			$result = $snmp->get_request("$clavister_descr");
			if (!defined($result))
			{
				printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
				$snmp->close;
				exit $ERRORS{"UNKNOWN"};
			}
		}
		else
		{
			printf("UNKNOWN ERROR: Description table : %s.\n",
			$snmp->error);
			$snmp->close;
			exit $ERRORS{"UNKNOWN"};
		}
	}
}

sub readOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,      "warning=i"      => \$warning,
	"c=i" => \$critical,     "critical=i"     => \$critical,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
 			-version    => 'snmpv1',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}

my $return=$ERRORS{OK};
if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
	exit $ERRORS{"UNKNOWN"};
}
else
{
	$result = $snmp->get_table(Baseoid => $clavister_clvHWSensorIndex);
	if (!defined($result))
	{
	   print "The sensor : na\n";
	}
	else
	{
		foreach my $key ( keys %$result)
		{
		    my $id = $$result{$key};
		    if(defined($id))
		    {
		        my $name_sensor = readOID($snmp,"$clavister_clvHWSensorName.$id");
		        my $value_sensor = readOID($snmp,"$clavister_clvHWSensorValue.$id");
		        my $unit_sensor = readOID($snmp,"$clavister_clvHWSensorUnit.$id");

				if($value_sensor < $warning)
				{
					print("OK : TEM : temperature CPU is $value_sensor °C (< $warning °C) |tem=$value_sensor,warning=$warning,critical=$critical\n");					
				}	
				else
				{
					if($value_sensor > $critical)
					{
						print("CRITICAL : TEM : temperature CPU is $value_sensor (> $critical °C) |tem=$value_sensor,warning=$warning,critical=$critical\n");
						$return = priority_state($return,$ERRORS{CRITICAL}) 
					}
					else
					{
						print("WARNING : TEM : temperature CPU is $value_sensor ( $warning °C <= X < $critical °C ) |tem=$value_sensor,warning=$warning,critical=$critical\n");
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
				}
			}
		}
	}
}
exit $return;
