#!/usr/bin/perl -w
############################## check_snmp_cyberoam_mem_swap #################
my $Version='1.0';
# Date : Sept 12 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
my $PROGNAME='check_snmp_cyberoam_mem_swap.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $warningMem='70';
my $criticalMem='80';
my $warningSwap='10';
my $criticalSwap='20';
my $port='161';
my @tab_value_read;
my $result;
my $o_warn=	undef;		# warning level option
my $o_crit=	undef;		# Critical level option

my $cyberoam_memoryCapacity	    = "1.3.6.1.4.1.21067.2.1.2.4.1"; # Memory capacity in MB
my $cyberoam_memoryPercentUsage	= "1.3.6.1.4.1.21067.2.1.2.4.2"; # % usage of main memory

my $cyberoam_swapCapacity       = "1.3.6.1.4.1.21067.2.1.2.4.3"; # Swap Capacity in MB
my $cyberoam_swapPercentUsage   = "1.3.6.1.4.1.21067.2.1.2.4.4"; # % usage of swap

my $cyberoam_descr 				= "1.3.6.1.2.1.1.1.0";


sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)		Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -w warning value in % (default: 70)\n";
	print " -c critical value in % (default: 80)\n";
	print " -V (--version) Plugin version\n";
	print " -h (--help) usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print " $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] [-w warning] [-c critical]\n";
	print " $PROGNAME [-h | --help]\n";
	print " $PROGNAME [-V | --version]\n";
}

sub isnnum { # Return true if arg is not a number
  my $num = shift;
  if ( $num =~ /^(\d+\.?\d*)|(^\.\d+)$/ ) { return 0 ;}
  return 1;
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
	"w=s" => \$o_warn,       "warning=s"      => \$o_warn,
	"c=s" => \$o_crit,       "critical=s"     => \$o_crit
);

# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

if (!defined($o_warn) || !defined($o_crit))
{ 
	print "Put warning and critical values!\n"; print_help(); exit $ERRORS{"UNKNOWN"};
}
# Get rid of % sign
$o_warn =~ s/\%//g; 
$o_crit =~ s/\%//g;
# if -N or -E switch , undef $o_netsnmp
if ( isnnum($o_warn) || isnnum($o_crit)) 
{ 
	print "Numeric value for warning or critical !\n";print_usage(); exit $ERRORS{"UNKNOWN"};
}
if ( ($o_crit != 0) && ($o_warn > $o_crit) ) 
{ 
  	print "warning <= critical ! \n";print_usage();exit $ERRORS{"UNKNOWN"};
}
   	
my @o_warnL=split(/,/ , $o_warn);
my @o_critL=split(/,/ , $o_crit);
if (($#o_warnL != 1) || ($#o_critL != 1)) 
{ 
	print "2 warnings and critical !\n";print_usage(); exit $ERRORS{"UNKNOWN"}
}
for (my $i=0;$i<2;$i++) 
{
	if ( isnnum($o_warnL[$i]) || isnnum($o_critL[$i])) 
   	{
   		print "Numeric value for warning or critical !\n";print_usage(); exit $ERRORS{"UNKNOWN"} 
   	}
	if (($o_critL[$i]!= 0) && ($o_warnL[$i] > $o_critL[$i]))
	{ 
		print "warning <= critical ! \n";print_usage(); exit $ERRORS{"UNKNOWN"}
	}
	if ( $o_critL[$i] > 100)
	{
		print "critical percent must be < 100 !\n";print_usage(); exit $ERRORS{"UNKNOWN"}
	}
}
$warningMem=$o_warnL[0];
$criticalMem=$o_critL[0];
$warningSwap=$o_warnL[1];
$criticalSwap=$o_critL[1];

my ($snmp, $snmperror);
if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname  => $hostname,
			-version   => 'snmpv1',
			-domain    => $domain,
			-port      => $port,
			-community => $community,
			-timeout   => 2
		);
}

my $return=$ERRORS{OK};
if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
	exit $ERRORS{"UNKNOWN"};
}
else
{
	$result = $snmp->get_request($cyberoam_memoryCapacity);
	if(defined($result))
	{
		my $memCapacity = $$result{"$cyberoam_memoryCapacity"};
		$result = $snmp->get_request($cyberoam_swapCapacity);
		if(defined($result))
		{			
			my $swapCapacity = $$result{"$cyberoam_swapCapacity"};
			$result = $snmp->get_request($cyberoam_memoryPercentUsage);
			if(defined($result))
			{
				my $memPercent = $$result{"$cyberoam_memoryPercentUsage"};
				$result = $snmp->get_request($cyberoam_swapPercentUsage);
				if(defined($result))
				{		
					my $swapPercent = $$result{"$cyberoam_swapPercentUsage"};
					my $sizeMem = sprintf("%.2f",($memCapacity/100)*$memPercent);
					my $sizeSwap = sprintf("%.2f",($swapCapacity/100)*$swapPercent);

					if($memPercent < $warningMem)
					{
						print ("OK : RAM : $memPercent% used ($sizeMem MB/$memCapacity MB) (< $warningMem%) : ");
					}
					else
					{
						if($memPercent > $criticalMem)
						{
							print ("CRITICAL : RAM : $memPercent% used ($sizeMem MB/$memCapacity MB) (> $criticalMem%) : ");
							$return = priority_state($return,$ERRORS{CRITICAL}) 
						}
						else
						{
							print ("WARNING : RAM : $memPercent% used ($sizeMem MB/$memCapacity MB) ($warningMem% <= X < $criticalMem%) : ");
							$return = priority_state($return,$ERRORS{WARNING}) 
						}
					}

					if($swapPercent < $warningSwap)
					{
						print ("OK : SWAP : $swapPercent% used ($sizeSwap MB/$swapCapacity MB) (< $warningSwap%) |ram=$memPercent,sizeMem=$sizeMem,memCapacity=$memCapacity,swap=$swapPercent,sizeSwap=$sizeSwap,swapCapacity=$swapCapacity,warningMem=$warningMem,criticalMem=$criticalMem,warningSwap=$warningSwap,criticalSwap=$criticalSwap \n");
					}
					else
					{
						if($swapPercent > $criticalSwap)
						{
							print ("CRITICAL : SWAP : $swapPercent% used ($sizeSwap MB/$swapCapacity MB) (> $criticalSwap%) |ram=$memPercent,sizeMem=$sizeMem,memCapacity=$memCapacity,swap=$swapPercent,sizeSwap=$sizeSwap,swapCapacity=$swapCapacity,warningMem=$warningMem,criticalMem=$criticalMem,warningSwap=$warningSwap,criticalSwap=$criticalSwap \n");
							$return = priority_state($return,$ERRORS{CRITICAL}) 
						}
						else
						{
							print ("WARNING : SWAP : $swapPercent% used ($sizeSwap MB/$swapCapacity MB) ($warningSwap% <= X < $criticalSwap%) |ram=$memPercent,sizeMem=$sizeMem,memCapacity=$memCapacity,swap=$swapPercent,sizeSwap=$sizeSwap,swapCapacity=$swapCapacity,warningMem=$warningMem,criticalMem=$criticalMem,warningSwap=$warningSwap,criticalSwap=$criticalSwap \n");
							$return = priority_state($return,$ERRORS{WARNING}) 
						}
					}
				}
			}
		}
	}	
	else
	{
		print (" ERROR Ram/Swap !!!! \n");
		#si l'host ne répond pas on blocke le script
		if (!defined($result))
		{
			if($snmp_version == 2)
			{
				($snmp, $snmperror) = Net::SNMP->session(
						-hostname  => $hostname,
						-domain    => $domain,
						-version   => 'snmpv1',
						-port      => $port,
						-community => $community
					);
				
				$result = $snmp->get_request("$cyberoam_descr");
				if (!defined($result))
				{
					printf("UNKNOWN ERROR: Description table :%s.\n", $snmp->error);
					$snmp->close;
					exit $ERRORS{"UNKNOWN"};
				}
			}
			else
			{
				printf("UNKNOWN ERROR: Description table : %s.\n",$snmp->error);
				$snmp->close;
				exit $ERRORS{"UNKNOWN"};
			}
		}
	}
}
exit $return;
