#!/usr/bin/perl -w 
############################## check_snmp_hp5400_temp.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_hp_switch.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning='';
my $critical='';
my $type = 'env';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";

#procurve switchs
my $oid_dell_chassis    	= "1.3.6.1.4.1.674.11000.5000.100.4.1.1.3.1.2"; 	
my $oid_dell_chassis_serial	= "1.3.6.1.4.1.674.11000.5000.100.4.1.1.3.1.4"; 
my $oid_dell_chassis_temp   = "1.3.6.1.4.1.674.11000.5000.100.4.1.1.3.1.11"; 
my %chassis_type = (
	'1'=> 's6000on',
	'2'=> 's4048on',
	'3'=> 's4048Ton',
	'4'=> 's3048on',
	'5'=> 's6010on',
	'6'=> 's4148Fon',
	'7'=> 's4128Fon',
	'8'=> 's4148Ton',
	'9'=> 's4128Ton',
	'10'=> 's4148FEon',
	'11'=> 's4148Uon',
	'12'=> 's4200on',
	'13'=> 'mx5108Non',
	'14'=> 'mx9116Non',
	'15'=> 's5148Fon',
	'16'=> 'z9100on',
	'17'=> 's4248FBon',
	'18'=> 's4248FBLon',
	'19'=> 's4112Fon',
	'20'=> 's4112Ton',
	'21'=> 'z9264Fon',
	'9999'=> 'unknown');
my $oid_dell_card_descr    	= "1.3.6.1.4.1.674.11000.5000.100.4.1.1.4.1.3"; 	
my $oid_dell_card_status	= "1.3.6.1.4.1.674.11000.5000.100.4.1.1.4.1.4"; 
my $oid_dell_card_temp	    = "1.3.6.1.4.1.674.11000.5000.100.4.1.1.4.1.5"; 
my %oid_dell_card_status_message = (
	'1'=> 'ready',
	'2'=> 'cardMisMatch',
	'3'=> 'cardProblem',
	'4'=> 'diagMode',
	'5'=> 'cardAbsent',
	'6'=> 'offline'
);
my $oid_dell_psu_state	 ="1.3.6.1.4.1.674.11000.5000.100.4.1.2.1.1.4";
my $oid_dell_psu_id	     ="1.3.6.1.4.1.674.11000.5000.100.4.1.2.1.1.6";
my $oid_dell_fan_state	 ="1.3.6.1.4.1.674.11000.5000.100.4.1.2.2.1.4";
my $oid_dell_fan_id	     ="1.3.6.1.4.1.674.11000.5000.100.4.1.2.2.1.5";
my %oid_dell_component_status_message =(
	'1'=> 'up',
	'2'=> 'down',
	'3'=> 'testing',
	'4'=> 'unknown',
	'5'=> 'dormant',
	'6'=> 'notPresent',
	'7'=> 'lowerLayerDown',
	'8'=> 'failed'
);
my %oid_dell_component_state =(
	'1'=> 'OK',
	'2'=> 'CRITICAL',
	'3'=> 'WARNING',
	'4'=> 'UNKNOWN',
	'5'=> 'OK',
	'6'=> 'OK',
	'7'=> 'WARNING',
	'8'=> 'CRITICAL'
);
my $oid_default_cpu ='1.3.6.1.2.1.25.3.3.1.2';
my $oid_dell_cpu ='1.3.6.1.4.1.674.10895.5000.2.6132.1.1.1.1.4.9.0';
my $oid_ram_free	= "1.3.6.1.4.1.2021.4.6.0";  # Real memory free
my $oid_ram_total	= "1.3.6.1.4.1.2021.4.5.0";  # Real memory total
my $oid_ram_cache      = "1.3.6.1.4.1.2021.4.15.0";
my $oid_dell_ram_free = ".1.3.6.1.4.1.674.10895.5000.2.6132.1.1.1.1.4.1.0";
my $oid_dell_ram_total = ".1.3.6.1.4.1.674.10895.5000.2.6132.1.1.1.1.4.2.0";
#DELL N2048
my $oid_dell_temperature_state = "1.3.6.1.4.1.674.10895.5000.2.6132.1.1.43.1.8.1.4";
my $oid_dell_temperature_value = "1.3.6.1.4.1.674.10895.5000.2.6132.1.1.43.1.8.1.5";

my $oid_dell_n2048_fan_state = "1.3.6.1.4.1.674.10895.5000.2.6132.1.1.43.1.6.1.3";
my %oid_translate_dell_fan_state =(
	'1'=> 'notpresent',
	'2'=> 'operational',
	'3'=> 'failed',
	'4'=> 'powering',
	'5'=> 'nopower',
	'6'=> 'notpowering',
	'7'=> 'incompatible',
);
my $oid_dell_n2048_power_state = "1.3.6.1.4.1.674.10895.5000.2.6132.1.1.43.1.7.1.3";
my %oid_translate_dell_power_state =(
	'1'=> 'notpresent',
	'2'=> 'operational',
	'3'=> 'failed',
	'4'=> 'powering',
	'5'=> 'nopower',
	'6'=> 'notpowering',
	'7'=> 'incompatible',
);
my %oid_translate_dell_component_state =(
	'1'=> 'OK',
	'2'=> 'OK',
	'3'=> 'CRITICAL',
	'4'=> 'WARNING',
	'5'=> 'CRITICAL',
	'6'=> 'WARNING',
	'7'=> 'CRITICAL',
);
my $oid_dell_54xx_fan_descr = "1.3.6.1.4.1.674.10895.3000.1.2.110.7.1.1.2";
my $oid_dell_54xx_fan_state = "1.3.6.1.4.1.674.10895.3000.1.2.110.7.1.1.3";
my $oid_dell_54xx_power_descr = "1.3.6.1.4.1.674.10895.3000.1.2.110.7.2.1.2";
my $oid_dell_54xx_power_state = "1.3.6.1.4.1.674.10895.3000.1.2.110.7.2.1.3";
my $oid_dell_54xx_cpu_value = "1.3.6.1.4.1.89.1.9.0";
my $oid_dell_54xx_temp_value ="1.3.6.1.4.1.89.53.15.1.9.1";
my %oid_translate_dell_54xx_envmom_state =(
	'1'=> 'normal',
	'2'=> 'warning',
	'3'=> 'critical',
	'4'=> 'shutdown',
	'5'=> 'notPresent',
	'6'=> 'notFunctioning',
);
my %oid_translate_dell_54xx_envmom_status =(
	'1'=> 'OK',
	'2'=> 'WARNING',
	'3'=> 'CRITICAL',
	'4'=> 'WARNING',
	'5'=> 'OK',
	'6'=> 'WARNING',
);

# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password

sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

sub print_help() {
	printf "check snmp HP switch.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -T                  Test type (cpu,mem,env)\n";
	print " -w                  warning threshold\n";
	print " -c                  critical threshold\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	'l=s'	=> \$o_login,		'login=s'	=> \$o_login,
	'x=s'	=> \$o_passwd,		'passwd=s'	=> \$o_passwd,
	'X=s'	=> \$o_privpass,		'privpass=s'	=> \$o_privpass,
	'L=s'	=> \$v3protocols,		'protocols=s'	=> \$v3protocols,   
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if (defined($o_login) || defined($o_passwd)) 
{
    $snmp_version =3;
}
if ($snmp_version == 3) 
{
    my @v3proto=split(/,/,$v3protocols);
    if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
    if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
    if (!defined ($o_privpass)) 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -timeout          => 2
        );  
    }
    else 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -privpassword	=> $o_privpass,
	      -privprotocol => $o_privproto,
          -timeout          => 5
        );
    }
}
elsif ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysObjectOID");


#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysObjectOID");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $Sysoid_value = $$result{"$oid_sysObjectOID"};
my $full_sysoid_value = $Sysoid_value;
if($Sysoid_value =~/3\.6\.1\.4\.1\.([0-9]*)/)
{
    $Sysoid_value = $1;
}
# test des paramètre par défaut
if($warning eq '' )
{
    if($type eq 'cpu' || $type eq 'mem')
    {
        $warning=80;
    }
}
if($critical eq '')
{
    if($type eq 'cpu'|| $type eq 'mem')
    {
        $critical=80;
    }
}

if($type eq 'env')
{
    if($full_sysoid_value=~/3031$/)
    {
        $result = $snmp->get_table(Baseoid => $oid_dell_54xx_temp_value);
        foreach my $key ( keys %$result) 
        {	
            my $state="OK";
            #on récupère l'index
            if($key=~ /([0-9]*)$/ )
            {
                my $index =$1;
                my $temp =$$result{"$key"};

                my $message="OK <$warning °C";
                if($temp > $critical)
                {
                    $state="CRITICAL";
                    $message="very high > $critical";
                }
                elsif($temp > $warning)
                {
                    $state="WARNING";
                    $message="high > $warning";
                }
                print "$state : Temperature sensors $index: $temp °C is $message  \n";
                $return = priority_state($return,$ERRORS{$state});
            } 
        }
        $result = $snmp->get_table(Baseoid => $oid_dell_54xx_power_descr);
        foreach my $key ( keys %$result) 
        {	
            my $state="OK";
            #on récupère l'index
            if($key=~ /([0-9]*)$/ )
            {
                my $index =$1;
                my $descr =$$result{"$key"};
                my $status = getsnmpOID($snmp,"$oid_dell_54xx_power_state.$index");
                if($status > 6)
                {
                    $state="UNKNOWN";
                    print "$state : PSU $index bad status return $status\n";
                }
                else
                {
                    $state = $oid_translate_dell_54xx_envmom_status{$status};
                    print "$state : PSU $descr is ".$oid_translate_dell_54xx_envmom_state{$status}."\n";
                }
                $return = priority_state($return,$ERRORS{$state});
            } 
        }
        $result = $snmp->get_table(Baseoid => $oid_dell_54xx_fan_descr);
        foreach my $key ( keys %$result) 
        {	
            my $state="OK";
            #on récupère l'index
            if($key=~ /([0-9]*)$/ )
            {
                my $index =$1;
                my $descr =$$result{"$key"};
                my $status = getsnmpOID($snmp,"$oid_dell_54xx_fan_state.$index");
                if($status > 6)
                {
                    $state="UNKNOWN";
                    print "$state : FAN $index bad status return $status\n";
                }
                else
                {
                    $state = $oid_translate_dell_54xx_envmom_status{$status};
                    print "$state : FAN $descr is ".$oid_translate_dell_54xx_envmom_state{$status}."\n";
                }
                $return = priority_state($return,$ERRORS{$state});
            } 
        }
    }
    elsif($full_sysoid_value=~/3054$/)
    {
#my $oid_dell_temperature_state = "1.3.6.1.4.1.674.10895.5000.2.6132.1.1.43.1.8.1.4";
#my $oid_dell_temperature_value = "1.3.6.1.4.1.674.10895.5000.2.6132.1.1.43.1.8.1.5";
        $result = $snmp->get_table(Baseoid => $oid_dell_temperature_state);
        foreach my $key ( keys %$result) 
        {	
            my $state="OK";
            #on récupère l'index
            if($key=~ /([0-9]*\.[0-9]*)$/ )
            {
                my $index =$1;
                my $status =$$result{"$key"};
                my $temp = getsnmpOID($snmp,"$oid_dell_temperature_value.$index");
                my $message="OK <$warning °C";
                if($status == 6)
                {
                    $state="CRITICAL";
                    $message="sensors is defect  ";
                }
                else
                {
                    if($temp > $critical)
                    {
                        $state="CRITICAL";
                        $message="very high > $critical";
                    }
                    elsif($temp > $warning)
                    {
                        $state="WARNING";
                        $message="high > $warning";
                    }
                }
                print "$state : Temperature sensors $index: $temp °C is $message  \n";
                $return = priority_state($return,$ERRORS{$state});
            } 
        }
        $result = $snmp->get_table(Baseoid => $oid_dell_n2048_fan_state);
        foreach my $key ( keys %$result) 
        {	
            my $state="OK";
            #on récupère l'index
            if($key=~ /([0-9]*\.[0-9]*)$/ )
            {
                my $index =$1;
                my $status =$$result{"$key"};
                if($status > 8)
                {
                    $state="UNKNOWN";
                    print "$state : FAN $index bad status return $status\n";
                }
                else
                {
                    $state = $oid_translate_dell_component_state{$status};
                    print "$state : FAN $index is ".$oid_translate_dell_fan_state{$status}."\n";
                }
                $return = priority_state($return,$ERRORS{$state});
            } 
        }
        $result = $snmp->get_table(Baseoid => $oid_dell_n2048_power_state);
        foreach my $key ( keys %$result) 
        {	
            my $state="OK";
            #on récupère l'index
            if($key=~ /([0-9]*\.[0-9]*)$/ )
            {
                my $index =$1;
                my $status =$$result{"$key"};
                if($status > 8)
                {
                    $state="UNKNOWN";
                    print "$state : PSU $index bad status return $status\n";
                }
                else
                {
                    $state = $oid_translate_dell_component_state{$status};
                    print "$state : PSU $index is ".$oid_translate_dell_power_state{$status}."\n";
                }
                $return = priority_state($return,$ERRORS{$state});
            } 
        }
    }
    else
    {
        $result = $snmp->get_table(Baseoid => $oid_dell_chassis);
        foreach my $key ( keys %$result) 
        {	
            my $state="OK";
            #on récupère l'index
            if($key=~ /([0-9]*)$/ )
            {
                my $index =$1;
                my $type =$$result{"$key"};
                my $sn = getsnmpOID($snmp,"$oid_dell_chassis_serial.$index");
                my $temp = getsnmpOID($snmp,"$oid_dell_chassis_temp.$index");
                my $message="OK <$warning °C";
                if($temp > $critical)
                {
                    $state="CRITICAL";
                    $message="very high > $critical";
                }
                elsif($temp > $warning)
                {
                    $state="WARNING";
                    $message="high > $warning";
                }
                
                print "$state : chassis $index ".$chassis_type{$type}." (PN:$sn) temperature $temp °C is $message  \n";
                $return = priority_state($return,$ERRORS{$state});
            } 
        }
        $result = $snmp->get_table(Baseoid => $oid_dell_card_descr);
        foreach my $key ( keys %$result) 
        {	
            my $state="OK";
            #on récupère l'index
            if($key=~ /([0-9]*\.[0-9]*)$/ )
            {
                my $index =$1;
                my $descr =$$result{"$key"};
                my $status = getsnmpOID($snmp,"$oid_dell_card_status.$index");
                my $temp = getsnmpOID($snmp,"$oid_dell_card_temp.$index");
                my $message="OK <$warning °C";
                if($status == 1)
                {
                     $state="OK";                 
                }   
                elsif($status == 4 || $status == 6)
                {
                    $state="WARNING";  
                }    
                else 
                {
                    $state="CRITICAL";  
                }
                if($temp > $critical)
                {
                    $state="CRITICAL";
                    $message="very high > $critical";
                }
                elsif($temp > $warning && $state ne "CRITICAL")
                {
                    $state="WARNING";
                    $message="high > $warning";
                }
                print "$state : card $descr ($index) is ".$oid_dell_card_status_message{$status}.", temperature $temp °C is $message  \n";
                $return = priority_state($return,$ERRORS{$state});
            } 
        }
        $result = $snmp->get_table(Baseoid => $oid_dell_psu_state);
        foreach my $key ( keys %$result) 
        {
            my $state="OK";
            #on récupère l'index
            if($key=~ /([0-9]*)$/ )
            {
                my $index =$1;
                my $status =$$result{"$key"};
                if($status > 8)
                {
                    $state="UNKNOWN";
                    print "$state : PSU $index bad status return $status\n";
                }
                else
                {
                    my $psuid = getsnmpOID($snmp,"$oid_dell_psu_id.$index");
                    $state = $oid_dell_component_state{$status};
                    print "$state : PSU $index ($psuid) is ".$oid_dell_component_status_message{$status}."\n";
                }
                $return = priority_state($return,$ERRORS{$state});
            }	
        }
        $result = $snmp->get_table(Baseoid => $oid_dell_fan_state);
        foreach my $key ( keys %$result) 
        {
            my $state="OK";
            #on récupère l'index
            if($key=~ /([0-9]*)$/ )
            {
                my $index =$1;
                my $status =$$result{"$key"};
                if($status > 8)
                {
                    $state="UNKNOWN";
                    print "$state : PSU $index bad status return $status\n";
                }
                else
                {
                    my $psuid = getsnmpOID($snmp,"$oid_dell_fan_id.$index");
                    $state = $oid_dell_component_state{$status};
                    print "$state : FAN $index ($psuid) is ".$oid_dell_component_status_message{$status}."\n";
                }
                $return = priority_state($return,$ERRORS{$state});
            }	
        }

    }
}
elsif($type eq 'cpu')
{
    my $load = 0;
    my $cpu_number=0;
    if($full_sysoid_value=~/3031$/)
    {
        my $cpu = getsnmpOID($snmp,"$oid_dell_54xx_cpu_value");
        $load =$cpu;
    }
    elsif($full_sysoid_value=~/3054$/)
    {
        my $cpu = getsnmpOID($snmp,"$oid_dell_cpu");
        if($cpu =~ /([0-9\.]*)%\)$/ )
        {
            #print "$cpu\n LOAD = $1\n";
            $load = int($1);
        }
    }
    else
    {
        $result = $snmp->get_table(Baseoid => $oid_default_cpu);
        foreach my $key ( keys %$result) 
        {
            $load = $load + $$result{"$key"};
            $cpu_number++;
        }
        $load = int($load/$cpu_number);
    }
    my $state="OK";
    my $message="< $warning";
    if($load > $critical)
    {
        $state="CRITICAL";
        $message=" > $critical";
    }
    elsif($load > $warning)
    {
        $state="WARNING";
        $message=" > $warning";
    }
    print "$state : CPU average load $load% $message|cpu=$load\n";
    $return = priority_state($return,$ERRORS{$state});
}
elsif($type eq 'mem')
{
    my $percent_used=-1;
    my $total = getsnmpOID($snmp,"$oid_ram_total");
    if($total ne 'noSuchObject')
    {
        my $free = getsnmpOID($snmp,"$oid_ram_free");
        my $cache = getsnmpOID($snmp,"$oid_ram_cache");
        my $used = $total-($free+$cache);
        $percent_used =int(($used/$total)*100);
    }
    else
    {
        $total = getsnmpOID($snmp,"$oid_dell_ram_total");
        my $free = getsnmpOID($snmp,"$oid_dell_ram_free");
        my $used = $total-$free;
        $percent_used =int(($used/$total)*100);
    }
    my $state="OK";
    my $message="< $warning";
    if($percent_used > $critical)
    {
        $state="CRITICAL";
        $message=" > $critical";
    }
    elsif($percent_used > $warning)
    {
        $state="WARNING";
        $message=" > $warning";
    }
    print "$state : RAM used $percent_used% $message|ram=$percent_used\n";
    $return = priority_state($return,$ERRORS{$state});
}  

exit($return);
