#!/usr/bin/perl -w
############################## check_snmp_dlink_switch.pl ########
my $Version='1.0';
# Date : Jan 15 2021
# Author  : Julien George
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

#use Data::Dumper;

my $PROGNAME='check_snmp_dlink_switch.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning='';
my $critical='';
my $type = 'env';

my $exclude_list;
my $select_index;

# Mib Standard
my $oid_sysDescr = "1.3.6.1.2.1.1.1.0";
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";

# Switches like DGS_1210
my $oid_dgs_1210_version = '1.3.6.1.4.1.171.11.153.1000.1.3.0';

my $oid_dgs_1210_load5m         = '1.3.6.1.4.1.171.11.153.1000.1.46.3.0';
my $oid_dgs_1210_mem_percent    = '1.3.6.1.4.1.171.11.153.1000.1.47.3.0';
my $oid_dgs_1210_flash_percent  = '1.3.6.1.4.1.171.11.153.1000.1.33.3.1.5';

# Status
my %esEntityExtEnvPowerStatus   = (1 => 'inOperation', 2 => 'failed', 3 => 'empty');
my %esEntityExtEnvTempStatus    = (1 => 'ok', 2 => 'abnormal');
my %esEntityExtEnvFanStatus     = (1 => 'ok', 2 => 'fault');

my %esiaPowerStatus =  (1 => 'OK', 2 => 'CRITICAL', 3 => 'WARNING');
my %esiaTempStatus  =  (1 => 'OK', 2 => 'CRITICAL');
my %esiaFanStatus   =  (1 => 'OK', 2 => 'CRITICAL');

# Switches like DGS_1510/DGS_3630
my $oid_dgs_1510_load5m     = '1.3.6.1.4.1.171.14.5.1.7.1.5.1.1';
my $oid_dgs_1510_mem_total  = '1.3.6.1.4.1.171.14.5.1.4.1.3.1.1';
my $oid_dgs_1510_mem_used   = '1.3.6.1.4.1.171.14.5.1.4.1.4.1.1';
my $oid_dgs_1510_flash_total= '1.3.6.1.4.1.171.14.5.1.4.1.3.1.2';
my $oid_dgs_1510_flash_used = '1.3.6.1.4.1.171.14.5.1.4.1.4.1.2';

my $oid_dgs_1510_pwr_descr  = '1.3.6.1.4.1.171.14.5.1.1.3.1.3.1';
my $oid_dgs_1510_pwr_status = '1.3.6.1.4.1.171.14.5.1.1.3.1.6.1';

my $oid_dgs_1510_temp_descr     = '1.3.6.1.4.1.171.14.5.1.1.1.1.3.1';
my $oid_dgs_1510_temp_current   = '1.3.6.1.4.1.171.14.5.1.1.1.1.4.1';
my $oid_dgs_1510_temp_low       = '1.3.6.1.4.1.171.14.5.1.1.1.1.5.1';
my $oid_dgs_1510_temp_high      = '1.3.6.1.4.1.171.14.5.1.1.1.1.6.1';
my $oid_dgs_1510_temp_status    = '1.3.6.1.4.1.171.14.5.1.1.1.1.7.1';

my $oid_dgs_1510_fan_descr  = '1.3.6.1.4.1.171.14.5.1.1.2.1.3.1';
my $oid_dgs_1510_fan_status = '1.3.6.1.4.1.171.14.5.1.1.2.1.4.1';

# Switches like DGS_3130
my $oid_dgs_3130_load5m     = '1.3.6.1.4.1.171.17.5.1.7.1.5.1.1';
my $oid_dgs_3130_mem_total  = '1.3.6.1.4.1.171.17.5.1.4.1.3.1.1';
my $oid_dgs_3130_mem_used   = '1.3.6.1.4.1.171.17.5.1.4.1.4.1.1';
my $oid_dgs_3130_flash_total= '1.3.6.1.4.1.171.17.5.1.4.1.3.1.2';
my $oid_dgs_3130_flash_used = '1.3.6.1.4.1.171.17.5.1.4.1.4.1.2';

my $oid_dgs_3130_pwr_descr  = '1.3.6.1.4.1.171.17.5.1.1.3.1.3.1';
my $oid_dgs_3130_pwr_status = '1.3.6.1.4.1.171.17.5.1.1.3.1.6.1';

my $oid_dgs_3130_temp_descr     = '1.3.6.1.4.1.171.17.5.1.1.1.1.3.1';
my $oid_dgs_3130_temp_current   = '1.3.6.1.4.1.171.17.5.1.1.1.1.4.1';
my $oid_dgs_3130_temp_low       = '1.3.6.1.4.1.171.17.5.1.1.1.1.5.1';
my $oid_dgs_3130_temp_high      = '1.3.6.1.4.1.171.17.5.1.1.1.1.6.1';
my $oid_dgs_3130_temp_status    = '1.3.6.1.4.1.171.17.5.1.1.1.1.7.1';

my $oid_dgs_3130_fan_descr  = '1.3.6.1.4.1.171.17.5.1.1.2.1.3.1';
my $oid_dgs_3130_fan_status = '1.3.6.1.4.1.171.17.5.1.1.2.1.4.1';

# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password

sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

sub print_help() {
	printf "check snmp HP switch.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -T (--type)         Test type (cpu|mem|flash|pwr|temp|fan)\n";
    print " -e (--exclude)      Exclude components\n";
    print " -i (--index)        Port or item index\n";
	print " -w                  warning threshold\n";
	print " -c                  critical threshold\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] -T cpu|mem|flash|pwr|temp|fan [-w warning] [-c critical] [-i index]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
    "T=s" => \$type,"type=s" => \$type,
    "e=s" => \$exclude_list,"exclude=s" => \$exclude_list,
    "i=s" => \$select_index,"index=s" => \$select_index,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	'l=s'	=> \$o_login,		'login=s'	=> \$o_login,
	'x=s'	=> \$o_passwd,		'passwd=s'	=> \$o_passwd,
	'X=s'	=> \$o_privpass,		'privpass=s'	=> \$o_privpass,
	'L=s'	=> \$v3protocols,		'protocols=s'	=> \$v3protocols,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) {
    print "ERROR : You forget to specify -H, --host option\n\n";
    print_help();
    exit $ERRORS{'UNKNOWN'};
}

if ($type eq 'cpu' || $type eq 'mem' || $type eq 'flash') {
    if(!$warning || !$critical){
        print "ERROR : You forget to specify -w|-c, --warning|--critical options\n\n";
        print_help();
        exit $ERRORS{'UNKNOWN'};
    }
}
else{
    if($warning || $critical){
        print "ERROR : -w|-c, --warning|--critical options are not used for type $type\n\n";
        print_help();
        exit $ERRORS{'UNKNOWN'};
    }
}
my ($snmp, $snmperror);
if (defined($o_login) || defined($o_passwd))
{
    $snmp_version =3;
}
if ($snmp_version == 3)
{
    my @v3proto=split(/,/,$v3protocols);
    if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
    if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
    if (!defined ($o_privpass))
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -timeout          => 2
        );
    }
    else
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -privpassword	=> $o_privpass,
	      -privprotocol => $o_privproto,
          -timeout          => 5
        );
    }
}
elsif ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}

#on interroge en SNMP pour récupérer les informations
my $result = $snmp->get_request("$oid_sysObjectOID");

#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysObjectOID");
		if (!defined($result))
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}

my $dlink_class;
my $firmware_version;
my $cpu_load = $snmp->get_request(-varbindlist => [ $oid_dgs_3130_load5m,$oid_dgs_1510_load5m,$oid_dgs_1210_load5m ]);

if (!($cpu_load->{$oid_dgs_3130_load5m} =~ "noSuch")){
    $dlink_class='DGS_3130';
}
elsif(!($cpu_load->{$oid_dgs_1510_load5m} =~ "noSuch")){
    $dlink_class='DGS_1510';
}
elsif(!($cpu_load->{$oid_dgs_1210_load5m} =~ "noSuch")){
    $dlink_class='DGS_1210';
}
else{
    $firmware_version = $snmp->get_request(-varbindlist => [ $oid_dgs_1210_version ]);
    if(!defined $firmware_version || $firmware_version->{ $oid_dgs_1210_version} =~ "noSuch"){ #La classe du switch n'a pas pu être identifiée
        printf("UNKNOWN ERROR: Unable to identify D-LINK switch class\n");
        $snmp->close;
        exit $ERRORS{"UNKNOWN"};
    }
    $dlink_class='DGS_1210';
    print "$dlink_class\n";
}

my $state = 'OK';
if( $dlink_class eq 'DGS_3130' ){
    if( $type eq 'cpu' ){
                my $threshold = $cpu_load->{$oid_dgs_3130_load5m};
                my $message="(< $warning)";
                if($threshold >= $critical)
                {
                    $state='CRITICAL';
                    $return = priority_state($return,$ERRORS{CRITICAL});
                    $message="(> $critical)";
                }
                elsif($threshold >= $warning)
                {
                    $state='WARNING';
                    $return = priority_state($return,$ERRORS{WARNING});
                    $message="(> $warning)";
                }
                printf("%s : CPU used %.1f%% %s\n",$state,$threshold,$message);
    }
    elsif( $type eq 'mem' ){
        my $total = getsnmpOID($snmp,$oid_dgs_3130_mem_total);
        my $used = getsnmpOID($snmp,$oid_dgs_3130_mem_used);
        my $percent = $used/$total*100;

        my $message="(< $warning)";
        if($percent >= $critical)
        {
            $state='CRITICAL';
            $return = priority_state($return,$ERRORS{CRITICAL});
            $message="(> $critical)";
        }
        elsif($percent >= $warning)
        {
            $state='WARNING';
            $return = priority_state($return,$ERRORS{WARNING});
            $message="(> $warning)";
        }
        printf("%s : Mem used %.1f%% %s\n",$state,$percent,$message);
    }
    elsif ($type eq 'flash' ){
        my $total = getsnmpOID($snmp,$oid_dgs_3130_flash_total);
        my $used = getsnmpOID($snmp,$oid_dgs_3130_flash_used);
        my $percent = $used/$total*100;

        my $message="(< $warning)";
        if($percent >= $critical)
        {
            $state='CRITICAL';
            $return = priority_state($return,$ERRORS{CRITICAL});
            $message="(> $critical)";
        }
        elsif($percent >= $warning)
        {
            $state='WARNING';
            $return = priority_state($return,$ERRORS{WARNING});
            $message="(> $warning)";
        }
        printf("%s : Flash used %.1f%% %s\n",$state,$percent,$message);
    }
    elsif ($type eq 'pwr' ){
        $result = $snmp->get_table(-baseoid => $oid_dgs_3130_pwr_descr);
        my $found_index = 0;
        foreach my $key ( sort keys %$result)
        {
            #on récupère l'index
            if($key=~ /([0-9]*)$/ )
            {
                my $index = $1;
                if( defined $select_index && $select_index != $index) {
                    next;
                }
                my $descr = $$result{$key};
                my $status = getsnmpOID($snmp,"$oid_dgs_3130_pwr_status.$index");
                $found_index = 1;

                $state = $esiaPowerStatus{$status};
                print "$state : id $index $descr status is ".$esEntityExtEnvPowerStatus{$status}."\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
        if( $found_index == 0 ){
            $state = 'UNKNOWN';
            print "$state : ";
            print "Index $select_index not found. You should try index between 2 and 4\n";
            $return = priority_state($return,$ERRORS{$state});
        }
    }
    elsif ($type eq 'fan' ){
        $result = $snmp->get_table(-baseoid => $oid_dgs_3130_fan_descr);
        my $found_index = 0;
        foreach my $key ( sort keys %$result)
        {
            #on récupère l'index
            if($key=~ /([0-9]*)$/ )
            {
                my $index = $1;
                if( defined $select_index && $select_index != $index) {
                    next;
                }
                my $descr = $$result{$key};
                my $status = getsnmpOID($snmp,"$oid_dgs_3130_fan_status.$index");
                $found_index = 1;

                $state = $esiaFanStatus{$status};
                print "$state : $index. $descr status is ".$esEntityExtEnvFanStatus{$status}."\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
        if( $found_index == 0 ){
            $state = 'UNKNOWN';
            print "$state : ";
            print "Index $select_index not found. You should try index between 2 and 4\n";
            $return = priority_state($return,$ERRORS{$state});
        }
    }
    elsif ($type eq 'temp' ){
        $result = $snmp->get_table(-baseoid => $oid_dgs_3130_temp_descr);
        my $found_index = 0;
        foreach my $key ( sort keys %$result)
        {
            #on récupère l'index
            if($key=~ /([0-9]*)$/ )
            {
                my $index = $1;
                if( defined $select_index && $select_index != $index) {
                    next;
                }
                my $descr = $$result{$key};
                my $status = getsnmpOID($snmp,"$oid_dgs_3130_temp_status.$index");
                my $current = getsnmpOID($snmp,"$oid_dgs_3130_temp_current.$index");
                my $low = getsnmpOID($snmp,"$oid_dgs_3130_temp_low.$index");
                my $high = getsnmpOID($snmp,"$oid_dgs_3130_temp_high.$index");
                $found_index = 1;

                $state = $esiaTempStatus{$status};

                my $message="";
                if($current >= $high)
                {
                    #state always CRITICAL
                    $state="CRITICAL";
                    $message="high (> $high)";
                }
                elsif($current <= $low)
                {
                    #state always CRITICAL
                    $state="CRITICAL";
                    $message="lower (< $low)";
                }
                else
                {
                    #state OK or esiaTempStatus value
                    $message="normal (< $high)";
                }

                print "$state : id $index $descr status is ".$esEntityExtEnvTempStatus{$status}. ". Value $current is $message\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
        if( $found_index == 0 ){
            $state = 'UNKNOWN';
            print "$state : ";
            print "Index $select_index not found. You should try index between 2 and 4\n";
            $return = priority_state($return,$ERRORS{$state});
        }
    }
    else{
        print "ERROR : type $type not implemented\n\n";
        print_help();
        exit $ERRORS{'UNKNOWN'};
    }
}
elsif( $dlink_class eq 'DGS_1510' ){
    if( $type eq 'cpu' ){
                my $threshold = $cpu_load->{$oid_dgs_1210_load5m};
                my $message="(< $warning)";
                if($threshold >= $critical)
                {
                    $state='CRITICAL';
                    $return = priority_state($return,$ERRORS{CRITICAL});
                    $message="(> $critical)";
                }
                elsif($threshold >= $warning)
                {
                    $state='WARNING';
                    $return = priority_state($return,$ERRORS{WARNING});
                    $message="(> $warning)";
                }
                printf("%s : CPU used %.1f%% %s\n",$state,$threshold,$message);
                $return = priority_state($return,$ERRORS{$state});
    }
    elsif( $type eq 'mem' ){
        my $total = getsnmpOID($snmp,$oid_dgs_1510_mem_total);
        my $used = getsnmpOID($snmp,$oid_dgs_1510_mem_used);
        my $percent = $used/$total*100;

        my $message="(< $warning)";
        if($percent >= $critical)
        {
            $state='CRITICAL';
            $return = priority_state($return,$ERRORS{CRITICAL});
            $message="(> $critical)";
        }
        elsif($percent >= $warning)
        {
            $state='WARNING';
            $return = priority_state($return,$ERRORS{WARNING});
            $message="(> $warning)";
        }
        printf("%s : Mem used %.1f%% %s\n",$state,$percent,$message);
    }
    elsif ($type eq 'flash' ){
        my $total = getsnmpOID($snmp,$oid_dgs_1510_flash_total);
        my $used = getsnmpOID($snmp,$oid_dgs_1510_flash_used);
        my $percent = $used/$total*100;

        my $message="(< $warning)";
        if($percent >= $critical)
        {
            $state='CRITICAL';
            $return = priority_state($return,$ERRORS{CRITICAL});
            $message="(> $critical)";
        }
        elsif($percent >= $warning)
        {
            $state='WARNING';
            $return = priority_state($return,$ERRORS{WARNING});
            $message="(> $warning)";
        }
        printf("%s : Flash used %.1f%% %s\n",$state,$percent,$message);
    }
    else{
        print "ERROR : type $type not implemented\n\n";
        print_help();
        exit $ERRORS{'UNKNOWN'};
    }
}
elsif( $dlink_class eq 'DGS_1210' ){
    if( $type eq 'cpu' ){
                if( defined $firmware_version ){
                    $state = 'UNKNOWN';
                    print "$state : ";
                    print 'Unable to get snmp status with this D-Link firmware version : '.$firmware_version->{$oid_dgs_1210_version}.". You must upgrade your firmware for getting this info\n";
                    $state = 'UNKNOWN';
                    $return = priority_state($return,$ERRORS{$state});
                }
                else{
                    $state = 'OK';
                    my $threshold = $cpu_load->{$oid_dgs_1210_load5m};
                    my $message="(< $warning)";
                    if($threshold >= $critical)
                    {
                        $state='CRITICAL';
                        $return = priority_state($return,$ERRORS{CRITICAL});
                        $message="(> $critical)";
                    }
                    elsif($threshold >= $warning)
                    {
                        $state='WARNING';
                        $return = priority_state($return,$ERRORS{WARNING});
                        $message="(> $warning)";
                    }
                    printf("%s : CPU used %.1f%% %s\n",$state,$threshold,$message);
                    $return = priority_state($return,$ERRORS{$state});
                }
    }
    elsif( $type eq 'mem' ){
                    if( defined $firmware_version ){
                        $state = 'UNKNOWN';
                        print "$state : ";
                        print 'Unable to get snmp status with this D-Link firmware version : '.$firmware_version->{$oid_dgs_1210_version}.". You must upgrade your firmware for getting this info\n";
                        $return = priority_state($return,$ERRORS{$state});
                    }
                    else{
                        my $threshold = getsnmpOID($snmp,$oid_dgs_1210_mem_percent);
                        my $message="(< $warning)";
                        if($threshold >= $critical)
                        {
                            $state='CRITICAL';
                            $return = priority_state($return,$ERRORS{CRITICAL});
                            $message="(> $critical)";
                        }
                        elsif($threshold >= $warning)
                        {
                            $state='WARNING';
                            $return = priority_state($return,$ERRORS{WARNING});
                            $message="(> $warning)";
                        }
                        printf("%s : Mem used %.1f%% %s\n",$state,$threshold,$message);
                        $return = priority_state($return,$ERRORS{$state});
                    }
        }
        elsif ($type eq 'flash' ){
            $result = $snmp->get_table(-baseoid => $oid_dgs_1210_flash_percent);
            my @desc = ('NULL','Boot','Image1','Image2','Jffs2');
            my $found_index = 0;
            foreach my $key ( sort keys %$result)
            {
                #on récupère l'index
                if($key=~ /([0-9]*)$/ )
                {
                    my $index =$1;
                    if ($index == 1){
                        next;
                    }
                    if( defined $select_index && $select_index != $index) {
                        next;
                    }
                    $found_index = 1;
                    $state = 'OK';
                    my $threshold = $$result{"$key"};
                    my $message="(< $warning)";
                    if($threshold >= $critical)
                    {
                        $state='CRITICAL';
                        $return = priority_state($return,$ERRORS{CRITICAL});
                        $message="(> $critical)";
                    }
                    elsif($threshold >= $warning)
                    {
                        $state='WARNING';
                        $return = priority_state($return,$ERRORS{WARNING});
                        $message="(> $warning)";
                    }
                    printf("%s : Flash id %s (%s) : %.0f%% %s\n",$state,$index,$desc[$index],$threshold,$message);
                }
            }
            if( $found_index == 0 ){
                $state = 'UNKNOWN';
                print "$state : ";
                print "Index $select_index not found. You should try index between 2 and 4\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
        elsif( $type eq 'pwr' || $type eq 'temp' || $type eq 'fan' ){
            $state = 'OK';
            print "$state : ";
            print "Data types (pwr,temp and fan) are not availables for this D-Link switches class\n";
            $return = priority_state($return,$ERRORS{$state});
        }
        elsif ($type eq 'pwr' ){
            $result = $snmp->get_table(-baseoid => $oid_dgs_1510_pwr_descr);
            my $found_index = 0;
            foreach my $key ( sort keys %$result)
            {
                #on récupère l'index
                if($key=~ /([0-9]*)$/ )
                {
                    my $index = $1;
                    if( defined $select_index && $select_index != $index) {
                        next;
                    }
                    my $descr = $$result{$key};
                    my $status = getsnmpOID($snmp,"$oid_dgs_1510_pwr_status.$index");
                    $found_index = 1;

                    $state = $esiaPowerStatus{$status};
                    print "$state : $index. $descr status is ".$esEntityExtEnvPowerStatus{$status}."\n";
                    $return = priority_state($return,$ERRORS{$state});
                }
            }
            if( $found_index == 0 ){
                $state = 'UNKNOWN';
                print "$state : ";
                print "Index $select_index not found. You should try index between 2 and 4\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
        elsif ($type eq 'fan' ){
            $result = $snmp->get_table(-baseoid => $oid_dgs_1510_fan_descr);
            my $found_index = 0;
            foreach my $key ( sort keys %$result)
            {
                #on récupère l'index
                if($key=~ /([0-9]*)$/ )
                {
                    my $index = $1;
                    if( defined $select_index && $select_index != $index) {
                        next;
                    }
                    my $descr = $$result{$key};
                    my $status = getsnmpOID($snmp,"$oid_dgs_1510_fan_status.$index");
                    $found_index = 1;

                    $state = $esiaFanStatus{$status};
                    print "$state : $index. $descr status is ".$esEntityExtEnvFanStatus{$status}."\n";
                    $return = priority_state($return,$ERRORS{$state});
                }
            }
            if( $found_index == 0 ){
                $state = 'UNKNOWN';
                print "$state : ";
                print "Index $select_index not found. You should try index between 2 and 4\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
        elsif ($type eq 'temp' ){
            $result = $snmp->get_table(-baseoid => $oid_dgs_1510_temp_descr);
            my $found_index = 0;
            foreach my $key ( sort keys %$result)
            {
                #on récupère l'index
                if($key=~ /([0-9]*)$/ )
                {
                    my $index = $1;
                    if( defined $select_index && $select_index != $index) {
                        next;
                    }
                    my $descr = $$result{$key};
                    my $status = getsnmpOID($snmp,"$oid_dgs_1510_temp_status.$index");
                    my $current = getsnmpOID($snmp,"$oid_dgs_1510_temp_current.$index");
                    my $low = getsnmpOID($snmp,"$oid_dgs_1510_temp_low.$index");
                    my $high = getsnmpOID($snmp,"$oid_dgs_1510_temp_high.$index");
                    $found_index = 1;

                    $state = $esiaTempStatus{$status};

                    my $message="";
                    if($current >= $high)
                    {
                        #state always CRITICAL
                        $state="CRITICAL";
                        $message="high $current > $high";
                    }
                    elsif($current <= $low)
                    {
                        #state always CRITICAL
                        $state="CRITICAL";
                        $message="lower $current < $low";
                    }
                    else
                    {
                        #state OK or esiaTempStatus value
                        $message="normal $current < $high";
                    }

                    print "$state : $index. $descr status is ".$esEntityExtEnvTempStatus{$status}. " value is $message\n";
                    $return = priority_state($return,$ERRORS{$state});
                }
            }
            if( $found_index == 0 ){
                $state = 'UNKNOWN';
                print "$state : ";
                print "Index $select_index not found. You should try index between 2 and 4\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
        else{
            print "ERROR : type $type not implemented\n\n";
            print_help();
            exit $ERRORS{'UNKNOWN'};
        }
}
exit($return);
