#!/usr/bin/perl -w 
############################## check_snmp_emc.pl ########
my $Version='1.0';
# Date : 15 mars 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_emc.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning='55';
my $critical='60';
my $type = 'node';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";


# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password


#OID ISILON
my $oid_cluster_name        ="1.3.6.1.4.1.12124.1.1.1.0";
my $oid_cluster_Health      ="1.3.6.1.4.1.12124.1.1.2.0";
my $oid_cluster_node_conf   ="1.3.6.1.4.1.12124.1.1.5.0";
my $oid_cluster_node_online ="1.3.6.1.4.1.12124.1.1.6.0";
my $oid_node_name        ="1.3.6.1.4.1.12124.2.1.1.0";
my $oid_node_Health      ="1.3.6.1.4.1.12124.2.1.2.0";
my $oid_node_readOnly    ="1.3.6.1.4.1.12124.2.1.4.0";
my $oid_disk_name    ="1.3.6.1.4.1.12124.2.52.1.4";
my $oid_disk_bay     ="1.3.6.1.4.1.12124.2.52.1.1";
my $oid_disk_status  ="1.3.6.1.4.1.12124.2.52.1.5";
my $oid_disk_serial  ="1.3.6.1.4.1.12124.2.52.1.7";
my $oid_temp_name   ="1.3.6.1.4.1.12124.2.54.1.3";
my $oid_temp_value  ="1.3.6.1.4.1.12124.2.54.1.4";

my $oid_node_cpu_perf  ="1.3.6.1.2.1.25.3.3.1.2";

my $oid_node_fan_name  ="1.3.6.1.4.1.12124.2.53.1.2";
my $oid_node_fan_descr  ="1.3.6.1.4.1.12124.2.53.1.3";
my $oid_node_fan_rpm  ="1.3.6.1.4.1.12124.2.53.1.4";
sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

sub print_help() {
	printf "check snmp HP switch.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -T                  Test type (cluster,node,disk)\n";
	print " -w                  warning temperature threshold\n";
	print " -c                  critical temperature threshold\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	'l=s'	=> \$o_login,		'login=s'	=> \$o_login,
	'x=s'	=> \$o_passwd,		'passwd=s'	=> \$o_passwd,
	'X=s'	=> \$o_privpass,		'privpass=s'	=> \$o_privpass,
	'L=s'	=> \$v3protocols,		'protocols=s'	=> \$v3protocols,   
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if (defined($o_login) || defined($o_passwd)) 
{
    $snmp_version =3;
}
if ($snmp_version == 3) 
{
    my @v3proto=split(/,/,$v3protocols);
    if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
    if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
    if (!defined ($o_privpass)) 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -timeout          => 5
        );  
    }
    else 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -privpassword	=> $o_privpass,
	      -privprotocol => $o_privproto,
          -timeout          => 5
        );
    }
}
elsif ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>5
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>5
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysObjectOID");


#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysObjectOID");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $Sysoid_value = $$result{"$oid_sysObjectOID"};
if($Sysoid_value =~/3\.6\.1\.4\.1\.([0-9]*)/)
{
    $Sysoid_value = $1;
}
#ISILON
if($Sysoid_value == 12325)
{
    if($type eq 'cluster')
    {
        my $name = getsnmpOID($snmp,$oid_cluster_name);
        my $status = getsnmpOID($snmp,$oid_cluster_Health);
        my $confnode = getsnmpOID($snmp,$oid_cluster_node_conf);
        my $nodeonline = getsnmpOID($snmp,$oid_cluster_node_online);
        my $state = "OK";
        my $mode="status is OK";
        if($status == 1)
        {
            $mode="is in ATTN mode";
            $state = "CRITICAL";
        }
        elsif($status == 2)
        {
            $mode="is down";
            $state = "CRITICAL";
        }
        elsif($status == 3)
        {
            $mode="has some other problems";
            $state = "CRITICAL";
        }
        elsif($status != 0)
        {
            $mode="error checking cluster";
            $state = "UNKNOWN";
        }
        print "$state : cluster $name $mode. Online is $nodeonline of $confnode\n";
        $return = priority_state($return,$ERRORS{$state});
    }   
    elsif($type eq 'node')
    {
        my $state = "OK";
        my $mode="status is OK";
        my $name = getsnmpOID($snmp,$oid_node_name);
        my $status = getsnmpOID($snmp,$oid_node_Health);
        my $readonly = getsnmpOID($snmp,$oid_node_readOnly);
        if($status == 1)
        {
            $mode="is in ATTN mode";
            $state = "CRITICAL";
        }
        elsif($status == 2)
        {
            $mode="is down";
            $state = "CRITICAL";
        }
        elsif($status == 3)
        {
            $mode="has some other problems";
            $state = "CRITICAL";
        }
        elsif($status != 0)
        {
            $mode="error checking cluster";
            $state = "UNKNOWN";
        }
        if($readonly == 1)
        {
            $mode.=". Node is read-only state";
            $state = "CRITICAL";       
        }
        print "$state : node $name $mode\n";
        $return = priority_state($return,$ERRORS{$state});
    }
    elsif($type eq 'disk')
    {
        $result = $snmp->get_table(Baseoid => $oid_disk_name);
        foreach my $key ( keys %$result) 
        {	
            my $state = 'OK';
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
                my $name =$$result{"$key"};
		        my $bay = getsnmpOID($snmp,"$oid_disk_bay.$index"); 
		        my $status = getsnmpOID($snmp,"$oid_disk_status.$index"); 
		        my $serial = getsnmpOID($snmp,"$oid_disk_serial.$index");       
                my $state="OK";
                my $error="";
                if($status eq 'HEALTHY')
                {
                    $state="OK";
                    $error="is OK";
                }
                elsif($status eq 'SMARTFAIL')
                {
                    $state="WARNING";
                    $error="is smart failed status";
                }
                elsif($status eq 'L3')
                {
                    $state="OK";
                    $error="is caching L3";
                }
                elsif($status eq 'DEAD')
                {
                    $state="CRITICAL";
                    $error="is dead";
                }
                else
                {
                    $state="UNKNOWN";
                    $error="is unknown state : $status";
                }
                print "$state : disk $name ($serial) $error on bay $bay\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
    }
    elsif($type eq 'temp')
    {
        $result = $snmp->get_table(Baseoid => $oid_temp_name);
        foreach my $key ( keys %$result) 
        {	
            my $state = 'OK';
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
                my $name =$$result{"$key"};
		        my $temp = getsnmpOID($snmp,"$oid_temp_value.$index"); 
                if($temp=~ /^([0-9]*)\./ )
                {
                    $temp = $1;
                }
                my $state="OK";
                my $error=" < $warning Celsius";
                if($temp > $critical)
                {
                    $state="CRITICAL";
                    $error=" > $critical Celsius";
                }
                elsif($temp > $warning)
                {
                    $state="WARNING";
                    $error=" > $warning Celsius";
                }
               
                print "$state : $name is $temp $error\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
    }
    elsif($type eq 'fan')
    {
        $result = $snmp->get_table(Baseoid => $oid_node_fan_name);
        foreach my $key ( keys %$result) 
        {	
            my $state = 'OK';
	        #on récupère l'index
	        if($key=~ /([0-9]*)$/ )
	        {
                my $index =$1;
                my $name =$$result{"$key"};
		        my $rpm = getsnmpOID($snmp,"$oid_node_fan_rpm.$index"); 
                my $state="OK";
                my $message=" < $warning";
                if($rpm > $critical)
                {
                    $state="CRITICAL";
                    $message=" > $critical";
                }
                elsif($rpm > $warning)
                {
                    $state="WARNING";
                    $message=" > $warning";
                }
               
                print "$state : $name is $rpm rpm $message|rpm=$rpm\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
    }
    elsif($type eq 'cpu')
    {
        my $state = "OK";
        $result = $snmp->get_table(Baseoid => $oid_node_cpu_perf);
        my $cpuload=0;
        my $i=0;
        foreach my $key ( keys %$result) 
        {
            $cpuload = $cpuload + $$result{"$key"};
            $i++;
        }
        my $used = $cpuload/$i;
        $used = sprintf( "%.1f", $used); 
        my $message=" < $warning";    
        if($used > $critical)
        {
            $state="CRITICAL";
            $message=" > $critical";
        }
        elsif($used > $warning)
        {
            $state="WARNING";
            $message=" > $warning";
        }      
        print "$state : CPU load is $used% $message|cpu=$used\n";
        $return = priority_state($return,$ERRORS{$state});
    }

}
else
{
    print "OID is unknown. Model not support contact your support\@esia-sa.com\n";
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
exit($return);
