#!/usr/bin/perl -w
################## check_snmp_epdu_env #################
my $Version='1.0';
# Date : Novembre 17 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use POSIX qw(strftime);
use Switch;
use strict;
use Net::SNMP;
use Getopt::Long;
my $PROGNAME='check_snmp_epdu_env.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "esia_read";
my $snmp_version = 2;
my ($hostname);
my $warning='70';
my $port='161';
my $critical='80';
my $output="";
my $status="";
my $state="";
my $service="";
my $result;
my $maxWh = 0;
my $maxWatts = 0;
my $maxVA = 0;
my $maxmA = 0;
my ($snmp, $snmperror);
my @tab_value_read;

my $epdu_descr   			  = "1.3.6.1.2.1.1.1.0";

my $epdu_units 				  = "1.3.6.1.4.1.534.6.6.7.1";
my $epdu_productName 		  = "1.3.6.1.4.1.534.6.6.7.1.2.1.2.0";  # Name of the product.
my $epdu_partNumber 		  = "1.3.6.1.4.1.534.6.6.7.1.2.1.3.0";  # Part number of the unit.
my $epdu_serialNumber 		  = "1.3.6.1.4.1.534.6.6.7.1.2.1.4.0";  # Serial number of the unit.
my $epdu_firmwareVersion 	  = "1.3.6.1.4.1.534.6.6.7.1.2.1.5.0";  # Firmware version.
my $epdu_unitName 			  = "1.3.6.1.4.1.534.6.6.7.1.2.1.6.0";  # Descriptive name for the unit.
my $epdu_inputCount	 		  = "1.3.6.1.4.1.534.6.6.7.1.2.1.20.0"; # Number of inputs on this ePDU.
my $epdu_groupCount 		  = "1.3.6.1.4.1.534.6.6.7.1.2.1.21.0"; # Number of groups on this ePDU. Groups include breakers, outlet sections,and user-defined groups.
my $epdu_outletCount 		  = "1.3.6.1.4.1.534.6.6.7.1.2.1.22.0"; # Number of outlets on this ePDU.
my $epdu_temperatureCount 	  = "1.3.6.1.4.1.534.6.6.7.1.2.1.23.0"; # Max number of temperature measurements on this ePDU.
my $epdu_humidityCount 		  = "1.3.6.1.4.1.534.6.6.7.1.2.1.24.0"; # Max number of humidity measurements on this ePDU.
my $epdu_contactCount 		  = "1.3.6.1.4.1.534.6.6.7.1.2.1.25.0"; # Max number of contact sensors on this ePDU.

my $epdu_inputs 			  = "1.3.6.1.4.1.534.6.6.7.3";

my $epdu_inputVoltage 		  = "1.3.6.1.4.1.534.6.6.7.3.2.1.3"; # Units are millivolts
my $epdu_inputVoltageThStatus = "1.3.6.1.4.1.534.6.6.7.3.2.1.4"; # Status  #good(0),lowWarning(1),lowCritical(2),highWarning(3),highCritical(4)
my $epdu_inputCurrentCapacity = "1.3.6.1.4.1.534.6.6.7.3.3.1.3"; # Units are milliamps
my $epdu_inputCurrent 		  = "1.3.6.1.4.1.534.6.6.7.3.3.1.4"; # Units are milliamps
my $epdu_inputCurrentThStatus = "1.3.6.1.4.1.534.6.6.7.3.3.1.5"; # Status  #good(0),lowWarning(1),lowCritical(2),highWarning(3),highCritical(4)
my $epdu_inputVA 			  = "1.3.6.1.4.1.534.6.6.7.3.4.1.3"; # Units are VA. A value of -1 indicates that this object is not available for a particular ePDU model
my $epdu_inputWatts 		  = "1.3.6.1.4.1.534.6.6.7.3.4.1.4"; # Units are Watts
my $epdu_inputWh 			  = "1.3.6.1.4.1.534.6.6.7.3.4.1.5"; # Units are Watt-hours

my $epdu_groups 			  = "1.3.6.1.4.1.534.6.6.7.5";

my $epdu_groupVoltage		  = "1.3.6.1.4.1.534.6.6.7.5.3.1.3"; # Units are millivolts.
my $epdu_groupVoltageThStatus = "1.3.6.1.4.1.534.6.6.7.5.3.1.4"; # Status  #good(0),lowWarning(1),lowCritical(2),highWarning(3),highCritical(4)
my $epdu_groupCurrent		  = "1.3.6.1.4.1.534.6.6.7.5.4.1.3"; # Units are milliamps.
my $epdu_groupCurrentThStatus = "1.3.6.1.4.1.534.6.6.7.5.4.1.4"; # Status  #good(0),lowWarning(1),lowCritical(2),highWarning(3),highCritical(4)
my $epdu_groupVA			  = "1.3.6.1.4.1.534.6.6.7.5.5.1.2"; # Units are VA.
my $epdu_groupWatts			  = "1.3.6.1.4.1.534.6.6.7.5.5.1.3"; # Units are Watts.
my $epdu_groupWh			  = "1.3.6.1.4.1.534.6.6.7.5.5.1.4"; # Units are Watt-hours
 
my $epdu_outlets 			 = "1.3.6.1.4.1.534.6.6.7.6";

my $epdu_environmental		 = "1.3.6.1.4.1.534.6.6.7.7";

my $epdu_temperature_status	 = "1.3.6.1.4.1.534.6.6.7.7.1.1.5.0.1"; #good(0),lowWarning(1),lowCritical(2),highWarning(3),highCritical(4)
my $epdu_humidity_status     = "1.3.6.1.4.1.534.6.6.7.7.2.1.5.0.1"; #good(0),lowWarning(1),lowCritical(2),highWarning(3),highCritical(4)
my $epdu_contact 		 	 = "1.3.6.1.4.1.534.6.6.7.7.3";


sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname) 	Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -w	warning value in % (default: 70)\n";
	print " -c	critical value in % (default: 80)\n";
	print " -V (--version)	Plugin version\n";
	print " -h (--help)	usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print " $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] [-w warning] [-c critical] \n";
	print " $PROGNAME [-h | --help]\n";
	print " $PROGNAME [-V | --version]\n";
}

sub error() {
	print (" ERROR SNMP ENV\n");
	#si l'host ne répond pas on blocke le script
	if (!defined($result))
	{
		if($snmp_version == 2)
		{
			($snmp, $snmperror) = Net::SNMP->session(
					-hostname  => $hostname,
					-domain    => $domain,
					-version   => 'snmpv1',
					-port      => $port,
					-community => $community
				);

			$result = $snmp->get_request("$epdu_descr");
			if (!defined($result))
			{
				printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
				$snmp->close;
				exit $ERRORS{"UNKNOWN"};
			}
		}
		else
		{
			printf("UNKNOWN ERROR: Description table : %s.\n",
			$snmp->error);
			$snmp->close;
			exit $ERRORS{"UNKNOWN"};
		}
	}
}

sub readOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,      "warning=i"      => \$warning,
	"c=i" => \$critical,     "critical=i"     => \$critical,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2,
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
 			-version    => 'snmpv1',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2,			
		);
}

my $return=$ERRORS{OK};
if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
	exit $ERRORS{"UNKNOWN"};
}
else
{
	$result = $snmp->get_request($epdu_productName);
	if (defined($result))
	{
	    my $model_name = $$result{"$epdu_productName"};	
	    my $serial_number = readOID($snmp,"$epdu_serialNumber");
		my $device_name = readOID($snmp,"$epdu_unitName");
		my $software_version = readOID($snmp,"$epdu_firmwareVersion");

		print "Model : $model_name, serial number : $serial_number, version : $software_version \n";		

		$result = $snmp->get_table(Baseoid => $epdu_inputVoltage);
		if (!defined($result))
		{
		   print "The input table : na\n";
		   $return = priority_state($return,$ERRORS{UNKNOWN})
		}
		else
		{
			foreach my $key ( keys %$result)
			{
			    my $id = ($key =~ /1.3.6.1.4.1.534.6.6.7.3.2.1.3.(\S+)/);
			    $id = $1;			   

		    	if(defined($id))
		    	{
		    		my $index_capteur = ($id =~ /(\d+)$/);
		    		$index_capteur=$1;
			        my $input_voltage = readOID($snmp,"$epdu_inputVoltage.$id");
			        #my $input_current_capacity = readOID($snmp,"$epdu_inputCurrentCapacity.$id");
			        my $input_current = readOID($snmp,"$epdu_inputCurrent.$id");
			        my $input_VA = readOID($snmp,"$epdu_inputVA.$id");
			   	    my $input_Watts = readOID($snmp,"$epdu_inputWatts.$id");
			        my $input_Wh = readOID($snmp,"$epdu_inputWh.$id");
			        my $input_voltage_status = readOID($snmp,"$epdu_inputVoltageThStatus.$id");
			        my $input_current_status = readOID($snmp,"$epdu_inputCurrentThStatus.$id");

			        my $state_voltage = "OK";
			        my $state_current = "OK";

					switch ($input_voltage_status) 
					{
						case 0	{ $input_voltage_status="good"; }
						case 1	{ $state_voltage = "WARNING"; $input_voltage_status="lowWarning"; $return = priority_state($return,$ERRORS{WARNING})}
						case 2	{ $state_voltage = "CRITICAL"; $input_voltage_status="lowCritical"; $return = priority_state($return,$ERRORS{CRITICAL})}
						case 3  { $state_voltage = "WARNING"; $input_voltage_status="highWarning"; $return = priority_state($return,$ERRORS{WARNING})}
						case 4 	{ $state_voltage = "CRITICAL"; $input_voltage_status="highCritical"; $return = priority_state($return,$ERRORS{CRITICAL})}
						else	{ $state_voltage = "UNKNOWN"; $input_voltage_status="Unknown"; $return = priority_state($return,$ERRORS{UNKNOWN})}
    				}

    				switch ($input_current_status) 
					{
						case 0	{ $input_current_status="good"; }
						case 1	{ $state_current = "WARNING"; $input_current_status="lowWarning"; $return = priority_state($return,$ERRORS{WARNING})}
						case 2	{ $state_current = "CRITICAL"; $input_current_status="lowCritical"; $return = priority_state($return,$ERRORS{CRITICAL})}
						case 3  { $state_current = "WARNING"; $input_current_status="highWarning"; $return = priority_state($return,$ERRORS{WARNING})}
						case 4 	{ $state_current = "CRITICAL"; $input_current_status="highCritical"; $return = priority_state($return,$ERRORS{CRITICAL})}
						else	{ $state_current = "UNKNOWN"; $input_current_status="Unknown"; $return = priority_state($return,$ERRORS{UNKNOWN})}
    				}
					print "$state_voltage Input $index_capteur : Voltage ($input_voltage_status) : $input_voltage mV, $state_current Current ($input_current_status) : $input_current mA, VA : $input_VA, Watts : $input_Watts, Wh : $input_Wh \n";				
			    	$maxWh = $maxWh + $input_Wh;
			    	$maxWatts = $maxWatts + $input_Watts;
			    	$maxVA = $maxVA + $input_VA;
			    	$maxmA = $maxmA + $input_current;
		        }
		        else
		        {
		        	print "The input table : na\n";
		        	$return = priority_state($return,$ERRORS{UNKNOWN})
		        }
		    }		    
		}

		$result = $snmp->get_table(Baseoid => $epdu_groupVoltage);
		if (!defined($result))
		{
		   print "The group table : na\n";
		   $return = priority_state($return,$ERRORS{UNKNOWN})
		}
		else
		{
			foreach my $key ( keys %$result)
			{
			    my $id = ($key =~ /1.3.6.1.4.1.534.6.6.7.5.3.1.3.(\S+)/);
			    $id = $1;
		    	if(defined($id))
		    	{
		    		my $index_capteur = ($id =~ /(\d+)$/);
		    		$index_capteur=$1;
			        my $group_voltage = readOID($snmp,"$epdu_groupVoltage.$id");
			        my $group_current = readOID($snmp,"$epdu_groupCurrent.$id");
			        my $group_VA = readOID($snmp,"$epdu_groupVA.$id");
			        my $group_Watts = readOID($snmp,"$epdu_groupWatts.$id");
			   	    my $group_Wh = readOID($snmp,"$epdu_groupWh.$id");
			   	    my $group_voltage_status = readOID($snmp,"$epdu_groupVoltageThStatus.$id");
			   	    my $group_current_status = readOID($snmp,"$epdu_groupCurrentThStatus.$id");

					my $state_voltage = "OK";
					my $state_current = "OK";

					switch ($group_voltage_status) 
					{
						case 0	{ $group_voltage_status="good"; }
						case 1	{ $state_voltage = "WARNING"; $group_voltage_status="lowWarning"; $return = priority_state($return,$ERRORS{WARNING}) }
						case 2	{ $state_voltage = "CRITICAL"; $group_voltage_status="lowCritical"; $return = priority_state($return,$ERRORS{CRITICAL})}
						case 3  { $state_voltage = "WARNING"; $group_voltage_status="highWarning"; $return = priority_state($return,$ERRORS{WARNING}) }
						case 4 	{ $state_voltage = "CRITICAL"; $group_voltage_status="highCritical"; $return = priority_state($return,$ERRORS{CRITICAL})}
						else	{ $state_voltage = "UNKNOWN"; $group_voltage_status="Unknown"; $return = priority_state($return,$ERRORS{UNKNOWN}) }
    				}

    				switch ($group_current_status) 
					{
						case 0	{ $group_current_status="good"; }
						case 1	{ $state_current = "WARNING"; $group_current_status="lowWarning"; $return = priority_state($return,$ERRORS{WARNING}) }
						case 2	{ $state_current = "CRITICAL"; $group_current_status="lowCritical"; $return = priority_state($return,$ERRORS{CRITICAL}) }
						case 3  { $state_current = "WARNING"; $group_current_status="highWarning"; $return = priority_state($return,$ERRORS{WARNING}) }
						case 4 	{ $state_current = "CRITICAL"; $group_current_status="highCritical"; $return = priority_state($return,$ERRORS{CRITICAL}) }
						else	{ $state_current = "UNKNOWN"; $group_current_status="Unknown"; $return = priority_state($return,$ERRORS{UNKNOWN}) }
    				}
					print "$state_voltage Group $index_capteur : Voltage ($group_voltage_status) : $group_voltage mV, $state_current Current ($group_current_status) : $group_current mA, VA : $group_VA, Watts : $group_Watts, Wh : $group_Wh \n";
		        }
		        else
		        {
		        	print "The group table : na\n";
		        	$return = priority_state($return,$ERRORS{UNKNOWN}) 
		        }
		    }		  
		}
		#good(0),lowWarning(1),lowCritical(2),highWarning(3),highCritical(4)
		my $temperature_status = readOID($snmp,"$epdu_temperature_status");
		my $humidity_status = readOID($snmp,"$epdu_humidity_status");
		my $state_temp = "OK";
		my $state_humidity = "OK";

		switch ($temperature_status) 
		{
			case 0	{ $temperature_status="good"; }
			case 1	{ $state_temp = "WARNING"; $temperature_status="lowWarning";  $return = priority_state($return,$ERRORS{WARNING})}
			case 2	{ $state_temp = "CRITICAL"; $temperature_status="lowCritical";  $return = priority_state($return,$ERRORS{CRITICAL})}
			case 3  { $state_temp = "WARNING"; $temperature_status="highWarning";  $return = priority_state($return,$ERRORS{WARNING})}
			case 4 	{ $state_temp = "CRITICAL"; $temperature_status="highCritical";  $return = priority_state($return,$ERRORS{CRITICAL})}
			else	{ $state_temp = "UNKNOWN"; $temperature_status="Unknown";  $return = priority_state($return,$ERRORS{UNKNOWN})}
		}

		switch ($humidity_status) 
		{
			case 0	{ $humidity_status="good"; }
			case 1	{ $state_humidity = "WARNING"; $humidity_status="lowWarning";  $return = priority_state($return,$ERRORS{WARNING})}
			case 2	{ $state_humidity = "CRITICAL"; $humidity_status="lowCritical";  $return = priority_state($return,$ERRORS{CRITICAL})}
			case 3  { $state_humidity = "WARNING"; $humidity_status="highWarning";  $return = priority_state($return,$ERRORS{WARNING})}
			case 4 	{ $state_humidity = "CRITICAL"; $humidity_status="highCritical";  $return = priority_state($return,$ERRORS{CRITICAL})}
			else	{ $state_humidity = "UNKNOWN"; $humidity_status="Unknown";  $return = priority_state($return,$ERRORS{UNKNOWN})}
		}
		print "$state_temp Temperature : $temperature_status \n";
		print "$state_humidity Humidity : $humidity_status \n";

		print "Total Input : mA : $maxmA, VA : $maxVA, Watts : $maxWatts, Wh : $maxWh |mA=$maxmA,VA=$maxVA,Watts=$maxWatts,Wh=$maxWh \n";
	}
 	else
 	{
 		error();
 	}
}
exit $return;
