#!/usr/bin/perl -w
############################## check_snmp_esx_mem #################
my $Version='1.0';
# Date : Oct 24 2016
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_esx_mem.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=$ERRORS{OK};
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 80;
my $critical = 90;
#OID ESX
my $oid_hrmemory= "1.3.6.1.2.1.25.2.2.0";
my $oid_perf_mem="1.3.6.1.2.1.25.5.1.1.2";

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}
sub print_help() {
	printf "check ESX Memory status\n";
	print "\n";
	print " -H (--hostname)     Hostname to query with https or http - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -T	target value (default: ALL)\n";
	print " -w	warning value (default: 70)\n";
	print " -c	critical value (default: 80)\n";
	print " -h (--help)         usage help\n";
	print "\n";
}
#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
);

# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations
my $result = $snmp->get_request("$oid_hrmemory");
#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_hrmemory");
		if (!defined($result))
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $totalmemory = $$result{"$oid_hrmemory"};

$result = $snmp->get_table(Baseoid => $oid_perf_mem);
if (!defined($result))
{
	printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	$snmp->close;
	exit $ERRORS{"UNKNOWN"};
}
my $mem_used = 0;
foreach my $key ( keys %$result)
{
	$mem_used = $$result{"$key"} + $mem_used;
}
#
#Calcul du warning et du critique
#
my $mem_percent_used = ($mem_used / $totalmemory)*100;
my $state = "OK";
my $message="< $warning";
$mem_percent_used = sprintf("%.2f", $mem_percent_used);
if($mem_percent_used >= $critical)
{
	$message="> $critical";
	$state = "CRITICAL";
	$return = priority_state($return,$ERRORS{CRITICAL});
}
elsif($mem_percent_used >= $warning)
{
	$message="> $warning";
	$state = "WARNING";
	$return = priority_state($return,$ERRORS{WARNING});
}

my $intunity ="KB";
my $totaloctet = $totalmemory;
my $memoctet_used = $mem_used;
# on regarde l'unité d'affiche en kb/s ou en Mb/S
if( int($mem_used/1024)>0)
{
	$mem_used = sprintf("%.3f", $mem_used/1024);
	$totalmemory = sprintf("%.3f", $totalmemory/1024);
	$intunity ="MB";
}
if( int($mem_used/1024)>0)
{
	$mem_used = sprintf("%.3f", $mem_used/1024);
	$totalmemory = sprintf("%.3f", $totalmemory/1024);
	$intunity ="GB";
}
print "$state: memory used $mem_percent_used% ($mem_used $intunity / $totalmemory $intunity) $message|used=$mem_percent_used;total=$totaloctet;mem=$memoctet_used\n";
exit($return);
