#!/usr/bin/perl -w
############################## check_snmp_syno_diskhealth.pl ########
my $Version='1.0';
# Date : Mar 14 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;

my $PROGNAME='check_snmp_fortinet.pl';
my $PROGDESC = "Check fortigate (fortinet manufacturer) sensors by SNMP (CPU, RAM,Session,VPN) ";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 80;
my $critical = 90;
my $type = "CPU";
## OIDs ##
my $oid_unitdesc         = ".1.3.6.1.2.1.1.5.0";                   # Location of Fortinet device description... (String)
my $oid_serial           = ".1.3.6.1.4.1.12356.100.1.1.1.0";       # Location of Fortinet serial number (String)
my $oid_firmware         = ".1.3.6.1.4.1.12356.101.4.1.1.0";       # Location of Fortinet firmware
my $oid_cpu              = ".1.3.6.1.4.1.12356.101.13.2.1.1.3";    # Location of cluster member CPU (%)
my $oid_net              = ".1.3.6.1.4.1.12356.101.13.2.1.1.5";    # Location of cluster member Net (kbps)
my $oid_mem              = ".1.3.6.1.4.1.12356.101.13.2.1.1.4";    # Location of cluster member Mem (%)
my $oid_ses              = ".1.3.6.1.4.1.12356.101.13.2.1.1.6";    # Location of cluster member Sessions (int)

## FortiAnalyzer OIDs ##
my $oid_faz_cpu_used     = ".1.3.6.1.4.1.12356.103.2.1.1.0";       # Location of CPU for FortiAnalyzer (%)
my $oid_faz_mem_used     = ".1.3.6.1.4.1.12356.103.2.1.2.0";       # Location of Memory used for FortiAnalyzer (kb)
my $oid_faz_mem_avail    = ".1.3.6.1.4.1.12356.103.2.1.3.0";       # Location of Memory available for FortiAnalyzer (kb)
my $oid_faz_disk_used    = ".1.3.6.1.4.1.12356.103.2.1.4.0";       # Location of Disk used for FortiAnalyzer (Mb)
my $oid_faz_disk_avail   = ".1.3.6.1.4.1.12356.103.2.1.5.0";       # Location of Disk available for FortiAnalyzer (Mb)

## FortiMail OIDs ##
my $oid_fe_cpu           = ".1.3.6.1.4.1.12356.105.1.6.0";         # Location of CPU for FortiMail (%)
my $oid_fe_mem           = ".1.3.6.1.4.1.12356.105.1.7.0";         # Location of Memory used for FortiMail (%)
my $oid_fe_ldisk         = ".1.3.6.1.4.1.12356.105.1.8.0";         # Location of Log Disk used for FortiMail (%)
my $oid_fe_mdisk         = ".1.3.6.1.4.1.12356.105.1.9.0";         # Location of Mail Disk used for FortiMail (%)
my $oid_fe_load          = ".1.3.6.1.4.1.12356.105.1.30.0";        # Location of Load used for FortiMail (%)
my $oid_fe_ses           = ".1.3.6.1.4.1.12356.105.1.10.0";        # Location of cluster member Sessions for FortiMail (int)

# Cluster
my $oid_cluster_type     = ".1.3.6.1.4.1.12356.101.13.1.1.0";      # Location of Fortinet cluster type (String)
my $oid_cluster_master_serial = ".1.3.6.1.4.1.12356.101.13.2.1.1.16.1"; # Location of cluster master Serial
my $oid_cluster_serials  = ".1.3.6.1.4.1.12356.101.13.2.1.1.2";    # Location of Cluster serials (String)
my $oid_cluster_sync_state = ".1.3.6.1.4.1.12356.101.13.2.1.1.12"; # Location of cluster sync state (int)

# VPN OIDs
# XXX to be checked
my $oid_ActiveSSL         = ".1.3.6.1.4.1.12356.101.12.2.3.1.2"; # Location of Fortinet firewall SSL VPN Tunnel connection count
my $oid_ActiveSSLTunnel   = ".1.3.6.1.4.1.12356.101.12.2.3.1.6"; # Location of Fortinet firewall SSL VPN Tunnel connection count
my $oid_ipsectuntableroot = ".1.3.6.1.4.1.12356.101.12.2.2.1";     # Table of IPSec VPN tunnels
my $oid_VPNtunStatus        = ".1.3.6.1.4.1.12356.101.12.2.2.1.20";                                 # Location of a tunnel's connection status
my $oid_VPNtunIndex       = ".1.3.6.1.4.1.12356.101.12.2.2.1.1";                                  # Location of a tunnel's index...
my $oidf_tunname          = ".3";                                  # Location of a tunnel's name...

# WTP
my $oid_apstatetableroot  = ".1.3.6.1.4.1.12356.101.14.4.4.1.7";   # Represents the connection state of a WTP to AC : offLine(1), onLine(2), downloadingImage(3), connectedImage(4), other(0)
my $oid_wtpsessions       = ".1.3.6.1.4.1.12356.101.14.2.5.0";     # Represents the number of WTPs that are connecting to the AC.
my $oid_wtpmanaged        = ".1.3.6.1.4.1.12356.101.14.2.4.0";     # Represents the number of WTPs being managed on the AC
my $oid_apipaddrtableroot = ".1.3.6.1.4.1.12356.101.14.4.4.1.3" ;  # Represents the IP address of a WTP
my $oid_apidtableroot     = ".1.3.6.1.4.1.12356.101.14.4.4.1.1" ;  # Represents the unique identifier of a WTP

# HARDWARE SENSORS
# "A list of device specific hardware sensors and values. Because different devices have different hardware sensor capabilities, this table may or may not contain any values."
my $oid_hwsensor_cnt     = ".1.3.6.1.4.1.12356.101.4.3.1.0";       # Hardware Sensor count
my $oid_hwsensorid       = ".1.3.6.1.4.1.12356.101.4.3.2.1.1";     # Hardware Sensor index
my $oid_hwsensorname     = ".1.3.6.1.4.1.12356.101.4.3.2.1.2";     # Hardware Sensor Name
my $oid_hwsensorvalue    = ".1.3.6.1.4.1.12356.101.4.3.2.1.3";     # Hardware Sensor Value
my $oid_hwsensoralarm    = ".1.3.6.1.4.1.12356.101.4.3.2.1.4";     # Hardware Sensor Alarm (not all sensors have alarms!)


sub print_help() {
	printf "check la charge CPU des Fortinet.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning disk temperature (default: 40)\n";
	print " -c                  critical disk temperature (default: 50)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

ESIA::PLUGIN::arm_end_signal($TIMEOUT);

#
#MAIN
#
&Getopt::Long::config('bundling');
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
);
# -h & --help print help
if ($opt_h || !defined($snmp))
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-T","--type","type of check [CPU|MEM|Ses|VPN] (default: CPU)" ]);
	push(@arr,["-w","--warning","warning threshold in percent (default: 80)" ]);
	push(@arr,["-c","--critical","critical threshold in percent(default: 90) "]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on interroge en SNMP pour récupérer les informations

my $result = ESIA::SNMP::get($snmp,"$oid_serial");

#si l'host ne répond pas on blocke le script
if (!defined($result))
{
    printf("MERDEUNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
my $curr_serial =  $result;
my $curr_desc = ESIA::SNMP::get($snmp,"$oid_unitdesc");
my $message ="";
my $return_state = "OK";
# Get cluster master ID to fetch data from it and not from the slaves
my $cluster_master_id = get_cluster_master_id();
if($type eq 'CPU' || $type eq 'cpu')
{
	$message = get_cpu();
}
elsif($type eq 'MEM'|| $type eq 'mem')
{
	$message = get_mem();
}
elsif($type eq 'Ses')
{
	$message = get_session();
}
elsif($type eq 'VPN')
{
	$message = get_vpn_state();
}
print "$message";
exit($return);

sub get_cluster_master_id{
	# Get the cluster master device snmp id to use in oid results
	my $master_id = 1;
	my $master_serial = ESIA::SNMP::get($snmp,"$oid_cluster_master_serial");
	if($master_serial eq"")
	{
		my $serials = $snmp->get_table(Baseoid => $oid_cluster_serials);
		if (defined($master_serial) && defined($serials))
		{
			foreach my $key ( keys %$serials)
			{
				if($key =~/\.([0-9]*)$/)
				{
					my $id = $1;
					if ($$serials{$key} eq $master_serial)
					{
						$master_id = $id;
						last;
					}
				}
			}
		}
	}
	return $master_id;
}
sub get_cpu
{
	my $message="";
	print "$oid_cpu.$cluster_master_id\n";
	my $cpu_val = ESIA::SNMP::get($snmp,"$oid_cpu.$cluster_master_id");
	if (!defined($cpu_val))
	{
		print "CPU na\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
	}
	else
	{
		my $status = "OK";
		my $endline="< ".$warning."%;";
		if ( $cpu_val >= $critical )
		{
			$status = "CRITICAL";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			$endline="> ".$critical."%;";
		}
		elsif ( $cpu_val >= $warning )
		{
			$status = "WARNING";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			$endline="> ".$warning."%;";
		}
		else
		{
			$status = "OK";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
		}
		$message = "$status: $curr_desc(Current device: $curr_serial) load ".$cpu_val."% $endline |'cpu'=".$cpu_val."%;".$warning.";".$critical."\n";

	}
	return $message;
}
sub get_mem
{
	my $message="";
	my $mem_val = ESIA::SNMP::get($snmp,"$oid_mem.$cluster_master_id");
	if (!defined($mem_val))
	{
		print "Memory na\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
	}
	else
	{
		my $status = "OK";
		my $endline="< ".$warning."%;";
		if ( $mem_val >= $critical )
		{
			$status = "CRITICAL";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			$endline="> ".$critical."%;";
		}
		elsif ( $mem_val >= $warning )
		{
			$status = "WARNING";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			$endline="> ".$warning."%;";
		}
		else
		{
			$status = "OK";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
		}
		$message = "$status: $curr_desc(Current device: $curr_serial) memory is ".$mem_val."% $endline |'memory'=".$mem_val."%;".$warning.";".$critical."\n";

	}
	return $message;
}
sub get_session
{
	my $message="";
	my $ses_val = ESIA::SNMP::get($snmp,"$oid_ses.$cluster_master_id");
	if (!defined($ses_val))
	{
		print "Session na\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
	}
	else
	{
		my $status = "OK";
		my $endline="< ".$warning.";";
		if ( $ses_val >= $critical )
		{
			$status = "CRITICAL";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
			$endline="> ".$critical.";";
		}
		elsif ( $ses_val >= $warning )
		{
			$status = "WARNING";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
			$endline="> ".$warning.";";
		}
		else
		{
			$status = "OK";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
		}
		$message = "$status: $curr_desc(Current device: $curr_serial) number session: ".$ses_val." $endline |'session'=".$ses_val.";".$warning.";".$critical."\n";

	}
	return $message;
}
sub get_vpn_state
{
	my $ipsectunsdown = 0;
	my $ipsectunsopen = 0;
	my $ActiveSSL = 0;
	my $ActiveSSLTunnel = 0;
	my $ipsecMsg = "";
	my $SSLMsg = "";
    #si valeur par défaut on indique les valeurs VPN par défaut
    if($warning == 80)
    {
        $warning = 1;
        $critical= 2;
    }
	# Enumeration for the tunnel up/down states
	my %entitystate = (
		                  '1' => 'down',
		                  '2' => 'up'
		                );
	my $status = "OK";
	# Work on the table to get connections from all virtual domains
	my $activeSSLTable = $snmp->get_table(Baseoid => $oid_ActiveSSL);
	my $activeSSLTunnelTable = $snmp->get_table(Baseoid => $oid_ActiveSSLTunnel);
	if (!defined($activeSSLTable) || !defined($activeSSLTunnelTable))
	{
		$SSLMsg = "No SSL connection na\n";
	}
	else
	{
		foreach my $key (keys %$activeSSLTable)
		{
			$ActiveSSL = $ActiveSSL + $$activeSSLTable{$key};
		}
		foreach my $key (keys %$activeSSLTunnelTable)
		{
			$ActiveSSLTunnel = $ActiveSSLTunnel + $$activeSSLTunnelTable{$key};
		}
		$SSLMsg = "Active SSL-VPN Connections/Tunnels: " . $ActiveSSL."/".$ActiveSSLTunnel."\n";
	}
	#TUNNEL IPSEC
	my $VPNtunStatusTable = $snmp->get_table(Baseoid => $oid_VPNtunStatus);
	my $nbr_ipsec = 0;

	# Create new hash where keys are index entry (unique index by ipsec tunnel) and value is an array of status (it's possible to have several states for a ipsec connection)
	if (!defined($VPNtunStatusTable))
	{
		$ipsecMsg = "No IPSEC Tunnels na\n";
	}
	else
	{
		my %ipsecStatesByEntryIndex;
		foreach my $key (keys %$VPNtunStatusTable)
		{
			if($key=~ /([0-9]*)\.[0-9]*$/)
	        {
				push @{$ipsecStatesByEntryIndex{$1}}, $$VPNtunStatusTable{"$key"};
	        }
		}
		# For each ipsec entry check if there is an open state
		foreach my $indexKey (keys %ipsecStatesByEntryIndex)
		{
			$nbr_ipsec++;
			foreach my $vpn_state (@{$ipsecStatesByEntryIndex{$indexKey}})
			{
				if($vpn_state == 2 || $vpn_state eq "up")
				{
					$ipsectunsopen++;
					last; # Break loop if we found an up state for this index
				}
			}
		}
		$ipsectunsdown = $nbr_ipsec - $ipsectunsopen;
		$ipsecMsg = "IPSEC Tunnels: Configured/Active: " . $nbr_ipsec . "/" . $ipsectunsopen;
	}

	if ($SSLMsg eq "" && $ipsecMsg eq "")
	{
		print $SSLMsg . $ipsecMsg;
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
	}
	else
	{
		if ($ipsectunsdown >= $critical)
	    {
	        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
	        $status = "CRITICAL";
	    }
		elsif ($ipsectunsdown >= $warning)
	    {
	        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
	        $status = "WARNING";
	    }

		$message = "$status: $curr_desc(Current device: $curr_serial)"."\n".$SSLMsg.$ipsecMsg." |'ActiveSSL-VPN'=".$ActiveSSL." 'ActiveIPSEC'=".$ipsectunsopen."\n";
	}

	return $message;
}
