#!/usr/bin/perl -w
############################## check_snmp_fujitsu_diskhealth #################
my $Version='1.0';
# Date : Juillet 06 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use Switch;
use strict;
use Net::SNMP;
use Getopt::Long;
my $PROGNAME='check_snmp_fujitsu_diskhealth.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $port='161';
my $result;
my $state;
my $smartstate;
#my @tab_value_read;
my ($snmp, $snmperror);

my $fujitsu_descr = "1.3.6.1.2.1.1.1.0";

####RAID####
my $fujitsu_raid_number_of_physical_disks="1.3.6.1.4.1.231.2.49.1.4.2.1.11";
my $fujitsu_raid_number_of_logical_drives="1.3.6.1.4.1.231.2.49.1.4.2.1.12";
my $fujitsu_raid_status					 ="1.3.6.1.4.1.231.2.49.1.4.2.1.15";
# 1: OK, 2: Warning, 3: Error, 5: Unknown

####PHYSICAL DRIVE####
my $fujitsu_number_physical_drives       ="1.3.6.1.4.1.231.2.49.1.5.1.0"; # (GET) (Number of Physical Drive)
my $fujitsu_physical_device_model	     ="1.3.6.1.4.1.231.2.49.1.5.2.1.5";
my $fujitsu_physical_device_SmartStatus	 ="1.3.6.1.4.1.231.2.49.1.5.2.1.14"; # 1: OK, 2: Failure Predicted, 3: S.M.A.R.T. Not Available, 4: S.M.A.R.T. Monitoring Disabled
my $fujitsu_physical_device_Status		 ="1.3.6.1.4.1.231.2.49.1.5.2.1.15";
# Normal: 3, 4; Warning: 6, 7, 8, 11; Failed: 1, 2, 5, 9, 10, 12
# 1: Unknown, 2: No Disk, 3: Online, 4: Ready, 5: Failed, 6: Rebuilding, 7: Global Hot Spare, 8: Dedicated Hot Spare, 9: Offline, 10: Failed Unconfigured, 11:Formatting, 12: Dead
my $fujitsu_physical_device_SerialNumber ="1.3.6.1.4.1.231.2.49.1.5.2.1.17";
my $fujitsu_physical_device_CapacityMB	 ="1.3.6.1.4.1.231.2.49.1.5.2.1.21";


####LOGICAL DRIVE####
my $fujitsu_number_logical_drives        ="1.3.6.1.4.1.231.2.49.1.6.1.0"; # (GET) (Number of Logical Drive)
my $fujitsu_logical_drive_entry          ="1.3.6.1.4.1.231.2.49.1.6.2.1";
my $fujitsu_logical_drive_RaidLevelStr   ="1.3.6.1.4.1.231.2.49.1.6.2.1.6";
my $fujitsu_logical_drive_status         ="1.3.6.1.4.1.231.2.49.1.6.2.1.19";
# Normal: 2, Failed: 1, 3-14
# 1: Unknown, 2: Operational, 3: Partially Degraded, 4: Degraded, 5: Failed, 6: Rebuilding, 7: Checking, 8: MDCing, 9: Initializing, 10: Bac kground Initializing, 11:Migrating, 12: Copying, 13: Offline, 14: In Use Spare
my $fujitsu_logical_drive_name			 ="1.3.6.1.4.1.231.2.49.1.6.2.1.11";
my $fujitsu_logical_drive_TotalSizeMB	 ="1.3.6.1.4.1.231.2.49.1.6.2.1.13";


sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)		Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -V (--version)	Plugin version\n";
	print " -h (--help)	usage help\n";
	print "\n";
}

sub error() {
	print (" ERROR SNMP diskhealth !!!! \n");
	#si l'host ne répond pas on blocke le script
	if (!defined($result))
	{
		if($snmp_version == 2)
		{
			($snmp, $snmperror) = Net::SNMP->session(
					-hostname  => $hostname,
					-domain    => $domain,
					-version   => 'snmpv1',
					-port      => $port,
					-community => $community
				);

			$result = $snmp->get_request("$fujitsu_descr");
			if (!defined($result))
			{
				printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
				$snmp->close;
				exit $ERRORS{"UNKNOWN"};
			}
		}
		else
		{
			printf("UNKNOWN ERROR: Description table : %s.\n",
			$snmp->error);
			$snmp->close;
			exit $ERRORS{"UNKNOWN"};
		}
	}
}

sub print_usage() {
	print "Usage: \n";
	print " $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] \n";
	print " $PROGNAME [-h | --help]\n";
	print "	$PROGNAME [-V | --version]\n";
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
);

# -h & --help print help
if($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname  => $hostname,
			-version   => 'snmpv1',
			-domain    => $domain,
			-port      => $port,
			-community => $community,
			-timeout   => 2
		);
}

my $return=$ERRORS{OK};
if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
	exit $ERRORS{"UNKNOWN"};
}
else
{
	####LOGICAL DRIVE####
	$result = $snmp->get_request($fujitsu_number_logical_drives);
	if(defined($result))
	{
		my $nbrlogicaldrive = $$result{"$fujitsu_number_logical_drives"};
		print "Number of Logical Drives : $nbrlogicaldrive\n";
		$result = $snmp->get_table(Baseoid => $fujitsu_logical_drive_RaidLevelStr);
		foreach my $key ( keys %$result)
		{
			if($key=~ /1.3.6.1.4.1.231.2.49.1.6.2.1.6.(.*)/ )
			{
				my $id =$1;
				$result = $snmp->get_request("$fujitsu_logical_drive_RaidLevelStr.$id");
				my $raid_level =  $$result{"$fujitsu_logical_drive_RaidLevelStr.$id"};
				$result = $snmp->get_request("$fujitsu_logical_drive_name.$id");
				my $logical_drvie_name =  $$result{"$fujitsu_logical_drive_name.$id"};
				$result = $snmp->get_request("$fujitsu_logical_drive_TotalSizeMB.$id");
				my $logical_drvie_total_size =  $$result{"$fujitsu_logical_drive_TotalSizeMB.$id"};
				$result = $snmp->get_request("$fujitsu_logical_drive_status.$id");
				my $logical_drive_status = $$result{"$fujitsu_logical_drive_status.$id"};
				
   				switch ($logical_drive_status) 
   				{
					case 1	{
						$logical_drive_status = "Unknown"; 			
						$state = "UNKNOWN";
						$return = priority_state($return,$ERRORS{UNKNOWN}) 
					}
					case 2	{
						$logical_drive_status = "Operational"; 
						$state = "OK";
					}
					case 3	{
						$logical_drive_status = "Partially Degraded"; 
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 4	{
						$logical_drive_status = "Degraded"; 
						$state = "CRITICAL";
						$return = priority_state($return,$ERRORS{CRITICAL}) 
					}
					case 5	{
						$logical_drive_status = "Failed"; 
						$state = "CRITICAL";
						$return = priority_state($return,$ERRORS{CRITICAL}) 
					}
					case 6 	{
						$logical_drive_status = "Rebuilding"; 
						$state = "WARNING";						
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 7	{
						$logical_drive_status = "Checking"; 
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 8	{
						$logical_drive_status = "MDCing"; 
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 9	{
						$logical_drive_status = "Initializing"; 
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 10	{
						$logical_drive_status = "Bac kground Initializing"; 
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 11	{
						$logical_drive_status = "Migrating"; 
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 12	{
						$logical_drive_status = "Copying"; 
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 13	{
						$logical_drive_status = "Offline"; 
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 14	{
						$logical_drive_status = "In Use Spare"; 
						$state = "WARNING";						
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					else	{
						$logical_drive_status = "Unknown";
						$state = "UNKNOWN";
						$return = priority_state($return,$ERRORS{UNKNOWN}) 
					}
    			}	

    			if($logical_drvie_total_size > 1000000)
    			{
    				$logical_drvie_total_size=sprintf("%.2f",(($logical_drvie_total_size/1024)/1024));
    				$logical_drvie_total_size="$logical_drvie_total_size TB";
    			}
    			elsif($logical_drvie_total_size > 1000)
    			{
    				$logical_drvie_total_size=sprintf("%.2f",$logical_drvie_total_size/1024);
    				$logical_drvie_total_size="$logical_drvie_total_size GB";
    			}

    			print ("$state : $logical_drvie_name ; $raid_level ; Capacity : $logical_drvie_total_size ; Status : $logical_drive_status\n");
			}
		}
	}
	else
	{
		error();
	}


	####PHYSICAL DRIVE####
	$result = $snmp->get_request($fujitsu_number_physical_drives);
	if(defined($result))
	{
		my $nbrphysicaldrive = $$result{"$fujitsu_number_physical_drives"};
		print "Number of Physical Devices : $nbrphysicaldrive\n";
		$result = $snmp->get_table(Baseoid => $fujitsu_physical_device_model);
		foreach my $key ( keys %$result)
		{
			if($key=~ /1.3.6.1.4.1.231.2.49.1.5.2.1.5.(.*)/ )
			{
				my $id =$1;
				$result = $snmp->get_request("$fujitsu_physical_device_SerialNumber.$id");
				my $serial_number_physical_drive = $$result{"$fujitsu_physical_device_SerialNumber.$id"};
				$result = $snmp->get_request("$fujitsu_physical_device_CapacityMB.$id");
				my $physical_drive_total_size = $$result{"$fujitsu_physical_device_CapacityMB.$id"};
				$result = $snmp->get_request("$fujitsu_physical_device_model.$id");
				my $physical_drive_model = $$result{"$fujitsu_physical_device_model.$id"};
				$result = $snmp->get_request("$fujitsu_physical_device_Status.$id");
				my $physical_device_status = $$result{"$fujitsu_physical_device_Status.$id"};

				switch ($physical_device_status) 
   				{
					case 1	{
						$physical_device_status = "Unknown"; 
						$state = "UNKNOWN";
						$return = priority_state($return,$ERRORS{UNKNOWN}) 
					}
					case 2	{
						$physical_device_status = "No Disk";
						$state = "UNKNOWN";
						$return = priority_state($return,$ERRORS{UNKNOWN}) 
					}
					case 3	{
						$physical_device_status = "Online"; 
						$state = "OK";
					}
					case 4	{
						$physical_device_status = "Ready";
						$state = "OK";
					}
					case 5	{
						$physical_device_status = "Failed"; 
						$state = "CRITICAL";
						$return = priority_state($return,$ERRORS{CRITICAL}) 
					}
					case 6 	{
						$physical_device_status = "Rebuilding"; 
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 7	{
						$physical_device_status = "Global Hot Spare"; 
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 8	{
						$physical_device_status = "Dedicated Hot Spare"; 
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 9	{
						$physical_device_status = "Offline"; 
						$state = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 10	{
						$physical_device_status = "Failed Unconfigured"; 
						$state = "CRITICAL";						
						$return = priority_state($return,$ERRORS{CRITICAL}) 
					}
					case 11	{
						$physical_device_status = "Formatting"; 
						$state = "WARNING";						
						$return = priority_state($return,$ERRORS{WARNING}) 
					}
					case 12	{
						$physical_device_status = "Dead"; 
						$state = "CRITICAL";					
						$return = priority_state($return,$ERRORS{CRITICAL})
					}
					else	{
						$physical_device_status = "Unknown";
						$state = "UNKNOWN";
						$return = priority_state($return,$ERRORS{UNKNOWN})
					}
    			}

				$result = $snmp->get_request("$fujitsu_physical_device_SmartStatus.$id");
				my $physical_drive_smart_status = $$result{"$fujitsu_physical_device_SmartStatus.$id"};
				switch ($physical_drive_smart_status) 
   				{
					case 1	{
						$physical_drive_smart_status = "OK"; 
						$smartstate = "OK"						;
					}
					case 2	{
						$physical_drive_smart_status = "Failure Predicted"; 					
						$smartstate = "CRITICAL";
						$return = priority_state($return,$ERRORS{CRITICAL})
					}
					case 3	{
						$physical_drive_smart_status = "S.M.A.R.T. Not Available"; 
						$smartstate = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING})
					}
					case 4	{
						$physical_drive_smart_status = "S.M.A.R.T. Monitoring Disabled"; 			
						$smartstate = "WARNING";
						$return = priority_state($return,$ERRORS{WARNING})
					}
					else	{
						$physical_drive_smart_status = "Unknown";
						$smartstate = "UNKNOWN";
						$return = priority_state($return,$ERRORS{UNKNOWN})
					}
    			}   			
 				
 				if($physical_drive_total_size > 1000000)
    			{    				
    				$physical_drive_total_size=sprintf("%.2f",(($physical_drive_total_size/1024)/1024));
    				$physical_drive_total_size="$physical_drive_total_size TB";
    			}
    			elsif($physical_drive_total_size > 1000)
    			{
    				$physical_drive_total_size=sprintf("%.2f",($physical_drive_total_size/1024));    				
    				$physical_drive_total_size="$physical_drive_total_size GB";
    			}

    			if($state eq $smartstate)
    			{
    				print ("$state : Serial Number : $serial_number_physical_drive, Capacity : $physical_drive_total_size, model : $physical_drive_model and status : $physical_device_status (SMART : $physical_drive_smart_status)\n"); 	
    			}
    			else
    			{
    				if($state eq "CRITICAL" || $smartstate eq "CRITICAL")
    				{
						print ("CRITICAL : Serial Number : $serial_number_physical_drive, Capacity : $physical_drive_total_size, model : $physical_drive_model and status : $physical_device_status (SMART : $physical_drive_smart_status)\n");
    				}
    				else
    				{
    					if($state eq "WARNING" || $smartstate eq "WARNING")
    					{
    						print ("WARNING : Serial Number : $serial_number_physical_drive, Capacity : $physical_drive_total_size, model : $physical_drive_model and status : $physical_device_status (SMART : $physical_drive_smart_status)\n");
    					}
    					else
    					{
    						if($state eq "UNKNOWN" || $smartstate eq "UNKNOWN")
    						{
    							print ("UNKNOWN : Serial Number : $serial_number_physical_drive, Capacity : $physical_drive_total_size, model : $physical_drive_model and status : $physical_device_status (SMART : $physical_drive_smart_status)\n");
    						}
    					}
    				}
    			}
 				
			} 
		}
	}
	else
	{
		error();
	}


	####RAID####
	$result = $snmp->get_table(Baseoid => $fujitsu_raid_number_of_physical_disks);
	if(defined($result))
	{
		foreach my $key ( keys %$result)
		{
			if($key=~ /1.3.6.1.4.1.231.2.49.1.4.2.1.11.(.*)/ )
			{
				my $id =$1;
				$result = $snmp->get_request("$fujitsu_raid_number_of_physical_disks.$id");
				my $raid_number_of_physical_disks = $$result{"$fujitsu_raid_number_of_physical_disks.$id"};
				$result = $snmp->get_request("$fujitsu_raid_number_of_logical_drives.$id");
				my $raid_number_of_logical_drives = $$result{"$fujitsu_raid_number_of_logical_drives.$id"};
				$result = $snmp->get_request("$fujitsu_raid_status.$id");
				my $raid_status = $$result{"$fujitsu_raid_status.$id"};
				
				if($raid_number_of_logical_drives > 0)
				{
					switch ($raid_status) 
	   				{
						case 1	{
							$raid_status = "OK"; 						
							print ("OK : Raid status : $raid_status \n");
						}
						case 2	{
							$raid_status = "Warning"; 
							print ("WARNING : Raid status : $raid_status \n");
							$return = priority_state($return,$ERRORS{WARNING})
						}
						case 3	{
							$raid_status = "Error"; 
							print ("CRITICAL : Raid status : $raid_status \n");
							$return = priority_state($return,$ERRORS{CRITICAL})
						}
						case [4,5]	{
							$raid_status = "Unknown"; 
							print ("UNKNOWN : Raid status : $raid_status \n");
							$return = priority_state($return,$ERRORS{UNKNOWN})
						}
						else	{
							$raid_status = "Unknown";
							print ("UNKNOWN : Raid status : $raid_status \n");
							$return = priority_state($return,$ERRORS{UNKNOWN})
						}
					}
				}				
			}
		}
	}
	else
	{
		error();
	}
}
exit $return;