#!/usr/bin/perl -w
############################## check_snmp_fujitsu_systemhealth #################
my $Version='1.0';
# Date : Juillet 06 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use Switch;
use strict;
use Net::SNMP;
use Getopt::Long;
my $PROGNAME='check_snmp_fujitsu_systemhealth.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $warning='2500';
my $port='161';
my $critical='3500';
my $result;
my $stateStatus;
#my @tab_value_read;
my ($snmp, $snmperror);

my $fujitsu_descr = "1.3.6.1.2.1.1.1.0";

####FAN####
my $fujitsu_fan_table                   ="1.3.6.1.4.1.231.2.10.2.2.5.2.2.1";
my $fujitsu_fan_status	   				="1.3.6.1.4.1.231.2.10.2.2.5.2.2.1.3";
# Normal: 3; Warning: 6; Failed: 1, 2, 4, 5, 7, 8
# 1: Unknown, 2: Disabled, 3: OK,4: Failed, 5: Failure Predicted, 6: Redundant Fan Failed, 7: Not Manageable, 8: Not Present
my $fujitsu_fan_CurrentSpeed	        ="1.3.6.1.4.1.231.2.10.2.2.5.2.2.1.8"; # measured in revolutions per minute
my $fujitsu_fan_name 					="1.3.6.1.4.1.231.2.10.2.2.5.2.2.1.16";


####POWER SUPPLY####
my $fujitsu_power_supply_status         ="1.3.6.1.4.1.231.2.10.2.2.5.11.1.1.2.0"; # (GET) 1=unknown, 2=ok, 3=degraded, 4=cri


sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)		Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -w	warning value in rpm (default: 2500)\n";
	print " -c	critical value in rpm (default: 3500)\n";
	print " -V (--version)	Plugin version\n";
	print " -h (--help)	usage help\n";
	print "\n";
}

sub error() {
	print (" ERROR SNMP syshealth !!!! \n");
	#si l'host ne répond pas on blocke le script
	if (!defined($result))
	{
		if($snmp_version == 2)
		{
			($snmp, $snmperror) = Net::SNMP->session(
					-hostname  => $hostname,
					-domain    => $domain,
					-version   => 'snmpv1',
					-port      => $port,
					-community => $community
				);

			$result = $snmp->get_request("$fujitsu_descr");
			if (!defined($result))
			{
				printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
				$snmp->close;
				exit $ERRORS{"UNKNOWN"};
			}
		}
		else
		{
			printf("UNKNOWN ERROR: Description table : %s.\n",
			$snmp->error);
			$snmp->close;
			exit $ERRORS{"UNKNOWN"};
		}
	}
}

sub print_usage() {
	print "Usage: \n";
	print " $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] \n";
	print " $PROGNAME [-h | --help]\n";
	print "	$PROGNAME [-V | --version]\n";
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,      "warning=i"      => \$warning,
	"c=i" => \$critical,     "critical=i"     => \$critical,
);

# -h & --help print help
if($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname  => $hostname,
			-version   => 'snmpv1',
			-domain    => $domain,
			-port      => $port,
			-community => $community,
			-timeout   => 2
		);
}

my $return=$ERRORS{OK};
if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
	exit $ERRORS{"UNKNOWN"};
}
else
{	
	####POWER SUPPLY####
	$result = $snmp->get_request($fujitsu_power_supply_status);
	if(defined($result))
	{
		my $status_power_supply = $$result{"$fujitsu_power_supply_status"};
		switch ($status_power_supply) 
   		{
   			case 1	{
				$status_power_supply = "Unknown"; 
				print ("UNKNOWN : Power supply is $status_power_supply\n");
				$return = priority_state($return,$ERRORS{UNKNOWN}) 				
			}
			case 2	{
				$status_power_supply = "OK"; 
				print ("OK : Power supply is $status_power_supply\n");
			}
			case 3	{
				$status_power_supply = "Degraded"; 
				print ("WARNING : Power supply is $status_power_supply\n");
				$return = priority_state($return,$ERRORS{WARNING}) 				
			}
			case 4	{
				$status_power_supply = "Error"; 
				print ("CRITICAL : Power supply is $status_power_supply\n");
				$return = priority_state($return,$ERRORS{CRITICAL})				
			}
			else	{
				print "status Unknown\n";				
				print ("UNKNOWN : Power supply is $status_power_supply\n");
				$return = priority_state($return,$ERRORS{UNKNOWN}) 
			}
		}			
	}
	else
	{
		error();
	}

	####FAN####
	$result = $snmp->get_table(Baseoid => $fujitsu_fan_name);
	if(defined($result))
	{
		foreach my $key ( keys %$result)
		{
			if($key=~ /1.3.6.1.4.1.231.2.10.2.2.5.2.2.1.16.(.*)/ )
			{
				my $id =$1;
				$result = $snmp->get_request("$fujitsu_fan_name.$id");
				my $fan_name = $$result{"$fujitsu_fan_name.$id"};
				$result = $snmp->get_request("$fujitsu_fan_CurrentSpeed.$id");
				my $fan_CurrentSpeed = $$result{"$fujitsu_fan_CurrentSpeed.$id"};
				$result = $snmp->get_request("$fujitsu_fan_status.$id");
				my $fan_status = $$result{"$fujitsu_fan_status.$id"};
				
				if($fan_CurrentSpeed > 0)
				{
					switch ($fan_status) 
	   				{
						case 1	{
							$fan_status = "Unknown"; 							
							$stateStatus = "UNKNOWN";
							$return = priority_state($return,$ERRORS{UNKNOWN})
						}
						case 2	{
							$fan_status = "Disabled"; 			
							$stateStatus = "WARNING";
							$return = priority_state($return,$ERRORS{WARNING})
						}
						case 3	{
							$fan_status = "OK"; 
							$stateStatus = "OK";
						}
						case 4	{
							$fan_status = "Failed"; 												
							$stateStatus = "CRITICAL";
							$return = priority_state($return,$ERRORS{CRITICAL})
						}
						case 5	{
							$fan_status = "Failure Predicted"; 			
							$stateStatus = "CRITICAL";					
							$return = priority_state($return,$ERRORS{CRITICAL})
						}
						case 6	{
							$fan_status = "Redundant Fan Failed"; 							
							$stateStatus = "WARNING";
							$return = priority_state($return,$ERRORS{WARNING})
						}
						case 7	{
							$fan_status = "Not Manageable"; 
							$stateStatus = "UNKNOWN";
							$return = priority_state($return,$ERRORS{UNKNOWN})
						}
						case 8	{
							$fan_status = "Not Present"; 							
							$stateStatus = "UNKNOWN";
							$return = priority_state($return,$ERRORS{UNKNOWN})
						}
						else	{
							print "status Unknown\n";
							$stateStatus = "UNKNOWN";
							$return = priority_state($return,$ERRORS{UNKNOWN})
						}
					}

					if($fan_CurrentSpeed < $warning)
					{
						if($stateStatus eq "CRITICAL")
						{
							print ("CRITICAL : $fan_name ; Speed : $fan_CurrentSpeed rpm ; Status : $fan_status\n");
							$return = priority_state($return,$ERRORS{CRITICAL})
						}
						elsif($stateStatus eq "WARNING")
						{
							print ("WARNING : $fan_name ; Speed : $fan_CurrentSpeed rpm ; Status : $fan_status\n");
							$return = priority_state($return,$ERRORS{WARNING})
						}
						else
						{
							print ("OK : $fan_name ; Speed : $fan_CurrentSpeed rpm ; Status : $fan_status\n");
						}
					}
					else
					{
						if($fan_CurrentSpeed > $critical)
						{
							print ("CRITICAL : $fan_name ; Speed : $fan_CurrentSpeed rpm (too fast) ; Status : $fan_status\n");
							$return = priority_state($return,$ERRORS{CRITICAL})
						}
						else
						{
							if($stateStatus eq "CRITICAL")
							{	
								print ("CRITICAL : $fan_name ; Speed : $fan_CurrentSpeed rpm (too fast) ; Status : $fan_status\n");
								$return = priority_state($return,$ERRORS{CRITICAL})
							}
							else
							{
								print ("WARNING : $fan_name ; Speed : $fan_CurrentSpeed rpm (too fast) ; Status : $fan_status\n");
								$return = priority_state($return,$ERRORS{WARNING})
							}							
						}
					}	
				}		
			}
		}
	}
	else
	{
		error();
	}
}
exit $return;