#!/usr/bin/perl -w
############################## check_snmp_fujitsu_syshealth #################
my $Version='1.0';
# Date : Juillet 06 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use Switch;
use strict;
use Net::SNMP;
use Getopt::Long;
my $PROGNAME='check_snmp_fujitsu_syshealth.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $warning='35';
my $port='161';
my $critical='45';
my $result;
#my @tab_value_read;
my ($snmp, $snmperror);
my $output = "";
my $val;

my $fujitsu_descr = "1.3.6.1.2.1.1.1.0";

####TEMPERATURE####
my $fujitsu_temp_name                   ="1.3.6.1.4.1.231.2.10.2.2.10.5.1.1.4";
my $fujitsu_status_temperature          ="1.3.6.1.4.1.231.2.10.2.2.10.5.1.1.5";
# Normal: 3, 8; Warning: 6, 9; Failed: 1, 2, 4, 5, 7
# 1: Unknown, 2: Not Available, 3: OK, 4: Sensor Failed, 5: Failed, 6: Temperature Warning – Too Hot, 7: Temperature – Critical, 8: Temperature – Normal, 9: Temperature Warning
my $fujitsu_current_temperature         ="1.3.6.1.4.1.231.2.10.2.2.10.5.1.1.6"; # in degrees Celsius
my $fujitsu_warning_level_temperature   ="1.3.6.1.4.1.231.2.10.2.2.10.5.1.1.7"; # in degrees Celsius
my $fujitsu_critical_level_temperature  ="1.3.6.1.4.1.231.2.10.2.2.10.5.1.1.8"; # in degrees Celsius


sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)		Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -w	warning value in Celsius (default: 35)\n";
	print " -c	critical value in Celsius (default: 45)\n";
	print " -V (--version)	Plugin version\n";
	print " -h (--help)	usage help\n";
	print "\n";
}

sub error() {
	print (" ERROR SNMP syshealth !!!! \n");
	#si l'host ne répond pas on blocke le script
	if (!defined($result))
	{
		if($snmp_version == 2)
		{
			($snmp, $snmperror) = Net::SNMP->session(
					-hostname  => $hostname,
					-domain    => $domain,
					-version   => 'snmpv1',
					-port      => $port,
					-community => $community
				);

			$result = $snmp->get_request("$fujitsu_descr");
			if (!defined($result))
			{
				printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
				$snmp->close;
				exit $ERRORS{"UNKNOWN"};
			}
		}
		else
		{
			printf("UNKNOWN ERROR: Description table : %s.\n",
			$snmp->error);
			$snmp->close;
			exit $ERRORS{"UNKNOWN"};
		}
	}
}

sub print_usage() {
	print "Usage: \n";
	print " $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] \n";
	print " $PROGNAME [-h | --help]\n";
	print "	$PROGNAME [-V | --version]\n";
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,      "warning=i"      => \$warning,
	"c=i" => \$critical,     "critical=i"     => \$critical,
);

# -h & --help print help
if($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname  => $hostname,
			-version   => 'snmpv1',
			-domain    => $domain,
			-port      => $port,
			-community => $community,
			-timeout   => 2
		);
}

my $return=$ERRORS{OK};
if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
	exit $ERRORS{"UNKNOWN"};
}
else
{	
	####TEMPERATURE####
	$result = $snmp->get_table(Baseoid => $fujitsu_temp_name);
	if(defined($result))
	{
		foreach my $key ( keys %$result)
		{
			if($key=~ /1.3.6.1.4.1.231.2.10.2.2.10.5.1.1.4.(.*)/ )
			{
				my $id =$1;
				$result = $snmp->get_request("$fujitsu_temp_name.$id");				
				my $temp_name = $$result{"$fujitsu_temp_name.$id"};
				
				$result = $snmp->get_request("$fujitsu_current_temperature.$id");				
				my $current_temp = $$result{"$fujitsu_current_temperature.$id"};							

				if($current_temp > 0)
				{
					$result = $snmp->get_request("$fujitsu_warning_level_temperature.$id");				
					my $war_level_temp = $$result{"$fujitsu_warning_level_temperature.$id"};
				
					$result = $snmp->get_request("$fujitsu_critical_level_temperature.$id");				
					my $cri_level_temp = $$result{"$fujitsu_critical_level_temperature.$id"};
				
					$result = $snmp->get_request("$fujitsu_status_temperature.$id");				
					my $status_temp = $$result{"$fujitsu_status_temperature.$id"};

					switch ($status_temp) 
	   				{
						case 1	{
							$status_temp = "Unknown"; 
							$output=$output."UNKNOWN : $temp_name : $current_temp  degree celsius ; Status : $status_temp\n";
							$return = priority_state($return,$ERRORS{UNKNOWN}) 					
						}
						case [2,6,9]	{
							if($status_temp==2) {$status_temp = "Not Available"; }
							else {$status_temp = "Temperature Warning";}
							$output=$output."WARNING : $temp_name : $current_temp  degree celsius ; Status : $status_temp\n";
							$return = priority_state($return,$ERRORS{WARNING}) 
						}
						case [3,8]	{
							$status_temp = "OK";
							if($temp_name eq "Ambient")
							{							
								if($current_temp < $warning)
								{
									$output=$output."OK : $temp_name : $current_temp  degree celsius ; Status : $status_temp\n";
								}
								else
								{
									if($current_temp > $critical)
									{
										$output=$output."CRITICAL : $temp_name : $current_temp  degree celsius ; Status : $status_temp\n";
										$return = priority_state($return,$ERRORS{CRITICAL}) 
									}
									else
									{
										$output=$output."WARNING : $temp_name : $current_temp  degree celsius ; Status : $status_temp\n";
										$return = priority_state($return,$ERRORS{WARNING}) 
									}
								}	
								$val = $current_temp;
							}
							else
							{
								$output=$output."OK : $temp_name : $current_temp  degree celsius ; Status : $status_temp\n";
							}	
						}
						case [4,7]	{
							if($status_temp==4) {$status_temp = "Nensor Failed"; }
							else {$status_temp = "Temperature Critical";}
							$output=$output."CRITICAL :$temp_name : $current_temp  degree celsius ; Status : $status_temp\n";
							$return = priority_state($return,$ERRORS{CRITICAL}) 
						}
						case 5	{
							$status_temp = "Failed"; 
							$output=$output."CRITICAL : $temp_name : $current_temp  degree celsius ; Status : $status_temp\n";
							$return = priority_state($return,$ERRORS{CRITICAL}) 					
						}
					}		
				}
			}
		}
		print $output."|tempAmb=$val";		
	}
	else
	{
		error();
		$return = priority_state($return,$ERRORS{UNKNOWN}) 	
	}
}
exit $return;