#!/usr/bin/perl -w 
############################## check_snmp_load #################
my $Version='1.0';
# Date : Mars 21 2016
# Author  : Victor Marin
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_Switch_POE.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);

my ($opt_h,$opt_V);
my $domain ="UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $warning='70';
my $port='161';
my $critical='80';
my $result;
my $verbose = "Debug";


my $hp_MPS ="1.3.6.1.2.1.105.1.3.1.1.2.1"; # Maximum Power Supply    Watts
my $hp_PowerConsu = ".1.3.6.1.2.1.105.1.3.1.1.4.1"; # Total Power Consumption Watts
my $hp_LevelAlarm = ".1.3.6.1.2.1.105.1.3.1.1.5.1"; # Power Over-Usage in % limite en % qd il envoit une alarme
my $hp_descr = "1.3.6.1.2.1.1.1.0"; 

# pethPsePortMPSAbsentCounter
my @Tab_MPSAC =
(".1.3.6.1.2.1.105.1.1.1.8.1.1",".1.3.6.1.2.1.105.1.1.1.8.1.2",
".1.3.6.1.2.1.105.1.1.1.8.1.3",".1.3.6.1.2.1.105.1.1.1.8.1.4",
".1.3.6.1.2.1.105.1.1.1.8.1.5",".1.3.6.1.2.1.105.1.1.1.8.1.6",
".1.3.6.1.2.1.105.1.1.1.8.1.7",".1.3.6.1.2.1.105.1.1.1.8.1.8");

# pethPsePortInvalidSignatureCounter
my @Tab_PortInvaSignCount = 
(".1.3.6.1.2.1.105.1.1.1.11.1.1",".1.3.6.1.2.1.105.1.1.1.11.1.2",
".1.3.6.1.2.1.105.1.1.1.11.1.3",".1.3.6.1.2.1.105.1.1.1.11.1.4",
".1.3.6.1.2.1.105.1.1.1.11.1.5",".1.3.6.1.2.1.105.1.1.1.11.1.6",
".1.3.6.1.2.1.105.1.1.1.11.1.7",".1.3.6.1.2.1.105.1.1.1.11.1.8");

# pethPsePortOverLoadCounter
my @Tab_PortOverLoadCount = 
(".1.3.6.1.2.1.105.1.1.1.13.1.1",".1.3.6.1.2.1.105.1.1.1.13.1.2",
".1.3.6.1.2.1.105.1.1.1.13.1.3",".1.3.6.1.2.1.105.1.1.1.13.1.4",
".1.3.6.1.2.1.105.1.1.1.13.1.5",".1.3.6.1.2.1.105.1.1.1.13.1.6",
".1.3.6.1.2.1.105.1.1.1.13.1.7",".1.3.6.1.2.1.105.1.1.1.13.1.8");

sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -D (--domain) 	    UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning value in % (default: 70)\n";
	print " -c                  critical value in % (default: 80)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print " -d (--debug mode)   Detail counter (encode 'option')\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"D=s"   => \$domain,	"domain=s"    => \$domain,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,	"warning=i" => \$warning,
	"c=i" => \$critical,	"critical=i" => \$critical,
	"d=s" => \$verbose,	"verbose=s" => \$verbose
);
if($verbose eq"option")
{
	print "coucou";
}
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-domain=>$domain,
		-port => $port,
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-domain=>$domain,
		-port => $port,
		-community => $community,
		-timeout=>2
	);
}

if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
	exit $ERRORS{"UNKNOWN"};
}
else
{	
	my $MPS;
	$result = $snmp->get_request($hp_MPS);
	if(defined($result))
	{
		$MPS = $$result{"$hp_MPS"};
		print (" Maximum Power Supply : $MPS watt(s) \n");
	}
	else
	{
		print (" ERROR MPS !!!! \n");
		fct_Error();
	}


	$result = $snmp->get_request($hp_PowerConsu);
	if(defined($result))
	{
		my $PowerConsu = $$result{"$hp_PowerConsu"};
		print (" Total Power Consumption : $PowerConsu watt(s) \n");
		$PowerConsu=($PowerConsu*$MPS)/100;
		print (" En pourcentage : $PowerConsu % \n");
	}
	else
	{
		print (" ERROR PowerConsu !!!! \n");
		fct_Error();
	}
	
	$result = $snmp->get_request($hp_LevelAlarm);
	if(defined($result))
	{
		my $LevelAlarm = $$result{"$hp_LevelAlarm"};
		print (" Power Over-Usage (Level Alarm) : $LevelAlarm % \n");
	}
	else
	{
		print (" ERROR LevelAlarm !!!! \n");
		fct_Error();
	}

	if($verbose eq"option")
	{
		my $i=1;

		foreach my $v (@Tab_MPSAC)
		{	
			$result = $snmp->get_request($v);
			if(defined($result))
			{
				my $PsePortMPSAC = $$result{"$v"};
				print (" pethPsePortMPSAbsentCounter $i :  $PsePortMPSAC \n");
			}
			else
			{
				print (" PsePortMPSAC $i !!!! \n");
				fct_Error();
			}
			$i++;
		}
	
		$i=1;
		foreach my $v (@Tab_PortInvaSignCount)
		{
			$result = $snmp->get_request($v);
			if(defined($result))
			{
				my $PortInvaSignCount = $$result{"$v"};
				print (" pethPsePortInvalidSignatureCounter $i : $PortInvaSignCount \n");
			}
			else
			{
				print (" PortInvaSignCount $i !!!! \n");
				fct_Error();
			}
			$i++;
		}
	
		$i=1;
		foreach my $v (@Tab_PortOverLoadCount)
		{
			$result = $snmp->get_request($v);
			if(defined($result))
			{
				my $PortOverLoadCount = $$result{"$v"};
				print (" pethPsePortOverLoadCounter $i : $PortOverLoadCount \n");
			}
			else
			{
				print (" PortOverLoadCount $i !!!! \n");
				fct_Error();
			}
			$i++;
		}
	}
}

#si l'host ne répond pas on blocke le script
sub fct_Error
{
	if (!defined($result)) 
	{
		if($snmp_version == 2)
		{
			($snmp, $snmperror) = Net::SNMP->session(
				-hostname => $hostname,
				-domain=>$domain,
				-version => 'snmpv1',
				-port => $port,
				-community => $community
			);
			$result = $snmp->get_request("$hp_descr");
			if (!defined($result)) 
			{
		   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		   		$snmp->close;
		  		 exit $ERRORS{"UNKNOWN"};
			}
		}
		else
		{
			printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
			$snmp->close;
			exit $ERRORS{"UNKNOWN"};
		}
	}
}
