#!/usr/bin/perl -w 
############################## check_snmp_temp #################
my $Version='1.0';
# Date : Mars 14 2016
# Author  : Victor Marin
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_Switch_temp.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);

my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $warning='60';
my $port='161';
my $critical='65';
my $result;

my $hp_tem	= "1.3.6.1.4.1.25506.2.6.1.1.1.1.12.8";   # HP tem
my $hp_descr    = "1.3.6.1.2.1.1.1.0"; 


sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -D (--domain) 	    UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning value in % (default: 60)\n";
	print " -c                  critical value in % (default: 65)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"D=s" => \$domain,	"domain=s"   => \$domain,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s"=> \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,	"warning=i" => \$warning,
	"c=i" => \$critical,	"critical=i" => \$critical
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-domain=>$domain,
		-port => $port,
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-domain=>$domain,
		-port => $port,
		-community => $community,
		-timeout=>2
	);
}

if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
}
else
{
	$result = $snmp->get_request($hp_tem);

	if(defined($result))
	{
		my $tem = $$result{"$hp_tem"};
		if($tem < $warning)
		{
			print (" Temperature : $tem° used (< $warning°) : OK\n");
			exit $ERRORS{"OK"};
		}
		else
		{
			if($tem > $critical)
			{
				print (" Temperature : $tem° used (> $critical°) : CRITICAL\n");
				exit $ERRORS{"CRITICAL"};
			}
			else
			{
				print (" Temperature : $tem° used ( $warning° <= X < $critical° ) : WARNING\n");
				exit $ERRORS{"WARNING"};
			}
		}
	}
	else
	{
		print (" ERROR TEM !!!! \n");
		#si l'host ne répond pas on blocke le script
		if (!defined($result)) 
		{
			if($snmp_version == 2)
			{
				($snmp, $snmperror) = Net::SNMP->session(
					-hostname => $hostname,
					-domain=>$domain,
					-version => 'snmpv1',
					-port => $port,
					-community => $community
				);
				$result = $snmp->get_request("$hp_descr");
				if (!defined($result)) 
				{
			   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		   			$snmp->close;
			  		exit $ERRORS{"UNKNOWN"};
				}
			}
			else
			{
				printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
				$snmp->close;
				exit $ERRORS{"UNKNOWN"};
			}
		}
	}
}

