#!/usr/bin/perl -w 
############################## check_snmp_hp_vsa.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_hp_vsa.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning='';
my $critical='';
my $type = 'cluster';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";

#HP VSA
my $oid_hp_clustername= "1.3.6.1.4.1.9804.3.1.1.2.12.44.1.2";
my $oid_hp_clusterversion= "1.3.6.1.4.1.9804.3.1.1.2.12.44.1.3";
my $oid_hp_clustersn= "1.3.6.1.4.1.9804.3.1.1.2.12.44.1.4";
my $oid_hp_clusterstate= "1.3.6.1.4.1.9804.3.1.1.2.12.44.1.5"; 	# Component description
my %oid_hp_clusterstate_text  = ( "0"  => "--Invalid--","1" => "up","2"  => "down");

my $oid_hp_moduleraid= "1.3.6.1.4.1.9804.3.1.1.2.12.46.1.8";
my $oid_hp_modulestate= "1.3.6.1.4.1.9804.3.1.1.2.12.46.1.10";
my $oid_hp_moduledescr= "1.3.6.1.4.1.9804.3.1.1.2.12.46.1.2";
my $oid_hp_moduleclustername= "1.3.6.1.4.1.9804.3.1.1.2.12.46.1.14";
my %oid_hp_modulestate_text  = ( "0"  => "--Invalid--","1" => "up","2"  => "down");

my $oid_hp_module_space_total="1.3.6.1.4.1.9804.3.1.1.2.12.46.1.5";
my $oid_hp_module_space_used="1.3.6.1.4.1.9804.3.1.1.2.12.46.1.29";

my $oid_hp_volume_name="1.3.6.1.4.1.9804.3.1.1.2.12.97.1.2";
my $oid_hp_volume_space_total="1.3.6.1.4.1.9804.3.1.1.2.12.97.1.5";
my $oid_hp_volume_space_used="1.3.6.1.4.1.9804.3.1.1.2.12.97.1.31";
my $oid_hp_volume_space_percent="1.3.6.1.4.1.9804.3.1.1.2.12.97.1.52";

my $oid_hp_raid_disk_name="1.3.6.1.4.1.9804.3.1.1.2.4.2.1.14";
my $oid_hp_raid_disk_smart="1.3.6.1.4.1.9804.3.1.1.2.4.2.1.17";
my $oid_hp_raid_disk_smarthealth="1.3.6.1.4.1.9804.3.1.1.2.4.2.1.18";
my $oid_hp_raid_disk_label ="1.3.6.1.4.1.9804.3.1.1.2.4.2.1.13";

my $oid_hp_raid_name="1.3.6.1.4.1.9804.3.1.1.2.4.4.1.2";
my $oid_hp_raid_level="1.3.6.1.4.1.9804.3.1.1.2.4.4.1.3";
my $oid_hp_raid_state="1.3.6.1.4.1.9804.3.1.1.2.4.4.1.90";
my $oid_hp_raid_rebuild="1.3.6.1.4.1.9804.3.1.1.2.4.4.1.12";

my $oid_hp_raid_controleur="1.3.6.1.4.1.9804.3.1.1.2.4.4.1.91";

# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password
my $match= undef;
sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

sub print_help() {
	printf "check snmp HP switch.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -T                  Test type (cpu,mem,env)\n";
	print " -w                  warning threshold\n";
	print " -c                  critical threshold\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	'l=s'	=> \$o_login,		'login=s'	=> \$o_login,
	'x=s'	=> \$o_passwd,		'passwd=s'	=> \$o_passwd,
	'X=s'	=> \$o_privpass,		'privpass=s'	=> \$o_privpass,
	'L=s'	=> \$v3protocols,		'protocols=s'	=> \$v3protocols, 
    "n=s"   =>\$match,		'name=s'	=> \$match,   
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if (defined($o_login) || defined($o_passwd)) 
{
    $snmp_version =3;
}
if ($snmp_version == 3) 
{
    my @v3proto=split(/,/,$v3protocols);
    if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
    if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
    if (!defined ($o_privpass)) 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -timeout          => 2
        );  
    }
    else 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -privpassword	=> $o_privpass,
	      -privprotocol => $o_privproto,
          -timeout          => 5
        );
    }
}
elsif ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
if (!defined($snmp))
{
    printf("Switch Down\n");
    exit $ERRORS{"OK"};	
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysObjectOID");


#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysObjectOID");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $Sysoid_value = $$result{"$oid_sysObjectOID"};
if($Sysoid_value =~/3\.6\.1\.4\.1\.([0-9]*)/)
{
    $Sysoid_value = $1;
}


#
# END SNMP INSTANCE
#
if($type eq 'cluster')
{
    $result = $snmp->get_table(Baseoid => $oid_hp_clustername);
    foreach my $key ( keys %$result) 
    {	
        my $state='OK';
        #on récupère l'index
	    if($key=~ /([0-9]*)$/ )
	    {
            my $index =$1;
            my $name =$$result{"$key"};
            my $status = getsnmpOID($snmp,"$oid_hp_clusterstate.$index");
            my $version = getsnmpOID($snmp,"$oid_hp_clusterversion.$index");
            my $sn = getsnmpOID($snmp,"$oid_hp_clustersn.$index");
            if($status !=1)
            {
                $state='CRITICAL';
            }

            print "$state : $name (SN: $sn, version: $version) is ".$oid_hp_clusterstate_text{$status}."\n";
            $return = priority_state($return,$ERRORS{$state});
        }
    } 
}
elsif($type eq 'modules')
{
    $result = $snmp->get_table(Baseoid => $oid_hp_moduledescr);
    foreach my $key ( keys %$result) 
    {	
        my $state='OK';
        #on récupère l'index
	    if($key=~ /([0-9]*)$/)
	    {
            my $index =$1;
            my $descr =$$result{"$key"};
            my $status = getsnmpOID($snmp,"$oid_hp_modulestate.$index");
            my $cluster = getsnmpOID($snmp,"$oid_hp_moduleclustername.$index");
            my $raid = getsnmpOID($snmp,"$oid_hp_moduleraid.$index");
            if($status !=1)
            {
                $state='CRITICAL';
            }

            print "$state : Module $descr on $cluster is ".$oid_hp_modulestate_text{$status}." (raid: $raid)\n";
            $return = priority_state($return,$ERRORS{$state});
        }        
    } 
}
elsif($type eq 'modules-space')
{
    my $flag=0;
    $result = $snmp->get_table(Baseoid => $oid_hp_moduledescr);
    foreach my $key ( keys %$result) 
    {	
        my $state='OK';
        #on récupère l'index
	    if($key=~ /([0-9]*)$/  && defined($match))
	    {
            my $index =$1;
            my $descr =$$result{"$key"};
            if($descr =~/$match/)
            {
                $flag=1;
                my $total = getsnmpOID($snmp,"$oid_hp_module_space_total.$index");
                my $used = getsnmpOID($snmp,"$oid_hp_module_space_used.$index");
                my $message = "< $warning";
                # conversion de KB en byte
                $total = $total*1024;
                $used = $used*1024;
                my $percent = int(($used/$total)*100);            
                if($percent > $critical)
                {
                    $message = "> $critical";
                    $state='CRITICAL';
                }
                elsif($percent > $warning)
                {
                    $message = "> $warning";
                    $state='WARNING';
                }
                print "$state : Module $descr: $percent% used $message |hdd=$percent,total=$total,used=$used\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
        else
        {
            print "match possible (with -n parameter) ".$$result{"$key"}."\n";
        }   
    } 
    if($flag==0)
    {
        print "UNKNOWN: Module not found\n";
        $return = priority_state($return,$ERRORS{'UNKNOWN'});
    }
}
elsif($type eq 'volumes')
{
    my $flag=0;
    $result = $snmp->get_table(Baseoid => $oid_hp_volume_name);
    foreach my $key ( keys %$result) 
    {	
        my $state='OK';
        #on récupère l'index
	    if($key=~ /([0-9]*)$/  && defined($match))
	    {
            my $index =$1;
            my $descr =$$result{"$key"};
            if($descr eq $match)
            {
                $flag=1;
                my $total = getsnmpOID($snmp,"$oid_hp_volume_space_total.$index");
                my $used = getsnmpOID($snmp,"$oid_hp_volume_space_used.$index");
                my $message = "< $warning";
                # conversion de KB en byte
                $total = $total*1024;
                $used = $used*1024;
                my $percent = getsnmpOID($snmp,"$oid_hp_volume_space_percent.$index");
                if($percent > $critical)
                {
                    $message = "> $critical";
                    $state='CRITICAL';
                }
                elsif($percent > $warning)
                {
                    $message = "> $warning";
                    $state='WARNING';
                }
                print "$state : Module $descr: $percent% used $message |hdd=$percent,total=$total,used=$used\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
        else
        {
            print "match possible (with -n parameter) ".$$result{"$key"}."\n";
        }   
    } 
    if($flag==0)
    {
        print "UNKNOWN: Module not found\n";
        $return = priority_state($return,$ERRORS{'UNKNOWN'});
    }
}
elsif($type eq 'raid')
{
    $result = $snmp->get_table(Baseoid => $oid_hp_raid_name);
    foreach my $key ( keys %$result) 
    {	
        my $state='OK';
        #on récupère l'index
	    if($key=~ /([0-9]*)$/)
	    {
            my $index =$1;
            my $name =$$result{"$key"};
            my $level = getsnmpOID($snmp,"$oid_hp_raid_level.$index");
            my $status = getsnmpOID($snmp,"$oid_hp_raid_state.$index");
            my $message="";
            if($status eq 'Normal')
            {
                $state='OK';
            }
            elsif($status eq 'rebuilding')
            {
                $state='WARNING';
                my $rebuild = getsnmpOID($snmp,"$oid_hp_raid_rebuild.$index");
                $message="rebuilding $rebuild%";
            }
            else
            {
                $state='CRITICAL';
            }
            print "$state : raid $name ($level) is $status $message\n";
            $return = priority_state($return,$ERRORS{$state});
        }
    } 
}
elsif($type eq 'disks')
{
    $result = $snmp->get_table(Baseoid => $oid_hp_raid_disk_name);
    foreach my $key ( keys %$result) 
    {	
        my $state='OK';
        #on récupère l'index
	    if($key=~ /([0-9]*)$/)
	    {
            my $index =$1;
            my $name =$$result{"$key"};
            my $label = getsnmpOID($snmp,"$oid_hp_raid_disk_label.$index");
            my $smart = getsnmpOID($snmp,"$oid_hp_raid_disk_smart.$index");
            my $message="";
            if($smart eq 'Normal')
            {
                $state='OK';
            }
            else
            {
                $state='WARNING';
            }
            print "$state : disque $name ($label) smart is $smart \n";
            $return = priority_state($return,$ERRORS{$state});
        }
    } 
}
exit($return);
