#!/usr/bin/perl -w

# basic requirements
use strict;
use Getopt::Long;
use Net::SNMP;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;
use Data::Dumper;
my $PROGNAME='check_snmp_hwg.pl';
my $PROGDESC = "Check 1wire or contact sensors on HWG poseidon, STE and STE2";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

my $snmp_version  = "1";
# options
my $opt_sensor;
my $opt_contact;
my $opt_warning=25;
my $opt_critical=30;
my $opt_calibre=0;
my $opt_help;

# oids
my $oid_sysObjectOID        = "1.3.6.1.2.1.1.2.0";
my $enterprise = '.1.3.6.1.4.1.21796';
my $oid_poseidon_1wireid = '.1.3.6.1.4.1.21796.3.3.3.1.8';
my $oid_poseidon_sensName = '.1.3.6.1.4.1.21796.3.3.3.1.2';# POSEIDON-MIB::sensName
my $oid_poseidon_sensValue = '.1.3.6.1.4.1.21796.3.3.3.1.6';# POSEIDON-MIB::sensName
my $oid_poseidon_unit = '.1.3.6.1.4.1.21796.3.3.3.1.9';# POSEIDON-MIB::sensName

my $oid_ste2_input_name =  '.1.3.6.1.4.1.21796.4.9.1.1.3';
my $oid_ste2_input_value = '.1.3.6.1.4.1.21796.4.9.1.1.2';

my $oid_poseidon_input_name =  '.1.3.6.1.4.1.21796.3.3.1.1.3';
my $oid_poseidon_input_value = '.1.3.6.1.4.1.21796.3.3.1.1.2';

my $oid_ste2_1wireid  = '.1.3.6.1.4.1.21796.4.9.3.1.8';
my $oid_ste2_sensName = '.1.3.6.1.4.1.21796.4.9.3.1.2';
my $oid_ste2_sensValue= '.1.3.6.1.4.1.21796.4.9.3.1.5';
my $oid_ste2_unit= '.1.3.6.1.4.1.21796.4.9.3.1.7';
my $oid_ste2_sensString= '.1.3.6.1.4.1.21796.4.9.3.1.4';

my $oid_ste_1wireid  = '.1.3.6.1.4.1.21796.4.1.3.1.8';
my $oid_ste_sensName = '.1.3.6.1.4.1.21796.4.1.3.1.2';
my $oid_ste_sensString= '.1.3.6.1.4.1.21796.4.1.3.1.4';
my $oid_ste_sensValue= '.1.3.6.1.4.1.21796.4.1.3.1.5';
my $oid_ste_unit= '.1.3.6.1.4.1.21796.4.1.3.1.7';
my $deviceid = '.1.3.6.1.2.1.1.1.0';
my @oids;
my $response;
my $output;
my $sensor_id;
my $return=0;
my %unit_text  = ( "0"  => "C°","1" => "fahrenheit","2"  => "kelvin","3"  => "%","4"  => "volt","5"=> "miliamper","6"  => "nounit","7"=> "pulse","8"=> "switch"   );
my %unit_ste_text  = ("0"  => "none","1"  => "C°","2" => "fahrenheit","3"  => "kelvin","4"=>"%");

sub checkthreshold
{
	# ICI CHECK VALEUR WARNING, CRITICAL (-w -c)
	my ($value,$warning,$critical,$calibre) = @_;
	my $ret = 'OK';
	if($calibre != 0)
	{
		$value += $calibre;
	}
	if($critical =~ m/([\-0-9]*)\,([\-0-9]*)/i)
	{
		if($value <= $1 || $value >= $2)
		{
			$ret = 'CRITICAL';
		}
	}
	else
	{
		if($value >= $critical)
		{
			$ret = 'CRITICAL';
		}
	}
	if($ret eq 'OK')
	{
		if($warning =~ m/([\-0-9]*)\,([\-0-9]*)/i)
		{
			if($value <= $1 || $value >= $2)
			{
				$ret = 'WARNING';
			}
		}
		else
		{
			if($value >= $warning)
			{
				$ret = 'WARNING';
			}
		}
	}
	return $ret;
}
sub sensorArray
{
	my %sensorname;
	my ($snmp, $searchsensor, $oid_wireid, $oid_name) = @_;
	if($searchsensor=~/^([0-9]*)$/)
	{
		# get the sensor ID
		my $result = $snmp->get_table($oid_wireid);
		foreach my $key ( keys %$result)
		{
			my $sensorid = $$result{"$key"};
			if($sensorid  =~/$searchsensor/)
			{
				if($key=~/\.([0-9]*)$/)
				{
					my $id = $1;
					$sensorname{$id} =  ESIA::SNMP::get($snmp,"$oid_name.$id");
				}
			}
		}
	}
	else
	{
		# get the sensor ID
		my $result = $snmp->get_table($oid_name);
		foreach my $key ( keys %$result)
		{
			my $sensorname = $$result{"$key"};
			if($sensorname =~/$searchsensor/)
			{
				if($key=~/\.([0-9]*)$/)
				{
					$sensorname{$1} = $sensorname;
				}
			}
		}
	}
	return (\%sensorname);
}

#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"h"   => \$opt_help,	"help"       => \$opt_help,
	"I=s" => \$opt_contact,"contact=s" => \$opt_contact,
	"S=s" => \$opt_sensor,"sensor=s"=>\$opt_sensor,
	"w=i" => \$opt_warning,"warning=i" => \$opt_warning,
	"c=i" => \$opt_critical,"critical=i" => \$opt_critical,
	"d=s" => \$opt_calibre,"calibre=s" => \$opt_calibre,
);
# -h & --help print help
if ($opt_help || !defined($snmp))
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-S sensors');
	my @arr;
	push(@arr,["-I","--contact","contact sensors id" ]);
	push(@arr,["-S","--sensors","1wire sensors id" ]);
	push(@arr,["-d","--calibre","calibration value" ]);
	push(@arr,["-w","--warning","warning threshold in percent (default: 80)" ]);
	push(@arr,["-c","--critical","critical threshold in percent(default: 90) "]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
my $hostname = $snmp->hostname();
# Check if all needed options present.
unless ( $hostname && ( $opt_sensor || $opt_contact ))
{
	print "\nNot enough options specified !\n";
	print "Display manual with option -h or --help\n";
	exit $ERRORS{'UNKNOWN'};
}
#on interroge en SNMP pour récupérer les informations
my $result = ESIA::SNMP::get($snmp,"$oid_sysObjectOID");
if (!defined($result))
{
	printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	$snmp->close;
	exit $ERRORS{"UNKNOWN"};
}
if (defined $opt_contact)
{
	my $sensorfound=0;
	my $device = ESIA::SNMP::get($snmp,"$deviceid");
	# setting per device OIDs for sensor values
	if ($device =~ m/Poseidon/i)
	{
		my $result = $snmp->get_table(Baseoid =>$oid_poseidon_input_name);
		foreach my $key (keys %$result)
		{
			if($key =~ /([0-9]*)$/)
			{
				my $index =$1;
				my $name =$$result{"$key"};
				if($name =~ /$opt_sensor/)
				{
					my $state = 'OK';
					my $val = ESIA::SNMP::get($snmp,"$oid_poseidon_input_value.$index");
					my $output="";
					if($val == 1)
					{
						$output .= "open";
						if( $opt_contact eq 'on')
						{
							$state='CRITICAL';
						}
					}
					else
					{
						$output .= "closed";
						if( $opt_contact eq 'off')
						{
							$state='CRITICAL';
						}
					}
					$sensorfound=1;
					print "$state : contact $name is $output\n";
					$return =  ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
				}
			}
		}
	}
	elsif ($device =~ m/STE2/i)
	{
		my $result = $snmp->get_table(Baseoid =>$oid_ste2_input_name);
		foreach my $key (keys %$result)
		{
			if($key =~ /([0-9]*)$/)
			{
				my $index =$1;
				my $name =$$result{"$key"};
				if($index =~ /$opt_sensor/)
				{
					my $state = 'OK';
					my $val = ESIA::SNMP::get($snmp,"$oid_ste2_input_value.$index");
					my $output="";
					if($val == 1)
					{
						$output .= "close";
						if( $opt_contact eq 'off')
						{
							$state='CRITICAL';
						}
					}
					else
					{
						$output .= "open";
						if( $opt_contact eq 'on')
						{
							$state='CRITICAL';
						}
					}
					$sensorfound=1;
					print "$state : contact $name is $output\n";
					$return =  ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
				}
			}
		}
	}
	if($sensorfound==0)
	{
		print "UNKNOWN: contact not found\n";
		$return =  ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
	}
}
# Si un ID sensor est bien défini
elsif (defined $opt_sensor)
{
	my $globalval=0;
	my $num=0;
	my $device = ESIA::SNMP::get($snmp,"$deviceid");
	# setting per device OIDs for sensor values
	if ($device =~ m/Poseidon/i)
	{
		my ($arr) = sensorArray($snmp, $opt_sensor, $oid_poseidon_1wireid, $oid_poseidon_sensName);
		my %arrid = %$arr;
		foreach my $id ( keys %arrid)
		{
			my $name =$arrid{$id};
			my $state="OK";
			my $value = ESIA::SNMP::get($snmp,"$oid_poseidon_sensValue.$id");
			if($value==0)
			{

			}
			else
			{
				$value = $value/10;
				$value = sprintf( "%.1f", $value);
				$state = checkthreshold($value,$opt_warning,$opt_critical,$opt_calibre);
			}
			my $type="sensors";
			my $unit = ESIA::SNMP::get($snmp,"$oid_poseidon_unit.$id");
			if($unit ==0||$unit ==1)
			{
				$type="Temperature of";
			}
			elsif($unit ==3)
			{
				$type="Humidity sensor";
			}
			print "$state : $type $name is $value ".$unit_text{$unit}."\n";
			$return =  ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
			$globalval=$globalval + $value;
			$num++;
		}
	}
	else
	{
		if ($device =~ m/STE2/i)
		{
			my ($arr) = sensorArray($snmp, $opt_sensor, $oid_ste2_1wireid, $oid_ste2_sensName);
			my %arrid = %$arr;
			foreach my $id ( keys %arrid)
			{
				my $name =$arrid{$id};
				my $value = ESIA::SNMP::get($snmp,"$oid_ste2_sensValue.$id");
				my $state="OK";
				my $flag=0;
				if($value==0)
				{
					my $str = ESIA::SNMP::get($snmp,"$oid_ste2_sensString.$id");
					if($str eq "")
					{
						print "UNKNOWN: $name is error state. check STE web page\n";
						$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
						$flag=1;
					}
					else
					{
						$state = checkthreshold($value,$opt_warning,$opt_critical,$opt_calibre);
					}
				}
				else
				{
					$value = $value/10;
					$value = sprintf( "%.1f", $value);
				}
				if($flag == 0)
				{
					$state = checkthreshold($value,$opt_warning,$opt_critical,$opt_calibre);
					my $unit = ESIA::SNMP::get($snmp,"$oid_ste2_unit.$id");
					my $type="sensors";
					if($unit ==1 || $unit ==2)
					{
						$type="Temperature of";
					}
					elsif($unit ==4)
					{
						$type="Humidity sensor";
					}
					print "$state : $type $name is $value ".$unit_ste_text{$unit}."\n";
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
					$globalval=$globalval + $value;
					$num++;
				}
			}
		}
		elsif ($device =~ m/STE/i)
		{
			my ($arr) = sensorArray($snmp,$opt_sensor,$oid_ste_1wireid,$oid_ste_sensName);
			my %arrid = %$arr;
			foreach my $id ( keys %arrid)
			{
				my $name =$arrid{$id};
				my $value = ESIA::SNMP::get($snmp,"$oid_ste_sensValue.$id");
				my $state="OK";
				my $flag=0;
				if($value==0)
				{
					my $str = ESIA::SNMP::get($snmp,"$oid_ste_sensString.$id");
					if($str eq "")
					{
						print "UNKNOWN: $name is error state. check STE web page\n";
						$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
						$flag=1;
					}
					else
					{
						$state = checkthreshold($value,$opt_warning,$opt_critical,$opt_calibre);
					}
				}
				else
				{
					$value = $value/10;
					$value = sprintf( "%.1f", $value);
				}
				if($flag == 0)
				{
					$state = checkthreshold($value,$opt_warning,$opt_critical,$opt_calibre);
					my $unit = ESIA::SNMP::get($snmp,"$oid_ste_unit.$id");
					my $type="sensors";
					if($unit ==1 || $unit ==2)
					{
						$type="Temperature of";
					}
					elsif($unit ==4)
					{
						$type="Humidity sensor";
					}
					print "$state : $type $name is $value ".$unit_ste_text{$unit}."\n";
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
					$globalval=$globalval + $value;
					$num++;
				}
			}
		}
		else
		{
			print "ERROR: device not supported";
			$snmp->close();
			exit($ERRORS{UNKNOWN});
		}
	}
	if($num == 0)
	{
		print "UNKNOWN: no sensors found $opt_sensor\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
	}
	else
	{
		$globalval = $globalval/$num;
		$globalval = sprintf( "%.1f", $globalval);
		print "|data=$globalval";
	}
}
exit($return);
