#!/usr/bin/perl -w 
############################## check_snmp_hp_vsa.pl ########
my $Version='1.0';
# Date : 19 juin 2021
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_ibm_ts.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning='';
my $critical='';
my $type = 'status';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";
# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password
my $match= undef;

sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

my $oid_hostname = ".1.3.6.1.2.1.1.5.0";
my $oid_ps1status =".1.3.6.1.4.1.2.6.257.1.3.2.1.3.0";
my $oid_ps2status =".1.3.6.1.4.1.2.6.257.1.3.2.1.4.0";


sub print_help() {
	printf "check snmp HP switch.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -T                  Test type (cpu,mem,env)\n";
	print " -w                  warning threshold\n";
	print " -c                  critical threshold\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 
#ne sert que pour la migration
my $mparam;
#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	"m=s" => \$mparam,"model=s" => \$mparam,
	'l=s'	=> \$o_login,		'login=s'	=> \$o_login,
	'x=s'	=> \$o_passwd,		'passwd=s'	=> \$o_passwd,
	'X=s'	=> \$o_privpass,		'privpass=s'	=> \$o_privpass,
	'L=s'	=> \$v3protocols,		'protocols=s'	=> \$v3protocols, 
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if (defined($o_login) || defined($o_passwd)) 
{
    $snmp_version =3;
}
if ($snmp_version == 3) 
{
    my @v3proto=split(/,/,$v3protocols);
    if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
    if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
    if (!defined ($o_privpass)) 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -timeout          => 2
        );  
    }
    else 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -privpassword	=> $o_privpass,
	      -privprotocol => $o_privproto,
          -timeout          => 5
        );
    }
}
elsif ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysObjectOID");
if (!defined($result)) 
{
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}
my $Sysoid_value = $$result{"$oid_sysObjectOID"};
if($Sysoid_value =~/noSuchObject/)
{
    my $hostname = getsnmpOID($snmp,"1.3.6.1.4.1.14851.3.1.3.5.0");
#    print "ICI $hostname";
    if(!($hostname =~/noSuchObject/))
    {
        if($type eq 'status')
        {
            my %oid_globalstatus_text  = ( "0" => "unknown","1"  => "other","2" => "ok","3"  => "degraded","4"  => "stressed","5"  => "predictiveFailure",
                                            "6"  => "error","7"  => "non-RecoverableError","8" => "starting","9"  => "stopping","10"  => "stopped",
                                            "11"  => "inService","12"  => "noContact","13" => "lostCommunication","14"  => "aborted","15"  => "dormant","16"  => "supportingEntityInError",
                                            "18"  => "powerMode","19"  => "dMTFReserved","32768" => "vendorReserved");
            my %oid_globalstatus_text_status = ( "0" => "OK","1"  => "WARNING","2" => "OK","3"  => "WARNING","4"  => "WARNING","5"  => "WARNING",
                                            "6"  => "CRITICAL","7"  => "CRITICAL","8" => "OK","9"  => "CRITICAL","10"  => "CRITICAL",
                                            "11"  => "OK","12"  => "WARNING","13" => "CRITICAL","14"  => "WARNING","15"  => "WARNING","16"  => "CRITICAL",
                                            "18"  => "WARNING","19"  => "unknown","32768" => "unknown");
            my $productname = getsnmpOID($snmp,"1.3.6.1.4.1.14851.3.1.3.1.0");
            print "$hostname is a $productname\n";
            my $state ='OK';
	        my $globalstatus = getsnmpOID($snmp,"1.3.6.1.4.1.14851.3.1.10.2.0");
            if($globalstatus < 1 && $globalstatus > 19 )
            {
                $state='UNKNOWN';
                print "UNKNOWN: unknown global status code ($globalstatus)\n";
                $return = priority_state($return,$ERRORS{'UNKNOWN'});
            }
            else
            {
                $state = $oid_globalstatus_text_status{$globalstatus};
                print "$state : Current status is ".$oid_globalstatus_text{$globalstatus}."\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
        elsif($type eq 'clean')
        {
           print "$hostname oid to clean state disk not exist\n";
        }
        elsif($type eq 'power')
        {
            my $ps1 = getsnmpOID($snmp,$oid_ps1status);
            my $ps2 = getsnmpOID($snmp,$oid_ps2status);
            if($ps1 == 1)
            {
                print "OK: power supply 1 not installed\n";
            }
            elsif($ps1 == 2 || $ps1 == 0)
            {
                print "OK: power supply 1 is ok \n";
            }
            else
            {
                print "CRITICAL: power supply 1 is not ok (code: $ps1) \n";
                $return = priority_state($return,$ERRORS{'CRITICAL'});
            }
            if($ps2 == 1)
            {
                print "OK: power supply 2 not installed\n";
            }
            elsif($ps2 == 2 || $ps2 == 0)
            {
                print "OK: power supply 2 is ok \n";
            }
            else
            {
                print "CRITICAL: power supply 2 is not ok (code: $ps1) \n";
                $return = priority_state($return,$ERRORS{'CRITICAL'});
            }
        }
	else{
		print "UNKNOWN: -T, --type $type not implemented. Parameter must be set with status, clean or power\n";
		$return = priority_state($return,$ERRORS{'UNKNOWN'});
	}
    }
}
else
{
    if($Sysoid_value =~/\.([0-9]*)$/)
    {
        $Sysoid_value = $1;
    }
    if($Sysoid_value == 210 || $Sysoid_value == 211)
    {
        my $oid_base = '.1.3.6.1.4.1.2.6.'.$Sysoid_value;
        my $oid_productname = "$oid_base.1.1.0";
        my $oid_vendorname = "$oid_base.1.3.0";
        my $oid_productid = "$oid_base.3.1.1.8.1";
        my $oid_serialnumber = "$oid_base.3.1.1.10.1";
        my $oid_firmware = "$oid_base.3.1.1.9.1";
        my $oid_globalstatus = "$oid_base.2.1.0";
        my $oid_drivenumber = "$oid_base.3.1.1.11.1";
        my $oid_cartridges = "$oid_base.3.1.1.12.1";
        my $oid_faulterror = "$oid_base.3.1.1.22.1";
        my $oid_faultseverity = "$oid_base.3.1.1.23.1";
        my $oid_faultdesc = "$oid_base.3.1.1.24.1";
        my $oid_cleanstate = "$oid_base.3.2.1.2"; 
        my %oid_globalstatus_text  = ( "1"  => "other","2" => "unknown","3"  => "ok","4"  => "non-critical","5"  => "critical","6"  => "non-recoverable");
        my $oid_diskvendor = "$oid_base.3.2.1.7"; 
        my $oid_diskmodel  = "$oid_base.3.2.1.8"; 
        if($type eq 'status')
        {
            my $hostname = getsnmpOID($snmp,$oid_hostname);
            my $productname = getsnmpOID($snmp,$oid_productname);
            my $serialnumber = getsnmpOID($snmp,$oid_serialnumber);
            my $firmware = getsnmpOID($snmp,$oid_firmware);
            print "$hostname is a $productname (SN: $serialnumber). Firmware version is $firmware\n";
            my $state ='OK';
	        my $globalstatus = getsnmpOID($snmp,$oid_globalstatus);
            my  $message="";
            my $faultdesc = getsnmpOID($snmp,$oid_faultdesc);
            #print  "global $globalstatus\n";
	    if($globalstatus == 1 || $globalstatus == 2 || ($globalstatus == 4 && $faultdesc ne 'No error'))
            {
	            my $faulterror = getsnmpOID($snmp,$oid_faulterror);
	    #        my $faultdesc = getsnmpOID($snmp,$oid_faultdesc);
                $message=" - $faultdesc (error code: $faulterror)";
                $state='WARNING';
            }
            elsif($globalstatus == 5 || $globalstatus == 6)
            {
	            my $faulterror = getsnmpOID($snmp,$oid_faulterror);
	     #       my $faultdesc = getsnmpOID($snmp,$oid_faultdesc);
                $message=" - $faultdesc (error code: $faulterror)";
                $state='CRITICAL';
            }
            elsif($globalstatus == 3 || $faultdesc eq 'No error')
            {
                $state='OK';
            }
            else
            {
                $state='UNKNOWN';
                $message="unknown statut code\n";
            }
            print "$state: Current status is $oid_globalstatus_text{$globalstatus} $message\n";
            $return = priority_state($return,$ERRORS{$state});
        }
        elsif($type eq 'clean')
        {
            $result = $snmp->get_table(Baseoid => $oid_diskvendor);
            foreach my $key ( keys %$result) 
            {	
                #on récupère l'index
	            if($key=~ /([0-9]*)$/)
	            {
                    my $index =$1;
                    my $vendor =$$result{"$key"};
                    $vendor =~ s/^\s+|\s+$//g;
                    my $diskproduct = getsnmpOID($snmp,$oid_diskmodel.".$index");
                    $diskproduct =~ s/^\s+|\s+$//g;
                    my $cleanstate = getsnmpOID($snmp,$oid_cleanstate.".$index");
		    #print "clean $cleanstate";
                    if($cleanstate == 34)
                    {
                        print "WARNING: drive $index ($vendor $diskproduct) needs to be cleaned\n";
                        $return = priority_state($return,$ERRORS{'WARNING'});
                    }
                    elsif($cleanstate == 41)
                    {
                        print "WARNING: drive $index ($vendor $diskproduct) cleaning suggested\n";
                        $return = priority_state($return,$ERRORS{'WARNING'});
                    }
                    elsif($cleanstate == 137)
                    {
                        print "WARNING: drive $index ($vendor $diskproduct) media attention\n";
                        $return = priority_state($return,$ERRORS{'WARNING'});
                    }
                    elsif($cleanstate == 2)
                    {
                        print "OK: drive $index ($vendor $diskproduct) is clean ($cleanstate)\n";
                        $return = priority_state($return,$ERRORS{'OK'});
                    }
                    else
                    {
                        print "UNKNOWN: drive $index ($vendor $diskproduct) cleanstate is unknown ($cleanstate)\n";
                        $return = priority_state($return,$ERRORS{'UNKNOWN'});
                    }
                }
            }
        }
        elsif($type eq 'power')
        {
            my $ps1 = getsnmpOID($snmp,$oid_ps1status);
            my $ps2 = getsnmpOID($snmp,$oid_ps2status);
            if($ps1 == 1)
            {
                print "OK: power supply 1 not installed\n";
            }
            elsif($ps1 == 2 || $ps1 == 0)
            {
                print "OK: power supply 1 is ok \n";
            }
            else
            {
                print "CRITICAL: power supply 1 is not ok (code: $ps1) \n";
                $return = priority_state($return,$ERRORS{'CRITICAL'});
            }
            if($ps2 == 1)
            {
                print "OK: power supply 2 not installed\n";
            }
            elsif($ps2 == 2 || $ps2 == 0)
            {
                print "OK: power supply 2 is ok \n";
            }
            else
            {
                print "CRITICAL: power supply 2 is not ok (code: $ps1) \n";
                $return = priority_state($return,$ERRORS{'CRITICAL'});
            }
        }
        else{
                print "UNKNOWN: -T, --type $type not implemented. Parameter must be set with status, clean or power\n";
                $return = priority_state($return,$ERRORS{'UNKNOWN'});
        }
    }
    elsif($Sysoid_value == 21){
        my $oid_base = '.1.3.6.1.4.1.2.6.219';
        my %statusToStr = (1=>'unknown',2=>'ok',3=>'warning',4=>'failed');
        my %statusToEsia = ('unknown'=>'UNKNOWN','ok'=>'OK','warning'=>'WARNING','failed'=>'CRITICAL');

	if( $type eq 'status' ){
		my $oid_productname = "$oid_base.2.1.1.6.0";
		my $oid_hostname = "$oid_base.2.1.1.3.0";
		my $oid_serialnumber = "$oid_base.2.1.1.7.0";
		my $oid_firmware = "$oid_base.2.1.1.8.0";

		my $oid_magstatus = "$oid_base.2.2.1.1.4.1";
		my $oid_robotstatus = "$oid_base.2.2.1.1.6.1";
		my $oid_cntlstatus = "$oid_base.2.2.1.1.7.1";
		my $oid_cntlmode = "$oid_base.2.2.1.1.8.1";

		my $host = getsnmpOID($snmp,$oid_hostname);
		if( $host eq "" ){
			$host = $hostname;
		}
		my $productname = getsnmpOID($snmp,$oid_productname);
		my $serialnumber = getsnmpOID($snmp,$oid_serialnumber);
		my $firmware = getsnmpOID($snmp,$oid_firmware);
		
		print "$host is a $productname (SN: $serialnumber). Firmware version is $firmware\n";

		my $magstatus = getsnmpOID($snmp,$oid_magstatus);
		my $robotstatus = getsnmpOID($snmp,$oid_robotstatus);
		my $cntlstatus = getsnmpOID($snmp,$oid_cntlstatus);
		my $cntlmode = getsnmpOID($snmp,$oid_cntlmode);

		$magstatus = $statusToStr{$magstatus};
		$robotstatus = $statusToStr{$robotstatus};
		$cntlstatus = $statusToStr{$cntlstatus};

		my $esia_status = $statusToEsia{$magstatus};
		print "$esia_status: Magazine Status is $magstatus \n";
		$return = priority_state($return,$ERRORS{$esia_status});

		$esia_status = $statusToEsia{$robotstatus};
		print "$esia_status: Robot Status is $robotstatus \n";
		$return = priority_state($return,$ERRORS{$esia_status});

		$esia_status = $statusToEsia{$cntlstatus};
		print "$esia_status: Controller Status is $cntlstatus \n";
		$return = priority_state($return,$ERRORS{$esia_status});

		if( $cntlmode == 3 ){
			print "CRITICAL: Controller Status is offline\n";
			$return = priority_state($return,$ERRORS{'CRITICAL'});	
		}
		elsif( $cntlmode == 2 ){
			print "OK: Controller Status is online\n";
			$return = priority_state($return,$ERRORS{'OK'});
		}
		elsif( $cntlmode == 1 ) {
			print "UNKNOWN: Controller Status is unknown\n";
			$return = priority_state($return,$ERRORS{'UNKNOWN'});
		}

		if( $return == 0 ){
			$esia_status = 'OK';
			print "$esia_status: All components status are ok/online\n";
		}		
		elsif( $return == 1 ){
                        $esia_status = 'WARNING';
			print "$esia_status: Some components status are warning\n";
		}
		elsif( $return == 2 ){
                        $esia_status = 'CRITICAL';
			print "$esia_status: Some components status are failed/offline\n";
		}
		else{
                        $esia_status = 'UNKNOWN';
			print "$esia_status: Some components status are unknown\n";
		}
	}
	elsif( $type eq 'clean' ){
		my $oid_diskvendor = "$oid_base.2.1.2.1.2";
		my $oid_diskmodel = "$oid_base.2.1.2.1.3";
		my $oid_cleaninfo = "$oid_base.2.2.2.1.2";
		my $oid_cleanstate = "$oid_base.2.2.2.1.3";  


            	my %statusDriveToStr = (1=>'unknown',2=>'ok',3=>'needClean',4=>'failed',5=>'failed');
		my %infoDriveToStr = (1=>'unknown',2=>'empty',3=>'cartridgeIn',4=>'writing',5=>'reading',6=>'rewinding',7=>'locating',8=>'cleaning');
            	my %statusDriveToEsia = ('unknown'=>'UNKNOWN','ok'=>'OK','warning'=>'WARNING','needClean'=>'CRITICAL','failed'=>'CRITICAL');
            	$result = $snmp->get_table(Baseoid => $oid_diskvendor);
            	my $bcl = 0;
            	foreach my $key ( keys %$result)
            	{
                	$bcl = 1;
                	#on récupère l'index
                    	if($key=~ /([0-9]*)$/)
                    	{
                    		my $index =$1;
                    		my $vendor =$$result{"$key"};
                    		$vendor =~ s/^\s+|\s+$//g;
				$vendor =~ s/\s\s//g;
                                $vendor =~ s/\s\s//g;
                    		my $diskproduct = getsnmpOID($snmp,$oid_diskmodel.".$index");
                    		$diskproduct =~ s/^\s+|\s+$//g;
                    		my $cleanstate = getsnmpOID($snmp,$oid_cleanstate.".$index");
				my $infostate = getsnmpOID($snmp,$oid_cleaninfo.".$index");

                    		$cleanstate = $statusDriveToStr{$cleanstate};
				$infostate = $infoDriveToStr{$infostate};
                    		my $esia_status = $statusDriveToEsia{$cleanstate};
                    		print "$esia_status: drive $index ($vendor $diskproduct) clean status is $cleanstate ($infostate)\n";
                    		$return = priority_state($return,$ERRORS{$esia_status});
                	}
            	}

            	if( $bcl eq 0 ){
                 	print "UNKNOWN: No drive found\n";
                 	$return = priority_state($return,$ERRORS{'UNKNOWN'});
            	}
	}
	elsif( $type eq 'power' ){
                my $oid_pwrstatus = "$oid_base.2.2.1.1.2.1";
                my $pwrstatus = getsnmpOID($snmp,$oid_pwrstatus);
		$pwrstatus = $statusToStr{$pwrstatus};

                my $esia_status = $statusToEsia{$pwrstatus};
                print "$esia_status: Power status is $pwrstatus \n";
                $return = priority_state($return,$ERRORS{$esia_status});
	}
        else{
                print "UNKNOWN: -T, --type $type not implemented. Parameter must be set with status, clean or power\n";
                $return = priority_state($return,$ERRORS{'UNKNOWN'});
        }
    }
    else{
        print "UNKNOWN: Unable to detect IBM tape device type\n";
        $return=priority_state($return,$ERRORS{'UNKNOWN'});
    }
}

exit($return);
