#!/usr/bin/perl -w 
############################## check_snmp_idrac_diskhealth.pl ########
my $Version='1.0';
# Date : Mar 02 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_idrac_diskhealth.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my %RAID_layout= (
        1  => 'Other',
        2  => 'RAID-0',
        3  => 'RAID-1',
        4  => 'RAID-5',
        5  => 'RAID-6',
        6  => 'RAID-10',
        7  => 'RAID-50',
        8  => 'RAID-60',
        9  => 'ConcatRAID-1',
        10 => 'ConcatRAID-5',
	);
my $return=0;
my ($opt_h,$opt_V,$opt_u);
my $community = "public";
my $hostname;
my $snmp_version = 2;

my $oid_sysDescr = "1.3.6.1.2.1.1.1";
my $oid_fqdd = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.55";
my $oid_status = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.24";
my $oid_state = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.4";
my $oid_productid = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.6";
my $oid_serialNo = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.7";
my $oid_size = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.11";
my $oid_smartAlert = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.31";
my $oid_mediatype = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.35";
my $oid_freespace = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.19";
my $oid_firmware_ver = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.8";
my $oid_overall_status = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.24";
#RAID
my $oid_raid_fqdd = "1.3.6.1.4.1.674.10892.5.5.1.20.140.1.1.36";
my $oid_raid_status = "1.3.6.1.4.1.674.10892.5.5.1.20.140.1.1.20";
my $oid_raid_state = "1.3.6.1.4.1.674.10892.5.5.1.20.140.1.1.4";
my $oid_raid_layout = "1.3.6.1.4.1.674.10892.5.5.1.20.140.1.1.13";
my $oid_raid_size = "1.3.6.1.4.1.674.10892.5.5.1.20.140.1.1.6";
my $oid_raid_mediatype = "1.3.6.1.4.1.674.10892.5.5.1.20.140.1.1.33";

sub print_help() {
	printf "check l'état de santé des disques d'un serveur IDRAC Dell.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -u (--unknown)		ignore unknown\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"u"   => \$opt_u,	"unknown"    => \$opt_u,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysDescr");

#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysDescr");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $state = "OK";
my $snmpresult = $snmp->get_request("$oid_overall_status");
my $overall_status =  $$snmpresult{"$oid_overall_status"};

my $fqdd = $snmp->get_table(Baseoid => $oid_fqdd);

if (!defined($fqdd)) 
{
   print "FQDD na\n";
   $return = priority_state($return,$ERRORS{UNKNOWN});
}
else
{
	foreach my $key ( keys %$fqdd) 
	{	
		if($key =~/4\.1\.55\.([0-9]*)/)
		{
			my $id = "$1";
			$snmpresult = $snmp->get_request("$oid_status.$id");
			my $value_status = $$snmpresult{"$oid_status.$id"};
			my $state_status = "OK";
			if($value_status == 5)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state_status = "CRITICAL";
			}
			elsif($value_status == 3)
			{
				$return = priority_state($return,$ERRORS{OK});
				$state_status = "OK";
			}
			elsif($value_status == 2 && $opt_u)
			{
				$return = priority_state($return,$ERRORS{OK});
				$state_status = "OK (IGNORED disk state unknown)";
			}
			else
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state_status = "UNKNOWN";
			}

			$snmpresult = $snmp->get_request("$oid_state.$id");
			my $value_state = $$snmpresult{"$oid_state.$id"};
			my $state_state = "OK";
			if($value_state == 5 || $value_state == 6 || $value_state == 7 || $value_state == 9)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state_status = "CRITICAL";
				if($value_state == 5)
				{
					$state_state = "Offline";
				}
				elsif($value_state == 6)
				{
					$state_state = "Blocked";
				}				
				elsif($value_state == 7)
				{
					$state_state = "Failed";
				}
				elsif($value_state == 8)
				{
					$state_state = "Non-raid";
				}
				elsif($value_state == 9)
				{
					$state_state = "Removed";
				}
			}
			elsif($value_state == 3 || $value_state == 2 || $value_state == 8 )
			{
				$return = priority_state($return,$ERRORS{OK});
				if($value_state == 2)
				{
					$state_state = "Ready";
				}
				elsif($value_state == 3)
				{
					$state_state = "Online";
				}
                                elsif($value_state == 8)
                                {
                                        $state_state = "Non-raid";
                                }

			}
			else
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state_state = "UNKNOWN";
			}

			#print "State : $state_state\n";
			#$snmpresult = $snmp->get_request("$oid_productid.$id");
			#my $productid = $$snmpresult{"$oid_productid.$id"};
			$snmpresult = $snmp->get_request("$oid_serialNo.$id");
			my $serialno = $$snmpresult{"$oid_serialNo.$id"};
			$snmpresult = $snmp->get_request("$oid_size.$id");
			my $size = $$snmpresult{"$oid_size.$id"};
			$snmpresult = $snmp->get_request("$oid_mediatype.$id");
			my $mediatype;
			if($$snmpresult{"$oid_mediatype.$id"} == 2)
			{
				$mediatype = "hdd";
			}
			elsif($$snmpresult{"$oid_mediatype.$id"} == 3)
			{
				$mediatype = "ssd";
			}
			else
			{
				$mediatype = "unknown";
			}

			#$snmpresult = $snmp->get_request("$oid_freespace.$id");
			#print "Free Space : ".$$snmpresult{"$oid_freespace.$id"}." MB\n";
			#$snmpresult = $snmp->get_request("$oid_firmware_ver.$id");
			#print "Firmware Version : ".$$snmpresult{"$oid_firmware_ver.$id"}."\n";
			$overall_status = 4;
			$snmpresult = $snmp->get_request("$oid_overall_status.$id");
			$overall_status = $$snmpresult{"$oid_overall_status.$id"};
			my $state_ign='';
			if($overall_status == 5)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state = "CRITICAL";
			}
			elsif($overall_status == 3)
			{
				$return = priority_state($return,$ERRORS{OK});
				$state = "OK";
			}
			elsif($overall_status == 2 && $opt_u)
			{
				$return = priority_state($return,$ERRORS{OK});
				$state = "OK";
				$state_ign = "IGNORE (overall status)";
			}
			else
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state = "UNKNOWN ($overall_status)";
			}

			$snmpresult = $snmp->get_request("$oid_smartAlert.$id");
			my $smartAlert_status = $$snmpresult{"$oid_smartAlert.$id"};
			if($smartAlert_status == 0){
				$return = priority_state($return,$ERRORS{OK});
				print "$state_status : $$fqdd{$key} ($mediatype $size"."MB".") is $state_state. Smartcl OK. SN : $serialno\n";
			}
			else{
				$return = priority_state($return,$ERRORS{CRITICAL});
				print "CRITICAL : $$fqdd{$key} ($mediatype $size"."MB".") is $state_state. Any Smartcl alerts received. SN : $serialno\n";
			}
		}
	}
}

my $raid_fqdd = $snmp->get_table(Baseoid => $oid_raid_fqdd);

if (!defined($raid_fqdd)) 
{
   print "FQDD RAID na\n";
   $return = priority_state($return,$ERRORS{UNKNOWN});
}
else
{
	foreach my $key ( keys %$raid_fqdd) 
	{	
		if($key =~/1\.1\.36\.([0-9]*)/)
		{
			my $id = "$1";
			$snmpresult = $snmp->get_request("$oid_raid_status.$id");
			my $raid_value_status = $$snmpresult{"$oid_raid_status.$id"};
			my $raid_state_status = "OK";
			if($raid_value_status == 5)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$raid_state_status = "CRITICAL";
			}
			elsif($raid_value_status == 3)
			{
				$return = priority_state($return,$ERRORS{OK});
				$raid_state_status = "OK";
			}
			elsif($raid_value_status == 2 && $opt_u)
			{
				$return = priority_state($return,$ERRORS{OK});
				$raid_state_status = "OK (IGNORED raid state unknown)";
			}
			else
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$raid_state_status = "UNKNOWN";
			}

			$snmpresult = $snmp->get_request("$oid_raid_state.$id");
			my $raid_value_state = $$snmpresult{"$oid_raid_state.$id"};
			my $raid_state_state = "OK";
			if($raid_value_state == 5 || $raid_value_state == 6 || $raid_value_state == 7 || $raid_value_state == 8 || $raid_value_state == 9)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$raid_state_status = "CRITICAL";
				if($raid_value_state == 5)
				{
					$raid_state_state = "Offline";
				}
				elsif($raid_value_state == 6)
				{
					$raid_state_state = "Blocked";
				}				
				elsif($raid_value_state == 7)
				{
					$raid_state_state = "Failed";
				}
				elsif($raid_value_state == 8)
				{
					$raid_state_state = "Non-raid";
				}
				elsif($raid_value_state == 9)
				{
					$raid_state_state = "Removed";
				}
			}
			elsif($raid_value_state == 3 || $raid_value_state == 2)
			{
				$return = priority_state($return,$ERRORS{OK});
				if($raid_value_state == 2)
				{
					$raid_state_state = "Ready";
				}
				elsif($raid_value_state == 3)
				{
					$raid_state_state = "Online";
				}
			}
			else
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$raid_state_state = "UNKNOWN";
			}

			$snmpresult = $snmp->get_request("$oid_raid_size.$id");
			my $raid_size = $$snmpresult{"$oid_raid_size.$id"};

			$snmpresult = $snmp->get_request("$oid_raid_mediatype.$id");
			my $raid_mediatype = $$snmpresult{"$oid_raid_mediatype.$id"};

			$snmpresult = $snmp->get_request("$oid_raid_layout.$id");
			my $raid_layout = $$snmpresult{"$oid_raid_layout.$id"};

			print "$raid_state_status : ".$RAID_layout{$raid_layout}." - ".$$raid_fqdd{$key}." ($raid_mediatype) is $raid_state_state"."\n";

		}
	}
}

exit($return);
