#!/usr/bin/perl -w
############################## check_snmp_idrac_syshealth.pl ########
my $Version='1.0';
# Date : Mar 02 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use JSON;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;

my $PROGNAME='check_snmp_idrac_syshealth.pl';
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
my ($opt_h,$opt_V, $opt_G, $json, $PROGDESC);

my $return = $ERRORS{OK};
my $type = 'health';

my $community = "public";
my $hostname;
my $snmp_version = 2;

my $oid_sysDescr = "1.3.6.1.2.1.1.1";
my $oid_model = "1.3.6.1.4.1.674.10892.5.1.3.12.0";
my $oid_globalSystemStatus = '1.3.6.1.4.1.674.10892.5.2.1.0';

my $oid_eventLogTableEntry = '1.3.6.1.4.1.674.10892.5.4.300.40.1';
my $oid_eventLogRecord = '1.3.6.1.4.1.674.10892.5.4.300.40.1.5';
my $oid_eventLogSeverityStatus = '1.3.6.1.4.1.674.10892.5.4.300.40.1.7';
my $oid_eventLogDateName = '1.3.6.1.4.1.674.10892.5.4.300.40.1.8';

my $oid_fan_fqdd = "1.3.6.1.4.1.674.10892.5.4.700.12.1.8";
my $oid_fan_status = "1.3.6.1.4.1.674.10892.5.4.700.12.1.5";
my $oid_fan_state = "1.3.6.1.4.1.674.10892.5.4.700.12.1.4";
my $oid_fan_speed = "1.3.6.1.4.1.674.10892.5.4.700.12.1.6";
my $oid_cpu_fqdd = "1.3.6.1.4.1.674.10892.5.4.1100.30.1.26";
my $oid_cpu_status = "1.3.6.1.4.1.674.10892.5.4.1100.30.1.5";
my $oid_cpu_state = "1.3.6.1.4.1.674.10892.5.4.1100.30.1.4";
my $oid_cpu_currentspeed = "1.3.6.1.4.1.674.10892.5.4.1100.30.1.12";
my $oid_cpu_corecount = "1.3.6.1.4.1.674.10892.5.4.1100.30.1.17";
my $oid_overall_cpu_status = "1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.24";
my $oid_power_State = "1.3.6.1.4.1.674.10892.5.2.4.0";

my $PowerStatus = { 1 => 'other',2=>'unknown',3=>'off',4=>'on'};
my $ObjectStatus = { 1 => 'other',2=>'unknown',3=>'ok',4=>'nonCritical (non-critical)',5=>'critical (failure)',6=>'nonRecoverable (dead)'};
my $ObjectStatusToEsia = { 1 => 'OK',2=>'UNKNOWN',3=>'OK',4=>'WARNING',5=>'CRITICAL',6=>'CRITICAL'};

#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);

GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"J" => \$json,	 "json" => \$json,
	"T=s" => \$type,	 "type=s" => \$type,
);

# -h & --help print help
if ($opt_h)
{
    #affiche l'aide par défaut
    ESIA::SNMP::print_snmp_usage($PROGNAME);
    my @arr;
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	print "\n";
	exit $ERRORS{'OK'};
}

#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

my $state = "OK";
my $snmpresult;

if( $type eq 'health' )
{
	if ($opt_G)
	{
	    my $snmpresult = $snmp->get_request("$oid_globalSystemStatus");
	    my $global_status = $$snmpresult{"$oid_globalSystemStatus"};
	    my $global_status_str = $ObjectStatus->{"$global_status"};
	    my $global_status_esia = $ObjectStatusToEsia->{"$global_status"};

	    print "$global_status_esia: Device global health status is '$global_status_str'\n";

	    if( $global_status > 3 )
	    {
	        $snmpresult = $snmp->get_table(Baseoid => $oid_eventLogDateName);
	        my $str = '';

	        if(defined( $snmpresult))
	        {
	            my $list;
	            foreach my $key (keys %$snmpresult)
	            {
	                if($key =~ /([0-9]*)$/)
	                {
	                    my $id = $1;
	                    $$list{$id} = $$snmpresult{$key};
	                }
	            }

	            foreach my $id (sort { $$list{$b} cmp $$list{$a} } keys %$list)
	            {
	                my $severity = readOID($snmp,"$oid_eventLogSeverityStatus.1.$id");
	                if( $severity == $global_status )
	                {
	                    my $severity_str = $global_status_str;
	                    my $record = readOID($snmp,"$oid_eventLogRecord.1.$id");
	                    my $date = $$list{$id};
	                    $date =~ ( /(2\d{3})([0-1]\d)([0-3]\d)([0-2]\d)([0-6]\d)([0-6]\d).*([+-]\d+)/ );
	                    $str.= "Last '$severity_str' log is : $record - $4:$5:$6 $3/$2/$1 ($7)\n";
	                    last;
	                }
	            }
	        }
	        print "$str";
	    }
	    exit($ERRORS{$global_status_esia});
	}

	$snmpresult = $snmp->get_request("$oid_overall_cpu_status");
	my $overall_cpu_status =  $$snmpresult{"$oid_overall_cpu_status"};

	$snmpresult = $snmp->get_request("$oid_model");
	print "Model : ".$$snmpresult{"$oid_model"}."\n";
	############## POWER ##############
	$snmpresult = $snmp->get_request("$oid_power_State");
	my $power_status =  $$snmpresult{"$oid_power_State"};
	my $msg_power = 'OK';

	if($power_status == 3 )
	{
		$msg_power = 'CRITICAL';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
	}
	elsif($power_status == 2 || $power_status == 1)
	{
		$msg_power = 'UNKNOWN';
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
	}
	print "$msg_power: system power state is ".$PowerStatus->{"$power_status"}."\n";


	############### FAN ###############

	my $fan_fqdd = $snmp->get_table(Baseoid => $oid_fan_fqdd);

	if (!defined($fan_fqdd))
	{
	   print "FAN sensors not found\n";
	}
	else
	{
		foreach my $key ( keys %$fan_fqdd)
		{
			if($key =~/12\.1\.8\.([0-9]*)\.([0-9]*)/)
			{
				my $id = "$1.$2";
				$snmpresult = $snmp->get_request("$oid_fan_status.$id");
				my $value_status = $$snmpresult{"$oid_fan_status.$id"};
				my $state_status = "OK";
				if($value_status == 5 || $value_status == 8 || $value_status == 10)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
					if($value_status == 5)
					{
						$state_status = "Critical Upper";
					}
					elsif($value_status == 8)
					{
						$state_status = "Critical Lower";
					}
					else
					{
						$state_status = "Critical";
					}
				}
				elsif($value_status == 4 || $value_status == 6 || $value_status == 7 || $value_status == 9)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
					if($value_status == 4)
					{
						$state_status = "Non Critical Upper";
					}
					elsif($value_status == 6)
					{
						$state_status = "Non Recoverable Upper";
					}
					elsif($value_status == 7)
					{
						$state_status = "Non Critical Lower";
					}
					elsif($value_status == 9)
					{
						$state_status = "Non Recoverable Lower";
					}
					else
					{
						$state_status = "Warning";
					}
				}
				elsif($value_status == 3)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
					$state_status = "OK";
				}
				else
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
					$state_status = "UNKNOWN";
				}

				$snmpresult = $snmp->get_request("$oid_fan_state.$id");
				my $value_state = $$snmpresult{"$oid_fan_state.$id"};
				my $state_state = "OK";
				if($value_state == 4)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
					$state_state = "Not Ready";
				}
				elsif($value_state == 6)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
					$state_state = "Enabled and not ready";
				}
				elsif($value_state == 2)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
					$state_state = "Enabled";
				}
				else
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
					$state_state = "Unknown";
				}


				$snmpresult = $snmp->get_request("$oid_fan_speed.$id");
				print "$$fan_fqdd{$key} : $state_status ; $state_state ; Speed : ".$$snmpresult{"$oid_fan_speed.$id"}." RPM\n";

			}
		}
	}

	############### CPU ###############

	my $cpu_fqdd = $snmp->get_table(Baseoid => $oid_cpu_fqdd);

	if (!defined($cpu_fqdd))
	{
	   print "FQDD CPU na\n";
	}
	else
	{
		foreach my $key ( keys %$cpu_fqdd)
		{
			if($key =~/30\.1\.26\.([0-9]*)\.([0-9]*)/)
			{
				my $id = "$1.$2";
				$snmpresult = $snmp->get_request("$oid_cpu_status.$id");
				my $value_status = $$snmpresult{"$oid_cpu_status.$id"};
				my $state_status = "OK";
				if($value_status == 5 || $value_status == 8 || $value_status == 10)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
					if($value_status == 5)
					{
						$state_status = "Critical Upper";
					}
					elsif($value_status == 8)
					{
						$state_status = "Critical Lower";
					}
					else
					{
						$state_status = "Critical";
					}
				}
				elsif($value_status == 4 || $value_status == 6 || $value_status == 7 || $value_status == 9)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
					if($value_status == 4)
					{
						$state_status = "Non Critical Upper";
					}
					elsif($value_status == 6)
					{
						$state_status = "Non Recoverable Upper";
					}
					elsif($value_status == 7)
					{
						$state_status = "Non Critical Lower";
					}
					elsif($value_status == 9)
					{
						$state_status = "Non Recoverable Lower";
					}
					else
					{
						$state_status = "Warning";
					}
				}
				elsif($value_status == 3)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
					$state_status = "OK";
				}
				else
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
					$state_status = "UNKNOWN";
				}

				$snmpresult = $snmp->get_request("$oid_cpu_state.$id");
				my $value_state = $$snmpresult{"$oid_cpu_state.$id"};
				my $state_state = "OK";
				if($value_state == 4)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
					$state_state = "Not Ready";
				}
				elsif($value_state == 6)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
					$state_state = "Enabled and not ready";
				}
				elsif($value_state == 2)
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
					$state_state = "Enabled";
				}
				else
				{
					$return = ESIA::PLUGIN::priority_state($return,$ERRORS{UNKNOWN});
					$state_state = "Unknown";
				}


				$snmpresult = $snmp->get_request("$oid_cpu_currentspeed.$id");
				my $current_speed = $$snmpresult{"$oid_cpu_currentspeed.$id"};

				$snmpresult = $snmp->get_request("$oid_cpu_corecount.$id");
				my $core_count = $$snmpresult{"$oid_cpu_corecount.$id"};

				print "$$cpu_fqdd{$key} : $state_status ; $state_state ; Current Speed : ".$current_speed." MHz ; Core count : $core_count\n";
			}
		}
	}
	exit($return);
}
elsif($type eq 'inventory')
{
	my $result;
	my @output;

	#DELL-RAC-MIB
	my $oid_drsProductName 		= '1.3.6.1.4.1.674.10892.2.1.1.1';
	my $oid_drsProductShortName	= '1.3.6.1.4.1.674.10892.2.1.1.2';
	my $oid_drsProductVersion	= '1.3.6.1.4.1.674.10892.2.1.1.5';

	$result = ESIA::SNMP::get($snmp,"$oid_drsProductShortName.0");
	if( defined $oid_drsProductName )
	{
		my $name = $result;
		my $model = ESIA::SNMP::get($snmp,"$oid_drsProductName.0");
		my $version = ESIA::SNMP::get($snmp,"$oid_drsProductVersion.0");

		my %hash = (
			'name' => $name,
			'description'=>'',
			'manufacturer'=>'Dell Inc.',
			'model' => "$model",
			'type'=>'OS',
			'serial' => '',
			'hardwareVersion' => '',
			'firmwareVersion' => $version,
		);
		push(@output, \%hash);
	}

	#IDRAC-MIB-SMIv2
	my $oid_chassisModelTypeName	= '1.3.6.1.4.1.674.10892.5.4.300.10.1.9';
	my $oid_chassisServiceTagName	= '1.3.6.1.4.1.674.10892.5.4.300.10.1.11';

	$result = ESIA::SNMP::get($snmp,"$oid_chassisModelTypeName.1");
	if( defined $oid_chassisModelTypeName )
	{
		my $name = $result;
		my $serial = ESIA::SNMP::get($snmp,"$oid_chassisServiceTagName.1");

		my %hash = (
			'name' => $name,
			'description'=>'',
			'manufacturer'=>'Dell Inc.',
			'model' => "$name",
			'type'=>'CHASSIS',
			'serial' => $serial,
			'hardwareVersion' => '',
			'firmwareVersion' => '',
		);
		push(@output, \%hash);
	}

	#IDRAC-MIB-SMIv2
	my $oid_processorDevicechassisIndex = '1.3.6.1.4.1.674.10892.5.4.1100.30.1.1';
	my $oid_processorDeviceCurrentSpeed	= '1.3.6.1.4.1.674.10892.5.4.1100.30.1.12';
	my $oid_processorDeviceCoreCount 	= '1.3.6.1.4.1.674.10892.5.4.1100.30.1.17';
	my $oid_processorDeviceThreadCount 	= '1.3.6.1.4.1.674.10892.5.4.1100.30.1.19';
	my $oid_processorDeviceBrandName 	= '1.3.6.1.4.1.674.10892.5.4.1100.30.1.23';


	$result = $snmp->get_table( $oid_processorDevicechassisIndex);
	if (defined($result))
	{
		foreach my $key ( keys %$result)
		{
			my $index = $$result{"$key"};
			my $cpuName = ESIA::SNMP::get($snmp,"$oid_processorDeviceBrandName.1.$index");
			$cpuName =~ /\s+([vV]\d+)/;
			my $version = $1;

			my $cpuCore = ESIA::SNMP::get($snmp,"$oid_processorDeviceCoreCount.1.$index");
			my $cpuThread = ESIA::SNMP::get($snmp,"$oid_processorDeviceThreadCount.1.$index");

			my $cpuSpeed = ESIA::SNMP::get($snmp,"$oid_processorDeviceCurrentSpeed.1.$index");
			$cpuSpeed/=1000;

			$cpuName =~ /((\w+)\S*\s\S+)/;
			my %hash = (
				'name' => $1,
				'description'=>"core:$cpuCore,thread:$cpuThread,frequency:$cpuSpeed GHz",
				'manufacturer'=>$2,
				'model' => $cpuName,
				'type'=>'CPU',
				'serial' => '',
				'hardwareVersion' => $version,
				'firmwareVersion' => '',
			);
			push(@output, \%hash);
		}
	}

	#IDRAC-MIB-SMIv2
	my $oid_memoryDevicechassisIndex		= '1.3.6.1.4.1.674.10892.5.4.1100.50.1.1';
	my $oid_memoryDeviceSize				= '1.3.6.1.4.1.674.10892.5.4.1100.50.1.14';
	my $oid_memoryDeviceSpeed				= '1.3.6.1.4.1.674.10892.5.4.1100.50.1.15';
	my $oid_memoryDeviceManufacturerName	= '1.3.6.1.4.1.674.10892.5.4.1100.50.1.21';
	my $oid_memoryDevicePartNumberName		= '1.3.6.1.4.1.674.10892.5.4.1100.50.1.22';
	my $oid_memoryDeviceSerialNumberName	= '1.3.6.1.4.1.674.10892.5.4.1100.50.1.23';

	$result = $snmp->get_table( $oid_memoryDevicechassisIndex);
	if (defined($result))
	{
		foreach my $key ( keys %$result)
		{
			my $index = $$result{"$key"};
			my $size = ESIA::SNMP::get($snmp,"$oid_memoryDeviceSize.1.$index");
			if( $size == 0 )
			{
				next;
			}
			$size=POSIX::floor($size/1024);
			my $speed = ESIA::SNMP::get($snmp,"$oid_memoryDeviceSpeed.1.$index");


			my $vendor = ESIA::SNMP::get($snmp,"$oid_memoryDeviceManufacturerName.1.$index");
			$vendor =~ /(\S+)/;
			$vendor = $1;

			my $sn = ESIA::SNMP::get($snmp,"$oid_memoryDeviceSerialNumberName.1.$index");

			my $model = ESIA::SNMP::get($snmp,"$oid_memoryDevicePartNumberName.1.$index");
			$model =~ /(\S+)/;
			$model = $1;

			my %hash = (
				'name' => $model,
				'description'=>"Size:$size MB,Frequency:$speed ns",
				'manufacturer'=>$vendor,
				'model' => $model,
				'type'=>'RAM',
				'serial' => '',
				'hardwareVersion' => '',
				'firmwareVersion' => '',
			);
			push(@output, \%hash);
		}
	}

	#IDRAC-MIB-SMIv2
	my $oid_controllerNumber				= '1.3.6.1.4.1.674.10892.5.5.1.20.130.1.1.1';
	my $oid_controllerName					= '1.3.6.1.4.1.674.10892.5.5.1.20.130.1.1.2';
	my $oid_controllerFWVersion				= '1.3.6.1.4.1.674.10892.5.5.1.20.130.1.1.8';
	my $oid_controllerSASAddress			= '1.3.6.1.4.1.674.10892.5.5.1.20.130.1.1.77';
	my $oid_controllerFQDD					= '1.3.6.1.4.1.674.10892.5.5.1.20.130.1.1.78';

	$result = $snmp->get_table( $oid_controllerNumber);
	if (defined($result))
	{
		foreach my $key ( keys %$result)
		{
			my $index = $$result{"$key"};
			my $model = ESIA::SNMP::get($snmp,"$oid_controllerName.$index");
			my $fw = ESIA::SNMP::get($snmp,"$oid_controllerFWVersion.$index");
			my $sasAddr = ESIA::SNMP::get($snmp,"$oid_controllerSASAddress.$index");
			my $fqdd = ESIA::SNMP::get($snmp,"$oid_controllerFQDD.$index");

			$model =~ /^([^<(]+)/;
			$model = $1;

			my %hash = (
				'name' => $model,
				'description'=>"FQDD:$fqdd,SASAddr:$sasAddr",
				'manufacturer'=>'Dell Inc.',
				'model' => $model,
				'type'=>'RAID_CONTROLLER',
				'serial' => '',
				'hardwareVersion' => '',
				'firmwareVersion' => $fw,
			);
			push(@output, \%hash);
		}
	}

	#IDRAC-MIB-SMIv2
	my $oid_enclosureNumber					= '1.3.6.1.4.1.674.10892.5.5.1.20.130.3.1.1';
	my $oid_enclosureName					= '1.3.6.1.4.1.674.10892.5.5.1.20.130.3.1.2';
	my $oid_enclosureFirmwareVersion		= '1.3.6.1.4.1.674.10892.5.5.1.20.130.3.1.26';
	my $oid_enclosureTotalSlots				= '1.3.6.1.4.1.674.10892.5.5.1.20.130.3.1.32';
	my $oid_enclosureDisplayName			= '1.3.6.1.4.1.674.10892.5.5.1.20.130.3.1.48';

	$result = $snmp->get_table( $oid_enclosureNumber);
	if (defined($result))
	{
		foreach my $key ( keys %$result)
		{
			my $index = $$result{"$key"};
			my $model = ESIA::SNMP::get($snmp,"$oid_enclosureName.$index");
			my $fw = ESIA::SNMP::get($snmp,"$oid_enclosureFirmwareVersion.$index");
			my $slots = ESIA::SNMP::get($snmp,"$oid_enclosureTotalSlots.$index");
			my $description = ESIA::SNMP::get($snmp,"$oid_enclosureDisplayName.$index");

			$model =~ /(\S+)\s*\w*/;
			$model = $1;

			my %hash = (
				'name' => $model,
				'description'=>"$description ($slots slots)",
				'manufacturer'=>'Dell Inc.',
				'model' => $model,
				'type'=>'ENCLOSURE',
				'serial' => '',
				'hardwareVersion' => '',
				'firmwareVersion' => $fw,
			);
			push(@output, \%hash);
		}
	}

	#IDRAC-MIB-SMIv2
	my $oid_physicalDiskNumber				= '1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.1';
	my $oid_physicalDiskProductID			= '1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.6';
	my $oid_physicalDiskSerialNo			= '1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.7';
	my $oid_physicalDiskRevision			= '1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.8';
	my $oid_physicalDiskCapacityInMB		= '1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.11';
	my $oid_physicalDiskBusType				= '1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.21';
	my $oid_physicalDiskMediaType			= '1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.35';
	my $oid_physicalDiskFQDD				= '1.3.6.1.4.1.674.10892.5.5.1.20.130.4.1.54';

	my %physicalDiskBusType  = (
		"1"  => "unknown",
		"2"  => "scsi",
		"3"  => "sas",
		"4"  => "sata",
		"5"  => "fibre"
	);

	my %physicalDiskMediaType  = (
		"1"  => "unknown",
		"2"  => "hdd",
		"3"  => "ssd"
	);

	$result = $snmp->get_table( $oid_physicalDiskNumber);
	if (defined($result))
	{
		foreach my $key ( keys %$result)
		{
			my $index = $$result{"$key"};
			my $model = ESIA::SNMP::get($snmp,"$oid_physicalDiskProductID.$index");
			my $serial = ESIA::SNMP::get($snmp,"$oid_physicalDiskSerialNo.$index");
			my $version = ESIA::SNMP::get($snmp,"$oid_physicalDiskRevision.$index");

			my $size = ESIA::SNMP::get($snmp,"$oid_physicalDiskCapacityInMB.$index");
			$size=POSIX::floor($size/1024);

			my $bus = ESIA::SNMP::get($snmp,"$oid_physicalDiskBusType.$index");
			$bus = $physicalDiskBusType{$bus};

			my $type = ESIA::SNMP::get($snmp,"$oid_physicalDiskMediaType.$index");
			if( $physicalDiskMediaType{$type} eq 'hdd')
			{
				$type = 'HDD';
			}
			elsif($physicalDiskMediaType{$type} eq 'ssd')
			{
				$type = 'SSD';
			}
			else
			{
				$type = 'DISK';
			}

			my $fqdd = ESIA::SNMP::get($snmp,"$oid_physicalDiskFQDD.$index");

			$model =~ /(\S+)\s*(\S*)/;
			my $name = $1;

			my %hash = (
				'name' => $name,
				'description'=>"Size:$size GB,Bus:$bus,FQDD:$fqdd",
				'manufacturer'=>'',
				'model' => $model,
				'type'=>$type,
				'serial' => $serial,
				'hardwareVersion' => '',
				'firmwareVersion' => $version,
			);
			push(@output, \%hash);
		}
	}

	#IDRAC-MIB-SMIv2
	my $oid_virtualDiskNumber				= '1.3.6.1.4.1.674.10892.5.5.1.20.140.1.1.1';
	my $oid_virtualDiskSizeInMB				= '1.3.6.1.4.1.674.10892.5.5.1.20.140.1.1.6';
	my $oid_virtualDiskLayout				= '1.3.6.1.4.1.674.10892.5.5.1.20.140.1.1.13';

	my %virtualDiskLayout = (
		"1"  => "unknown",
		"2"  => "RAID-0",
		"3"  => "RAID-1",
		"4"  => "RAID-5",
		"5"  => "RAID-6",
		"6"  => "RAID-10",
		"7"  => "RAID-50",
		"8"  => "RAID-60",
		"9"  => "Concatenated RAID-1",
		"10"  => "Concatenated RAID-5"
	);

	$result = $snmp->get_table( $oid_virtualDiskNumber);
	if (defined($result))
	{
		foreach my $key ( keys %$result)
		{
			my $index = $$result{"$key"};
			my $model = ESIA::SNMP::get($snmp,"$oid_virtualDiskLayout.$index");
			$model = $virtualDiskLayout{$model};

			my $size = ESIA::SNMP::get($snmp,"$oid_virtualDiskSizeInMB.$index");
			$size=POSIX::floor($size/1024);

			my %hash = (
				'name' => $model,
				'description'=>"Size:$size GB",
				'manufacturer'=>'Dell Inc.',
				'model' => $model,
				'type'=>'VDISK',
				'serial' => '',
				'hardwareVersion' => '',
				'firmwareVersion' => '',
			);
			push(@output, \%hash);
		}
	}

	if($json)
	{
		my $out = encode_json(\@output);
		print $out;
	}
	else
	{
		my $size = @output;
		if($size != 0)
		{
			foreach my $line ( @output)
			{

				my $hash = \%$line;
				print "name: ".$hash->{name}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
				print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
			}
			exit 0;
		}
	}
}
else
{
	print ("UNKNOWN: Incorrect or missed option -T, --type\n");
	exit $ERRORS{'UNKNOWN'};
}
