#!/usr/bin/perl -w 
############################## check_snmp_idrac_temp.pl ########
my $Version='1.0';
# Date : Mar 02 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_idrac_temp.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;

my $oid_sysDescr = "1.3.6.1.2.1.1.1";
my $oid_location = "1.3.6.1.4.1.674.10892.5.4.700.20.1.8";
my $oid_status = "1.3.6.1.4.1.674.10892.5.4.700.20.1.5";
my $oid_state = "1.3.6.1.4.1.674.10892.5.4.700.20.1.4";
my $oid_value = "1.3.6.1.4.1.674.10892.5.4.700.20.1.6";
my $oid_overall_status = "1.3.6.1.4.1.674.10892.5.4.200.10.1.24.1";

sub print_help() {
	printf "check la tempéraure d'un serveur IDRAC Dell.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysDescr");

#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysDescr");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $state = "OK";
my $snmpresult = $snmp->get_request("$oid_overall_status");
my $overall_status =  $$snmpresult{"$oid_overall_status"};

my $location = $snmp->get_table(Baseoid => $oid_location);

if (!defined($location)) 
{
   print "Location na\n";
}
else
{
	foreach my $key ( keys %$location) 
	{	
		if($key =~/20\.1\.8\.([0-9]*)\.([0-9]*)/)
		{
			my $id = "$1.$2";
			$snmpresult = $snmp->get_request("$oid_status.$id");
			my $value_status = $$snmpresult{"$oid_status.$id"};
			my $state_status = "OK";
			if($value_status == 5)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state_status = "CRITICAL";
			}
			elsif($value_status == 3)
			{
				$return = priority_state($return,$ERRORS{OK});
				$state_status = "OK";
			}
			else
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state_status = "UNKNOWN";
			}

			#my $temp_location = "$$location{$key} : ";
			#my $temp_status = "$state_status";
			$snmpresult = $snmp->get_request("$oid_state.$id");
			my $value_state = $$snmpresult{"$oid_state.$id"};
			my $state_state = "OK";
			if($value_state == 4)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state_state = "Not Ready";
			}
			elsif($value_state == 6)
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state_state = "Enabled and not ready";
			}
			elsif($value_state == 2)
			{
				$return = priority_state($return,$ERRORS{OK});
				$state_state = "Enabled";
			}
			else
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state_state = "Unknown";
			}

			#$temp_state = "State : $state_state (".$value_state.")\n";
			$snmpresult = $snmp->get_request("$oid_value.$id");
			my $value_celsius = $$snmpresult{"$oid_value.$id"} / 10 ;
			print "$state_status : $$location{$key} ; Temperature : ".$value_celsius." Degree Celsius is $state_state\n";
		}
	}
}
if($overall_status eq 'noSuchInstance')
{
	print "OK: temperature sensors not found";
}
elsif($overall_status == 5)
{
	$return = priority_state($return,$ERRORS{CRITICAL});
	$state = "CRITICAL";
}
elsif($overall_status == 3)
{
	$return = priority_state($return,$ERRORS{OK});
	$state = "OK";
}
else
{
	$return = priority_state($return,$ERRORS{UNKNOWN});
	$state = "UNKNOWN ($overall_status)";
}

#print "Overall Status : $state";

exit($return);
