#!/usr/bin/perl -w
####################### check_snmp_juniper_temp #################
my $Version='1.0';
# Date : Mars 22 2018
# Author : Victor Marin
# Help : esia-sa.com
#################################################################
use strict;
use Net::SNMP;
use Getopt::Long;
my $PROGNAME='check_snmp_juniper_temp.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $warning='70';
my $port='161';
my $critical='80';
my $result;
my @tab_value_read;
my @tab_value_oid;
my ($snmp, $snmperror);

my $jnxOperatingStateOrdered = "1.3.6.1.4.1.2636.3.1.13.1.16";
my $juniper_tempDegCelsius   = "1.3.6.1.4.1.2636.3.1.13.1.7"; # Temp in Celsius (degrees C)
my $jnxOperatingState		 = "1.3.6.1.4.1.2636.3.1.13.1.6"; 
my $jnxOperatingDescr        = "1.3.6.1.4.1.2636.3.1.13.1.5";
my $juniper_descr 		     = "1.3.6.1.2.1.1.1.0";

=for

1.3.6.1.4.1.2636.3.1.13.1.16    jnxOperatingStateOrdered

running(1), -- up and running,-- as a active primary
standby(2), -- running as a standby backup
ready(3), -- ready to run, not running yet 
runningAtFullSpeed(4),-- valid for fans only
reset(5), -- held in reset, not ready yet
down(6), -- down or off, for power supply
unknown(7)

=cut

sub print_help() {
	printf "check temp via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)		Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -w	warning value in % (default: 70)\n";
	print " -c	critical value in % (default: 80)\n";
	print " -V (--version)	Plugin version\n";
	print " -h (--help)	usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print " $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] [-w warning] [-c critical]\n";
	print " $PROGNAME [-h | --help]\n";
	print "	$PROGNAME [-V | --version]\n";
}

sub error() {
	print (" ERROR SNMP TEMP \n Verify your OID");
	#si l'host ne répond pas on blocke le script
	if (!defined($result))
	{
		if($snmp_version == 2)
		{
			($snmp, $snmperror) = Net::SNMP->session(
					-hostname  => $hostname,
					-domain    => $domain,
					-version   => 'snmpv1',
					-port      => $port,
					-community => $community
				);
			
			$result = $snmp->get_request("$juniper_descr");
			if (!defined($result))
			{
				printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
				$snmp->close;
				exit $ERRORS{"UNKNOWN"};
			}
		}
		else
		{
			printf("UNKNOWN ERROR: Description table : %s.\n",
			$snmp->error);
			$snmp->close;
			exit $ERRORS{"UNKNOWN"};
		}
	}
	exit $ERRORS{"UNKNOWN"};
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 
sub readOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	if(defined($result))
	{
		return $$result{"$oid"};
	}
	error();	
}

########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,      "warning=i"      => \$warning,
	"c=i" => \$critical,     "critical=i"     => \$critical
);

# -h & --help print help
if($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname  => $hostname,
			-version   => 'snmpv1',
			-domain    => $domain,
			-port      => $port,
			-community => $community,
			-timeout   => 2
		);
}
my $return=$ERRORS{OK};
if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
	exit $ERRORS{"UNKNOWN"};
}
else
{	
	my $resultat = $snmp->get_table(Baseoid =>$jnxOperatingDescr);
	
	if(defined($resultat))
	{
		foreach my $key (keys %$resultat)
		{			
			 my $val = $$resultat{"$key"};
		  	if($val =~ /outing/)
	  		{			  		
				if($key=~ /1.3.6.1.4.1.2636.3.1.13.1.5.(.*)/ )
			  	{
			  		my $oid =$1;
		            my $temp = readOID($snmp,"$juniper_tempDegCelsius.$oid");
		            my $operatingState = readOID($snmp,"$jnxOperatingState.$oid");
		            #my $operatingStateOrdered  = readOID($snmp,"$jnxOperatingStateOrdered.$oid");
				    							
		            if($operatingState == 1)
		            {
			            print "UNKNOWN : The operating state is unknown \n"; 
                        $return = priority_state($return,$ERRORS{UNKNOWN}); 
                    }
                    elsif($operatingState == 2)
                    {
                        print "OK : The operating state is running \n";
                    }
                    elsif($operatingState == 3)
                    {
                        print "OK : The operating state is ready \n";
                    }
                    elsif($operatingState == 4)
                    {
                        print "WARNING : The operating state is reset \n"; $return = priority_state($return,$ERRORS{WARNING});
                    }
                    elsif($operatingState == 5)
                    {
                        print "WARNING : The operating state is runningAtFullSpeed \n";$return = priority_state($return,$ERRORS{WARNING});
                    }
                    elsif($operatingState == 6)
                    {
                        print "CRITICAL : The operating state is down \n"; $return = priority_state($return,$ERRORS{CRITICAL}); 
                    }
                    elsif($operatingState == 7)
                    {
                        print "OK : The operating state is running as a standby backup \n";
                    }
                    else 
                    { 
                        print "UNKNOWN : The operating state is unknown \n"; $return = priority_state($return,$ERRORS{UNKNOWN}); 
                    }


			        if($temp > $critical)
			        {
				        print("CRITICAL : Temperature : $temp °C (> $critical °C) |temp=$temp,warning=$warning,critical=$critical\n");
				        $return = priority_state($return,$ERRORS{CRITICAL});
			        }
			        elsif($temp > $warning)
			        {
				        print("WARNING : Temperature : $temp °C ( $warning °C <= X < $critical °C ) |temp=$temp,warning=$warning,critical=$critical\n");
				        $return = priority_state($return,$ERRORS{WARNING});
			        }
                    else
                    {
			            print("OK : Temperature :  $temp °C (< $warning °C) |temp=$temp,warning=$warning,critical=$critical\n");
		            }		  		
			  	}
	  		}
		}
	}	
	else{
		print "No SNMP result for this OID !!!\n";
		exit $ERRORS{"UNKNOWN"};
	}
}
exit $return;
