#!/usr/bin/perl -w 
############################## check_qnap_health #################
my $Version='1.0';
# Date : Jan 21 2015
# Author  : Julien George
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use RRDs;

my $PROGNAME='check_netgear_health.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning_temp = 50;
my $critical_temp = 70;

my $oid_descr = '1.3.6.1.2.1.1.1.0';
my $oid_nasMgrSoftwareVersion	='1.3.6.1.4.1.4526.22.1.0';
my $oid_nasMgrSerialNUM		='1.3.6.1.4.1.4526.22.2.0';

my $oidEntry_diskNumber		='1.3.6.1.4.1.4526.22.3.1.1';
my $oidEntry_diskID	        ='1.3.6.1.4.1.4526.22.3.1.2';
my $oidEntry_diskSlotName       ='1.3.6.1.4.1.4526.22.3.1.3';
my $oidEntry_diskSerial         ='1.3.6.1.4.1.4526.22.3.1.4';
my $oidEntry_diskModel         	='1.3.6.1.4.1.4526.22.3.1.5';
my $oidEntry_ataError           ='1.3.6.1.4.1.4526.22.3.1.6';
my $oidEntry_diskCapacity       ='1.3.6.1.4.1.4526.22.3.1.7';
my $oidEntry_diskInterface      ='1.3.6.1.4.1.4526.22.3.1.8';
my $oidEntry_diskState          ='1.3.6.1.4.1.4526.22.3.1.9';
my $oidEntry_diskTemperature    ='1.3.6.1.4.1.4526.22.3.1.10';

my $oidEntry_fanNumber        	='1.3.6.1.4.1.4526.22.4.1.1';
my $oidEntry_fanRPM             ='1.3.6.1.4.1.4526.22.4.1.2';
my $oidEntry_fanStatus       	='1.3.6.1.4.1.4526.22.4.1.3';
my $oidEntry_fanType       	='1.3.6.1.4.1.4526.22.4.1.4';

my $oidEntry_temperatureNumber  ='1.3.6.1.4.1.4526.22.5.1.1';
my $oidEntry_temperatureValue   ='1.3.6.1.4.1.4526.22.5.1.2';
my $oidEntry_temperatureType    ='1.3.6.1.4.1.4526.22.5.1.3';
my $oidEntry_temperatureMin     ='1.3.6.1.4.1.4526.22.5.1.4';
my $oidEntry_temperatureMax     ='1.3.6.1.4.1.4526.22.5.1.5';

my $oidEntry_volumeNumber       ='1.3.6.1.4.1.4526.22.7.1.1';
my $oidEntry_volumeName         ='1.3.6.1.4.1.4526.22.7.1.2';
my $oidEntry_volumeRAIDLevel    ='1.3.6.1.4.1.4526.22.7.1.3';
my $oidEntry_volumeStatus       ='1.3.6.1.4.1.4526.22.7.1.4';
my $oidEntry_volumeSize         ='1.3.6.1.4.1.4526.22.7.1.5';
my $oidEntry_volumeFreespace    ='1.3.6.1.4.1.4526.22.7.1.6';

my $diskStates 		= { "ONLINE", "OFFLINE", "REMOVED"};
my $healthTypes		= { "SYS" => "System board", "CPU" => "CPU", "CASE" => "System case", "FAN" => "Fan" , "AUX" => "Auxialliary/Others" };
my $volumeStatusReadyNasOs	= { "REDUNDANT", "UNPROTECTED", "DEGRADED", "DEAD", "INACTIVE", "UNKNOWN"};
my $volumeStatusReadyDataOs	= { "ONLINE", "OFFLINE", "DEGRADED", "FAULTED", "REMOVED", "UNAVAILABLE", "UNKNOWN"};

sub print_help() {
	printf "check QNAP health.\n";
	printf "\tcheck system and CPU temperature\n";
	printf "\tcheck disk status\n";
	printf "\tcheck volume raid status\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -i                  ifindex\n";
	print " -w                  warning temperature in Celsius (default: 50)\n";
	print " -c                  critical temperature in Celsius (default: 70)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

sub readOID {
        my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning_temp,"warning=i" => \$warning_temp,
	"c=i" => \$critical_temp,"critical=i" => \$critical_temp,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}

#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_descr");
#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_descr");
		if (!defined($result)) 
		{ 
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $descr =  $$result{"$oid_descr"};
#SNMP OK on récupère le nom et le modèle
$result = $snmp->get_request("$oid_nasMgrSoftwareVersion");
my $softwareVersion =  $$result{"$oid_nasMgrSoftwareVersion"};
if($softwareVersion=~/noSuchObject/)
{
    #Récupération des températures
    $result = $snmp->get_table(Baseoid => "1.3.6.1.4.1.4526.18.5.1.3");
    if (!defined($result))
    {
        print "UNKNOWN : oid temperatures : not found\n";
        $return = priority_state($return,$ERRORS{UNKNOWN});
    }
    else
    {
	    foreach my $key ( keys %$result)
        {
            my $state = "OK";
            if($key =~/\.([0-9]*)$/)
            {
		        my $id = $1;
                my $status = $$result{$key};
                if($status eq "ok" || $status eq "OK")
                {
                    $state = "OK";
                }
                elsif($status eq "warning" || $status eq "WARNING")
                {
                    $state = "WARNING";
                }
                else
                {
                    $state = "CRITICAL";
                }
                print "$state : temperature $id is $status state\n";
                $return = priority_state($return,$ERRORS{$state});
		    }
	    }
    }
    #FAN INFORMATION
    $result = $snmp->get_table(Baseoid => "1.3.6.1.4.1.4526.18.4.1.2");
    if (!defined($result))
    {
        print "UNKNOWN : oid Fan not found\n";
        $return = priority_state($return,$ERRORS{UNKNOWN});
    }
    else
    {
        foreach my $key ( keys %$result)
        {
            if($key =~/\.([0-9]*)$/)
            {
		        my $id = $1;
                my $RPM = $$result{$key};
                my $type = readOID($snmp,"1.3.6.1.4.1.4526.18.4.1.3.$id");
                print "OK : FAN $type : $RPM rpm \n";
            }
        }
    }
    #PSU INFORMATION
    $result = $snmp->get_table(Baseoid => "1.3.6.1.4.1.4526.18.8.1.3");
    if (!defined($result))
    {
        print "UNKNOWN : oid Fan not found\n";
        $return = priority_state($return,$ERRORS{UNKNOWN});
    }
    else
    {
        foreach my $key ( keys %$result)
        {
            if($key =~/\.([0-9]*)$/)
            {
		        my $id = $1;
                my $state = "OK";
                my $status = $$result{$key};
                if($status eq "ok" || $status eq "OK")
                {
                    $state = "OK";
                }
                elsif($status eq "warning" || $status eq "WARNING")
                {
                    $state = "WARNING";
                }
                else
                {
                    $state = "CRITICAL";
                }
                print "$state : Power supply $id is $status state\n";
                $return = priority_state($return,$ERRORS{$state});
            }
        }
    }
    #DISK INFORMATION
    $result = $snmp->get_table(Baseoid => "1.3.6.1.4.1.4526.18.3.1.2");
    if (!defined($result))
    {
        print "UNKNOWN : oid disk not found\n";
        $return = priority_state($return,$ERRORS{UNKNOWN});
    }
    else
    {
        foreach my $key ( keys %$result)
        {
            if($key =~/\.([0-9]*)$/)
            {
		        my $id = $1;
                my $state       = "OK";
                my $slotName	= $$result{$key};
                my $model       = readOID($snmp,"1.3.6.1.4.1.4526.18.3.1.3.$id");
                my $status       = readOID($snmp,"1.3.6.1.4.1.4526.18.3.1.4.$id");
	            my $temperature = readOID($snmp,"1.3.6.1.4.1.4526.18.3.1.5.$id");
			    if($status eq "ok" || $status eq "OK")
                {
                    $state = "OK";
                }
                elsif($status eq "0")
                {
                    $state = "OK";
                }
                else
                {
                    $state = "WARNING";
                }
                my $msg = "< $warning_temp";
                if( $temperature > $critical_temp)
                {
                    $state = "CRITICAL";
                    $msg = "> $critical_temp";
                }
                elsif( $temperature > $warning_temp)
                {
                    $state = "WARNING";
                    $msg = "< $warning_temp";
                }
                print "$state : disk in slot $slotName ($model ) is $state state. Temperature is $temperature $msg\n";
                $return = priority_state($return,$ERRORS{$state});
            }        
        }
    }
    #VOLUME informations
    $result = $snmp->get_table(Baseoid => "1.3.6.1.4.1.4526.18.7.1.2");
    if (!defined($result))
    {
        print "UNKNOWN : oid disk not found\n";
        $return = priority_state($return,$ERRORS{UNKNOWN});
    }
    else
    {
        foreach my $key ( keys %$result)
        {
            if($key =~/\.([0-9]*)$/)
            {
		        my $id = $1;
                my $name = $$result{$key};
                my $RAIDLevel   = readOID($snmp,"1.3.6.1.4.1.4526.18.7.1.3.$id");
                my $status      = readOID($snmp,"1.3.6.1.4.1.4526.18.7.1.4.$id");
                my $size      	= readOID($snmp,"1.3.6.1.4.1.4526.18.7.1.5.$id");
			    $size=int($size/1024);
                my $state ="OK";
			    if($status eq "OK" || $status eq "ok" || $status eq "clean" || $status eq "Initialized")
                {
                    $state ="OK";
                }
                else
                {
                    $state ="CRITICAL";
                }
                $return = priority_state($return,$ERRORS{$state});
                print "$state : Volume name $name ($RAIDLevel: $size GB ) is $status state\n";
		    }
	    }
    }
}
else
{
    $result = $snmp->get_request("$oid_nasMgrSerialNUM");
    my $sn =  $$result{"$oid_nasMgrSerialNUM"};

    print "Software version: $softwareVersion SN: $sn\n";

    #Récupération des températures
    $result = $snmp->get_table(Baseoid => $oidEntry_temperatureNumber);
    if (!defined($result))
    {
        print "UNKNOWN : oid temperatures : not found\n";
        $return = priority_state($return,$ERRORS{UNKNOWN});
    }
    else
    {
	    foreach my $key ( keys %$result){
		    my $id = $$result{$key};
		    if(defined($id)){
                            my $type	= readOID($snmp,"$oidEntry_temperatureType.$id");
			    my $value	= readOID($snmp,"$oidEntry_temperatureValue.$id");
	                  	my $min        	= readOID($snmp,"$oidEntry_temperatureMin.$id");
                            my $max       	= readOID($snmp,"$oidEntry_temperatureMax.$id");

			    my $critical=int($max*0.9);
			    my $warning=int($max*0.8);
			    my $key=uc $type;

			    if($value >= $critical)
            		{
                    		print "CRITICAL: $$healthTypes{$key} temperature: $value > (w $warning/ c $critical) C MAX : $max |val$key=$value\n";
                    		$return = priority_state($return,$ERRORS{CRITICAL});
            		}
            		elsif($value >= $warning)
            		{
                    		print "WARNING: $$healthTypes{$key} temperature: $value > (w $warning/ c $critical) C  MAX : $max |val$key=$value\n";
                    		$return = priority_state($return,$ERRORS{WARNING});
            		}
            		else
            		{
                    		print "OK: $$healthTypes{$key} temperature: $value < (w $warning/ c $critical) C  MAX : $max |val$key=$value\n";
                    		$return = priority_state($return,$ERRORS{OK});
            		}

		    }
	    }
    }

    #FAN INFORMATION
    $result = $snmp->get_table(Baseoid => $oidEntry_fanNumber);
    if (!defined($result))
    {
        print "UNKNOWN : oid Fan not found\n";
        $return = priority_state($return,$ERRORS{UNKNOWN});
    }
    else
    {
            foreach my $key ( keys %$result){
                    my $id = $$result{$key};
                    if(defined($id)){
                            my $RPM        = readOID($snmp,"$oidEntry_fanRPM.$id");
                            my $status     = readOID($snmp,"$oidEntry_fanStatus.$id");
                            my $type       = readOID($snmp,"$oidEntry_fanType.$id");

			    $status=uc $status;
			    $type = uc $type;
			    
			    if( $status ne "OK") {
				    $return = priority_state($return,$ERRORS{CRITICAL});
			    }

			    print "$status: $type : $RPM rpm \n";
                    }
            }
    }


    #DISK INFORMATION
    $result = $snmp->get_table(Baseoid => $oidEntry_diskNumber);
    if (!defined($result))
    {
        print "UNKNOWN : oid disk not found\n";
        $return = priority_state($return,$ERRORS{UNKNOWN});
    }
    else
    {
	      foreach my $key ( keys %$result){
                    my $id = $$result{$key};
                    if(defined($id)){
                            my $ID		= readOID($snmp,"$oidEntry_diskID.$id");
                            my $slotName	= readOID($snmp,"$oidEntry_diskSlotName.$id");
                            my $serial      = readOID($snmp,"$oidEntry_diskSerial.$id");
                            my $model       = readOID($snmp,"$oidEntry_diskModel.$id");
		            my $ataError    = readOID($snmp,"$oidEntry_ataError.$id");
                            my $capacity    = readOID($snmp,"$oidEntry_diskCapacity.$id");
                            my $interface   = readOID($snmp,"$oidEntry_diskInterface.$id");
                            my $state       = readOID($snmp,"$oidEntry_diskState.$id");
	            my $temperature = readOID($snmp,"$oidEntry_diskTemperature.$id");

			    my $smart="$ataError errors";
			    if( $ataError == 0 ) { $smart = "OK" }
			    my $str="$state Slot : '$slotName $interface' Disk : '$model $serial [$ID]' SMART : $smart Temperature : $temperature < ($warning_temp/$critical_temp)\n";
			    

			    $state=uc $state;

                            if(  $ataError == 0 && $temperature < $warning_temp && $state eq "ONLINE" )
                            {
                                    $return = priority_state($return,$ERRORS{OK});
                                    print "OK: $str";
                            }
			    elsif( $temperature >= $critical_temp ){
                                    $return = priority_state($return,$ERRORS{CRITICAL});
                                    print "CRITICAL: $str";
			    }
                            else
                            {
                                    $return = priority_state($return,$ERRORS{WARNING});
                                    print "WARNING: $str";
                            }
                    }
            }
	    
    }

    #VOLUME informations
    $result = $snmp->get_table(Baseoid => $oidEntry_volumeNumber);
    if (!defined($result))
    {
       print "VOLUME : na\n";
    }
    else
    {
              foreach my $key ( keys %$result){
                    my $id = $$result{$key};
                    if(defined($id)){
                            my $name    	= readOID($snmp,"$oidEntry_volumeName.$id");
                            my $RAIDLevel   = readOID($snmp,"$oidEntry_volumeRAIDLevel.$id");
                            my $status      = readOID($snmp,"$oidEntry_volumeStatus.$id");
                            my $size      	= readOID($snmp,"$oidEntry_volumeSize.$id");
                            my $freespace   = readOID($snmp,"$oidEntry_volumeFreespace.$id");

			    $size=int($size/1024);
			    $freespace=int($freespace/1024);
			    my $usage=int($size-$freespace);
			    my $percentUsage=int(100/$size*$usage);
			    my $percentFree=int(100/$size*$freespace);
			    

			    my $usage_str="SIZE $size GB USAGE $usage GB ($percentUsage%) FREE $freespace GB ($percentFree%)";
                            my $str="Status RAID '$name' Level $RAIDLevel $usage_str = '$status'\n";
			    $status = uc $status;

			    if( ($status eq "REDUNDANT" || $status eq "ONLINE" ) && $percentUsage < 80 ){
			    print "OK: $str";
                                $return = priority_state($return,$ERRORS{OK});
			    }
      			elsif ( ($status eq "DEAD" || $status eq "DEGRADED") || $percentUsage >=90 ){
                                print "CRITICAL: $str";
                                $return = priority_state($return,$ERRORS{CRITICAL});
                        }
			    elsif ( $status eq "UNPROTECTED" || $status eq "DEGRADED" || $status eq "INACTIVE" || $status eq "OFFLINE" || $status eq "FAULTED" || $percentUsage >= 80){
                                print "WARNING: $str";
                                $return = priority_state($return,$ERRORS{WARNING});
			    }
			    else {
                                print "UNKNOWN: $str";
                                $return = priority_state($return,$ERRORS{UNKNOWN});
			    }
		    }
	    }
    }
}
exit $return;

