#!/usr/bin/perl -w 
############################## check_snmp_hp5400_temp.pl ########
my $Version='1.0';
# Date : 19 février 2019
# Author  : Nicolas Biersart
# Help : support@esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_netgear_switch.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning=60;
my $critical=65;
my $type = 'env';
my $oid_sysObjectOID = "1.3.6.1.2.1.1.2.0";

#
#CPU
my $oid_netgear_cpu = "1.3.6.1.4.1.4526.10.1.1.4.9"; 
# MEM
my $oid_netgear_mem_free = "1.3.6.1.4.1.4526.10.1.1.4.1"; 
my $oid_netgear_mem_total = "1.3.6.1.4.1.4526.10.1.1.4.2"; 
# power
my $oid_netgear_power = "1.3.6.1.4.1.4526.10.43.1.7.1.3";
my %netgear_powerStatus_text  = ( "1"  => "notpresent","2" => "operational","3"  => "failed","4"  => "powering","5"  => "nopower","6"  => "notpowering","7"  => "incompatible");
my %netgear_powerStatus_text_status = ( "1"  => "OK","2"  => "OK","3" => "CRITICAL","4"  => "WARNING","5"  => "WARNING","6"  => "WARNING","7"  => "CRITICAL");

my $oid_netgear_sensor = "1.3.6.1.4.1.4526.10.43.1.8.1.5";
my $oid_netgear_sensorState = "1.3.6.1.4.1.4526.10.43.1.8.1.4";
my %netgear_sensorStatus_text  = ( "0"  => "low","1"  => "normal","2" => "warning","3"  => "critical","4"  => "shutdown","5"  => "notpresent","6"  => "notoperational");
my %netgear_sensorStatus_text_status = ( "0"=>"WARNING","1"  => "OK","2"  => "WARNING","3" => "CRITICAL","4"  => "WARNING","5"  => "UNKNOWN","6"  => "CRITICAL");

my $oid_netgear_fanspeed = "1.3.6.1.4.1.4526.10.43.1.6.1.4";
my $oid_netgear_fanstate = "1.3.6.1.4.1.4526.10.43.1.6.1.3";
my %netgear_fanStatus_text  = ("1"  => "notpresent","2" => "operational","3"  => "failed","4"  => "powering","5"  => "nopower","6"  => "notpowering","7"  => "incompatible");
my %netgear_fanStatus_text_status = ("1"  => "UNKNOWN","2"  => "OK","3" => "CRITICAL","4"  => "WARNING","5"  => "UNKNOWN","6"  => "CRITICAL","7"  => "CRITICAL");
# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password

sub getsnmpOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

sub print_help() {
	printf "check snmp Netgear switch.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -T                  Test type (cpu,mem,env)\n";
	print " -w                  warning threshold\n";
	print " -c                  critical threshold\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
    "T=s" => \$type,"type=s" => \$type,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	'l=s'	=> \$o_login,		'login=s'	=> \$o_login,
	'x=s'	=> \$o_passwd,		'passwd=s'	=> \$o_passwd,
	'X=s'	=> \$o_privpass,		'privpass=s'	=> \$o_privpass,
	'L=s'	=> \$v3protocols,		'protocols=s'	=> \$v3protocols,   
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if (defined($o_login) || defined($o_passwd)) 
{
    $snmp_version =3;
}
if ($snmp_version == 3) 
{
    my @v3proto=split(/,/,$v3protocols);
    if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
    if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
    if (!defined ($o_privpass)) 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -timeout          => 2
        );  
    }
    else 
    {
        ($snmp, $snmperror) = Net::SNMP->session(
          -hostname   	=> $hostname,
          -version		=> '3',
          -username		=> $o_login,
          -authpassword	=> $o_passwd,
          -authprotocol	=> $o_authproto,
          -privpassword	=> $o_privpass,
	      -privprotocol => $o_privproto,
          -timeout          => 5
        );
    }
}
elsif ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>10
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>10
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysObjectOID");

#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysObjectOID");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN SERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: V1 Description table : %s.\n", $snmp->error); 
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $Sysoid_value = $$result{"$oid_sysObjectOID"};
if($Sysoid_value =~/3\.6\.1\.4\.1\.([0-9]*)/)
{
    $Sysoid_value = $1;
}
# test des paramètre par défaut
if($warning eq '' )
{
    if($type eq 'cpu' || $type eq 'mem')
    {
        $warning=80;
    }
}
if($critical eq '')
{
    if($type eq 'cpu'|| $type eq 'mem')
    {
        $critical=80;
    }
}
if($type eq 'env')
{
    my $state = 'OK';
    $result = $snmp->get_table(Baseoid => $oid_netgear_power);
    foreach my $key ( keys %$result) 
    {	
        #on récupère l'index
        if($key=~ /([0-9]*)\.[0-9]*$/ )
        {
            my $index =$1;
            my $status =$$result{"$key"};
            my $state = $netgear_powerStatus_text_status{$status};
            print "$state : PSU $index is ".$netgear_powerStatus_text{$status}."\n";
            $return = priority_state($return,$ERRORS{$state});
        }
    } 
    $result = $snmp->get_table(Baseoid => $oid_netgear_sensor);
    foreach my $key ( keys %$result) 
    {	
        #on récupère l'index
        if($key=~ /([0-9]\.[0-9]*)$/ )
        {
            my $index =$1;
            my $temperature =$$result{"$key"};
            my $status = getsnmpOID($snmp,"$oid_netgear_sensorState.$index");
            my $state = $netgear_sensorStatus_text_status{$status};
            my $message =" Temperature is normal ($temperature°C < $warning)";
            if($temperature >= $critical)
            {
                $state='CRITICAL';
                $return = priority_state($return,$ERRORS{CRITICAL});
                $message="Temperature is very high ($temperature°C > $critical)";
            }
            elsif($temperature >= $warning)
            {
                $state='WARNING';
                $return = priority_state($return,$ERRORS{WARNING});
                $message="Temperature is high ($temperature°C > $warning)";
            }

            print "$state : Sensors $index is ".$netgear_sensorStatus_text{$status}." $message\n";
            $return = priority_state($return,$ERRORS{$state});
        }
    } 
    $result = $snmp->get_table(Baseoid => $oid_netgear_fanstate);
    foreach my $key ( keys %$result) 
    {	
        #on récupère l'index
        if($key=~ /([0-9]*\.[0-9]*)$/ )
        {
            my $index =$1;
            my $status =$$result{"$key"};
            my $state = $netgear_fanStatus_text_status{$status};
            my $rpm = getsnmpOID($snmp,"$oid_netgear_fanspeed.$index");
            print "$state : FAN $index is ".$netgear_fanStatus_text{$status}." ($rpm rpm) \n";
            $return = priority_state($return,$ERRORS{$state});
        }
    } 
}
elsif($type eq 'cpu')
{
    my $state = 'OK';

    $result = $snmp->get_table(Baseoid => $oid_netgear_cpu);
    foreach my $key ( keys %$result) 
    {	
        #on récupère l'index
        if($key=~ /([0-9]*)$/ )
        {
            my $index =$1;
            my $cpuused =$$result{"$key"};
            if($cpuused=~ /300\s+Secs\s+\(\s+([0-9\.]*)/ )
            {
                $cpuused= sprintf("%.1f",$1);
                my $message="(< $warning)";
                if($cpuused >= $critical)
                {
                    $state='CRITICAL';
                    $return = priority_state($return,$ERRORS{CRITICAL});
                    $message="(> $critical)";
                }
                elsif($cpuused >= $warning)
                {
                    $state='WARNING';
                    $return = priority_state($return,$ERRORS{WARNING});
                    $message="(> $warning)";
                }
                
                print "$state : slot $index CPU used $cpuused % $message|cpu=$cpuused\n";
            }
            else
            {
                $return = priority_state($return,$ERRORS{UNKNOWN});
                print "UNKNWON : No match cpu usage\n";
            }
        }
    }
}
elsif($type eq 'mem')
{
    $result = $snmp->get_table(Baseoid => $oid_netgear_mem_total);
    my $state = 'OK';
    foreach my $key ( keys %$result) 
    {	
        #on récupère l'index
        if($key=~ /([0-9]*)$/ )
        {
            my $index =$1;
            my $memtotal =$$result{"$key"};
            my $memfree = getsnmpOID($snmp,"$oid_netgear_mem_free.$index");
            my $memused = $memtotal - $memfree;
            my $memusedpercent = ($memused/$memtotal)*100;
            $memusedpercent= sprintf("%.1f",$memusedpercent);
            my $message="(< $warning)";
            if($memusedpercent >= $critical)
            {
                $state='CRITICAL';
                $return = priority_state($return,$ERRORS{CRITICAL});
                $message="> $critical% usage";
            }
            elsif($memusedpercent >= $warning)
            {
                $state='WARNING';
                $return = priority_state($return,$ERRORS{WARNING});
                $message=" > $warning% usage";
            }
            print "$state : slot $index memory used $memusedpercent % $message |mem=$memusedpercent\n";
        }
    }   
}
exit($return);
