#!/usr/bin/perl -w 
############################## check_snmp_printer #################
my $Version='1.0';
# Date : Jan 21 2015
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use Encode qw(decode encode);

my $PROGNAME='check_snmp_printer.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=$ERRORS{OK};
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 1;
my $warning = 20;
my $critical = 10;

my $oid_sysDescr=".1.3.6.1.2.1.1.1";
my $oid_supplies_marker_index="1.3.6.1.2.1.43.11.1.1.2";
my $oid_supplies_type="1.3.6.1.2.1.43.11.1.1.5";
my $oid_supplies_description="1.3.6.1.2.1.43.11.1.1.6";
my $oid_supplies_unit="1.3.6.1.2.1.43.11.1.1.7";
my $oid_supplies_maxcapacity="1.3.6.1.2.1.43.11.1.1.8";
my $oid_supplies_level=".1.3.6.1.2.1.43.11.1.1.9";	
my $oid_alert_description="1.3.6.1.2.1.43.18.1.1.8";
my $oid_colorant_description="1.3.6.1.2.1.43.12.1.1.4";
my $oid_lexmark_sn ="1.3.6.1.2.1.25.3.2.1.3.1";
my $oid_kyocera_kmMediaName="1.3.6.1.4.1.1347.42.2.1.1.1.2";
my $oid_kyocera_kmMediaCounterItem="1.3.6.1.4.1.1347.42.2.1.1.1.6";

my @device_types = ("unknown",
	"other",
	"unknown",
	"toner",
	"wasteToner",
	"ink",
	"inkCartridge",
	"inkRibbon",
	"wasteInk",
	"opc",
	"developer",
	"fuserOil",
	"solidWax",
	"ribbonWax",
	"wasteWax",
	"fuser",
	"coronaWire",
	"fuserOilWick",
	"cleanerUnit",
	"fuserCleaningPad",
	"transferUnit",
	"tonerCartridge",
	"fuserOiler",
	"water",
	"wasteWater",
	"glueWaterAdditive",
	"wastePaper",
	"bindingSupply",
	"bandingSupply",
	"stitchingWire",
	"shrinkWrap",
	"paperWrap",
	"staples",
	"inserts",
	"covers");

sub searchEliminateMessage{
	my ($receivemessage) = @_;
	#list des message à ne pas prendre en compte dans le standart d'erreur
	my @list_eliminate_message = ('Ready',
					'0x4d6f646520e9636f6e6f6d6965206427e96e6572676965207b31303033337d',
					'paper is out',
					'0x4e6f6e2064e974656374e9286529203a204d61676173696e2032207b31323330317d',
					'13300',
					'13500',
					'10074',
					'Veille',
				
		);
	my $result=0;
	foreach my $key ( @list_eliminate_message ) 
	{
		if($receivemessage =~ /$key/i)
		{
			$result=1;
			last;
		}
	}
	return $result;
}

sub convert_hexa{
	my ($string) = @_;
	if($string =~ /0x([a-fA-F0-9][a-fA-F0-9]*)/){

		my $to_convert = $1;
		## convert each two digit into char code
		$to_convert =~ s/([a-fA-F0-9][a-fA-F0-9])/chr(hex($1))/eg;

		$string =~ s/0x([a-fA-F0-9][a-fA-F0-9]*)/$to_convert/g;
	}
	return $string;
}

sub print_help() {
	printf "check SNMP printer.\n";
	printf "\tcheck system, \n";
	printf "\tcheck disk status\n";
	printf "\tcheck volume raid status\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -i                  ifindex\n";
	print " -w                  warning temperature in Celsius (default: 50)\n";
	print " -c                  critical temperature in Celsius (default: 70)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>3
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>3
	);
}
#sysDescr INFORMATION
my $resultat = $snmp->get_request("$oid_sysDescr.0");
if (!defined($resultat)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community,
		);
		$resultat = $snmp->get_request("$oid_sysDescr");
		if (!defined($resultat)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
   		$snmp->close;
  		 exit $ERRORS{"UNKNOWN"};
	}
}
my $description= $$resultat{"$oid_sysDescr.0"};
#Printer Status
print "Description: ".$description."\n";
if($description =~/RICOH/ || $description =~/NRG/)
{
        #Supplies Status
        $resultat = $snmp->get_table(Baseoid => ".1.3.6.1.4.1.367.3.2.1.2.24.1.1.3");
        foreach my $key ( keys %$resultat)
        {
                if($key =~ /\.([0-9]*)$/)
                {
			my $id=$1;
			my $state='OK';
			my $supplie_descr = $$resultat{"$key"};
  			my $result = $snmp->get_request(".1.3.6.1.4.1.367.3.2.1.2.24.1.1.5.$id");
			my $supplie_level = $$result{".1.3.6.1.4.1.367.3.2.1.2.24.1.1.5.$id"};
			my $error_message="";
			#si il retourne -100 sa signigie que qu'il ne reste qu'entre 1 et 10% d'encre
			if($supplie_level == -100)
			{
				$supplie_level="1-10%";
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state ="CRITICAL";
				 $error_message="<= $critical%";
			}
			else
			{
		                if($supplie_level <= $critical)
		                {
		                        $return = priority_state($return,$ERRORS{CRITICAL});
		                        $state ="CRITICAL";
		                        $error_message="<= $critical%";
		                }
		                elsif($supplie_level <= $warning)
		                {
		                        $return = priority_state($return,$ERRORS{WARNING});
		                        $state ="WARNING";
		                        $error_message="=< $warning%";
		                }
			}
			print convert_hexa("$state : $id. Description: $supplie_descr State: $supplie_level\% $error_message \n");
		}
	}
}
elsif($description =~/HP/)
{
	my $stdout="";
        #Supplies Status
        $resultat = $snmp->get_table(Baseoid => $oid_supplies_description);
        if (!defined($resultat))
        {
                print "Supplies: na\n";
        }
        else
        {
				my $result_maxcapacity = $snmp->get_table(Baseoid => $oid_supplies_maxcapacity);
				my $result_colorant = $snmp->get_table(Baseoid => $oid_colorant_description);
				my $result_level = $snmp->get_table(Baseoid => $oid_supplies_level);
                foreach my $key ( keys %$resultat)
                {
                        if($key =~ /\.([0-9]*)\.([0-9]*)$/)
                        {
                                my $id = $2;
                                my $marker = $1;
                                my $state = "OK";
                               # my $result = $snmp->get_request("$oid_supplies_description.$marker.$id");
                                my $supplie_descr =  $$resultat{"$oid_supplies_description.$marker.$id"};
							#print "ici $marker.$id $supplie_descr\n";
								#
								#Merdouille sur les HPs avec une sequence d'octet qui fou la sortie en l'air
								#								
								if($description =~/JETDIRECT/)
								{
									
									$supplie_descr =~ s/^\s+|\s+$//g;
									if($supplie_descr =~/^([a-zA-Z\s]*[0-9\s]*[\S\s]*)HP\s*([0-9A-Z]*)/)
									{
										my $tmpdes = $1;
										$tmpdes =~ s/^\s+|\s+$//g;
										if($supplie_descr =~/HP\s*([0-9A-Z]*)/)
										{
											if (!$1 =~ m/None/)
											{
												$tmpdes = $tmpdes.' '.$1;
											}
										}
										$supplie_descr = $tmpdes;
									}
									$supplie_descr =~ s/^\s+|\s+$//g;
								}
								

                                #my $result = $snmp->get_request("$oid_supplies_maxcapacity.$marker.$id");
                                my $supplie_maxcapacity =  $$result_maxcapacity{"$oid_supplies_maxcapacity.$marker.$id"};

                                #$result = $snmp->get_request("$oid_colorant_description.$marker.$id");
                                my $colorant = "";
                                if(  $$result_colorant{"$oid_colorant_description.$marker.$id"} )
                                {
			
                                        $colorant =  $$result_colorant{"$oid_colorant_description.$marker.$id"};
										if (!$colorant eq "")
										{
											if($colorant =~/^([a-zA-Z]*)/)
											{
												$colorant = $1;
											}
										}
                                }
                              #  $result = $snmp->get_request("$oid_supplies_level.$marker.$id");
                                my $supplie_level =  $$result_level{"$oid_supplies_level.$marker.$id"};
								my $error_message="";
								#si le supplie level est égal à -3 c'est que les valeurs récupérée sont justes OK / PAS OK
								# voici ce que dit la mib A value of (-3) means that the printer knows that there is some supply/remaining space
								if(defined($supplie_level))
								{
									if($supplie_level == -3)
									{
										 $state ="OK";
										 $return = priority_state($return,$ERRORS{OK});	
										 $supplie_level =' is OK';
									}
									#si c'est égal à -2 alors la valeurs est inconnue
									elsif($supplie_level == -2)
									{
										$state ="UNKNOWN";
										$return = priority_state($return,$ERRORS{OK});	
										$supplie_level =' is unknown';
									}
									#si c'est égal à 0 alors la valeurs est vide 
									elsif($supplie_level == 0)
									{
										$state ="CRITICAL";
										$return = priority_state($return,$ERRORS{CRITICAL});	
										$supplie_level =' is empty';
									}
									else
									{
										#converti le niveau en %
										$supplie_level = ($supplie_level*100)/$supplie_maxcapacity;
										if($supplie_level <= $critical)
										{
											$return = priority_state($return,$ERRORS{CRITICAL});
											$state ="CRITICAL";
											$error_message="<= $critical%";
										}
										elsif($supplie_level <= $warning)
										{
											$return = priority_state($return,$ERRORS{WARNING});
											$state ="WARNING";
											$error_message="=< $warning%";
										}
										$supplie_level =  sprintf("%.2f",$supplie_level).'%';     
									}
								#	my $flag = utf8::valid($state);
								#	print "$state: $flag\n";
								#	$flag = utf8::valid($id);
								#	print "$id: $flag\n";
								#	$flag = utf8::valid($supplie_descr);
								#	print "$supplie_descr: $flag\n";
								#	$flag = utf8::valid($colorant);
								#	print "$colorant: $flag\n";
								#	$flag = utf8::valid($supplie_level);
								#	print "$supplie_level: $flag\n";
		                           $stdout = $stdout."$state : $id. Description: $supplie_descr ($colorant) State: $supplie_level $error_message \n";
								}
                        }
                }
        }
	print convert_hexa($stdout);
}
elsif($description =~/KONICA/)
{
  #Supplies Status
        $resultat = $snmp->get_table(Baseoid => $oid_supplies_marker_index);
        if (!defined($resultat))
        {
                print "Supplies: na\n";
        }
        else
        {
                #correction d'un problème avec certaine imprimante qui ne prennent pas en compte la dernière oid 
                #avec un walk on récupère tout mais si on interrgoe avec un get on a un no found
                # exemple les deskjet qui on un $marker =0 
                        #iso.3.6.1.2.1.43.11.1.1.6.0.1 = STRING: "black ink"
                        #iso.3.6.1.2.1.43.11.1.1.6.0.2 = STRING: "yellow ink"
                        #iso.3.6.1.2.1.43.11.1.1.6.0.3 = STRING: "cyan ink"
                        #iso.3.6.1.2.1.43.11.1.1.6.0.4 = STRING: "magenta ink"
                # SNMP échoue sur le magenta dans notre cas.

                my $supplie_description_table = $snmp->get_table(Baseoid => $oid_supplies_description);
                my $supplie_type_table        = $snmp->get_table(Baseoid => $oid_supplies_type);
                my $supplie_maxcapacity_table = $snmp->get_table(Baseoid => $oid_supplies_maxcapacity);
                my $supplie_level_table = $snmp->get_table(Baseoid => $oid_supplies_level);
                my $colorant_description_table = $snmp->get_table(Baseoid => $oid_colorant_description);

		my $previous_device_type=-1;
                foreach my $key ( sort {$$supplie_type_table{$a} <=> $$supplie_type_table{$b} or ( $a =~ /\.(\d+$)/ )[0] <=> ( $b =~ /\.(\d+$)/ )[0]} keys %$supplie_type_table) #On trie par device_type et ensuite par key
                {
                        if($key =~ /\.([0-9]*)\.([0-9]*)$/)
                        {
                                my $id = $2;
                                my $marker = $1;
                                my $state = "OK";
                                my $supplie_descr = $$supplie_description_table{"$oid_supplies_description.$marker.$id"};
				my $supplie_type = $$supplie_type_table{"$oid_supplies_type.$marker.$id"};
                                my $supplie_maxcapacity =  $$supplie_maxcapacity_table{"$oid_supplies_maxcapacity.$marker.$id"};

				if( $supplie_type != $previous_device_type )
				{
					$previous_device_type = $supplie_type;
					my $type_name = $device_types[$supplie_type];
					print "\n=> $type_name : \n";
				}

                                my $colorant = "";
                                if(  $$colorant_description_table{"$oid_colorant_description.$marker.$id"} )
                                {
                                        $colorant = $$colorant_description_table{"$oid_colorant_description.$marker.$id"};
                                }
                                my $supplie_level =  $$supplie_level_table{"$oid_supplies_level.$marker.$id"};
                                #converti le niveau en %
                                my $error_message="";
                                if($supplie_maxcapacity > 0)
                                {
					$supplie_level = ($supplie_level*100)/$supplie_maxcapacity;
					if($supplie_level <= $critical)
                                	{
                                       		$return = priority_state($return,$ERRORS{CRITICAL});
                                        	$state ="CRITICAL";
                                        	$error_message="<= $critical%";
                                	}
                                	elsif($supplie_level <= $warning)
                                	{
                                       		 $return = priority_state($return,$ERRORS{WARNING});
                                       		 $state ="WARNING";
                                        	$error_message="=< $warning%";
                                	}
					print convert_hexa("$state : $id. Description: $supplie_descr ($colorant) State: $supplie_level\% $error_message \n");
				}
				else {
                                        print convert_hexa("NOCHECKED : $id. Description: $supplie_descr ($colorant) no state given $error_message \n");
				}
                        }
                }
	}
}
else
{	
	#Supplies Status
	$resultat = $snmp->get_table(Baseoid => $oid_supplies_marker_index);
	if (!defined($resultat)) 
	{
   		print "Supplies: na\n";
	}
	else
	{
		if($description =~/Lexmark/)
		{
			my $lexsn = $snmp->get_request("$oid_lexmark_sn");
			print "SN:".$$lexsn{"$oid_lexmark_sn"}."\n";
		}
		
		foreach my $key ( keys %$resultat) 
		{	
			if($key =~ /\.([0-9]*)\.([0-9]*)$/)
			{
				my $id = $2;
				my $marker = $1;
				my $state = "OK";
				my $result = $snmp->get_request("$oid_supplies_description.$marker.$id");
				my $supplie_descr =  $$result{"$oid_supplies_description.$marker.$id"};
				$result = $snmp->get_request("$oid_supplies_unit.$marker.$id");
				my $supplie_unit =  $$result{"$oid_supplies_unit.$marker.$id"};
				$result = $snmp->get_request("$oid_supplies_maxcapacity.$marker.$id");
				my $supplie_maxcapacity =  $$result{"$oid_supplies_maxcapacity.$marker.$id"};
				$result = $snmp->get_request("$oid_colorant_description.$marker.$id");
				my $colorant = "";
				if(  $$result{"$oid_colorant_description.$marker.$id"} )
				{
					$colorant =  $$result{"$oid_colorant_description.$marker.$id"};
				}
				$result = $snmp->get_request("$oid_supplies_level.$marker.$id");
				my $supplie_level =  $$result{"$oid_supplies_level.$marker.$id"};
				my $error_message="";
				#si le supplie level est égal à -3 c'est que les valeurs récupérée sont justes OK / PAS OK
				# voici ce que dit la mib A value of (-3) means that the printer knows that there is some supply/remaining space

				if($supplie_level == -3)
				{
					 $state ="OK";
					 $return = priority_state($return,$ERRORS{OK});	
					 $supplie_level ='is OK';
				}
				#si c'est égal à -2 alors la valeurs est inconnue
				elsif($supplie_level == -2)
				{
					$state ="UNKNOWN";
					$return = priority_state($return,$ERRORS{OK});	
					$supplie_level ='is unknown';
				}
				#si c'est égal à 0 alors la valeurs est vide 
				elsif($supplie_level == 0)
				{
					$state ="CRITICAL";
					$return = priority_state($return,$ERRORS{CRITICAL});	
					$supplie_level ='is empty';
				}
				else
				{
					if($supplie_maxcapacity == -2 && $supplie_unit == 7) #unité 7 = impressions et pas pourcentage
					{
						$return = priority_state($return,$ERRORS{OK});
						$state="OK";
						$supplie_level =  $supplie_level.' impressions'; 
					}					
					else
					{
						if($supplie_unit == 19) #19 = pourcentage
						{
							$supplie_maxcapacity=100;
						}
						#converti le niveau en %
						$supplie_level = ($supplie_level*100)/$supplie_maxcapacity;
					
						if($supplie_level <= $critical)
						{
							$return = priority_state($return,$ERRORS{CRITICAL});
							$state ="CRITICAL";
							$error_message="<= $critical%";
						}
						elsif($supplie_level <= $warning)
						{
							$return = priority_state($return,$ERRORS{WARNING});
							$state ="WARNING";
							$error_message="=< $warning%";
						}
						$supplie_level =  'State: '.sprintf("%.2f",$supplie_level).'%';   
					}    
				}
				#$supplie_descr = encode('ASCII', $supplie_descr);
				my $output = convert_hexa("$state : $id. Description: $supplie_descr ($colorant) $supplie_level $error_message \n");
				$output = encode('ASCII', $output);
				$output=~s/[^[:ascii:]]//g;
				#$output=~s/[^[:ascii:]]//g;
				print $output;
			}
		}
	}
	if($description =~/KYOCERA/)
	{
		$resultat = $snmp->get_table(Baseoid => $oid_kyocera_kmMediaName);
		foreach my $key ( keys %$resultat) 
		{
			if($key =~ /\.([0-9]*)\.([0-9]*)$/)
			{
				my $id = $2;
				my $media = $1;
				my $result = $snmp->get_request("$oid_kyocera_kmMediaCounterItem.$media.$id");
				print convert_hexa("OK: Counter ".$$resultat{"$key"}." (". $$result{"$oid_kyocera_kmMediaCounterItem.$media.$id"}.")\n");
			}
		}
	}
}
#TRAY Status
#$resultat = $snmp->get_table(Baseoid => $oid_supplies_marker_index);

exit($return);
