#!/usr/bin/perl -w 
############################## get_printer_stats #################
my $Version='1.0';
# Date : Mai 16 2017
# Author  : Victor Marin
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use Encode qw(decode encode);

my $PROGNAME='check_snmp_printer_count.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=$ERRORS{OK};
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 1;

my $oid_sysDescr     		  	  =".1.3.6.1.2.1.1.1";
my $oid_total_lifetime_impressions=".1.3.6.1.2.1.43.10.2.1.4";
my $oid_color_lexmark		  	  =".1.3.6.1.4.1.641.2.1.5.3.0";
my $oid_black_lexmark		  	  =".1.3.6.1.4.1.641.2.1.5.2.0";
my $oid_total_lexmark		      =".1.3.6.1.4.1.641.2.1.5.1.0";
my $oid_color_hp     		  	  =".1.3.6.1.4.1.11.2.3.9.4.2.1.4.1.2.7.0";	
my $oid_black_hp     		  	  =".1.3.6.1.4.1.11.2.3.9.4.2.1.4.1.2.6.0";
my $oid_total_hp     		  	  =".1.3.6.1.4.1.11.2.3.9.4.2.1.4.1.2.5.0";
my $oid_color_xerox  		  	  =".1.3.6.1.4.1.253.8.53.13.2.1.6.1.20.33.0";
my $oid_black_xerox 		  	  =".1.3.6.1.4.1.253.8.53.13.2.1.6.1.20.34.0";
my $oid_total_xerox  		  	  =".1.3.6.1.4.1.253.8.53.13.2.1.6.1.20.1.0";

sub print_help() {
	printf "get SNMP printer.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2,
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2,
	);
}
#sysDescr INFORMATION
my $resultat = $snmp->get_request("$oid_sysDescr.0");
if (!defined($resultat)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community,
		);
		$resultat = $snmp->get_request("$oid_sysDescr");
		if (!defined($resultat)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
   		$snmp->close;
  		 exit $ERRORS{"UNKNOWN"};
	}
}
my $description= $$resultat{"$oid_sysDescr.0"};
#Printer Status
my $stdout="";
my $oid_black="";
my $oid_color="";
my $oid_total="";

if($description =~/Lexmark/)
{  
	$oid_black = $oid_black_lexmark;
	$oid_color = $oid_color_lexmark;
	$oid_total = $oid_total_lexmark;
}
elsif($description =~/HP/)
{  
	$oid_black = $oid_black_hp;
	$oid_color = $oid_color_hp;
	$oid_total = $oid_total_hp;
}
elsif($description =~/XEROX/ || $description =~/Xerox/)
{
	$oid_black = $oid_black_xerox;
	$oid_color = $oid_color_xerox;
	$oid_total = $oid_total_xerox;
}
else
{ 
	$stdout=printer_defaut();
	print $stdout;
	exit(0);
}

my $result = $snmp->get_request("$oid_black");
my $number_page_black =  $$result{"$oid_black"};
if(defined($number_page_black))
{
	$result = $snmp->get_request("$oid_color");
    my $number_page_color =  $$result{"$oid_color"};

    $result = $snmp->get_request("$oid_total");
    my $number_total_page =  $$result{"$oid_total"};

	if(defined($number_page_color))
	{
        $stdout="Number of color pages : $number_page_color \nNumber of black pages : $number_page_black \nTotal page number : $number_total_page |color=$number_page_color,black=$number_page_black,total=$number_total_page";
	}
	else # printer mono
	{
		$stdout="Number of black pages : $number_page_black \nTotal page number : $number_total_page |color=0,black=$number_page_black,total=$number_total_page";
	}
}
else
{
	$stdout=printer_defaut();
}
print $stdout;
exit(0);

sub printer_defaut{
	my $number_page = "";
	$result = $snmp->get_table(-baseoid => $oid_total_lifetime_impressions); 
	
	my %bla = %{$result}; 		
	my $key; 

	foreach $key (keys %bla)
	{ 			
		$number_page = $bla{$key};
	} 		
	return "Total page number : $number_page |color=0,black=0,total=$number_page \n";
}
