#!/usr/bin/perl -w 
############################## check_qnap_health #################
my $Version='1.0';
# Date : Jan 21 2015
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use RRDs;
 use Data::Dumper;
my $PROGNAME='check_qnap_health.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V,$opt_u);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning_temp = 50;
my $critical_temp = 70;
#OID QNAP
my $oid_cputemp = "1.3.6.1.4.1.24681.1.2.5.0";
my $oid_systemtemp = "1.3.6.1.4.1.24681.1.2.6.0";
my $oid_model = "1.3.6.1.4.1.24681.1.2.12.0";
my $oid_hostname = "1.3.6.1.4.1.24681.1.2.13.0";
my $oid_fan="1.3.6.1.4.1.24681.1.2.15.1";
my $oid_disk_index="1.3.6.1.4.1.24681.1.2.11.1.1";
my $oid_disk_descr="1.3.6.1.4.1.24681.1.2.11.1.2";
my $oid_disk_temp="1.3.6.1.4.1.24681.1.2.11.1.3";
my $oid_disk_status="1.3.6.1.4.1.24681.1.2.11.1.4";
my $oid_disk_model="1.3.6.1.4.1.24681.1.2.11.1.5";
my $oid_disk_capacity="1.3.6.1.4.1.24681.1.2.11.1.6";
my $oid_disk_smart="1.3.6.1.4.1.24681.1.2.11.1.7";

my $oid_raid_index="1.3.6.1.4.1.24681.1.2.17.1.1";
my $oid_raid_descr="1.3.6.1.4.1.24681.1.2.17.1.2";
my $oid_raid_fs="1.3.6.1.4.1.24681.1.2.17.1.3";
my $oid_raid_status="1.3.6.1.4.1.24681.1.2.17.1.6";

my $oid_new_raid_index="1.3.6.1.4.1.24681.1.4.1.1.1.2.1.2.1.2";
my $oid_new_raid_size="1.3.6.1.4.1.24681.1.4.1.1.1.2.1.2.1.3";
my $oid_new_raid_level="1.3.6.1.4.1.24681.1.4.1.1.1.2.1.2.1.7";
my $oid_new_raid_status="1.3.6.1.4.1.24681.1.4.1.1.1.2.1.2.1.5";

my $oid_raid_alert_status="1.3.6.1.4.1.24681.1.4.1.1.1.2.3.2.1.5";
my $oid_raid_alert_name="1.3.6.1.4.1.24681.1.4.1.1.1.2.3.2.1.8";

my $HdStatus = { 0 => 'ready',-4=>'unknown', -5 => 'noDisk',-6 => 'invalid',-9 => 'rwError'  }; 	

sub print_help() {
	printf "check QNAP health.\n";
	printf "\tcheck system and CPU temperature\n";
	printf "\tcheck disk status\n";
	printf "\tcheck volume raid status\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -u (--unknown)		ignore unknown\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -i                  ifindex\n";
	print " -w                  warning temperature in Celsius (default: 50)\n";
	print " -c                  critical temperature in Celsius (default: 70)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"u"   => \$opt_u,	"unknown"    => \$opt_u,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning_temp,"warning=i" => \$warning_temp,
	"c=i" => \$critical_temp,"critical=i" => \$critical_temp,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>5
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>5
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_model");
#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community,
            -timeout=>5
		);
		$result = $snmp->get_request("$oid_model");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
#SNMP OK on récupère le nom et le modèle
my $model =  $$result{"$oid_model"};
$result = $snmp->get_request("$oid_hostname");
my $host =  $$result{"$oid_hostname"};
$host =~ s/\n+//g;
print "Model: $model Hostname: $host\n";

#Récupération des températures
$result = $snmp->get_request("$oid_cputemp");
my $cputemp =  $$result{"$oid_cputemp"};
$result = $snmp->get_request("$oid_systemtemp");
my $systemtemp =  $$result{"$oid_systemtemp"};
$systemtemp =~ s/\n+//g;
$cputemp =~ s/\n+//g;
#vérifaction de la température système.
if($systemtemp =~ /([0-9]*)\s+C/)
{
	if($1 >= $critical_temp)
	{
		print "CRITICAL: System temperature: $systemtemp > $critical_temp C\n"; 
		$return = priority_state($return,$ERRORS{CRITICAL});
	}
	elsif($1 >= $warning_temp)
	{
		print "WARNING: System temperature: $systemtemp > $warning_temp C\n"; 
		$return = priority_state($return,$ERRORS{WARNING});
	}
	else
	{
		print "OK: System temperature: $systemtemp < $warning_temp C\n"; 
		$return = priority_state($return,$ERRORS{OK});
	}
}
else
{
	print "OK: System temperature: noInfo\n";
}
#vérifaction de la température CPU.
if($cputemp =~ /([0-9]*)\s+C/)
{
	if($1 >= $critical_temp)
	{
		print "CRITICAL: System temperature: $cputemp > $critical_temp C\n"; 
		$return = priority_state($return,$ERRORS{CRITICAL});
	}
	elsif($1 >= $warning_temp)
	{
		print "WARNING: System temperature: $cputemp > $warning_temp C\n"; 
		$return = priority_state($return,$ERRORS{WARNING});
	}
	else
	{
		print "OK: System temperature: $cputemp < $warning_temp C\n"; 
		$return = priority_state($return,$ERRORS{OK});
	}
}
else
{
	print "OK: CPU temperature: noInfo\n";
}
#DISK INFORMATION
$result = $snmp->get_table(Baseoid => $oid_disk_descr);
if (!defined($result)) 
{
   print "DISK: na\n";
}
else
{
	foreach my $key ( keys %$result) 
	{	
		if($key=~ /([0-9]*)$/ )
		{
			my $id =$1;
			my $state = "OK";
            my $disk_descr = $$result{$key};
			my $disk_result = $snmp->get_request("$oid_disk_temp.$id");
			my $disk_temp =  $$disk_result{"$oid_disk_temp.$id"};
			$disk_result = $snmp->get_request("$oid_disk_status.$id");
			my $disk_status =  $$disk_result{"$oid_disk_status.$id"};
			$disk_result = $snmp->get_request("$oid_disk_model.$id");
			my $disk_model =  $$disk_result{"$oid_disk_model.$id"};
			$disk_result = $snmp->get_request("$oid_disk_capacity.$id");
			my $disk_capacity =  $$disk_result{"$oid_disk_capacity.$id"};
			$disk_result = $snmp->get_request("$oid_disk_smart.$id");
			my $disk_smart =  $$disk_result{"$oid_disk_smart.$id"};
			#nettoyage des variables
			$disk_descr =~ s/\n+//g;
			$disk_model =~ s/\n+//g;
			$disk_smart =~ s/\n+//g;
			$disk_temp =~ s/\n+//g;
			$disk_capacity =~ s/\n+//g;
            #INFO SMART
			if($disk_smart =~ m/noSuchInstance/i)
			{
				$disk_smart = 'noInfo';
			}
            elsif($disk_smart =~ m/GOOD/i || $HdStatus->{$disk_status} eq "noDisk")
            {
                $state ="OK";
            }
            else
			{
				$state ="WARNING";
                $return = priority_state($return,$ERRORS{WARNING});
			}
            #INFO DISK
			if($disk_status == -4 && $state ne "WARNING")
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state ="UNKNOWN";
			}
			elsif($disk_status == -6 || $disk_status == -9)
			{
				$return = priority_state($return,$ERRORS{WARNING});
				$state ="WARNING";
			}
			if( $HdStatus->{$disk_status} eq "noDisk" ){
				print "$state: DISK $id : $disk_descr (Missing)\n";
			}
			else{
			print "$state: DISK $id : $disk_descr (Model: $disk_model Capacity: $disk_capacity) status is: $HdStatus->{$disk_status} (SMART:$disk_smart) . Temperature is $disk_temp\n";
			}
		}
	}
}
#RAID INFORMATION
$result = $snmp->get_table(Baseoid => $oid_raid_status);
if (!defined($result)) 
{
    $result = $snmp->get_table(Baseoid => $oid_new_raid_size);
    if (!defined($result)) 
    {
        print "RAID: na\n";
    }
	foreach my $key ( keys %$result) 
	{	
		if($key=~ /([0-9]*)$/ )
		{
            my $id =$1;
			my $state = "OK";
            my $raid_size =  $$result{"$oid_new_raid_size.$id"};
            my $raid_result = $snmp->get_request("$oid_new_raid_level.$id");
            my $raid_level =  $$raid_result{"$oid_new_raid_level.$id"};
            $raid_result = $snmp->get_request("$oid_new_raid_status.$id");
            my $raid_status =  $$raid_result{"$oid_new_raid_status.$id"};
            $raid_size = int($raid_size/1000/1000/1000);
			#nettoyage des variables
			$raid_status =~ s/\n+//g;
			if( $raid_status ne "Ready" ) 
			{
			    print "CRITICAL: raid $id (level: $raid_level, capacity $raid_size GB) is $raid_status.\n";
				$return = priority_state($return,$ERRORS{CRITICAL});
			}
			else
			{
			    print "OK: raid $id (level: $raid_level, capacity $raid_size GB) is $raid_status.\n";
				$return = priority_state($return,$ERRORS{OK});
			}
        }
    }
}
else
{
	my $result_name = $snmp->get_table(Baseoid => $oid_raid_descr);
	my $result_aname;
	my $result_astatus;
	foreach my $key ( keys %$result) 
	{	
		if($key=~ /([0-9]*)$/ )
		{
            my $id =$1;
			my $state = "OK";
			my $raid_status =  $$result{"$oid_raid_status.$id"};
			my $raid_desc = $$result_name{"$oid_raid_descr.$id"};
            if( !($raid_desc =~ /Single\s+Disk/) )
            {
			    if ( $raid_desc =~ /([Vv]olume )?(\w+)/ ){
				    $raid_desc = $2;
			    }
		    
			    #nettoyage des variables
			    $raid_status =~ s/\n+//g;
			    if( $raid_status eq "Ready" ) 
			    {
			        print "OK: Volume $id ($raid_desc) software raid status is $raid_status.\n";
				    $return = priority_state($return,$ERRORS{OK});
			    }
			    elsif( $raid_status =~ /Rebuilding/ ) 
			    {
			        	print "WARNING: Volume $id ($raid_desc) software raid status is $raid_status.\n";
				    $return = priority_state($return,$ERRORS{WARNING});
			    }
			    elsif( $raid_status =~ /^degraded.*$/ ) 
			    {
			        	print "CRITICAL: Volume $id ($raid_desc) software raid status is $raid_status.\n";
			       	$return = priority_state($return,$ERRORS{CRITICAL});
			    }
			    elsif( $raid_status =~ /[Uu]nknown/ ){
				    if( !defined($result_aname)){
				         	$result_aname = $snmp->get_table(Baseoid => $oid_raid_alert_name);
            				$result_astatus = $snmp->get_table(Baseoid => $oid_raid_alert_status);
				    }
				    my $astatus = 'Unknown';
				    foreach my $key_alert ( keys %$result_aname){
					    if($key_alert=~ /([0-9]*)$/ ){
						    my $id_alert = $1;
						    if( $$result_aname{"$oid_raid_alert_name.$id_alert"} eq "$raid_desc" ){
							    $astatus = $$result_astatus{"$oid_raid_alert_status.$id_alert"};
						    }
					    }
				    }
				    if( $astatus =~ /[Ww]arning/ ){
					    print "WARNING: Volume $id ($raid_desc) software raid status $raid_status and disk space alert status is $astatus\n";
					    $return = priority_state($return,$ERRORS{WARNING});
				    }
				    else{
					    if( $opt_u ){
						     print "IGNORED: Volume $id ($raid_desc) software raid status is $raid_status and disk space alert status is $astatus\n";
                                    		$return = priority_state($return,$ERRORS{OK});
					    }
					    else{
						    print "UNKNOWN: Volume $id ($raid_desc) software raid status is $raid_status and disk space alert status is $astatus\n";
                                                    $return = priority_state($return,$ERRORS{UNKNOWN});

					    }
				    }
			    }
			    else 
			    {
			        	print "UNKNOWN: Volume $id ($raid_desc) software raid status is $raid_status.\n";
				    $return = priority_state($return,$ERRORS{UNKNOWN});
			    }
            }
		}
	}
}

exit($return);
