#!/usr/bin/perl -w 
############################## check_qnap_storage #################
my $Version='1.0';
# Date : Apr 24 2017
# Author  : Julien GEORGE
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use RRDs;

my $PROGNAME='check_qnap_ram.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=$ERRORS{OK};
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 80;
my $critical = 90;
#OID QNAP
my $oid_mem_total="1.3.6.1.4.1.24681.1.2.2.0";
my $oid_mem_free="1.3.6.1.4.1.24681.1.2.3.0";

sub print_help() {
	printf "check QNAP ram.\n";
	printf "\tcheck system and CPU memory\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning temperature in Celsius (default: 50)\n";
	print " -c                  critical temperature in Celsius (default: 70)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 
sub octet_conversion
{
	my ($size,$unit) = @_;
	my $octet_size = 0;
	if($unit eq 'TB')
	{
		$octet_size = $size * 1024 * 1024 * 1024 *1024;
	}
	elsif($unit eq 'GB')
	{
		$octet_size = $size * 1024 * 1024 * 1024;
	}
	elsif($unit eq 'MB')
	{
		$octet_size = $size * 1024 * 1024;
	}
	elsif($unit eq 'KB')
	{
		$octet_size = $size * 1024;
	}
	else
	{
		$octet_size = $size;
	} 
	return $octet_size;
}
#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#VOLUME INFORMATION
my $result = $snmp->get_request($oid_mem_total);
if (!defined($result)) 
{
	printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	$snmp->close;
	exit $ERRORS{"UNKNOWN"};
}

my $vol_size = $$result{$oid_mem_total};

$result = $snmp->get_request($oid_mem_free);
my $vol_free = $$result{$oid_mem_free};

my $vol_mb_used=0;
my $octet_size=0;
my $octet_free=0;

if($vol_size =~ /([0-9\.]*) ([A-Z]*)/)
{
	$octet_size = octet_conversion($1,$2);
}
if($vol_free =~ /([0-9\.]*) ([A-Z]*)/)
{
	$octet_free = octet_conversion($1,$2);
}
my $octet_used = $octet_size - $octet_free;
my $percent = ($octet_used * 100)/ $octet_size;
$percent = sprintf("%.2f", $percent);
$vol_size = $octet_size/1024/1024; #convert to MB
my $vol_used = $octet_used/1024/1024;
if($percent > $critical)
{
	print"CRITICAL: RAM : used $percent% ($vol_used MB/ $vol_size MB) > $critical\n";
	exit($ERRORS{CRITICAL});
}
elsif($percent > $warning)
{
	print"WARNING: RAM : used $percent% ($vol_used MB/ $vol_size MB) > $warning\n";
        exit($ERRORS{WARNING});
}
else
{
	print"OK: RAM : used $percent% ($vol_used MB/ $vol_size MB)\n";
        exit($ERRORS{OK});
}
