#!/usr/bin/perl -w
################## check_snmp_ratiran_env #################
my $Version='1.0';
# Date : Octob 12 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use POSIX qw(strftime);
use Switch;
use strict;
use Net::SNMP;
use Getopt::Long;
my $PROGNAME='check_snmp_ratiran_env.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "esia_read";
my $snmp_version = 2;
my ($hostname);
my $warning='70';
my $port='161';
my $critical='80';
my $output="";
my $status="";
my $state="";
my $service="";
my $result;
my ($snmp, $snmperror);
#my @tab_value_read;


my $ratiran_descr   			 = "1.3.6.1.2.1.1.1.0";
my $ratiran_device_model_name 	 = "1.3.6.1.4.1.13742.6.3.2.1.1.3.1";
my $ratiran_device_serial_number = "1.3.6.1.4.1.13742.6.3.2.1.1.4.1";
my $ratiran_device_name			 = "1.3.6.1.4.1.13742.6.3.2.2.1.13.1";
my $ratiran_software_version     = "1.3.6.1.4.1.13742.6.3.2.3.1.6.1.1.1";
my $ratiran_inlet_rated_voltage  = "1.3.6.1.4.1.13742.6.3.3.3.1.6";
my $ratiran_inlet_rated_current  = "1.3.6.1.4.1.13742.6.3.3.3.1.7.1.1";
my $ratiran_outlet_rated_voltage = "1.3.6.1.4.1.13742.6.3.5.3.1.6";
my $ratiran_outlet_rated_current = "1.3.6.1.4.1.13742.6.3.5.3.1.7";

my $ratiran_pdu_current_load 	 = "1.3.6.1.4.1.13742.6.5.2.3.1.4.1.1.1"; # Gauge32: 10 (A)  => 8,686
my $ratiran_pdu_input_voltage 	 = "1.3.6.1.4.1.13742.6.5.2.3.1.4.1.1.4"; # Gauge32: 119 (V) => 229
my $ratiran_pdu_input_realpower	 = "1.3.6.1.4.1.13742.6.5.2.3.1.4.1.1.5"; # Gauge32: 10 (W) => 1806
my $ratiran_pdu_input_power 	 = "1.3.6.1.4.1.13742.6.5.2.3.1.4.1.1.6"; # Gauge32: 122 (voltamp) => 1985
#my $ratiran_pdu_input_powerfactor= "1.3.6.1.4.1.13742.6.5.2.3.1.4.1.1.7"; # Gauge32: 8 (none) => 91
my $ratiran_pdu_activeEnergy     = "1.3.6.1.4.1.13742.6.5.2.3.1.4.1.1.8"; # Gauge32: 193 359 (wattHour) => 21844,973

#inletSensorDecimalDigits
my $inletSensorDecimalDigits_load 		= "1.3.6.1.4.1.13742.6.3.3.4.1.7.1.1.1";
my $inletSensorDecimalDigits_voltage 	= "1.3.6.1.4.1.13742.6.3.3.4.1.7.1.1.4";
my $inletSensorDecimalDigits_realpower   = "1.3.6.1.4.1.13742.6.3.3.4.1.7.1.1.5";
my $inletSensorDecimalDigits_power 		= "1.3.6.1.4.1.13742.6.3.3.4.1.7.1.1.6";
#my $inletSensorDecimalDigits_powerfactor = "1.3.6.1.4.1.13742.6.3.3.4.1.7.1.1.7";
my $inletSensorDecimalDigits_activeEnergy= "1.3.6.1.4.1.13742.6.3.3.4.1.7.1.1.8";

sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname) 	Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -w	warning value in % (default: 70)\n";
	print " -c	critical value in % (default: 80)\n";
	print " -V (--version)	Plugin version\n";
	print " -h (--help)	usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print " $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] [-w warning] [-c critical] \n";
	print " $PROGNAME [-h | --help]\n";
	print " $PROGNAME [-V | --version]\n";
}

sub error() {
	print (" ERROR SNMP ENV\n");
	#si l'host ne répond pas on blocke le script
	if (!defined($result))
	{
		if($snmp_version == 2)
		{
			($snmp, $snmperror) = Net::SNMP->session(
					-hostname  => $hostname,
					-domain    => $domain,
					-version   => 'snmpv1',
					-port      => $port,
					-community => $community
				);

			$result = $snmp->get_request("$ratiran_descr");
			if (!defined($result))
			{
				printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
				$snmp->close;
				exit $ERRORS{"UNKNOWN"};
			}
		}
		else
		{
			printf("UNKNOWN ERROR: Description table : %s.\n",
			$snmp->error);
			$snmp->close;
			exit $ERRORS{"UNKNOWN"};
		}
	}
}

sub readOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,      "warning=i"      => \$warning,
	"c=i" => \$critical,     "critical=i"     => \$critical,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
 			-version    => 'snmpv1',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}

my $return=$ERRORS{OK};
if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
	exit $ERRORS{"UNKNOWN"};
}
else
{
	$result = $snmp->get_request($ratiran_device_model_name);
	if (defined($result))
	{
	    my $model_name = $$result{"$ratiran_device_model_name"};
	    $result = $snmp->get_request($ratiran_device_serial_number);
	    my $serial_number = $$result{"$ratiran_device_serial_number"};
		$result = $snmp->get_request($ratiran_device_name);
		my $device_name = $$result{"$ratiran_device_name"};
		$result = $snmp->get_request($ratiran_software_version);
		my $software_version = $$result{"$ratiran_software_version"};

		print "Model : $model_name, serial number : $serial_number, version : $software_version \n";

		$result = $snmp->get_request($ratiran_pdu_current_load);
	 	if(defined($result))
	 	{
	 		my $current_load = $$result{"$ratiran_pdu_current_load"};
			$result = $snmp->get_request($inletSensorDecimalDigits_load);
	 		if(defined($result))
	 		{
	 			my $digit_load = $$result{"$inletSensorDecimalDigits_load"};
	 			$current_load=sprintf("%.2f",$current_load/(10**$digit_load));	 			

	 			$result = $snmp->get_request($ratiran_pdu_input_voltage);
				my $voltage = $$result{"$ratiran_pdu_input_voltage"}; # (V)

				$result = $snmp->get_request($inletSensorDecimalDigits_voltage);
				my $digit_voltage = $$result{"$inletSensorDecimalDigits_voltage"};

				$voltage = $voltage/(10**$digit_voltage);

				$result = $snmp->get_request($ratiran_pdu_input_power);
				my $power = $$result{"$ratiran_pdu_input_power"}; # (W)

				$result = $snmp->get_request($inletSensorDecimalDigits_voltage);
				my $digit_power = $$result{"$inletSensorDecimalDigits_voltage"};

				$power = $power/(10**$digit_power);

				$result = $snmp->get_request($ratiran_inlet_rated_current);
				my $max_amp = $$result{"$ratiran_inlet_rated_current"};

				if($current_load > 0)
				{
					my $max = ($max_amp =~ /(\d+)A/);
					$max=$1;
					my $used = sprintf("%.2f",(($current_load/$max)*100));

					if($used > $critical)
					{
						print "CRITICAL - PDU power is $power W \n";
						print "CRITICAL - PDU voltage is $voltage V \n";
	 					print "CRITICAL - PDU load is $current_load A ( $used % > $critical %)|percent=$used,amp=$current_load,vol=$voltage,pow=$power,warning=$warning,critical=$critical \n";
	 					$return = priority_state($return,$ERRORS{CRITICAL}) 
					}
					else
					{
						if($used > $warning)
						{
							print "WARNING - PDU power is $power W \n";
							print "WARNING - PDU voltage is $voltage V \n";
	 						print "WARNING - PDU load is $current_load A ( $used % > $warning %)|percent=$used,amp=$current_load,vol=$voltage,pow=$power,warning=$warning,critical=$critical \n";	
	 						$return = priority_state($return,$ERRORS{WARNING}) 
						}	
						else
						{
							print "OK - PDU power is $power W \n";
							print "OK - PDU voltage is $voltage V \n";
	 						print "OK - PDU load is $current_load A ( $used % < $warning %)|percent=$used,amp=$current_load,vol=$voltage,pow=$power,warning=$warning,critical=$critical \n";	 						
						}
					}
				}
				else
				{
					print "CRITICAL - PDU power is $power W \n";
					print "CRITICAL - PDU voltage is $voltage V \n";
	 				print "CRITICAL - PDU load is $current_load A device down !|percent=0,amp=$current_load,vol=$voltage,pow=$power,warning=$warning,critical=$critical \n";
	 				$return = priority_state($return,$ERRORS{CRITICAL}) 
				}
	 		}
	 		else
	 		{
	 			error();
	 		}
	 	}
	 	else
	 	{
	 		error();
	 	}
	}
 	else
 	{
 		error();
 	}
}
exit $return;
