#!/usr/bin/perl -w
################## check_snmp_ratiran_status #################
my $Version='1.0';
# Date : Octob 12 2017
# Author : Victor Marin
# Help : esia-sa.com
#################################################################

use POSIX qw(strftime);
use Switch;
use strict;
use Net::SNMP;
use Getopt::Long;
my $PROGNAME='check_snmp_ratiran_status.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "esia_read";
my $snmp_version = 2;
my ($hostname);
my $warning='70';
my $port='161';
my $critical='80';
my $output="";
my $status="";
my $state="";
my $service="";
my $result;
my ($snmp, $snmperror);


my $ratiran_descr   			 = "1.3.6.1.2.1.1.1.0";
my $ratiran_device_model_name 	 = "1.3.6.1.4.1.13742.6.3.2.1.1.3.1";
my $ratiran_device_serial_number = "1.3.6.1.4.1.13742.6.3.2.1.1.4.1";
my $ratiran_software_version     = "1.3.6.1.4.1.13742.6.3.2.3.1.6.1.1.1";
my $raritan_power_out			 = "1.3.6.1.4.1.13742.6.4.1.2.1.2.1"; # Power cycle an outlet   SI == 1 => ON
my $raritan_state_out 			 = "1.3.6.1.4.1.13742.6.4.1.2.1.3.1"; # Determine the status of an outlet


sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname) 	Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -V (--version)	Plugin version\n";
	print " -h (--help)	usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print " $PROGNAME -H <HOSTNAME> [-C <community>] [-D <domain>] \n";
	print " $PROGNAME [-h | --help]\n";
	print " $PROGNAME [-V | --version]\n";
}

sub error() {
	print (" ERROR SNMP ENV\n");
	#si l'host ne répond pas on blocke le script
	if (!defined($result))
	{
		if($snmp_version == 2)
		{
			($snmp, $snmperror) = Net::SNMP->session(
					-hostname  => $hostname,
					-domain    => $domain,
					-version   => 'snmpv1',
					-port      => $port,
					-community => $community
				);

			$result = $snmp->get_request("$ratiran_descr");
			if (!defined($result))
			{
				printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
				$snmp->close;
				exit $ERRORS{"UNKNOWN"};
			}
		}
		else
		{
			printf("UNKNOWN ERROR: Description table : %s.\n",
			$snmp->error);
			$snmp->close;
			exit $ERRORS{"UNKNOWN"};
		}
	}
}

sub readOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }

if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
 			-version    => 'snmpv1',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 2
		);
}

my $return=$ERRORS{OK};
if(!defined($snmp))
{
	print " Error SNMP not defined !!! \n";
	exit $ERRORS{"UNKNOWN"};
}
else
{
	$result = $snmp->get_request($ratiran_device_model_name);
	if (defined($result))
	{
	    my $model_name = $$result{"$ratiran_device_model_name"};
	    $result = $snmp->get_request($ratiran_device_serial_number);
	    my $serial_number = $$result{"$ratiran_device_serial_number"};		
		$result = $snmp->get_request($ratiran_software_version);
		my $software_version = $$result{"$ratiran_software_version"};

		print "Model : $model_name, serial number : $serial_number, version : $software_version \n";

 		$result = $snmp->get_table(Baseoid => $raritan_state_out);
		if (!defined($result))
		{			
	   		print "The status of an outlet : na\n";
	   		error();
		}
		else
		{
			foreach my $key ( keys %$result)
			{								    
			    if($key =~/1.3.6.1.4.1.13742.6.4.1.2.1.3.1\.([0-9]*)/)
				{
					my $id = "$1";		    	
			        my $power_out = readOID($snmp,"$raritan_power_out.$id");
		        	my $state_out = readOID($snmp,"$raritan_state_out.$id");	        	
	 				
	 				if($power_out==1)
	 				{
	 					$power_out="on";
	 				}
	 				else
	 				{
	 					$power_out="off";
	 					$return = priority_state($return,$ERRORS{WARNING})
	 				}
	 				
	 				if($state_out==-1)
	 				{
	 					$state_out="unavailable";
	 					$return = priority_state($return,$ERRORS{UNKNOWN})
	 				}
	 				elsif($state_out==0)
	 				{
	 					$state_out="open";
	 				}
	 				elsif($state_out==1)
	 				{
	 					$state_out="closed";
	 				}
	 				elsif($state_out==2)
	 				{
	 					$state_out="belowLowerCritical";
	 					$return = priority_state($return,$ERRORS{CRITICAL})
	 				}
	 				elsif($state_out==3)
	 				{
	 					$state_out="belowLowerWarning";
	 					$return = priority_state($return,$ERRORS{WARNING})
	 				}
	 				elsif($state_out==4)
	 				{
	 					$state_out="normal";
	 				}
	 				elsif($state_out==5)
	 				{
	 					$state_out="aboveUpperWarning";
	 					$return = priority_state($return,$ERRORS{WARNING})
	 				}
	 				elsif($state_out==6)
	 				{
	 					$state_out="aboveUpperCritical";
	 					$return = priority_state($return,$ERRORS{CRITICAL})
	 				}
	 				elsif($state_out==7)
	 				{
	 					$state_out="on";
	 				}
	 				elsif($state_out==8)
	 				{
	 					$state_out="off";
	 				}
	 				elsif($state_out==9)
	 				{
	 					$state_out="detected";
	 				}
	 				elsif($state_out==10)
	 				{
	 					$state_out="notDetected";
	 					$return = priority_state($return,$ERRORS{UNKNOWN})
	 				}
	 				elsif($state_out==11)
	 				{
	 					$state_out="alarmed";
	 					$return = priority_state($return,$ERRORS{WARNING})
	 				}
	 				elsif($state_out==12)
	 				{
	 					$state_out="ok";
	 				}
	 				elsif($state_out==13)
	 				{
	 					$state_out="marginal";
	 				}
	 				elsif($state_out==14)
	 				{
	 					$state_out="fail";
	 					$return = priority_state($return,$ERRORS{CRITICAL})
	 				}
	 				elsif($state_out==15)
	 				{
	 					$state_out="yes";
	 				}
	 				elsif($state_out==16)
	 				{
	 					$state_out="no";
	 				}
	 				elsif($state_out==17)
	 				{
	 					$state_out="standby";
	 				}
	 				elsif($state_out==18)
	 				{
	 					$state_out="one";
	 				}
	 				elsif($state_out==19)
	 				{
	 					$state_out="two";
	 				}
	 				elsif($state_out==20)
	 				{
	 					$state_out="inSync";
	 				}
	 				elsif($state_out==21)
	 				{
	 					$state_out="outOfSync";
	 					$return = priority_state($return,$ERRORS{WARNING})
	 				}
	 				else
	 				{
	 					$state_out="NULL";
	 					$return = priority_state($return,$ERRORS{UNKNOWN})
	 				}
	 				
	 				print "Outlet $id : power : $power_out ; state : $state_out \n";
		    	}
	 		}	 		
		}
	}
	else
	{
		error();
	}
}
exit $return;
