#!/usr/bin/perl -w 
############################## check_qnap_health #################
my $Version='1.0';
# Date : Jan 21 2015
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use Data::Dumper;

my $PROGNAME='check_qnap_health.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning_temp = 50;
my $critical_temp = 70;
#OID SEAGATE
my $oid_sysdescr = "1.3.6.1.2.1.1.1.0";
#RAID OID
my $oid_raid_status="1.3.6.1.4.1.3581.123.1.8.1.1.11";
my $oid_raid_name="1.3.6.1.4.1.3581.123.1.8.1.1.2";
my $oid_raid_syn_progress="1.3.6.1.4.1.3581.123.1.8.1.1.12"; 
my $oid_raid_syn_time="1.3.6.1.4.1.3581.123.1.8.1.1.13";
my $oid_raid_level="1.3.6.1.4.1.3581.123.1.8.1.1.15";
#SLOT OID
my $oid_slot_status=".1.3.6.1.4.1.3581.123.1.8.3.1.9";
my $oid_slot_name=".1.3.6.1.4.1.3581.123.1.8.3.1.3";

#DISK OID
my $oid_disk_serial=".1.3.6.1.4.1.3581.123.1.7.1.1.2";
my $oid_disk_model=".1.3.6.1.4.1.3581.123.1.7.1.1.3";
my $oid_disk_firmware=".1.3.6.1.4.1.3581.123.1.7.1.1.4";
my $oid_disk_slot=".1.3.6.1.4.1.3581.123.1.7.1.1.6";
my $oid_disk_status=".1.3.6.1.4.1.3581.123.1.7.1.1.7";
my $oid_disk_temp=".1.3.6.1.4.1.3581.123.1.7.1.1.9";
my $oid_disk_smart=".1.3.6.1.4.1.3581.123.1.7.1.1.11";

#FANSENSORS OID
my $oid_fan_name=".1.3.6.1.4.1.3581.123.1.3.1.1.2";
my $oid_fan_rpm=".1.3.6.1.4.1.3581.123.1.3.1.1.4";
my $oid_fan_max_rpm=".1.3.6.1.4.1.3581.123.1.3.1.1.5";

#POWERSUPPLY SENSORS OID
my $oid_supply_name=".1.3.6.1.4.1.3581.123.1.4.1.1.2";
my $oid_supply_status=".1.3.6.1.4.1.3581.123.1.4.1.1.3";



my $raidstatus  = { -1 => 'Unknown',1=>'Clean', 2 => 'Degraded',3 => 'Synchronizing',4 => 'Inactive'  }; 	
my $raidlevel  = { -2 => 'Unknown',-1 => 'Linear',0=>'Raid0',1=>'Raid1', 5 => 'Raid5',6 => 'Raid6',10 => 'Raid10'  }; 		
my $slotstatus  = { -1 => 'Unknown',
			1=>'New (not used in a RAID)', 
			2 => 'Empty (slot is empty)',
			3 => 'Missing (was a RAID member and is now missing)',
			4 => 'Clean (RAID member up to date)',
			5 => 'Synchronizing (RAID member synchronizing)',
			6 => 'Faulty (RAID member, faulty)',
			7 => 'Inactive (RAID member, array is inactive)', 
			8 => 'Spare (RAID member not currently in use)', 
			9 => 'Busy (an operation is being performed)'}; 
my $supplystatus = { -1 => 'Unknown',1=>'OK',2=>'NotPresent', 3 => 'Faulty' }; 		
my $diskstatus = { -1 => 'Unknown',0=>'Empty',1=>'OK',2=>'KO'}; 
my $smartstatus = { -1 => 'Unknown',0=>'Good',1=>'PreviousBadAttribute',2=>'BadSector', 3 => 'CurrentBadAttribute', 4 => 'ManyBadSectors', 5 => 'BadStatus'}; 
	
sub print_help() {
	printf "check QNAP health.\n";
	printf "\tcheck system and CPU temperature\n";
	printf "\tcheck disk status\n";
	printf "\tcheck volume raid status\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -i                  ifindex\n";
	print " -w                  warning temperature in Celsius (default: 50)\n";
	print " -c                  critical temperature in Celsius (default: 70)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning_temp,"warning=i" => \$warning_temp,
	"c=i" => \$critical_temp,"critical=i" => \$critical_temp,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysdescr");
#si l'host ne répond pas on blocke le script
if (!defined($result)) {
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysdescr");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
print $$result{"$oid_sysdescr"}."\n";

#SUPPLY INFORMATION
$result = $snmp->get_table(Baseoid => $oid_supply_name);
if (!defined($result)) 
{
   print "OK : SUPPLY: na\n";
}
else
{
	foreach my $key ( keys %$result) 
	{	
		if($key=~ /([0-9]*)$/ )
		{
			my $id =$1;
			my $state = "OK";
			my $name =$$result{"$key"};
			$result = $snmp->get_request("$oid_supply_status.$id");
			my $status =  $$result{"$oid_supply_status.$id"};
			if( $status == 1 || $status == 2) 
			{
				$return = priority_state($return,$ERRORS{OK});
				$state = "OK";
			}
			elsif( $status == -1 ) 
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state = "UNKNOWN";
			}
			elsif( $status == 3 ) 
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state = "CRITICAL";
			}			
			print "$state : supply $name state is ".$$supplystatus{"$status"}."\n";
		}
	}
}
#FAN INFORMATION
$result = $snmp->get_table(Baseoid => $oid_fan_name);
if (!defined($result)) 
{
   print "FAN: na\n";
}
else
{
	foreach my $key ( keys %$result) 
	{	
		if($key=~ /([0-9]*)$/ )
		{
			my $id =$1;
			my $state = "OK";
			my $name =$$result{"$key"};
			$result = $snmp->get_request("$oid_fan_rpm.$id");
			my $rpm = $$result{"$oid_fan_rpm.$id"};
			$result = $snmp->get_request("$oid_fan_max_rpm.$id");
			my $rpm_max = $$result{"$oid_fan_max_rpm.$id"};
			my $message = "$rpm rpm < $rpm_max max";
			if($rpm >= $rpm_max )
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state = "CRITICAL";
				$message = "$rpm rpm > $rpm_max rpm max";
			}
			print "$state : FAN $name $message\n";
		}
	}
}

#RAID INFORMATION
$result = $snmp->get_table(Baseoid => $oid_raid_name);
if (!defined($result)) 
{
   print "RAID: na\n";
}
else
{
	foreach my $key ( keys %$result) 
	{	
		if($key=~ /([0-9]*)$/ )
		{
			my $id =$1;
			my $message="";
			my $state = "OK";
			my $name =$$result{"$key"};
			$result = $snmp->get_request("$oid_raid_status.$id");
			my $status =  $$result{"$oid_raid_status.$id"};
			if( $status == 1 || $status == 4) 
			{
				$return = priority_state($return,$ERRORS{OK});
				$state = "OK";
			}
			elsif( $status == -1 ) 
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state = "UNKNOWN";
			}
			elsif( $status == 2 ) 
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state = "CRITICAL";
			}
			elsif( $status == 3 ) 
			{
				$return = priority_state($return,$ERRORS{WARNING});
				$state = "WARNING";
				$result = $snmp->get_request("$oid_raid_syn_progress.$id");
				my $sync_percent =  $$result{"$oid_raid_syn_progress.$id"};
				$result = $snmp->get_request("$oid_raid_syn_time.$id");
				my $sync_time =  $$result{"$oid_raid_syn_time.$id"};
				$message = " Process: $sync_percent%. Time remaining: ".($sync_time/60)." min"; 
			}
			$result = $snmp->get_request("$oid_raid_level.$id");
			my $level = $$result{"$oid_raid_level.$id"};
			print "$state : RAID $name (level raid: ".$$raidlevel{"$level"}.") state is ".$$raidstatus{"$status"}."$message\n";

		}	
	}
}
#DISK INFORMATION
$result = $snmp->get_table(Baseoid => $oid_disk_serial);
if (!defined($result)) 
{
   print "DISK: na\n";
}
else
{
	foreach my $key ( keys %$result) 
	{	
		if($key=~ /([0-9]*)$/ )
		{
			my $id =$1;
			my $state = "OK";
			$result = $snmp->get_request("$oid_disk_serial.$id");
			my $serial =$$result{"$oid_disk_serial.$id"};

			$result = $snmp->get_request("$oid_disk_model.$id");
			my $model= $$result{"$oid_disk_model.$id"};
			$result = $snmp->get_request("$oid_disk_firmware.$id");
			my $firmware= $$result{"$oid_disk_firmware.$id"};
			#DISK STATUS
			$result = $snmp->get_request("$oid_disk_status.$id");
			my $status= $$result{"$oid_disk_status.$id"};
			if( $status == 1 || $status == 0) 
			{
				$return = priority_state($return,$ERRORS{OK});
				$state = "OK";
			}
			elsif( $status == -1 ) 
			{
				$return = priority_state($return,$ERRORS{UNKNOWN});
				$state = "UNKNOWN";
			}
			elsif( $status == 2 ) 
			{
				$return = priority_state($return,$ERRORS{CRITICAL});
				$state = "CRITICAL";
			}
			#SMART
			$result = $snmp->get_request("$oid_disk_smart.$id");
			my $smart= $$result{"$oid_disk_smart.$id"};
			if( $smart == -1 || $smart == 0) 
			{
				$return = priority_state($return,$ERRORS{OK});
				$state = "OK";
			}
			else
			{
				$return = priority_state($return,$ERRORS{WARNING});
				$state = "WARNING";
			}
			#TEMP & SLOT
			$result = $snmp->get_request("$oid_disk_temp.$id");
			my $temp= $$result{"$oid_disk_temp.$id"};
			$result = $snmp->get_request("$oid_disk_slot.$id");
			my $slot= $$result{"$oid_disk_slot.$id"};

			print "$state : DISK $id $serial (model: $model, firmware: $firmware) state is ".$$diskstatus{"$status"}."($temp C°), SMART is ".$$smartstatus{"$smart"}." Disk is in slot$slot \n";
		}
	}
}

#SLOT INFORMATION
$result = $snmp->get_table(Baseoid => $oid_slot_name);
if (!defined($result)) 
{
   print "SLOT: na\n";
}
else
{
	#trie les disque
	my @diskid;
	foreach my $key ( keys %$result) 
	{	
		#print "$key => ".$$result{"$key"}."\n";
		if($key=~ /([0-9]*)\.([0-9]*)$/ )
		{
			push (@diskid, "$1.$2");
		}
	}
	@diskid = sort { $a cmp $b } @diskid;
	#affiche le status des disques 
	foreach my $id ( @diskid) 
	{
		my $state = "OK";
		$result = $snmp->get_request("$oid_slot_name.$id");
		my $name = $$result{"$oid_slot_name.$id"};
		$result = $snmp->get_request("$oid_slot_status.$id");
		my $status= $$result{"$oid_slot_status.$id"};
		if( $status == -1)
		{
			$return = priority_state($return,$ERRORS{UNKNOWN});
			$state = "UNKNOWN";
		} 
		elsif( $status == 6 || $status == 9 || $status == 3)
		{
			$return = priority_state($return,$ERRORS{CRITICAL});
			$state = "CRITICAL";
		}
		elsif( $status == 5)
		{
			$return = priority_state($return,$ERRORS{WARNING});
			$state = "WARNING";
		}
		else
		{
			$return = priority_state($return,$ERRORS{OK});
			$state = "OK";
		}
		print "$state : SLOT $name state is ".$$slotstatus{"$status"}."\n";
	}

}
exit($return);
