#!/usr/bin/perl -w
############################## check_snmp_load #################
my $Version='1.0';
# Date : Oct 20 2014
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use RRDs;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use ESIA::SNMP;
use ESIA::PLUGIN;

my $PROGNAME='check_snmp_sw_interface.pl';
my $PROGDESC = "Check switch, router and firewall bandwith by SNMP";
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

my ($opt_h,$opt_V);
my $ifindex;
my $path='/tmp/';
my $warning='70';
my $critical='80';
my $down = 'ok';
my $delta_interval = 300;
my $speed=1000000000; # default 1000000000 = 1Gb/s
my $detectspeed="auto";
my $oid_outoctet='1.3.6.1.2.1.2.2.1.16';
my $oid_outHCoctet='1.3.6.1.2.1.31.1.1.1.10';
my $oid_inoctet='1.3.6.1.2.1.2.2.1.10';
my $oid_inHCoctet='1.3.6.1.2.1.31.1.1.1.6';
my $oid_speed='1.3.6.1.2.1.2.2.1.5';
my $oid_highspeed='1.3.6.1.2.1.31.1.1.1.15';
my $oid_descr = "1.3.6.1.2.1.2.2.1.2";
my $oid_operstatus = "1.3.6.1.2.1.2.2.1.8";

my $oid_inUnicast ="1.3.6.1.2.1.2.2.1.11";
my $oid_inBroadcast ="1.3.6.1.2.1.2.2.1.12";
my $oid_inErrors = "1.3.6.1.2.1.2.2.1.14";

my $oid_outUnicast ="1.3.6.1.2.1.2.2.1.17";
my $oid_outBroadcast ="1.3.6.1.2.1.2.2.1.18";
my $oid_outErrors = "1.3.6.1.2.1.2.2.1.20";
my $oid_lastChange="1.3.6.1.2.1.2.2.1.9";
my $oid_sysUptime="1.3.6.1.2.1.1.3.0";
my $oid_counter_type='32bit';
sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -i                  ifindex\n";
	print " -w                  warning value in % (default: 70)\n";
	print " -c                  critical value in % (default: 80)\n";
	print " -p                  path of rrd's file\n";
	print " -s 		    vitesse de l'interface maximun en Mb/s(par défaut : autodétection)";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] -i ifindex [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
sub calc_bandwith_usage
{
	my $filename =  $_[0];
	my $actual_outoctet =  $_[1];
	my $actual_inoctet =  $_[2];
	my $timestamp = time();
	my $out_ocps=0;
	my $in_ocps=0;
	my $out_oc=0;
	my $in_oc=0;
	my $inter=0;
	#print "$filename\n";
	#on test si le fichier RRD existe
	if (! -e $filename)
	{
		open(FILECOUNTER,"+>$filename");
		print FILECOUNTER "$timestamp:$actual_outoctet:$actual_inoctet: ";
		close(FILECOUNTER);
	}
	else
	{
		open(FILECOUNTERS,"+<$filename") or die "Can't open $filename: $!";
		my $saveline="";
		while (my $line = <FILECOUNTERS>)
		{
			my @value = split(':',$line);
			$inter = ($timestamp -$value[0] );
			if($inter > 10)
			{
				$out_oc=($actual_outoctet -$value[1]);
				$in_oc=($actual_inoctet - $value[2]);

				if($out_oc>=0 && $in_oc >=0)
				{
					$out_ocps = sprintf("%.3f", ($actual_outoctet-$value[1])/$inter);
					$in_ocps = sprintf("%.3f", ($actual_inoctet-$value[2])/$inter);
				}
				last;
			}
			else
			{
				$saveline = $line;
			}

		}
		seek (FILECOUNTERS,0,0);
		print FILECOUNTERS "$timestamp:$actual_outoctet:$actual_inoctet:\n";
		print FILECOUNTERS "$saveline";
		close(FILECOUNTERS);
	}
	return ($out_oc,$in_oc,$out_ocps,$in_ocps);
}
#
#MAIN
#
&Getopt::Long::config('bundling');
#on récupère les paramètres SNMP
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"i=i" => \$ifindex,"ifindex=i" => \$ifindex,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	"p=s" => \$path,"path=s" => \$path,
	"s=s" => \$detectspeed,"speed=s"=>\$detectspeed,
	"d=s" => \$down,"down=s"=>\$down
);
# -h & --help print help
if ($opt_h || !defined($snmp))
{
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-i","--ifindex","ifindex of the interface" ]);
	push(@arr,["-d","--directory","directory for tmp file" ]);
	push(@arr,["-s","--speed","speed (default: auto)" ]);
	push(@arr,["-w","--warning","warning threshold in percent (default: 80)" ]);
	push(@arr,["-c","--critical","critical threshold in percent(default: 90) "]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}


#on interroge en SNMP pour récupérer les informations
my $result = ESIA::SNMP::get($snmp,"$oid_descr.$ifindex");
my $hostname = $snmp->hostname();
#si l'host ne répond pas on blocke le script
if (!defined($result))
{
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}

my $name =  $result;
my $la =chop($name);
if($la =~ m/[0-9a-zA-Z]$/)
{
        chomp($la);
      $name = $name.$la;
}

my $outoctet = 0;
my $inoctet = 0;
my $operstatus = ESIA::SNMP::get($snmp,"$oid_operstatus.$ifindex");
if($operstatus == 1) # Etat UP
{
	my $defaultspeed='';
	#récupération de la vitesse de l'interface
	if($detectspeed eq 'auto')
	{
		my $result =ESIA::SNMP::get($snmp,"$oid_speed.$ifindex");
		if($result !=0)
		{
			$speed =  $result;
			if($speed > 4294967200)
			{
				$speed =  ESIA::SNMP::get($snmp,"$oid_highspeed.$ifindex");
				$speed = $speed * 1000000;
			}
		}
		else
		{
			$defaultspeed='(speed recovery failed, use default speed)';
		}
	}
	else
	{
		$speed = $detectspeed*1000*1000 ;
	}


	#récupération du nombre d'octet en sortie avec le counter64 (ifHCOutOctets) si échoué utilise le counter32 (1.3.6.1.2.1.2.2.1.16)
	#récupère le nombre d'octet en entré suivant la valeur du dessus
	$result = $snmp->get_request("$oid_outHCoctet.$ifindex");
	if (!defined($result))
	{
		$result = $snmp->get_request("$oid_outoctet.$ifindex");
		$outoctet =  $$result{"$oid_outoctet.$ifindex"};
		$result = $snmp->get_request("$oid_inoctet.$ifindex");
		$inoctet =  $$result{"$oid_inoctet.$ifindex"};
	}
	#certain implémente un counter64 mais le laisse à 0 (ex: certain switch TP-LINK)
	elsif($$result{"$oid_outHCoctet.$ifindex"}==0)
	{
		$result = $snmp->get_request("$oid_outoctet.$ifindex");
		$outoctet =  $$result{"$oid_outoctet.$ifindex"};
		$result = $snmp->get_request("$oid_inoctet.$ifindex");
		$inoctet =  $$result{"$oid_inoctet.$ifindex"};
	}
	else
	{
		#OK le noeud est full compatible avec les compteurs 64 bit
		$outoctet =  $$result{"$oid_outHCoctet.$ifindex"};
		$result = $snmp->get_request("$oid_inHCoctet.$ifindex");
		$inoctet =  $$result{"$oid_inHCoctet.$ifindex"};
		$oid_counter_type='64bit';
	}
	my $timestamp = time();
	my $filename = $path.$hostname.'_'.$ifindex.".csv";

	#calcul de la bande passante utilisée
	my ($out,$in,$outkbs,$inkbs) = calc_bandwith_usage($filename,$outoctet,$inoctet);

	$in = $inkbs*8;
	$out = $outkbs*8;
    #calcul de congestion
    if($in > $speed)
    {
        #print "CONGESTION\n";
        my $maxspeed=0;
        if($speed < 100000000)#100Mbs
        {
            $maxspeed = $speed + ($speed/2);
        }
        elsif($speed < 1000000000)#1000Mbs
        {
            $maxspeed = $speed + ($speed/10);
        }
        elsif($speed < 10000000000)#10000Mbs
        {
            $maxspeed = $speed + ($speed/100);
        }
        # print "CONGESTION: $in < $maxspeed $speed?\n";
        if($in <= $maxspeed)
        {
         #   print "CONGESTION: mise au maximun\n";
            $in = $speed;
            $inkbs = $speed;
        }
    }
    if($out > $speed)
    {
        my $maxspeed=0;
        if($speed < 100000000) #100Mbs
        {
            $maxspeed = $speed + ($speed/2);
        }
        elsif($speed < 1000000000)#1000Mbs
        {
            $maxspeed = $speed + ($speed/10);
        }
        elsif($speed < 10000000000)#10000Mbs
        {
            $maxspeed = $speed + ($speed/10);
        }
        if($out <= $maxspeed)
        {
            $out = $speed;
            $outkbs = $speed;
        }
    }
	$inkbs = sprintf("%.3f", ($inkbs/1024)*8);
	$outkbs = sprintf("%.3f", ($outkbs/1024)*8);

	my $intunity ="kb/s";
	my $outunity="kb/s";
	# on regarde l'unité d'affiche en kb/s ou en Mb/S
	if( int($inkbs/1000)>0)
	{
		$inkbs = sprintf("%.3f", $inkbs/1000);
		$intunity ="Mb/s";
	}
	if( int($outkbs/1000)>0)
	{
		$outkbs = sprintf("%.3f", $outkbs/1000);
		$outunity ="Mb/s";
	}

	# teste pour savoir si on est en warning ou en critique.
	my $printspeed = $speed/1000/1000;
	my $warningspeed = ($speed/100)*$warning;
	my $criticalspeed = ($speed/100)*$critical;
	my $graphdata = "|data:in=".$in."data:out=$out:warning:$warningspeed:critical:$criticalspeed";

	#
	#récupération des informations sur les packet en erreur,broadcast,etc
	#
	#les infos en entrée
	$result = $snmp->get_request("$oid_inUnicast.$ifindex");
	my $inUnicast =  $$result{"$oid_inUnicast.$ifindex"};
	$result = $snmp->get_request("$oid_inBroadcast.$ifindex");
	my $inBroadcast =  $$result{"$oid_inBroadcast.$ifindex"};
	$result = $snmp->get_request("$oid_inErrors.$ifindex");
	my $inErrors =  $$result{"$oid_inErrors.$ifindex"};
	#les infos en sortie
	$result = $snmp->get_request("$oid_outUnicast.$ifindex");
	my $outUnicast =  $$result{"$oid_outUnicast.$ifindex"};
	$result = $snmp->get_request("$oid_outBroadcast.$ifindex");
	my $outBroadcast =  $$result{"$oid_outBroadcast.$ifindex"};
	$result = $snmp->get_request("$oid_outErrors.$ifindex");
	my $outErrors =  $$result{"$oid_outErrors.$ifindex"};
    #corrige un bug sur cisco
    if($inBroadcast eq 'noSuchInstance')
    {
        $inBroadcast=0;
        $outBroadcast=0;
    }
	#on test si le fichier RRD existe
	$filename = $path.$hostname.'_'.$ifindex."packet.rrd";
	if (! -e $filename)
	{
		RRDs::create($filename,'--start','N','--step','300','DS:inUnicast:COUNTER:600:0:4294967296','DS:inBroadcast:COUNTER:600:0:4294967296','DS:inErrors:COUNTER:600:0:4294967296',
								    'DS:outUnicast:COUNTER:600:0:4294967296','DS:outBroadcast:COUNTER:600:0:4294967296','DS:outErrors:COUNTER:600:0:4294967296','RRA:AVERAGE:0.5:1:12' );
	}
	#update de la DB
	RRDs::update($filename,"--template", "inUnicast:inBroadcast:inErrors:outUnicast:outBroadcast:outErrors","N:$inUnicast:$inBroadcast:$inErrors:$outUnicast:$outBroadcast:$outErrors");

	my ($start,$step,$names,$data) =  RRDs::fetch($filename,'AVERAGE','-s','N-5min','-e','N');
	my @arr = @$data;
	my @line = $arr[0];

	$inUnicast = sprintf("%.1f",$line[0][0]);
	$inBroadcast = sprintf("%.1f",$line[0][1]);
	$inErrors = sprintf("%.1f",$line[0][2]);
	$outUnicast = sprintf("%.1f",$line[0][3]);
	$outBroadcast = sprintf("%.1f",$line[0][4]);
	$outErrors = sprintf("%.1f",$line[0][5]);
	$graphdata .=":inUnicast:$inUnicast:inBroadcast:$inBroadcast:inErrors:$inErrors:outUnicast:$outUnicast:outBroadcast:$outBroadcast:outErrors:$outErrors";

#	print "$speed = $out = $in\n";
	#AFFICHAGE DES INFORMATIONS
#	if( $name =~ /TenGigabitEthernet/ &&  $name =~ /mpls/)
#	{
#		$speed= 10*1000*1000*1000;
#		# teste pour savoir si on est en warning ou en critique.
#		$printspeed = $speed/1000/1000;
#	}
	if($in <= $speed && $out <= $speed)
	{
		if($in >= $warningspeed)
		{
			if($in >= ($speed/100*$critical))
			{
				print "CRITICAL $name: in=$inkbs $intunity > $critical% out=$outkbs $outunity speed: $printspeed Mb/s Counter: $oid_counter_type $graphdata\n";
				exit $ERRORS{"CRITICAL"};
			}
			else
			{
				print "WARNING $name: in=$inkbs $intunity > $warning% out=$outkbs $outunity speed: $printspeed Mb/s Counter: $oid_counter_type $graphdata\n";
				exit $ERRORS{"WARNING"};
			}
		}
		elsif($out >= ($speed/100*$warning))
		{
			if($out >= ($speed/100*$critical))
			{
				print "CRITICAL $name: in=$inkbs $intunity out=$outkbs $outunity  > $critical% speed: $printspeed Mb/s Counter: $oid_counter_type $graphdata\n";
				exit $ERRORS{"CRITICAL"};
			}
			else
			{
				print "WARNING $name: in=$inkbs $intunity out=$outkbs $outunity > $warning% speed: $printspeed Mb/s Counter: $oid_counter_type $graphdata\n";
				exit $ERRORS{"WARNING"};
			}
		}
		else
		{
			print "OK $name: in=$inkbs $intunity out=$outkbs $outunity speed: $printspeed Mb/s $defaultspeed Counter: $oid_counter_type $graphdata\n";
			exit $ERRORS{"OK"};
		}
	}
	else
	{
		#Si $speed est plus petit que 1kb/s c'est qu'il y a une erreur
		if($speed < 1000)
		{
			print "UNKNOWN $name: Bad speed return by device. speed: $printspeed Mb/s\n";
			exit $ERRORS{"UNKNOWN"};
		}
		else
		{
			print "OK $name: RESET COUNTER OR SPEED MODIFY speed: $printspeed Mb/s $defaultspeed Counter: $oid_counter_type\n";
		}
	}
}
elsif($operstatus == 2 && $down eq 'critical')  # Etat DOWN
{
	print "CRITICAL $name: DOWN\n";
	exit $ERRORS{"CRITICAL"};
}
elsif($operstatus == 2 && $down eq 'warning')  # Etat DOWN
{
        print "WARNING $name: DOWN\n";
        exit $ERRORS{"WARNING"};
}
else
{
	print "OK $name: DOWN\n";
	exit $ERRORS{"OK"};
}
