#!/usr/bin/perl -w 
############################## check_snmp_syno_diskhealth.pl ########
my $Version='1.0';
# Date : Mar 14 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_syno_diskhealth.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 45;
my $critical = 50;

my $oid_sysDescr = "1.3.6.1.2.1.1.1";
my $oid_syno="1.3.6.1.4.1.6574";

my $oid_diskID="1.3.6.1.4.1.6574.2.1.1.2";
my $oid_diskModel="1.3.6.1.4.1.6574.2.1.1.3";
my $oid_diskStatus="1.3.6.1.4.1.6574.2.1.1.5";
my $oid_diskTemp="1.3.6.1.4.1.6574.2.1.1.6";

my %disks_status_layout= (
        1  => 'The hard disk functions normally.',
        2  => 'Initialized : The hard disk has system partition but no data.',
        3  => 'NotInitialized : The hard disk does not have system in system partition.',
        4  => 'SystemPartitionFailed : The system partitions on the hard disks are damaged.',
        5  => 'Crashed : The hard disk has damaged.',
	);

my $oid_RAIDName="1.3.6.1.4.1.6574.3.1.1.2";
my $oid_RAIDStatus="1.3.6.1.4.1.6574.3.1.1.3";

my %raids_status_layout= (
        1  => 'The raid functions normally.',
        2  => 'Repairing',
        3  => 'Migrating',
        4  => 'Expanding',
        5  => 'Deleting',
        6  => 'Creating',
        7  => 'RaidSyncing',
        8  => 'RaidParityChecking',
        9  => 'RaidAssembling',
        10  => 'Canceling',
        11  => 'Degrade : Degrade happens when a tolerable failure of disk(s) occurs.',
        12  => 'Crashed : Raid has crashed and just uses for read-only operation.',
	);

sub print_help() {
	printf "check l'état de santé des disques d'un NAS Synology.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning disk temperature (default: 40)\n";
	print " -c                  critical disk temperature (default: 50)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucle si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
        "w=i" => \$warning,     "warning=i" => \$warning,
        "c=i" => \$critical,     "critical=i" => \$critical,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysDescr");

#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysDescr");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}

############### DISK ###############

my $status = "OK";
my $snmpresult;
my $disks = $snmp->get_table(Baseoid => $oid_diskID);

if (!defined($disks)) 
{
   print "DISKS na\n";
}
else
{
	foreach my $key ( keys %$disks) 
	{	
		if($key =~/6574\.2\.1\.1\.2\.([0-9]*)/)
		{
			my $id = "$1";

			#DISK NAME
			my $disk_name = $$disks{$key};		
			
			#DISK MODEL
			$snmpresult = $snmp->get_request("$oid_diskModel.$id");
			my $disk_model = $$snmpresult{"$oid_diskModel.$id"};
			$disk_model =~ s/\s+//g;

			#DISK STATUS
			$snmpresult = $snmp->get_request("$oid_diskStatus.$id");
			my $disk_status = $$snmpresult{"$oid_diskStatus.$id"};
			if($disk_status == 1 || $disk_status==2)
			{
				$status = "OK";
			}
			elsif($disk_status == 3)
			{
				$status = "WARNING";
				$return = priority_state($return,$ERRORS{WARNING});
			}
			else
			{
				$status = "CRITICAL";
				$return = priority_state($return,$ERRORS{CRITICAL});
			}
			
			$snmpresult = $snmp->get_request("$oid_diskTemp.$id");
			my $disk_temp = $$snmpresult{"$oid_diskTemp.$id"};
			if($disk_temp < $warning)
			{
				$disk_temp = "Temperature : OK : $disk_temp < $warning Celsius";
			}
			else
			{
				if($disk_temp >= $critical)
				{
					$disk_temp = "Temperature : CRITICAL : $disk_temp >= $critical Celsius";
					$return = priority_state($return,$ERRORS{CRITICAL});
				}
				else # >= warning
				{
					$disk_temp = "Temperature : WARNING : $disk_temp >= $warning Celsius";
					$return = priority_state($return,$ERRORS{WARNING});		
				}	
			}

			print "$status : $disk_name ($disk_model) ; $disks_status_layout{$disk_status} $disk_temp\n";
		}
	}
}

############### RAID ###############

$status = "OK";
my $raids = $snmp->get_table(Baseoid => $oid_RAIDName);

if (!defined($raids)) 
{
   print "RAIDS na\n";
}
else
{
	foreach my $key ( keys %$raids) 
	{	
		if($key =~/6574\.3\.1\.1\.2\.([0-9]*)/)
		{
			my $id = "$1";

			#RAID NAME
			my $raid_name = $$raids{$key};		

			#RAID STATUS
			$snmpresult = $snmp->get_request("$oid_RAIDStatus.$id");
			my $raid_status = $$snmpresult{"$oid_RAIDStatus.$id"};
			if($raid_status == 1)
			{
				$status = "OK";
			}
			elsif($raid_status == 11 || $raid_status == 12)
			{
				$status = "CRITICAL";
				$return = priority_state($return,$ERRORS{CRITICAL});
			}
                        else
                        {
                                $status = "WARNING";
                                $return = priority_state($return,$ERRORS{WARNING});
                        }

			print "$status : $raid_name ; $raids_status_layout{$raid_status}\n";
		}
	}
}

exit($return);
