#!/usr/bin/perl -w 
############################## check_snmp_syno_syshealth.pl ########
my $Version='1.0';
# Date : Mar 14 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_syno_syshealth.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 50;
my $critical = 60;

my $oid_sysDescr = "1.3.6.1.2.1.1.1";
my $oid_syno="1.3.6.1.4.1.6574";
my $oid_sys_status="1.3.6.1.4.1.6574.1.1.0";
my $oid_model="1.3.6.1.4.1.6574.1.5.1.0";
my $oid_sn="1.3.6.1.4.1.6574.1.5.2.0";
my $oid_DSMVersion="1.3.6.1.4.1.6574.1.5.3.0";
my $oid_DSMUpgradeAvailable="1.3.6.1.4.1.6574.1.5.4.0";
my $oid_power_status="1.3.6.1.4.1.6574.1.3.0";
my $oid_sysfan_status="1.3.6.1.4.1.6574.1.3.0";
my $oid_cpufan_status="1.3.6.1.4.1.6574.1.3.0";
my $oid_temperature="1.3.6.1.4.1.6574.1.2.0";

sub print_help() {
	printf "check l'état de santé du système d'un NAS Synology.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning Temperature (default: 50)\n";
	print " -c                  critical Temperature (default: 60)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucle si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"w=i" => \$warning,     "warning=i" => \$warning,
	"c=i" => \$critical,     "critical=i" => \$critical,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysDescr");

#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysDescr");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}

my $state = "OK";

my $snmpresult = $snmp->get_request("$oid_model");
my $model = $$snmpresult{"$oid_model"};

$snmpresult = $snmp->get_request("$oid_sys_status");
my $sys_status = $$snmpresult{"$oid_sys_status"};
if($sys_status == 1)
{
	$sys_status = "OK : System functionals normally";
}
else
{
	$sys_status = "Failed : Volume has crashed.";
	$return = priority_state($return,$ERRORS{CRITICAL});
}

$snmpresult = $snmp->get_request("$oid_sn");
my $sn = $$snmpresult{"$oid_sn"};

$snmpresult = $snmp->get_request("$oid_DSMVersion");
my $version = $$snmpresult{"$oid_DSMVersion"};

$snmpresult = $snmp->get_request("$oid_DSMUpgradeAvailable");
my $upgradeAvail = $$snmpresult{"$oid_DSMUpgradeAvailable"};
if($upgradeAvail == 1)
{
	$upgradeAvail = "Upgrade available";
}
else
{
	$upgradeAvail = "Up to date";
}

$snmpresult = $snmp->get_request("$oid_power_status");
my $power_status = $$snmpresult{"$oid_power_status"};
if($power_status == 1)
{
	$power_status = "Normal : All power supplies functional normally."
}
else
{
	$power_status = "Failed : One of power supply has failed.";
	$return = priority_state($return,$ERRORS{CRITICAL});
}

$snmpresult = $snmp->get_request("$oid_sysfan_status");
my $sysfan_status = $$snmpresult{"$oid_sysfan_status"};
if($sysfan_status == 1)
{
	$sysfan_status = "Normal : All Internal fans functional normally."
}
else
{
	$sysfan_status = "Failed : One of internal fan stopped.";
	$return = priority_state($return,$ERRORS{WARNING});
}

$snmpresult = $snmp->get_request("$oid_cpufan_status");
my $cpufan_status = $$snmpresult{"$oid_cpufan_status"};
if($cpufan_status == 1)
{
	$cpufan_status = "Normal : All CPU fans functional normally."
}
else
{
	$cpufan_status = "Failed(2): One of CPU fan stopped.";
	$return = priority_state($return,$ERRORS{WARNING});
}

$snmpresult = $snmp->get_request("$oid_temperature");
my $temperature = $$snmpresult{"$oid_temperature"};
if($temperature < $warning)
{
	$temperature = "Temperature : OK : $temperature < $warning Celsius |temp=$temperature,warning=$warning,critical=$critical";
}
else
{
	if($temperature >= $critical)
	{
		$temperature = "Temperature : CRITICAL : $temperature >= $critical Celsius |temp=$temperature,warning=$warning,critical=$critical";
		$return = priority_state($return,$ERRORS{CRITICAL});
	}
	else # >= warning
	{
		$temperature = "Temperature : WARNING : $temperature >= $warning Celsius |temp=$temperature,warning=$warning,critical=$critical";
		$return = priority_state($return,$ERRORS{WARNING});		
	}	
}

print "Model : $model - SN : $sn - Version : $version - $upgradeAvail\n";
print "$sys_status\n";
print "Power : $power_status\n";
print "System Fans : $sysfan_status\n";
print "CPU Fans : $cpufan_status\n";
print "$temperature\n";

exit($return);
