#!/usr/bin/perl -w 
############################## check_snmp_syno_diskhealth.pl ########
my $Version='1.0';
# Date : Mar 14 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_syno_diskhealth.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 50;
my $critical = 60;
my $type="diskhealth";
my $oid_sysDescr = "1.3.6.1.2.1.1.1";
my $oid_syno="1.3.6.1.4.1.6574";

my $oid_diskID="1.3.6.1.4.1.6574.2.1.1.2";
my $oid_diskModel="1.3.6.1.4.1.6574.2.1.1.3";
my $oid_diskStatus="1.3.6.1.4.1.6574.2.1.1.5";
my $oid_diskTemp="1.3.6.1.4.1.6574.2.1.1.6";

my %disks_status_layout= (
        1  => 'The hard disk functions normally.',
        2  => 'Initialized : The hard disk has system partition but no data.',
        3  => 'NotInitialized : The hard disk does not have system in system partition.',
        4  => 'SystemPartitionFailed : The system partitions on the hard disks are damaged.',
        5  => 'Crashed : The hard disk has damaged.',
	);

my $oid_RAIDName="1.3.6.1.4.1.6574.3.1.1.2";
my $oid_RAIDStatus="1.3.6.1.4.1.6574.3.1.1.3";

my %raids_status_layout= (
        1  => 'The raid functions normally.',
        2  => 'Repairing',
        3  => 'Migrating',
        4  => 'Expanding',
        5  => 'Deleting',
        6  => 'Creating',
        7  => 'RaidSyncing',
        8  => 'RaidParityChecking',
        9  => 'RaidAssembling',
        10  => 'Canceling',
        11  => 'Degrade : Degrade happens when a tolerable failure of disk(s) occurs.',
        12  => 'Crashed : Raid has crashed and just uses for read-only operation.',
	);
my $oid_smart_test_name="1.3.6.1.4.1.6574.5.1.1.3";
my $oid_smart_test_status="1.3.6.1.4.1.6574.5.1.1.9";
my $oid_smart_test_raw="1.3.6.1.4.1.6574.5.1.1.8";
my $oid_smart_dev_name="1.3.6.1.4.1.6574.5.1.1.2";

my $oid_sys_status="1.3.6.1.4.1.6574.1.1.0";
my $oid_model="1.3.6.1.4.1.6574.1.5.1.0";
my $oid_sn="1.3.6.1.4.1.6574.1.5.2.0";
my $oid_DSMVersion="1.3.6.1.4.1.6574.1.5.3.0";
my $oid_DSMUpgradeAvailable="1.3.6.1.4.1.6574.1.5.4.0";
my $oid_power_status="1.3.6.1.4.1.6574.1.3.0";
my $oid_sysfan_status="1.3.6.1.4.1.6574.1.3.0";
my $oid_cpufan_status="1.3.6.1.4.1.6574.1.3.0";
my $oid_temperature="1.3.6.1.4.1.6574.1.2.0";

my $oid_storage_io_name="1.3.6.1.4.1.6574.101.1.1.2";
my $oid_storage_io_5min="1.3.6.1.4.1.6574.101.1.1.10";
my $oid_storage_io_15min="1.3.6.1.4.1.6574.101.1.1.11";

my $oid_service_name="1.3.6.1.4.1.6574.6.1.1.2";
my $oid_service_user="1.3.6.1.4.1.6574.6.1.1.3";

my $oid_cluster_name        ="1.3.6.1.4.1.6574.106.4.0";
my $oid_cluster_status      ="1.3.6.1.4.1.6574.106.5.0";
my $oid_cluster_activeNode  ="1.3.6.1.4.1.6574.106.1.0";
my $oid_cluster_passiveNode ="1.3.6.1.4.1.6574.106.2.0";
my $oid_cluster_autofailover="1.3.6.1.4.1.6574.106.3.0";
my $oid_cluster_heartbeat   ="1.3.6.1.4.1.6574.106.6.0";
my %cluster_status_layout= (
        0  => 'normal',
        1  => 'warning',
        2  => 'critical',
        3  => 'upgrading',
        4  => 'processing'
	);
my %cluster_heartbeatstatus_layout= (
        0  => 'normal',
        1  => 'abnormal',
        2  => 'disconnected',
        3  => 'empty',
	);
sub print_help() {
	printf "check l'état de santé des disques d'un NAS Synology.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning disk temperature (default: 40)\n";
	print " -c                  critical disk temperature (default: 50)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucle si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
    "w=i" => \$warning,     "warning=i" => \$warning,
    "c=i" => \$critical,     "critical=i" => \$critical,
    "T=s" => \$type,     "type=s" => \$type,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysDescr");

#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysDescr");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $status = "OK";
my $snmpresult;
############### DISK ###############
$status = "OK";
if($type eq 'diskhealth')
{
    ############### RAID ###############
    my $raids = $snmp->get_table(Baseoid => $oid_RAIDName);

    if (!defined($raids)) 
    {
       print "RAIDS na\n";
    }
    else
    {
	    foreach my $key ( keys %$raids) 
	    {	
		    if($key =~/6574\.3\.1\.1\.2\.([0-9]*)/)
		    {
			    my $id = "$1";

			    #RAID NAME
			    my $raid_name = $$raids{$key};		

			    #RAID STATUS
			    $snmpresult = $snmp->get_request("$oid_RAIDStatus.$id");
			    my $raid_status = $$snmpresult{"$oid_RAIDStatus.$id"};
			    if($raid_status == 1)
			    {
				    $status = "OK";
			    }
			    elsif($raid_status == 11 || $raid_status == 12)
			    {
				    $status = "CRITICAL";
				    $return = priority_state($return,$ERRORS{CRITICAL});
			    }
                            else
                            {
                                    $status = "WARNING";
                                    $return = priority_state($return,$ERRORS{WARNING});
                            }

			    print "$status : $raid_name ; $raids_status_layout{$raid_status}\n";
		    }
	    }
    }
    my $disks = $snmp->get_table(Baseoid => $oid_diskID);
    if (!defined($disks)) 
    {
       print "DISKS na\n";
    }
    else
    {
	    foreach my $key ( keys %$disks) 
	    {	
		    if($key =~/6574\.2\.1\.1\.2\.([0-9]*)/)
		    {
			    my $id = "$1";

			    #DISK NAME
			    my $disk_name = $$disks{$key};		
			    
			    #DISK MODEL
			    $snmpresult = $snmp->get_request("$oid_diskModel.$id");
			    my $disk_model = $$snmpresult{"$oid_diskModel.$id"};
			    $disk_model =~ s/\s+//g;

			    #DISK STATUS
			    $snmpresult = $snmp->get_request("$oid_diskStatus.$id");
			    my $disk_status = $$snmpresult{"$oid_diskStatus.$id"};
			    if($disk_status == 1 || $disk_status==2 || $disk_status==3)
			    {
				    $status = "OK";
			    }
			    elsif($disk_status == 3)
			    {
				    $status = "WARNING";
				    $return = priority_state($return,$ERRORS{WARNING});
			    }
			    else
			    {
				    $status = "CRITICAL";
				    $return = priority_state($return,$ERRORS{CRITICAL});
			    }
			    
			    $snmpresult = $snmp->get_request("$oid_diskTemp.$id");
			    my $disk_temp = $$snmpresult{"$oid_diskTemp.$id"};
			    if($disk_temp < $warning)
			    {
				    $disk_temp = "Temperature : OK : $disk_temp < $warning Celsius";
			    }
			    else
			    {
				    if($disk_temp >= $critical)
				    {
					    $disk_temp = "Temperature : $disk_temp >= $critical Celsius";
                        $status="CRITICAL";
					    $return = priority_state($return,$ERRORS{CRITICAL});
				    }
				    else # >= warning
				    {
                        $status="WARNING";
					    $disk_temp = "Temperature : $disk_temp >= $warning Celsius";
					    $return = priority_state($return,$ERRORS{WARNING});		
				    }	
			    }

			    print "$status : $disk_name ($disk_model) ; $disks_status_layout{$disk_status} $disk_temp\n";
		    }
	    }
    }

    ############### SMART ###############
    my $smart = $snmp->get_table(Baseoid => $oid_smart_test_name);

    #my $oid_smart_test_raw="1.3.6.1.4.1.6574.5.1.1.8";

    if (!defined($smart)) 
    {
       print "SMART na\n";
    }
    else
    {
        my $smartmessage="";
	    foreach my $key ( keys %$smart) 
	    {	
		    if($key =~/6574\.5\.1\.1\.3\.([0-9]*)/)
		    {
			    my $id = "$1";

			    #DISK NAME
			    my $test_name = $$smart{$key};		
			    if( $test_name ne 'Airflow_Temperature_Cel')
                {
			        #DISK MODEL
			        $snmpresult = $snmp->get_request("$oid_smart_test_status.$id");
			        my $test_status = $$snmpresult{"$oid_smart_test_status.$id"};
			        if($test_status eq 'OK' )
			        {
				        $status = "OK";
                        my $raw_value="";
                        if($test_name eq 'Reallocated_Sector_Ct' || $test_name eq 'Re-allocated_Sector_Count')
                        {
                            $snmpresult = $snmp->get_request("$oid_smart_test_raw.$id");
			                $raw_value = $$snmpresult{"$oid_smart_test_raw.$id"};
                            if($raw_value > 200)
                            {
				                $status = "WARNING";
				                $return = priority_state($return,$ERRORS{WARNING});
			                    $snmpresult = $snmp->get_request("$oid_smart_dev_name.$id");
			                    my $dev = $$snmpresult{"$oid_smart_dev_name.$id"};
                                $smartmessage.= "$status : Error on $dev hight $test_name\n";
                            }
                        }
			        }
			        else
			        {
				        $status = "WARNING";
				        $return = priority_state($return,$ERRORS{WARNING});
			            $snmpresult = $snmp->get_request("$oid_smart_dev_name.$id");
			            my $dev = $$snmpresult{"$oid_smart_dev_name.$id"};
                        $smartmessage.= "$status : $dev smart status ($test_name) is bad\n";
			        }
                }
		    }
	    }
        if($smartmessage eq "")
        {
            print "OK : All SMART test is good";      
        }
        else
        {
            print "WARNING: SMART test failed:\n$smartmessage";
        }
    }
}
############### SYSTEM ###############
elsif($type eq 'syshealth')
{
    my $snmpresult = $snmp->get_request("$oid_model");
    my $model = $$snmpresult{"$oid_model"};

    $snmpresult = $snmp->get_request("$oid_sys_status");
    my $sys_status = $$snmpresult{"$oid_sys_status"};
    if($sys_status == 1)
    {
	    $sys_status = "OK : System functionals normally";
    }
    else
    {
	    $sys_status = "CRITICAL : Volume has crashed.";
	    $return = priority_state($return,$ERRORS{CRITICAL});
    }

    $snmpresult = $snmp->get_request("$oid_sn");
    my $sn = $$snmpresult{"$oid_sn"};

    $snmpresult = $snmp->get_request("$oid_DSMVersion");
    my $version = $$snmpresult{"$oid_DSMVersion"};

    $snmpresult = $snmp->get_request("$oid_DSMUpgradeAvailable");
    my $upgradeAvail = $$snmpresult{"$oid_DSMUpgradeAvailable"};
    if($upgradeAvail == 1)
    {
	    $upgradeAvail = "Upgrade available";
    }
    else
    {
	    $upgradeAvail = "Up to date";
    }

    $snmpresult = $snmp->get_request("$oid_power_status");
    my $power_status = $$snmpresult{"$oid_power_status"};
    if($power_status == 1)
    {
	    $power_status = "OK : All power supplies functional normally."
    }
    else
    {
	    $power_status = "CRITICAL : One of power supply has failed.";
	    $return = priority_state($return,$ERRORS{CRITICAL});
    }

    $snmpresult = $snmp->get_request("$oid_sysfan_status");
    my $sysfan_status = $$snmpresult{"$oid_sysfan_status"};
    if($sysfan_status == 1)
    {
	    $sysfan_status = "OK : All Internal fans functional normally."
    }
    else
    {
	    $sysfan_status = "CRITICAL : One of internal fan stopped.";
	    $return = priority_state($return,$ERRORS{WARNING});
    }

    $snmpresult = $snmp->get_request("$oid_cpufan_status");
    my $cpufan_status = $$snmpresult{"$oid_cpufan_status"};
    if($cpufan_status == 1)
    {
	    $cpufan_status = "OK : All CPU fans functional normally."
    }
    else
    {
	    $cpufan_status = "CRITICAL : One of CPU fan stopped.";
	    $return = priority_state($return,$ERRORS{WARNING});
    }

    $snmpresult = $snmp->get_request("$oid_temperature");
    my $temperature = $$snmpresult{"$oid_temperature"};
    if($temperature < $warning)
    {
	    $temperature = "Temperature : OK : $temperature < $warning Celsius |temp=$temperature,warning=$warning,critical=$critical";
    }
    else
    {
	    if($temperature >= $critical)
	    {
		    $temperature = "CRITICAL : Temperature is very high $temperature >= $critical Celsius |temp=$temperature,warning=$warning,critical=$critical";
		    $return = priority_state($return,$ERRORS{CRITICAL});
	    }
	    else # >= warning
	    {
		    $temperature = "WARNING : Temperature is high $temperature >= $warning Celsius |temp=$temperature,warning=$warning,critical=$critical";
		    $return = priority_state($return,$ERRORS{WARNING});		
	    }	
    }

    print "Model : $model - SN : $sn - Version : $version - $upgradeAvail\n";
    print "$sys_status\n";
    print "$power_status\n";
    print "$sysfan_status\n";
    print "$cpufan_status\n";
    print "$temperature\n";
}
elsif($type eq 'io')
{
    my $io = $snmp->get_table(Baseoid => $oid_storage_io_name);

    if (!defined($io)) 
    {
       print "IO na\n";
    }
    else
    {
        print "IO 5 minute load average:\n";
        my $i = 0;
        my $total = 0;
	    foreach my $key ( keys %$io) 
	    {	
		    if($key =~/\.([0-9]*)$/)
		    {
                my $state="OK";
			    my $id = "$1";
                my $disk_name = $$io{$key};
                $snmpresult = $snmp->get_request("$oid_storage_io_5min.$id");
                my $io_5min = $$snmpresult{"$oid_storage_io_5min.$id"};
                my $message = "$disk_name load $io_5min%";   
                if($io_5min >=$critical)
                {
                    $state="CRITICAL";
                    $message.=" > $critical%";
                }
                elsif($io_5min >=$warning)
                {
                    $state="WARNING";
                    $message.=" > $warning%";
                }
                else
                {
                    $state="OK";
                    $message.=" < $warning%";
                }
                print "$state : $message\n";
		        $return = priority_state($return,$ERRORS{$state});		
                $total=$total+$io_5min;
                $i++;
            }
        }
        my $state="OK";
        my $load= sprintf("%.0f",($total/$i));      
        my $message = " < $warning%";       
        if($load >=$critical)
        {
            $state="CRITICAL";
            $message=" > $critical%";
        }
        elsif($load >=$warning)
        {
            $state="WARNING";
            $message=" > $warning%";
        }        
        print "$state : Average load : ".$load."$message|average=$load\n";
        $return = priority_state($return,$ERRORS{$state});	
    }
}
elsif($type eq 'service')
{
    my $service = $snmp->get_table(Baseoid => $oid_service_name);
    if (!defined($service)) 
    {
       print "Service na\n";
    }
    else
    {
        my $graph_data="|";
	    foreach my $key ( keys %$service) 
	    {	
		    if($key =~/\.([0-9]*)$/)
		    {
                my $state="OK";
			    my $id = "$1";
                my $name = $$service{$key};
                $snmpresult = $snmp->get_request("$oid_service_user.$id");
                my $user = $$snmpresult{"$oid_service_user.$id"};
                my $message ="< $warning";
                if($user >=$critical)
                {
                    $state="CRITICAL";
                    $message=" > $critical";
                }
                elsif($user >=$warning)
                {
                    $state="WARNING";
                    $message=" > $warning";
                }
                $graph_data.=$name.'='.$user.',';
                print"$state : service $name have $user user connected $message\n";
                $return = priority_state($return,$ERRORS{$state});	
            }
        }
        print $graph_data;
    }
}
elsif($type eq 'cluster')
{
    $snmpresult = $snmp->get_request("$oid_cluster_name");
    if (!defined($snmpresult)) 
    {
       print "Cluster na\n";
    }
    else
    {
        my $state = 'OK';
        my $name = $$snmpresult{"$oid_cluster_name"};
        $snmpresult = $snmp->get_request("$oid_cluster_status");
        my $status = $$snmpresult{"$oid_cluster_status"};
        $snmpresult = $snmp->get_request("$oid_cluster_activeNode");
        my $active = $$snmpresult{"$oid_cluster_activeNode"};
        $snmpresult = $snmp->get_request("$oid_cluster_passiveNode");
        my $pasive = $$snmpresult{"$oid_cluster_passiveNode"};
        if($status == 0 || $status == 3)
        {
            $state='OK';
        }
        elsif($status == 1 || $status == 4)
        {
            $state='WARNING';
        }
        else
        {
            $state='CRITICAL';
        }
        
        print "$state : cluster $name (active node: $active passive: $pasive) is  $cluster_status_layout{$status} state.\n";
        $return = priority_state($return,$ERRORS{$state});	
        $state = 'OK';
        $snmpresult = $snmp->get_request("$oid_cluster_heartbeat");
        my $heartbeat = $$snmpresult{"$oid_cluster_heartbeat"};
        if($heartbeat == 0)
        {
            $state = 'OK';
        }
        elsif($heartbeat == 1 || $heartbeat == 2)
        {
            $state = 'CRITICAL';
        }
        else
        {
            $state = 'WARNING';
        }
        print "$state : heartbeat is $cluster_heartbeatstatus_layout{$heartbeat} state.\n";
        $return = priority_state($return,$ERRORS{$state});	
        $state='OK';
        $snmpresult = $snmp->get_request("$oid_cluster_autofailover");
        my $autofail = $$snmpresult{"$oid_cluster_autofailover"};
        my $message = 'not activated';
        if($autofail ==1)
        {
            $message = 'activated'
        }
        print "$state : autofailover is $message\n";
    }
}
else
{
    print "UNKNOWN: test type not found\n";
    $return = priority_state($return,$ERRORS{UNKNOWN});

}
exit($return);
