#!/usr/bin/perl -w 
############################## check_snmp_synology_ups #################
my $Version='1.0';
# Date : Mars 21 2015
# Author  : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;


my $PROGNAME='check_snmp_synology_ups';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=$ERRORS{OK};
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 70;
my $critical = 50;
my $load_warning = 50;
my $load_critical = 70;
my $oid_ups_model=".1.3.6.1.4.1.6574.4.1.1.0";
my $oid_ups_manufacturer=".1.3.6.1.4.1.6574.4.1.2.0";
my $oid_ups_serial=".1.3.6.1.4.1.6574.4.1.3.0";
my $oid_ups_battery_value="1.3.6.1.4.1.6574.4.3.1.1.0";
my $oid_ups_battery_type="1.3.6.1.4.1.6574.4.3.12.0";
my $oid_ups_load="1.3.6.1.4.1.6574.4.2.12.1.0";
sub print_help() {
	printf "check SNMP SYNOLOGY UPS.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning baterry in %\n";
	print " -c                  critical baterry in %\n";
	print " --loadWarning=      warning UPS load in %\n";
	print " --loadCritical=     critical UPS load in %\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
	"loadWarning=i" => \$load_warning,
	"loadCritical=i" => \$load_critical,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
my ($snmp, $snmperror);

	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,

		-timeout=>2,
   
	);
# INFORMATION
my $resultat = $snmp->get_request("$oid_ups_manufacturer");
if (!defined($resultat)) 
{
	printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	$snmp->close;
	exit $ERRORS{"UNKNOWN"};
}
my $description= $$resultat{$oid_ups_manufacturer};
$resultat = $snmp->get_request("$oid_ups_model");
my $model= $$resultat{$oid_ups_model};
$resultat = $snmp->get_request("$oid_ups_serial");
my $serial= $$resultat{$oid_ups_serial};
# load UPS
#TRES TRES MOCHE mais pas moyen d'empécher perl d'interpreter le opaque: float: ><
my $commande = 'snmpget -v 2c -c '.$community.' '.$hostname.' '.$oid_ups_load.' | sed -e "s/\(.*Float\:\ \)\(.*\)/\2/g"';
my $load = `$commande`;
$load =~ s/\n+//g;
$load = int $load;
my $state = 'OK';
my $error_message=" <  $load_warning%";
if($load > $load_critical)
{
	$return = priority_state($return,$ERRORS{CRITICAL});
	$state ="CRITICAL";
	$error_message="> $load_critical%";
}
elsif($load > $load_warning)
{
	$return = priority_state($return,$ERRORS{WARNING});
	$state ="WARNING";
	$error_message="> $load_warning%";
}
print "Manufacturer: $description Model: $model SN: $serial\n";
print "$state: UPS load: $load% $error_message\n";

#Batterie Status
#TRES TRES MOCHE mais pas moyen d'empécher perl d'interpreter le opaque: float: ><
$commande = 'snmpget -v 2c -c '.$community.' '.$hostname.' '.$oid_ups_battery_value.' | sed -e "s/\(.*Float\:\ \)\(.*\)/\2/g"';
my $bat_value = `$commande`;
$bat_value =~ s/\n+//g;
$bat_value = int $bat_value;
$resultat = $snmp->get_request("$oid_ups_battery_type");
my $bat_type= $$resultat{$oid_ups_battery_type};

$state = 'OK';
$error_message=" > $warning%";
if($bat_value <= $critical)
{
	$return = priority_state($return,$ERRORS{CRITICAL});
	$state ="CRITICAL";
	$error_message=" < $critical%";
}
elsif($bat_value <= $warning)
{
	$return = priority_state($return,$ERRORS{WARNING});
	$state ="WARNING";
	$error_message="< $warning%";
}

print "$state: Battery type: $bat_type charge: $bat_value % $error_message\n";
exit($return);




