#!/usr/bin/php
<?php
$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);
$PRIORITIES_STATES=array("CRITICAL","WARNING","UNKNOWN","OK");

$DEFAULT_TIMEOUT = 50;

$tcpCurrEstab=".1.3.6.1.2.1.6.9.0";

$short_opts="hH:C:t:w:c:";
$long_opts=array("help","host:","community:","timeout:","warning:","critical:");

$options=getopt($short_opts,$long_opts);

function print_usage($bad_usage){
	echo 'USAGE : '.basename($_SERVER['PHP_SELF'])." -H <HOST> -C <COM> -w <WARN> -c <CRITIC> [-t <TIMEOUT>]\n";
	if($bad_usage) echo 'HELP  : '.basename($_SERVER['PHP_SELF'])." -h\n";
}

function print_help(){
	$col=20;
	print_usage(false);
	echo "\n";
	echo "Check the current number of established TCP connections\n\n";
	echo "\t-H, ".str_pad("--host=HOST",$col)			."Address or domain name\n";
	echo "\t-C, ".str_pad("--community=COM",$col)		."SNMP community\n";
	echo "\t-t, ".str_pad("--timeout=SECONDES",$col)	."Timeout in seconds\n";
	echo "\t-w, ".str_pad("--warning=INTEGER",$col)		."Max currents connections limit for warning\n";
	echo "\t-c, ".str_pad("--critical=INTEGER",$col)	."Max currents connections limit for critical\n";
	echo "\n";
	echo "\t-h, ".str_pad("--help",$col)				."Print this help\n";
	echo "\n";
}


function getParam($name1,$name2,$default=null){
	global $options;

	if( isset( $options[$name1] ) ) return $options[$name1];
	if( isset( $options[$name2] ) ) return $options[$name2];
	return $default;
}

function exit_state($state,$msg=null,$usage=false){
	$STR_STATUS=$GLOBALS["STR_STATUS"];
	$INT_STATUS=$GLOBALS["INT_STATUS"];

	$ret=$state;
	if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
	else $state=$STR_STATUS[$ret];

	if( $msg != null ) echo "$state - $msg\n";
	if( $usage ) print_usage(true);
	exit($ret);
}

function calcul_state($a,$b=null){
	$STR_STATUS=$GLOBALS["STR_STATUS"];
	$INT_STATUS=$GLOBALS["INT_STATUS"];
	$PRIORITIES_STATES=$GLOBALS["PRIORITIES_STATES"];

	if(is_string($a) ) $a=$INT_STATUS[strtoupper($a)];
	if(is_string($b) ) $b=$INT_STATUS[strtoupper($b)];

	foreach($PRIORITIES_STATES as $value){
		$value = $INT_STATUS[$value];

		if( $value == $a || $value == $b ) return $value;
	}
	return $INT_STATUS["UNKNOWN"];
}

if ( isset($options["help"]) || isset($options["h"]) ){
	print_help();
	exit($INT_STATUS["UNKNOWN"]);
}

$host = getParam("H","host");
$community = getParam("C","community");
$timeout = getParam("t","timeout",$DEFAULT_TIMEOUT);
$warning = getParam("w","warning");
$critical = getParam("c","critical");

if( $host == null ) exit_state(3,"You forget to specify host.",true);
if( $community == null ) exit_state(3,"You forget to specify community.",true);
if( $warning == null ) exit_state(3,"You forget to specify warning.",true);
if( $critical == null ) exit_state(3,"You forget to specify critical.",true);

if( !is_numeric($warning) || !is_numeric($critical) || $warning < 0) exit_state(3,"warning and critical values must isn't integers.",false);
if( $timeout < 0 ) exit_state(3,"The timeout must be > 0.",false);
if( $warning >= $critical ) exit_state(3,"warning must be lower than critical.",false);

$current_conn = @snmp2_get($host,$community,$tcpCurrEstab,$timeout*100000);
if( $current_conn === FALSE )  {
	$message = error_get_last();
	$message = $message["message"];
	$message = trim(explode(":",$message)[1]);
	exit_state(3,$message,false);
}

$current_conn = trim(explode(":",$current_conn)[1]);


if( $current_conn > $critical ){
	$state = 2;
}
else if( $current_conn > $warning ){
	$state = 1;
}
else {
	$state = 0;
}

$message = "$current_conn TCP connections (w:$warning,c:$critical)";

exit_state($state,$message,false);
?>
