#!/usr/bin/perl -w
######################### check_snmp_lenovo #################
my $Version='1.0';
# Date : August 01 2018
# Author : Nicolas Biersart
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;
use List::MoreUtils qw(uniq);
my $PROGNAME='check_snmp_cisco_ucs.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my ($opt_h,$opt_V);
my $domain = "UDP/IPV4"; # domain UDP/IPV6;UDP/IPV4;TCP/IPV6;TCP/IPV4
my $community = "public";
my $snmp_version = 2;
my ($hostname);
my $warning='40';
my $port='161';
my $critical='50';
my $type;
my ($snmp, $snmperror);
my $ciscoStatus = {
     0 => 'unknown',
     1 => 'operable',
     2 => 'inoperable',
     3 => 'degraded',
     4 => 'poweredOff',
     5 => 'powerProblem',
     6 => 'removed',
     7 => 'voltageProblem',
     8 => 'thermalProblem',
     9 => 'performanceProblem',
     10 => 'accessibilityProblem',
     11 => 'identityUnestablishable',
     12 => 'biosPostTimeout',
     13 => 'disabled',
     14 => 'malformedFru',
     15 => 'backplanePortProblem',
     16 => 'chassisIntrusion',
     51 => 'fabricConnProblem',
     52 => 'fabricUnsupportedConn',
     81 => 'config',
     82 => 'equipmentProblem',
     83 => 'decomissioning',
     84 => 'chassisLimitExceeded',
     100 => 'notSupported',
     101 => 'discovery',
     102 => 'discoveryFailed',
     103 => 'identify',
     104 => 'postFailure',
     105 => 'upgradeProblem',
     106 => 'peerCommProblem',
     107 => 'autoUpgrade',
     108 => 'linkActivateBlocked'
};
my $ciscoDriveStatus = {
     0 => 'unknown',
     1 => 'online',
     2 => 'unconfiguredGood',
     3 => 'globalHotSpare',
     4 => 'dedicatedHotSpare',
     5 => 'jbod',
     6 => 'offline',
     7 => 'rebuilding',
     8 => 'copyback',
     9 => 'failed',
     10 => 'unconfiguredBad',
     11 => 'predictiveFailure',
     12 => 'disabledForRemoval',
     13 => 'foreignConfiguration',
     14 => 'zeroing',
     15 => 'good',
     16 => 'bad',
     17 => 'lockedForeignConfiguration',
};

my $oid_sysdescr 		    ="1.3.6.1.2.1.1.1.0";
my $oid_cisco_fan_descr     ="1.3.6.1.4.1.9.9.719.1.15.12.1.2";
my $oid_cisco_fan_state     ="1.3.6.1.4.1.9.9.719.1.15.12.1.9";
my $oid_cisco_power_descr   ="1.3.6.1.4.1.9.9.719.1.15.56.1.2";
my $oid_cisco_power_state   ="1.3.6.1.4.1.9.9.719.1.15.56.1.7";
my $oid_cisco_mem_descr     ="1.3.6.1.4.1.9.9.719.1.30.11.1.2.1";
my $oid_cisco_drive_descr   ="1.3.6.1.4.1.9.9.719.1.45.4.1.2";
my $oid_cisco_drive_present ="1.3.6.1.4.1.9.9.719.1.45.4.1.10";
my $oid_cisco_drive_model   ="1.3.6.1.4.1.9.9.719.1.45.4.1.7";
my $oid_cisco_drive_size    ="1.3.6.1.4.1.9.9.719.1.45.4.1.13";
my $oid_cisco_drive_serial  ="1.3.6.1.4.1.9.9.719.1.45.4.1.12";
my $oid_cisco_drive_state   ="1.3.6.1.4.1.9.9.719.1.45.4.1.18";
my $oid_cisco_cpu_descr     ="1.3.6.1.4.1.9.9.719.1.41.2.1.2";
my $oid_cisco_cpu_temp      ="1.3.6.1.4.1.9.9.719.1.41.2.1.10";

sub print_help() {
	printf "check SNMP interface via SNMP.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)		Hostname to query - (required)\n";
	print " -D (--domain)		UDP/IPV6 UDP/IPV4 TCP/IPV6 TCP/IPV4 \n";
	print " -C (--community)	SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "		     2 for SNMP v2c (default)\n";
	print " -T	target value (default: ALL)\n";
	print " -w	warning value (default: 70)\n";
	print " -c	critical value (default: 80)\n";
	print " -V (--version)	Plugin version\n";
	print " -h (--help)	usage help\n";
	print "\n";
}



my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

sub print_usage() {
	 
	print "This snmpv2c plugin checks the hardware states on one HP server with the HP Insight System Management Agent \n";
	print "Usage : $0 <hostname or ip> <snmp community> <check> [device]";
	print "Usage : $0 <hostname or ip> <snmp community> <check> [device] [timeout en sec]";
	print "Usage : $0 <hostname or ip> <snmp community> <check> [min device] [max device] <timeout en sec> \n";
	print "The result also print <check name> <device> [details] : <state> => <description of this state> \n";
	exit $ERRORS{"UNKNOWN"};
}
sub readOID {
    my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}
########## MAIN ##########

&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,        "version"        => \$opt_V,
	"D=s" => \$domain,       "domain=s"       => \$domain,
	"h"   => \$opt_h,        "help"           => \$opt_h,
	"C=s" => \$community,    "community=s"    => \$community,
	"H=s" => \$hostname,     "hostname=s"     => \$hostname,
	"v=i" => \$snmp_version, "snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,      "warning=i"      => \$warning,
	"c=i" => \$critical,     "critical=i"     => \$critical,
	"T=s" => \$type,		 "target=s"		  => \$type,
	"t=s" => \$TIMEOUT,		 "timeout=s"	  => \$TIMEOUT,
);
# -h & --help print help
if($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }
if(!$community) {print_help(); exit $ERRORS{'OK'}; }
if(!$domain) {print_help(); exit $ERRORS{'OK'}; }
if(!$type) {print_help(); exit $ERRORS{'OK'}; }
if ($snmp_version == 2)
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname   => $hostname,
			-version    => 'snmpv2c',
			-domain     => $domain,
			-port       => $port,
			-community  => $community,
			-timeout    => 10
		);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
			-hostname  => $hostname,
			-version   => 'snmpv1',
			-domain    => $domain,
			-port      => $port,
			-community => $community,
			-timeout   => 10
		);
}
my $result = $snmp->get_request("$oid_sysdescr");
if (!defined($result))
{
	printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	$snmp->close;
	exit $ERRORS{"UNKNOWN"};
}
my $return=$ERRORS{OK};
if($type eq 'fan')
{
    my $result = $snmp->get_table(Baseoid =>"$oid_cisco_fan_descr");	         
    if (!defined($result))
    {
        printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
        $snmp->close;
        exit $ERRORS{"UNKNOWN"};
    }

    foreach my $key ( keys %$result)
    {
	    if($key =~ /\.([0-9]*)$/)
		{
            my $id = $1;
			my $fandescr = $$result{$key};
            my $status = readOID($snmp,"$oid_cisco_fan_state.$id");
            my $state = 'OK';
            if($status != 1 )
            {
                $state = 'WARNING';
                $return = priority_state($return,$ERRORS{WARNING});
            }
            print "$state : $fandescr status is ".$ciscoStatus->{$status}."\n";
        }
    }
}
elsif($type eq 'power')
{
    my $result = $snmp->get_table(Baseoid =>"$oid_cisco_power_descr");	         
    if (!defined($result))
    {
        printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
        $snmp->close;
        exit $ERRORS{"UNKNOWN"};
    }

    foreach my $key ( keys %$result)
    {
	    if($key =~ /\.([0-9]*)$/)
		{
            my $id = $1;
			my $powerdescr = $$result{$key};
            my $status = readOID($snmp,"$oid_cisco_power_state.$id");
            my $state = 'OK';
            if($status ==2 || $status ==3  || $status ==4 ||$status ==5  )
            {
                $state = 'CRITICAL';
                $return = priority_state($return,$ERRORS{CRITICAL});
            }
            elsif($status != 1 )
            {
                $state = 'WARNING';
                $return = priority_state($return,$ERRORS{WARNING});
            }
            print "$state : $powerdescr status is ".$ciscoStatus->{$status}."\n";
        }
    }
}
elsif($type eq 'drive')
{
    my $result = $snmp->get_table(Baseoid =>"$oid_cisco_drive_descr");	  
    my $resultdrive = $snmp->get_table(Baseoid =>"$oid_cisco_drive_state");	   

    if (!defined($result)||!defined($resultdrive))
    {
        printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
        $snmp->close;
        exit $ERRORS{"UNKNOWN"};
    }

    my $resultsize = $snmp->get_table(Baseoid =>"$oid_cisco_drive_size");
    my $resultpresent = $snmp->get_table(Baseoid =>"$oid_cisco_drive_present");
    my $resultserial = $snmp->get_table(Baseoid =>"$oid_cisco_drive_serial");

    foreach my $key ( keys %$result)
    {
	    if($key =~ /\.([0-9]*)$/)
		{
            my $id = $1;
			my $drivedescr = $$result{$key};
            my $present = $$resultpresent{"$oid_cisco_drive_present.$id"};
            if($present ==10)
            {
                my $status = $$resultdrive{"$oid_cisco_drive_state.$id"};
                my $size = $$resultsize{"$oid_cisco_drive_size.$id"};
                my $serial = $$resultserial{"$oid_cisco_drive_serial.$id"};
                my $state = 'OK';
                #$status=3;
                if($status ==1 || $status ==3  || $status ==4 ||$status ==15  )
                {
                    $state = 'OK';
                    $return = priority_state($return,$ERRORS{OK});
                }
                elsif($status ==9 || $status ==16 )
                {
                    $state = 'CRITICAL';
                    $return = priority_state($return,$ERRORS{CRITICAL});
                }
                else
                {
                    $state = 'WARNING';
                    $return = priority_state($return,$ERRORS{WARNING});
                }
                print "$state : $drivedescr (S/N: $serial, size: $size MB) status is ".$ciscoDriveStatus->{$status}."\n";
            }
            elsif($present==11||$present==1)
            {
                print "OK : $drivedescr is missing/empty\n";
            }
            else
            {
                print "UNKNOWN : $drivedescr bad code disk present\n";
            }
        }
    }
}
elsif($type eq 'temperature')
{
    my $result = $snmp->get_table(Baseoid =>"$oid_cisco_cpu_descr");	

    if (!defined($result))
    {
        printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
        $snmp->close;
        exit $ERRORS{"UNKNOWN"};
    }

    my $data="|";     
    my $i=0;    
    foreach my $key ( keys %$result)
    {
	    if($key =~ /\.([0-9]*)$/)
		{
            my $id = $1;
			my $cpudesct = $$result{$key};
            my $temp = readOID($snmp,"$oid_cisco_cpu_temp.$id");
            my $state = 'OK';
            my $message=" < $warning";
            if($temp > $critical)
            {
                     $message=" > $critical";
                    $state = 'CRITICAL';
                    $return = priority_state($return,$ERRORS{CRITICAL});
            }
            elsif($temp > $warning)
            {
                    $message=" > $warning";
                    $state = 'WARNING';
                    $return = priority_state($return,$ERRORS{WARNING});
            }
            print "$state : $cpudesct temperature is $temp $message\n";
            $i++;
            $data=$data."cpu$i=$temp,";
        }
    }
    print $data;
}
exit $return;
