#!/usr/bin/perl
#
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
#use lib "/root/julien/20220616/3.3.0";
#use lib "/root/libperl/"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Getopt::Long;
use JSON;
use ESIA::SNMP;
use ESIA::PLUGIN;

use Data::Dumper;

#nom du programme & description
my $PROGNAME = "check_snmp_ups.pl";
my $PROGDESC = "Check ups by SNMP";
my $PROGVERSION = "1.0";
#Chargement du timer par défaut et des erreurs
# a installer apt install libjson-perl
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

ESIA::PLUGIN::arm_end_signal($TIMEOUT);

$PROGNAME = "check_snmp_ups.pl";

Getopt::Long::Configure( "bundling" );
my ($opt_h,$opt_V);
my $community = "public";
my $snmp_version = 1;
my ($hostname);
my $warning=80;
my $critical=90;
my $return =0;
my $type ='state';

my $oid_sysObjectOID        = "1.3.6.1.2.1.1.2.0";
my $oid_upsAlarm            = "1.3.6.1.2.1.33.1.6.2.1";
my $oid_upsOutputSource     = "1.3.6.1.2.1.33.1.4.1.0";
my $oid_upsBatteryStatus    = "1.3.6.1.2.1.33.1.2.1.0";
my $oid_upsEstimatedMinutesRemaining ="1.3.6.1.2.1.33.1.2.3.0";
my $oid_upsOutputFrequency   ="1.3.6.1.2.1.33.1.4.2.0";
my $oid_upsOutputVoltage 	 ="1.3.6.1.2.1.33.1.4.4.1.2";
my $oid_upsOutputCurrent 	 ="1.3.6.1.2.1.33.1.4.4.1.3";
my $oid_upsOutputPower 	     ="1.3.6.1.2.1.33.1.4.4.1.4";
my $oid_upsOutputPercentLoad ="1.3.6.1.2.1.33.1.4.4.1.5";

my $oid_upsInputFrequency    ="1.3.6.1.2.1.33.1.3.3.1.2";
my $oid_upsInputVoltage 	 ="1.3.6.1.2.1.33.1.3.3.1.3";
my $oid_upsInputCurrent 	 ="1.3.6.1.2.1.33.1.3.3.1.4";
my $oid_upsInputTruePower    ="1.3.6.1.2.1.33.1.3.3.1.5";

my $oid_upsByPassFrequency ="1.3.6.1.2.1.33.1.5.1.0";
my $oid_upsByPassVoltage   ="1.3.6.1.2.1.33.1.5.3.1.2";
my $oid_upsByPassCurrent   ="1.3.6.1.2.1.33.1.5.3.1.3";
my $oid_upsByPassPower    ="1.3.6.1.2.1.33.1.5.3.1.4";




#affichage pour les status
my %upsBatteryStatuses = (
		1 => "unknown(1)",
		2 => "batteryNormal(2)",
		3 => "batteryLow(3)",
		4 => "batteryDepleted(4)",
        5 => "batteryDischarging(5)",
        6 => "batteryFailure(6)",
		);
#OID HP
my $oid_hp_input_source = "1.3.6.1.4.1.232.165.3.3.5.0";
my %oid_hp_input_sourceMessage = ("1" => "other(1)",
		"2" => "none(2)",
		"3" => "primaryUtility(3)",
		"4" => "bypassFeed(4)",
		"5" => "secondaryUtility(5)",
		"6" => "generator(6)",
		"7" => "flywheel(7)",
		"8" => "fuelcell(8)");
my %oid_hp_input_sourceStatus = ( "1" => "UNKNOWN",
		"2" => "OK",
		"3" => "OK",
		"4" => "CRITICAL",
		"5" => "WARNING",
		"6" => "WARNING",
		"7" => "WARNING",
		"8" => "WARNING" );
my %upsOutputSourcesMessage = (
		"1" => "other(1)",
		"2" => "none(2)",
		"3" => "normal(3)",
		"4" => "bypass(4)",
		"5" => "battery(5)",
		"6" => "booster(6)",
		"7" => "reducer(7)"
		);
my %upsOutputAlarmMessage = (
		"1.3.6.1.2.1.33.1.6.3.1" => "Battery is bad",
		"1.3.6.1.2.1.33.1.6.3.10" => "Bypass is bad",
		"1.3.6.1.2.1.33.1.6.3.11" => "Output off as resquested ",
		"1.3.6.1.2.1.33.1.6.3.12" => "UPS off as resquested",
		"1.3.6.1.2.1.33.1.6.3.13" => "Charger is failed",
		"1.3.6.1.2.1.33.1.6.3.14" => "Output is Off",
		"1.3.6.1.2.1.33.1.6.3.15" => "System is Off",
		"1.3.6.1.2.1.33.1.6.3.16" => "Fan is failed",
		"1.3.6.1.2.1.33.1.6.3.17" => "Fuse is failed",
		"1.3.6.1.2.1.33.1.6.3.18" => "General failure",
		"1.3.6.1.2.1.33.1.6.3.19" => "Diagnostic failed ",
		"1.3.6.1.2.1.33.1.6.3.2" => "Battery is a warning state",
		"1.3.6.1.2.1.33.1.6.3.20" => "Communication lost",
		"1.3.6.1.2.1.33.1.6.3.21" => "Awaiting power",
		"1.3.6.1.2.1.33.1.6.3.22" => "Shutdown pending",
		"1.3.6.1.2.1.33.1.6.3.23" => "Shutdown iminent",
		"1.3.6.1.2.1.33.1.6.3.24" => "Test in progress",
		"1.3.6.1.2.1.33.1.6.3.3" => "Battery low",
		"1.3.6.1.2.1.33.1.6.3.4" => "Battery is depleted",
		"1.3.6.1.2.1.33.1.6.3.5" => "Temperature alarm",
		"1.3.6.1.2.1.33.1.6.3.6" => "Input power alarm",
		"1.3.6.1.2.1.33.1.6.3.7" => "Ouput power alarm",
		"1.3.6.1.2.1.33.1.6.3.8" => "Overload",
		"1.3.6.1.2.1.33.1.6.3.9" => "By pass",
		);
my %upsOutputAlarmStatus = (
		"1.3.6.1.2.1.33.1.6.3.1" => "CRITICAL",
		"1.3.6.1.2.1.33.1.6.3.10" => "CRITICAL",
		"1.3.6.1.2.1.33.1.6.3.11" => "WARNING",
		"1.3.6.1.2.1.33.1.6.3.12" => "WARNING",
		"1.3.6.1.2.1.33.1.6.3.13" => "CRITICAL",
		"1.3.6.1.2.1.33.1.6.3.14" => "WARNING",
		"1.3.6.1.2.1.33.1.6.3.15" => "WARNING",
		"1.3.6.1.2.1.33.1.6.3.16" => "CRITICAL",
		"1.3.6.1.2.1.33.1.6.3.17" => "CRITICAL",
		"1.3.6.1.2.1.33.1.6.3.18" => "CRITICAL",
		"1.3.6.1.2.1.33.1.6.3.19" => "WARNING",
		"1.3.6.1.2.1.33.1.6.3.2" => "WARNING",
		"1.3.6.1.2.1.33.1.6.3.20" => "WARNING",
		"1.3.6.1.2.1.33.1.6.3.21" => "WARNING",
		"1.3.6.1.2.1.33.1.6.3.22" => "WARNING",
		"1.3.6.1.2.1.33.1.6.3.23" => "WARNING",
		"1.3.6.1.2.1.33.1.6.3.24" => "WARNING",
		"1.3.6.1.2.1.33.1.6.3.3" => "WARNING",
		"1.3.6.1.2.1.33.1.6.3.4" => "CRITICAL",
		"1.3.6.1.2.1.33.1.6.3.5" => "CRITICAL",
		"1.3.6.1.2.1.33.1.6.3.6" => "CRITICAL",
		"1.3.6.1.2.1.33.1.6.3.7" => "CRITICAL",
		"1.3.6.1.2.1.33.1.6.3.8" => "CRITICAL",
		"1.3.6.1.2.1.33.1.6.3.9" => "CRITICAL",
		);
#OID APC
my $oid_apc_battery_capacity = ".1.3.6.1.4.1.318.1.1.1.2.2.1.0";
my $oid_apc_battery_temperature = ".1.3.6.1.4.1.318.1.1.1.2.2.2.0";
my $oid_apc_battery_replace = ".1.3.6.1.4.1.318.1.1.1.2.2.4.0";

my $oid_apc_upsPhaseNumInputs = '1.3.6.1.4.1.318.1.1.1.9.2.1.0';

my $oid_apc_upsPhaseInputFrequency = '1.3.6.1.4.1.318.1.1.1.9.2.2.1.4';
my $oid_apc_upsPhaseInputVoltage = '1.3.6.1.4.1.318.1.1.1.9.2.3.1.3';
my $oid_apc_upsPhaseInputCurrent = '1.3.6.1.4.1.318.1.1.1.9.2.3.1.6';
my $oid_apc_upsPhaseInputPower   = '1.3.6.1.4.1.318.1.1.1.9.2.3.1.9';

my $oid_apc_upsPhaseNumOutputs = '1.3.6.1.4.1.318.1.1.1.9.3.1.0';

my $oid_apc_upsPhaseOutputFrequency = '1.3.6.1.4.1.318.1.1.1.9.3.2.1.4';
my $oid_apc_upsPhaseOutputVoltage = '1.3.6.1.4.1.318.1.1.1.9.3.3.1.3';
my $oid_apc_upsPhaseOutputCurrent = '1.3.6.1.4.1.318.1.1.1.9.3.3.1.4';
my $oid_apc_upsPhaseOutputPower   = '1.3.6.1.4.1.318.1.1.1.9.3.3.1.13';
my $oid_apc_upsPhaseOutputPercentLoad = '1.3.6.1.4.1.318.1.1.1.9.3.3.1.10';

my $oid_apc_upsAdvOutputActivePower = '1.3.6.1.4.1.318.1.1.1.4.2.8.0';

#Phoenixtec Power Co.,
my $oid_phoenix_load_battery = "1.3.6.1.4.1.935.1.1.1.2.2.1.0";
my $oid_phoenix_temp_battery = "1.3.6.1.4.1.935.1.1.1.2.2.3.0";
my $oid_phoenix_eppc_load_battery = "1.3.6.1.4.1.935.10.1.1.3.4.0";
my $oid_phoenix_eppc_temp_battery = "1.3.6.1.4.1.935.10.1.1.2.2.0";
my $oid_phoenix_upsOutputSource = ".1.3.6.1.4.1.935.1.1.1.4.1.1.0";
my $oid_phoenix_eppc_output_frequency = ".1.3.6.1.4.1.935.10.1.1.2.18.1.2";
my $oid_phoenix_eppc_output_volt = ".1.3.6.1.4.1.935.10.1.1.2.18.1.3";
my $oid_phoenix_eppc_output_current = ".1.3.6.1.4.1.935.10.1.1.2.18.1.4";
my $oid_phoenix_eppc_output_watt = ".1.3.6.1.4.1.935.10.1.1.2.18.1.5";
my $oid_phoenix_eppc_output_load = "1.3.6.1.4.1.935.10.1.1.2.18.1.7";
my %phoenixupsOutputSourcesMessage = (
		"1" => "unknown(1)",
		"2" => "onLine(2)",
		"3" => "onBattery(3)",
		"4" => "onBoost(4)",
		"5" => "sleeping(5)",
		"6" => "onBypass(6)",
		"7" => "rebooting(7)",
		"8" => "standBy(8)",
		"9" => "onBuck(9)",
		);
#Socomec
my $oid_socomec_battery_status = "1.3.6.1.4.1.4555.1.1.1.1.2.1";
my $oid_socomec_battery_time = ".1.3.6.1.4.1.4555.1.1.1.1.2.3";
my $oid_socomec_battery_temperature = ".1.3.6.1.4.1.4555.1.1.1.1.2.6";
my $oid_socomec_battery_charged = ".1.3.6.1.4.1.4555.1.1.1.1.2.4";
my $oid_socomec_alarm_present = "1.3.6.1.4.1.4555.1.1.1.1.6.1.0";
my %ups_socomec_BatteryStatus = (
		1 => "unknown(1)",
		2 => "batteryNormal(2)",
		3 => "batteryLow(3)",
		4 => "batteryDepleted(4)",
		5 => "batteryDischarging(5)",
		6 => "batteryFailure(6)",
		7 => "upsOff(7)"
		);

my $oid_socomec_output_source = "1.3.6.1.4.1.4555.1.1.1.1.4.1.0";
my %oid_socomec_output_source_status = ( "1" => "UNKNOWN",
		"2" => "WARNING",
		"3" => "OK",
		"4" => "CRITICAL",
		"5" => "WARNING",
		"6" => "WARNING",
		"7" => "WARNING",
		"8" => "WARNING",
		"9" => "OK" );
my %oid_socomec_output_source_message = (
		"1" => "unknown(1)",
		"2" => "onInverter(2)",
		"3" => "onMains(3)",
		"4" => "ecoMode(4)",
		"5" => "onBypass(5)",
		"6" => "standby(6)",
		"7" => "onMaintenanceBypass(7)",
		"8" => "upsOff(8)",
		"9" => "normalMode(9)"
		);

my $oid_socomec_input_volt = "1.3.6.1.4.1.4555.1.1.1.1.3.3.1.2";
my $oid_socomec_input_current = "1.3.6.1.4.1.4555.1.1.1.1.3.3.1.3";
my $oid_socomec_input_frequency = "1.3.6.1.4.1.4555.1.1.1.1.3.2.0";

my $oid_socomec_output_voltage = "1.3.6.1.4.1.4555.1.1.1.1.4.4.1.2";
my $oid_socomec_output_current = "1.3.6.1.4.1.4555.1.1.1.1.4.4.1.3";
my $oid_socomec_output_load = "1.3.6.1.4.1.4555.1.1.1.1.4.4.1.4";
my $oid_socomec_output_frequency = "1.3.6.1.4.1.4555.1.1.1.1.4.2.0";


#on récupère les paramètres SNMP et on utilise directe le V1
#push(@ARGV,('-v','1'));
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
my $match;
my $json;
my @jsonOutput;
GetOptions(
		"V"   => \$opt_V,	 "version"    => \$opt_V,
		"h"   => \$opt_h,	 "help"       => \$opt_h,
		"T=s" => \$type,	 "Type=s" => \$type,
		"m=s" => \$match,	 "match=s" => \$match,
		"w=i" => \$warning,	 "warning=i" => \$warning,
		"J" => \$json,	 "json" => \$json,
		"c=i" => \$critical, "critical=i" => \$critical,
	  );


# -h & --help print help
if ($opt_h)
{
#affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-T state');
	my @arr;
	push(@arr,["-T","--type","type of test [state,battery,line] (default: state)" ]);
	push(@arr,["-w","--warning","warning threshold in percent for the line power (default: 80) or warning threshold time remaining in minute for battery test (default: 30 min)" ]);
	push(@arr,["-c","--critical","critical threshold in percent for the line power (default: 90) or critical threshold time remaining in minute for battery test (default: 10 min) "]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
# -V & --version print version
if ($opt_V)
{
	print "$PROGNAME version $PROGVERSION\n";
	exit $ERRORS{'OK'};
}

#on vérifie qu'on a bien un objet SNMP
if (!defined($snmp))
{
	print ("UNKNOWN: SNMP configuration error: $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}

my $state = 'UNKNOWN';
my $output = "";


my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print ("UNKNOWN: Host not SNMP response $snmperror\n");
	exit $ERRORS{'UNKNOWN'};
}
my $upsOutputSource =  ESIA::SNMP::get($snmp,$oid_upsOutputSource);
if($type eq 'state')
{
	my $state = 'OK';
	if ($upsOutputSource !~ /^noSuch/)
	{
		#HP
		if($Sysoid_value==232)
		{
			my $inputstate = ESIA::SNMP::get($snmp,$oid_hp_input_source);
			my $state = $oid_hp_input_sourceStatus{$inputstate};
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
			my $output = "$state : UPS State is ".$oid_hp_input_sourceMessage{$inputstate};
		}
		#SOCOMEC
		elsif( $Sysoid_value==4555)
		{
			my $status = ESIA::SNMP::get($snmp,$oid_socomec_output_source);
			my $state = $oid_socomec_output_source_status{$status};
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
			$output = "$state : UPS State is ".$oid_socomec_output_source_message{$status}."\n";

			my $globalstate = ESIA::SNMP::get($snmp,$oid_socomec_alarm_present);
			my $message = "OK. There isn't global alarm";
			if ($globalstate == 0)
			{ # normal
				$state = 'OK';
			}
			else
			{
				$state = 'CRITICAL';
				$message = "CRITICAL. There are  $upsOutputSource global alarm";
			}
			$output .= "$state : UPS State is ".$message."\n";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		}
		#APC
		elsif($Sysoid_value==318)
		{
			my $apcreplace = ESIA::SNMP::get($snmp,$oid_apc_battery_replace);
			my $apctemperature = ESIA::SNMP::get($snmp,$oid_apc_battery_temperature);
			my $apcmessage="";
			if($apcreplace !=1)
			{
				$state='WARNING';
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
				$apcmessage='Battery must be change';
			}
			else
			{
				$upsOutputSource=3;
				$apcmessage="UPS state is OK (temperature: $apctemperature Celsius).";
			}
			$output = "$state : $apcmessage\n";
		}
		#PHOENIXTEC
		elsif( $Sysoid_value==935)
		{
			my $upsState= ESIA::SNMP::get($snmp,$oid_phoenix_upsOutputSource);
            if($upsState !~ /^noSuch/)
            {
			    if ($upsState == 2)
			    { # normal
				    $state = 'OK';
			    }
			    elsif (($upsState == 4))
			    { #6 ==booster 4
				    $state = 'WARNING';;
			    }
			    else
			    {
				    $state = 'CRITICAL';
			    }
			    $output = "$state : UPS State is ".$phoenixupsOutputSourcesMessage{$upsState}."\n";
            }
            else
            {
			    if ($upsOutputSource == 3)
			    { # normal
				    $state = 'OK';
			    }
			    elsif (($upsOutputSource == 6) || ($upsOutputSource == 7))
			    { #6 ==booster 7 ==reducer
				    $state = 'WARNING';
			    }
			    else
			    {
				    $state = 'CRITICAL';
			    }
			    $output = "$state : UPS State is ".$upsOutputSourcesMessage{$upsOutputSource}."\n";
            }
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		}
		else
		{
			if ($upsOutputSource == 3)
			{ # normal
				$state = 'OK';
			}
			elsif (($upsOutputSource == 6) || ($upsOutputSource == 7))
			{ #6 ==booster 7 ==reducer
				$state = 'WARNING';
			}
			else
			{oid_socomec_output_frequency
				$state = 'CRITICAL';
			}
			$output = "$state : UPS State is ".$upsOutputSourcesMessage{$upsOutputSource}."\n";
			$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		}
	}
}
elsif($type eq 'battery')
{
	my $state = 'OK';
	my $message="";
	if($Sysoid_value==318)
	{
		my $apccapacity = ESIA::SNMP::get($snmp,$oid_apc_battery_capacity);
		my $apctemperature = ESIA::SNMP::get($snmp,$oid_apc_battery_temperature);
		if($apccapacity < 10)
		{
			$state = 'CRITICAL';
		}
		elsif($apccapacity < 20)
		{
			$state = 'WARNING';
		}
		$output  = "$state : Battery load: $apccapacity%\nOK : battery temperature is $apctemperature Celsius\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		$state = 'OK';
	}
	elsif($Sysoid_value==935)
	{
		$state = 'OK';
		my $message = "> $warning";
        my $sysobject =  ESIA::SNMP::get($snmp,"$oid_sysObjectOID");
        my $upsBatteryload=undef;
		my $upsBatterytemp =  ESIA::SNMP::get($snmp,$oid_phoenix_temp_battery);
        if($sysobject =~ /.4.1.935.10.1/)
        {
		    $upsBatteryload =  ESIA::SNMP::get($snmp,"$oid_phoenix_eppc_load_battery");
		    $upsBatterytemp =  ESIA::SNMP::get($snmp,$oid_phoenix_eppc_temp_battery);
        }
        else
        {
		    $upsBatteryload =  ESIA::SNMP::get($snmp,"$oid_phoenix_load_battery");
		    $upsBatterytemp =  ESIA::SNMP::get($snmp,$oid_phoenix_temp_battery);
        }
		$upsBatterytemp = $upsBatterytemp / 10;
		if($upsBatteryload < $critical)
		{
			$state = 'CRITICAL';
			$message = "< $critical";
		}
		elsif($upsBatteryload < $warning)
		{
			$state = 'WARNING';
			$message = "< $warning";
		}
		$output .= "$state : battery load percentage ".$upsBatteryload." % ($message%)\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
		$state = 'OK';
		$message = "OK";
		if($upsBatterytemp > 40)
		{
			$state = 'CRITICAL';
			$message = "very high";
		}
		elsif($upsBatterytemp > 35)
		{
			$state = 'WARNING';
			$message = "high";
		}
		$output .= "$state : battery temperature is $message (".$upsBatterytemp."°C)\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});

	}
	#SOCOMEC
	elsif( $Sysoid_value==4555)
	{

		$oid_upsBatteryStatus = $oid_socomec_battery_status,
		$oid_upsEstimatedMinutesRemaining= $oid_socomec_battery_time;
		%upsBatteryStatuses = %ups_socomec_BatteryStatus;
		my $capacity = ESIA::SNMP::get($snmp,$oid_socomec_battery_charged);
		my $temperature = ESIA::SNMP::get($snmp,$oid_socomec_battery_temperature);
		if($capacity < 10)
		{
			$state = 'CRITICAL';
		}
		elsif($capacity < 20)
		{
			$state = 'WARNING';
		}
		$message = "$state : Battery load: $capacity%\n";
		if($temperature ne '')
		{
			$message .= "OK : battery temperature is $temperature Celsius\n";
		}
	}
	my $upsBatteryStatus =  ESIA::SNMP::get($snmp,$oid_upsBatteryStatus);
    if($upsBatteryStatus ne 'noSuchObject')
    {
	    if($upsBatteryStatus != 2 )
	    {
		    $state = 'WARNING';
		    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	    }
	    $output .= "$state : Battery state is ".$upsBatteryStatuses{$upsBatteryStatus}."\n$message";
    }
	$state = 'OK';
	my $upsEstimatedMinutesRemaining =  ESIA::SNMP::get($snmp,$oid_upsEstimatedMinutesRemaining);
	if($Sysoid_value==935 && $upsEstimatedMinutesRemaining != 0)
	{
		if($warning==80)
		{
			$warning=30;
		}
		if($critical==90)
		{
			$critical=10;
		}
		my $message = "< $warning";
		if($upsEstimatedMinutesRemaining < $critical)
		{
			$state = 'CRITICAL';
		}
		elsif($upsEstimatedMinutesRemaining < $warning)
		{
			$state = 'WARNING';
		}
		$output .= "$state : Time remaining is ".$upsEstimatedMinutesRemaining." min\n";
		$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
	}
}
elsif($type eq 'input')
{
    my $mode_apc_phase = 0;
    if( $Sysoid_value == 318 )
    {
        my $phaseNumInputs = ESIA::SNMP::get($snmp,"$oid_apc_upsPhaseNumInputs");
	    if( defined ($phaseNumInputs) )
	    {
            $mode_apc_phase = 1;
	    }
    }
	#SOCOMEC
	elsif( $Sysoid_value==4555)
	{
		$mode_apc_phase = -1;
		my $inputFrequency= ESIA::SNMP::get($snmp,"$oid_socomec_input_frequency");
		$inputFrequency /= 10;
		my $current = $snmp->get_table( $oid_socomec_input_current);
        my $result = $snmp->get_table( $oid_socomec_input_volt);
        foreach my $key ( sort keys %$result)
        {
            if($key=~/\.([0-9]*)$/)
		    {
                my $id = $1;
                if( !defined($match) ||$id == $match)
                {
                    my $inputVoltage = $$result{"$key"};
                	my $inputCurrent=  $$current{"$oid_socomec_input_current.$id"};
                    $inputVoltage /= 10;
                    $inputCurrent /= 10;
					my $inputPower = $inputVoltage * $inputCurrent;
                    if( $inputPower eq -1 )
				    {
					    $inputPower = 'N/A';
				    }
					my $linestatus = "OK";
				    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$linestatus});

				    if(!$json)
				    {
					    $output .= "$linestatus:  Line $id: $inputFrequency Hz, $inputCurrent Amperes, ".$inputVoltage." Volt, ".$inputPower." VA "."\n";
				    }
				    else
				    {
					    my %hash = ('id' => $id, 'state' => $linestatus, 'current'=>$inputCurrent,'volt' => $inputVoltage, 'va' => $inputPower, 'frequency'=>$inputFrequency);
					    push(@jsonOutput, \%hash);
				    }
                }
            }
       }
	}
    if( $mode_apc_phase eq 1 )
    {
        my $result = $snmp->get_table( $oid_apc_upsPhaseInputVoltage);
        foreach my $key ( sort keys %$result)
        {
            if($key=~/\.([0-9]*)\.1\.([0-9]*)$/)
		    {
                my $line = $1;
                my $phase = $2;
                my $id = "$line.$phase";
                if( !defined($match) ||$id == $match)
                {
                    my $inputVoltage = $$result{"$key"};
                    my $inputFrequency = ESIA::SNMP::get($snmp,"$oid_apc_upsPhaseInputFrequency.$line");
   		            if( $inputFrequency eq -1 )
			        {
				            next;
			        }

                    my $inputCurrent = ESIA::SNMP::get($snmp,"$oid_apc_upsPhaseInputCurrent.$line.1.$phase");
                    my $inputPower = ESIA::SNMP::get($snmp,"$oid_apc_upsPhaseInputPower.$line.1.$phase");

                    $inputFrequency /= 10;
                    $inputCurrent /= 10;

                    if( $inputPower eq -1 )
				    {
					    $inputPower = 'N/A';
				    }

                    my $linestatus = "OK";
				    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$linestatus});

				    if(!$json)
				    {
					    $output .= "$linestatus:  Line $id: $inputFrequency Hz, $inputCurrent Amperes, ".$inputVoltage." Volt, ".$inputPower." VA "."\n";
				    }
				    else
				    {
					    my %hash = ('id' => $id, 'state' => $linestatus, 'current'=>$inputCurrent,'volt' => $inputVoltage, 'va' => $inputPower, 'frequency'=>$inputFrequency);
					    push(@jsonOutput, \%hash);
				    }
                }
            }
        }
    }
    elsif( $mode_apc_phase eq 0 )
    {
	    my $result = $snmp->get_table( $oid_upsInputFrequency);
	    #Parfois l'inputFrequency n'est pas disponible, on utilise alors une autre table
	    if( !defined($result) )
        {
            $result = $snmp->get_table( $oid_upsInputCurrent);
        }
	    
	    foreach my $key ( sort keys %$result)
	    {
		    if($key=~/\.([0-9]*)$/)
		    {
			    my $id = $1;
			    if( !defined($match) ||$id == $match)
			    {
				    my $inputFrequency = $$result{"$key"};
				    $inputFrequency /= 10;

				    my $inputCurrent = ESIA::SNMP::get($snmp,"$oid_upsInputCurrent.$id");
				    if ( $inputCurrent eq 0 && $Sysoid_value eq 318 )
				    {
					    $inputCurrent = ESIA::SNMP::get($snmp,"$oid_upsByPassCurrent.$id");
				    }
				    $inputCurrent /= 10;


				    my $inputVoltage = ESIA::SNMP::get($snmp,"$oid_upsInputVoltage.$id");
				    my $inputPower = ESIA::SNMP::get($snmp,"$oid_upsInputTruePower.$id");
				    if( $inputPower eq 0 && $inputCurrent ne 0 )
				    {
					    $inputPower = 'N/A';
				    }

				    my $linestatus = "OK";
				    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$linestatus});

				    if(!$json)
				    {
					 	## Line vide
           		       	if( ($inputPower ne 0 && $inputCurrent ne 0) || $inputVoltage ne 0 )
                        {
					    	$output .= "$linestatus:  Line $id: $inputFrequency Hz, $inputCurrent Amperes, ".$inputVoltage." Volt, ".$inputPower." VA "."\n";
                        }
				    }
				    else
				    {
					    my %hash = ('id' => $id, 'state' => $linestatus, 'current'=>$inputCurrent,'volt' => $inputVoltage, 'va' => $inputPower, 'frequency'=>$inputFrequency);
					    push(@jsonOutput, \%hash);
				    }
			    }
		    }
	    }
    }
    if($json)
    {
	    $output = encode_json(\@jsonOutput);
    }
}
elsif($type eq 'output' || $type eq 'line')
{
    my $mode_apc_phase = 0;
    if( $Sysoid_value == 318 )
    {
        my $phaseNumOutputs = ESIA::SNMP::get($snmp,"$oid_apc_upsPhaseNumOutputs");
	    if( defined ($phaseNumOutputs) )
	    {
            $mode_apc_phase = 1;
	    }
    }
    #PHOENIX
    elsif($Sysoid_value==935)
	{
		$mode_apc_phase = -1;
		$state = 'OK';
		my $message = "> $warning";
        my $sysobject =  ESIA::SNMP::get($snmp,"$oid_sysObjectOID");
        if($sysobject =~ /.4.1.935.10.1/)
        {
            my $result = $snmp->get_table( $oid_phoenix_eppc_output_volt);
            foreach my $key ( sort keys %$result)
            {
                if($key=~/\.([0-9]*)$/)
		        {
                    my $id = $1;
                    if( !defined($match) ||$id == $match)
                    {
                        my $outputVoltage = $$result{"$key"};
                    	my $outputCurrent=  ESIA::SNMP::get($snmp,"$oid_phoenix_eppc_output_current.$id");
                        $outputVoltage /= 10;
                        $outputCurrent /= 10;
					    my $outputPower = ESIA::SNMP::get($snmp,"$oid_phoenix_eppc_output_watt.$id");
                        my $percentLoad = ESIA::SNMP::get($snmp,"$oid_phoenix_eppc_output_load.$id");
                        ## Calcul du status
			            my $linestatus = "OK";
			            my $loadmessage="";
			            if($percentLoad > $critical)
			            {
				            $loadmessage=$percentLoad."% load > $critical";
				            $linestatus='CRITICAL';
			            }
			            elsif($percentLoad > $warning)
			            {
				            $loadmessage=$percentLoad."% load > $warning";
				            $linestatus='WARNING';
			            }
			            else
			            {
				            $loadmessage=$percentLoad."% load < $warning";
			            }
			            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$linestatus});
                        my $outputFrequency = ESIA::SNMP::get($snmp,"$oid_phoenix_eppc_output_frequency.$id");
		                $outputFrequency /=10;

			            if(!$json)
			            {
				            $output .= "$linestatus: Line $id: $outputFrequency Hz, $outputCurrent Amperes, $outputVoltage Volt, $outputPower Watt, $loadmessage\n";
			            }
			            else
			            {
				            my %hash = ('id' => $id, 'state' => $linestatus, 'current'=>$outputCurrent, 'volt' => $outputVoltage, 'watt' => $outputPower, 'frequency'=>$outputFrequency);
				            push(@jsonOutput, \%hash);
			            }
                    }
                }
           }
        }
        else
        {
            $mode_apc_phase=0;
        }
    }
	#SOCOMEC
	elsif($Sysoid_value==4555)
	{
		$mode_apc_phase = -1;
		my $outputFrequency = ESIA::SNMP::get($snmp,"$oid_socomec_output_frequency");
		$outputFrequency /=10;
        my $load = $snmp->get_table( $oid_socomec_output_load);
        my $current = $snmp->get_table( $oid_socomec_output_current);
        my $result = $snmp->get_table( $oid_socomec_output_voltage);
        foreach my $key ( sort keys %$result)
        {
            if($key=~/\.([0-9]*)$/)
		    {
                my $id = $1;
                if( !defined($match) ||$id == $match)
                {
                    my $outputVoltage = $$result{"$key"};
                	my $outputCurrent=  $$current{"$oid_socomec_output_current.$id"};
                    $outputVoltage /= 10;
                    $outputCurrent /= 10;
					my $outputPower = $outputVoltage * $outputCurrent;
                    if( $outputPower eq -1 )
				    {
					    $outputPower = 'N/A';
				    }
                    my $percentLoad = $$load{"$oid_socomec_output_load.$id"};
                    ## Calcul du status
			        my $linestatus = "OK";
			        my $loadmessage="";
			        if($percentLoad > $critical)
			        {
				        $loadmessage=$percentLoad."% load > $critical";
				        $linestatus='CRITICAL';
			        }
			        elsif($percentLoad > $warning)
			        {
				        $loadmessage=$percentLoad."% load > $warning";
				        $linestatus='WARNING';
			        }
			        else
			        {
				        $loadmessage=$percentLoad."% load < $warning";
			        }
			        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$linestatus});


			        if(!$json)
			        {
				        $output .= "$linestatus: Line $id: $outputFrequency Hz, $outputCurrent Amperes, $outputVoltage Volt, $outputPower Watt, $loadmessage\n";
			        }
			        else
			        {
				        my %hash = ('id' => $id, 'state' => $linestatus, 'current'=>$outputCurrent, 'volt' => $outputVoltage, 'watt' => $outputPower, 'frequency'=>$outputFrequency);
				        push(@jsonOutput, \%hash);
			        }
                }
            }
       }
	}
    if( $mode_apc_phase eq 1 )
    {
        my $result = $snmp->get_table( $oid_apc_upsPhaseOutputVoltage);
        foreach my $key ( sort keys %$result)
        {
            if($key=~/\.([0-9]*)\.1\.([0-9]*)$/)
		    {
                my $line = $1;
                my $phase = $2;
                my $id = "$line.$phase";

                if( !defined($match) ||$id == $match)
                {
                    my $outputVoltage = $$result{"$key"};
                    my $outputFrequency = ESIA::SNMP::get($snmp,"$oid_apc_upsPhaseOutputFrequency.$line");
   		            if( $outputFrequency eq -1 && !$json)
			        {
				            next;
			        }

                    my $outputCurrent = ESIA::SNMP::get($snmp,"$oid_apc_upsPhaseOutputCurrent.$line.1.$phase");
                    my $outputPower = ESIA::SNMP::get($snmp,"$oid_apc_upsPhaseOutputPower.$line.1.$phase");

                    $outputFrequency /= 10;
                    $outputCurrent /= 10;

                    if( $outputPower eq -1 )
				    {
					    $outputPower = 'N/A';
				    }

                    my $percentLoad = ESIA::SNMP::get($snmp,"$oid_apc_upsPhaseOutputPercentLoad.$line.1.$phase");
                    ## Calcul du status
			        my $linestatus = "OK";
			        my $loadmessage="";
			        if($percentLoad > $critical)
			        {
				        $loadmessage=$percentLoad."% load > $critical";
				        $linestatus='CRITICAL';
			        }
			        elsif($percentLoad > $warning)
			        {
				        $loadmessage=$percentLoad."% load > $warning";
				        $linestatus='WARNING';
			        }
			        else
			        {
				        $loadmessage=$percentLoad."% load < $warning";
			        }
			        $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$linestatus});


			        if(!$json)
			        {
				        $output .= "$linestatus: Line $id: $outputFrequency Hz, $outputCurrent Amperes, $outputVoltage Volt, $outputPower Watt, $loadmessage\n";
			        }
			        else
			        {
				        my %hash = ('id' => $id, 'state' => $linestatus, 'current'=>$outputCurrent, 'volt' => $outputVoltage, 'watt' => $outputPower, 'frequency'=>$outputFrequency);
				        push(@jsonOutput, \%hash);
			        }
                }
            }
        }
    }
    elsif( $mode_apc_phase eq 0 )
    {
	    # Récupération des données
	    my $outputFreq = ESIA::SNMP::get($snmp,"$oid_upsOutputFrequency");
	    $outputFreq /= 10;
	    my $calculated_watt = 0;
	    my $nb_lines = 0;
	    my @lines_arr;
	    my $i=0;
		#my $result = $snmp->get_table($oid_upsOutputPercentLoad);
		my $result = $snmp->get_table($oid_upsOutputPercentLoad);
	    foreach my $key ( keys %$result)
	    {
		    if($key=~/\.([0-9]*)$/)
		    {
			    my $id = $1;
			    ## Récupération des données
			    my $percentLoad = $$result{"$key"};
			    my $outputPower = ESIA::SNMP::get($snmp,"$oid_upsOutputPower.$id");
			    my $outputCurrent = ESIA::SNMP::get($snmp,"$oid_upsOutputCurrent.$id");
			    my $outputVoltage = ESIA::SNMP::get($snmp,"$oid_upsOutputVoltage.$id");
			    ## Line vide
			    if( $percentLoad eq 0 && $outputPower eq 0 && $outputCurrent eq 0 && $outputVoltage eq 0 )
			    {
			    	if(!$json)
			    	{
				    	next;
				    }
			    }

			    ## Calcul des données
			    $outputCurrent /= 10;
			    $calculated_watt += ($outputCurrent * $outputVoltage );
			    $nb_lines++;

			    ## Enregistrement des données
			    my %hash = (id => $id, current=>$outputCurrent, volt => $outputVoltage, watt => $outputPower,load=>$percentLoad, frequency=>$outputFreq);
			    push(@lines_arr, \%hash);
		    }
	    }

	    my $totalPower = 0;

	    # Traitement des donnée
	    foreach my $hash (@lines_arr)
	    {
		    my $id = $$hash{id};
		    if( !defined($match) ||$id == $match)
		    {
			    my $outputFreq = $$hash{frequency};
			    my $outputCurrent = $$hash{current};
			    my $outputVoltage = $$hash{volt};
			    my $outputPower = $$hash{watt};
			    my $percentLoad = $$hash{load};

			    if( $outputPower eq 0 && $outputCurrent ne 0 && $Sysoid_value eq 318 )
			    {
				    my $ratio = 1;
				    if ( $totalPower == 0 )
				    {
					    $totalPower = ESIA::SNMP::get($snmp,$oid_apc_upsAdvOutputActivePower);
					    $ratio = $calculated_watt / $totalPower;
				    }
				    $outputPower = int($outputCurrent * $outputVoltage * $ratio);
			    }

			    ## Calcul du status
			    my $linestatus = "OK";
			    my $loadmessage="";
			    if($percentLoad > $critical)
			    {
				    $loadmessage=$percentLoad."% load > $critical";
				    $linestatus='CRITICAL';
			    }
			    elsif($percentLoad > $warning)
			    {
				    $loadmessage=$percentLoad."% load > $warning";
				    $linestatus='WARNING';
			    }
			    else
			    {
				    $loadmessage=$percentLoad."% load < $warning";
			    }
			    $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$linestatus});

			    ## Formattage pour l'affichage
			    if(!$json)
			    {
				    $output .= "$linestatus: Line $id: $outputFreq Hz, $outputCurrent Amperes, $outputVoltage Volt, $outputPower Watt, $loadmessage\n";
			    }
			    else
			    {
				    my %hash = ('id' => $id, 'state' => $linestatus, 'current'=>$outputCurrent, 'volt' => $outputVoltage, 'watt' => $outputPower, 'frequency'=>$outputFreq);
				    push(@jsonOutput, \%hash);
			    }
		    }
	    }
    }
	if($json)
	{
		$output = encode_json(\@jsonOutput);
	}
}
elsif($type eq 'alarm')
{
	my $result = $snmp->get_table(Baseoid => $oid_upsAlarm);
	foreach my $key ( keys %$result)
	{
		if($key=~/3\.6\.1\.2\.1\.33\.1\.6\.2\.1\.2/)
		{
			my $descr =$$result{"$key"};
			if($descr=~/1.3.6.1.2.1.33.1.6/)
			{
				my $state = $upsOutputAlarmStatus{$descr};
				$output .= "$state : alarm message: ".$upsOutputAlarmMessage{$descr}."\n";
				$return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
			}
		}
	}
	if($output eq "")
	{
		$output = "OK: No alarm\n";
	}
}
if($output eq "")
{
	print "UPS status retrieval failed.";
	exit $ERRORS{UNKNOWN};
}
else
{
	print $output;
	exit $return;
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
