#!/usr/bin/perl -w 
############################## check_snmp_watchguard_active_connections.pl ########
my $Version='1.0';
# Date : Jun 19 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_watchguard_active_connections.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 400;
my $critical = 500;

my $oid_sysDescr = "1.3.6.1.2.1.1.1";
my $oid_connections = "1.3.6.1.4.1.3097.6.3.80.0";

sub print_help() {
	printf "Watchguard : Total number of currently active connections of the system.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -w (--warning) 		warning value (DEFAULT : 400)\n";
	print " -c (--critical) 	critical value (DEFAULT : 500)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 
#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,		"version"    => \$opt_V,
	"h"   => \$opt_h,		"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"w=s" => \$warning,		"warning=s" => \$warning,
	"c=s" => \$critical,	"critical=s" => \$critical,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysDescr");

#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysDescr");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $state = "OK";
my $snmpresult = $snmp->get_request("$oid_connections");
my $connections =  $$snmpresult{"$oid_connections"};

if($connections >= $critical)
{
	$return = priority_state($return,$ERRORS{CRITICAL});
	$state = "CRITICAL : $connections active connections (>= $critical)\n";
}
elsif($connections >= $warning)
{
	$return = priority_state($return,$ERRORS{WARNING});
	$state = "WARNING : $connections active connections (>= $warning)\n";
}
else
{
	$return = priority_state($return,$ERRORS{OK});
	$state = "OK : $connections active connections (< $warning)\n";
}

print "$state";

exit($return);