#!/usr/bin/perl -w 
############################## check_snmp_watchguard_load.pl ########
my $Version='1.0';
# Date : Jun 19 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_watchguard_load.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 80;
my $critical = 90;
my $type = "cpu";
my $oid_sysDescr = "1.3.6.1.2.1.1.1";
my $oid_cpu = "1.3.6.1.4.1.3097.6.3.77.0";
my $oid_cluster_enabled = "1.3.6.1.4.1.3097.6.6.1.0";
my $oid_cluster_first_member_id = "1.3.6.1.4.1.3097.6.6.2.0";
my $oid_cluster_first_member_state = "1.3.6.1.4.1.3097.6.6.3.0";
my $oid_cluster_second_member_id = "1.3.6.1.4.1.3097.6.6.8.0";
my $oid_cluster_second_member_state = "1.3.6.1.4.1.3097.6.6.9.0";
my %cluster_text  = ( "0"  => "disabled","1" => "worker","2"  => "backup","3"  => "master","4"  => "idle","5"  => "standby");



sub print_help() {
	printf "Watchguard : CPU utilization of the system in last 1 minute.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -w (--warning) 		warning value (DEFAULT : 80)\n";
	print " -c (--critical) 	critical value (DEFAULT : 90)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 
#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,		"version"    => \$opt_V,
	"h"   => \$opt_h,		"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"T=s" => \$type,	"Type=s" => \$type,
	"w=s" => \$warning,		"warning=s" => \$warning,
	"c=s" => \$critical,	"critical=s" => \$critical,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysDescr");

#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysDescr");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}
my $state = "OK";
if($type eq 'cpu')
{
    my $snmpresult = $snmp->get_request("$oid_cpu");
    my $load =  $$snmpresult{"$oid_cpu"};
    $load = $load/100;

    if($load >= $critical)
    {
	    $return = priority_state($return,$ERRORS{CRITICAL});
	    $state = "CRITICAL : CPU : $load% (>= $critical)\n";
    }
    elsif($load >= $warning)
    {
	    $return = priority_state($return,$ERRORS{WARNING});
	    $state = "WARNING : CPU $load% (>= $warning)\n";
    }
    else
    {
	    $return = priority_state($return,$ERRORS{OK});
	    $state = "OK : CPU $load% (< $warning)\n";
    }

    print "$state";
}
else
{
    my $snmpresult = $snmp->get_request("$oid_cluster_enabled");
    my $cls_enabled =  $$snmpresult{"$oid_cluster_enabled"};
    if($cls_enabled == 1)
    {
        $snmpresult = $snmp->get_request("$oid_cluster_first_member_id");
        my $cls_id_1 =  $$snmpresult{"$oid_cluster_first_member_id"};
        $snmpresult = $snmp->get_request("$oid_cluster_second_member_id");
        my $cls_id_2 =  $$snmpresult{"$oid_cluster_second_member_id"};
        $snmpresult = $snmp->get_request("$oid_cluster_first_member_state");
        my $cls_state_1 =  $$snmpresult{"$oid_cluster_first_member_state"};
        $snmpresult = $snmp->get_request("$oid_cluster_second_member_state");
        my $cls_state_2 =  $$snmpresult{"$oid_cluster_second_member_state"};
        if($cls_state_1 == 3 || $cls_state_2 == 3)
        {
            print "OK: There is one master.\n";
            $return = priority_state($return,$ERRORS{OK});
        }
        else
        {
            print "CRITICAL: There isn't master.\n";
            $return = priority_state($return,$ERRORS{CRITICAL});
        }
        if($cls_state_1 == 2 || $cls_state_1 == 3)
        {
            $return = priority_state($return,$ERRORS{OK});
            print "OK: Node $cls_id_1 state is ".$cluster_text{$cls_state_1}."\n";            
        }
        else
        {
            $return = priority_state($return,$ERRORS{WARNING});
            print "WARNING: Node $cls_id_1 state is ".$cluster_text{$cls_state_1}."\n";            
        }
        if($cls_state_2 == 2 || $cls_state_2 == 3)
        {
            $return = priority_state($return,$ERRORS{OK});
            print "OK: Node $cls_id_2 state is ".$cluster_text{$cls_state_2}."\n";            
        }
        else
        {
            $return = priority_state($return,$ERRORS{WARNING});
            print "WARNING: Node $cls_id_2 state is ".$cluster_text{$cls_state_2}."\n";            
        }
    }
    else
    {
        $return = priority_state($return,$ERRORS{UNKNOWN});
        print "UNKNOWN: Cluster not configured\nCheck your firewall or delete this service";
    }
}
exit($return);
