#!/usr/bin/perl -w
############################## check_snmp_xclarity #################
my $Version='1.0';
# Date : Oct 18 2018
# Author  : Julien George (j.george@esia-sa.com)
# Help : http://wiki.esia-sa.com
#################################################################
#
# Help : ./check_snmp_xclarity.pl -h
#

use strict;
use Switch;
use Net::SNMP;
use Getopt::Long;

# Nagios specific

my $TIMEOUT = 50;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);

# SNMP Datas
## For CPU health
my $cpu_base_oid = "1.3.6.1.4.1.19046.11.1.1.5.20.1";
my $cpu_id 			= "1";	#1
my $cpu_desc 		= "2";	#"CPU 1"
#my $cpu_speed 		= "3";	#2100
#my $cpu_identifier = "4";	#"3030 3431 3635 3330 "
#my $cpu_type 		= "5";	#"CENTRAL"
#my $cpu_family 	= "6";	#"Intel Xeon"
my $cpu_cores 		= "7";	#8
my $cpu_threads 	= "8";	#16
#my $cpu_voltage 	= "9";	#1250
my $cpu_datawidth 	= "10";	#64
my $cpu_healthstatus= "11";	#"Normal"
my $cpu_model 		= "12";	#"Intel(R) Xeon(R) Silver 4110 CPU @ 2.10GHz"

## For diskHealth
my $disk_base_oid 		= "1.3.6.1.4.1.19046.11.1.1.12.2.1";
my $disk_id				= "1";	#0
my $disk_Name			= "2";	#"Drive 0"
my $disk_healthstatus	= "3";	#"Normal"

## For fanHealth
my $fan_base_oid 		= "1.3.6.1.4.1.19046.11.1.1.3.2.1";
my $fan_id				= "1";	#1
my $fan_Desc			= "2";	#"Fan 1 Tach"
my $fan_speed			= "3";	#" 26% of maximum" or "offline"
my $fan_healthstatus	= "10";	#"Normal" or "Unknown"

## For memoryHealth
my $mem_base_oid = "1.3.6.1.4.1.19046.11.1.1.5.21.1";
my $mem_id 				= "1";	#1
my $mem_desc 			= "2";	#"DIMM 1"
my $mem_partnum 		= "3";	#"HMA82GR7AFR8N-VK    "
my $mem_FRU_SN 			= "4";	#"423EE69C"
my $mem_manufacturerdate= "5";	#"423EE69C"
my $mem_type 			= "6";	#"DDR4"
my $mem_size 			= "7";	#16
my $mem_healthstatus 	= "8";	#"Normal"
my $mem_configspeed 	= "9";	#2400
my $mem_ratedspeed 		= "10";	#2666
my $mem_lenovo_partnum	= "11";	#"01DE973"

## For PowerHealth
my $pwr_base_oid 		= "1.3.6.1.4.1.19046.11.1.1.11.2.1";
my $pwr_id				= "1";	#1
my $pwr_FRU_name		= "2";	#"Power Supply 1"
my $pwr_partnum			= "3";	#"SP57A03099"
my $pwr_FRU_num			= "4";	#"01PE161"
my $pwr_FRU_SN			= "5";	#"P4LD84H00VL"
my $pwr_healthstatus	= "6";	#"Normal"

## For Temperatures
my $temp_base_oid = "1.3.6.1.4.1.19046.11.1.1.1.2.1";
my $temp_id 				= "1";	#1
my $temp_desc 				= "2";	#"CPU1 Temp" or "CPU1 DTS"
my $temp_reading 			= "3";	#"27" or "-51.0"
my $temp_nominalreading		= "4";	#"22" or "-27.0"
my $temp_nonRecovLimitHigh	= "5";	#"N/A" or "-0.2"
my $temp_CritLimitHigh		= "6";	#"N/A" or "-0.6"
my $temp_NonCritLimitHigh	= "7";	#"43" or "N/A"
#my $temp_NonRecovLimitLow 	= "8";	#"N/A"
#my $temp_CritLimitLow 		= "9";	#"N/A"
#my $temp_NonCritLimitLow	= "10";	#"N/A"
my $temp_healthstatus		= "11";	#"Normal"

## For Voltages
my $volt_base_oid = "1.3.6.1.4.1.19046.11.1.1.2.2.1";
my $volt_id 				= "1";	#1
my $volt_desc 				= "2";	#SysBrd 12V"
my $volt_reading 			= "3";	#"12.208"
my $volt_nominalreading		= "4";	#13.384"
#my $volt_nonRecovLimitHigh	= "5";	#"N/A"
my $volt_CritLimitHigh		= "6";	#"13.216"
my $volt_NonCritLimitHigh	= "7";	#"N/A"
#my $volt_NonRecovLimitLow 	= "8";	#"N/A"
my $volt_CritLimitLow 		= "9";	#"2.3920"
my $volt_NonCritLimitLow	= "10";	#"N/A"
my $volt_healthstatus		= "11";	#"Normal"

# Globals
my $o_host = 	undef; 		# hostname
my $o_community = undef; 	# community
my $o_port = 	161; 		# port
my $o_help=	undef; 		# wan't some help ?
my $o_verb=	undef;		# verbose mode
my $o_version=	undef;		# print version
# check type  : stand | netsc |  netsl | as400 | cisco | cata | ciscoASR | ciscoSB | nsc | fg | bc | nokia | hp | lp  | hpux
my $o_check_type= "stand";
# End compatibility
my $o_warn=	undef;		# warning level
my @o_warnL=	undef;		# warning levels for Linux Load or Cisco CPU
my $o_crit=	undef;		# critical level
my @o_critL=	undef;		# critical level for Linux Load or Cisco CPU
my $o_timeout=  undef; 		# Timeout (Default 5)
my $o_perf=     undef;          # Output performance data
my $o_version2= undef;          # use snmp v2c
# SNMPv3 specific
my $o_login=	undef;		# Login for snmpv3
my $o_passwd=	undef;		# Pass for snmpv3
my $v3protocols=undef;	# V3 protocol list.
my $o_authproto='md5';		# Auth protocol
my $o_privproto='des';		# Priv protocol
my $o_privpass= undef;		# priv password
my $o_ignore= undef;

# functions

sub p_version { print "check_snmp_xclarity : $Version\n"; }

sub print_usage {
    print "Usage: $0 [-v] -H <host> -C <snmp_community> -T [cpu|mem|disk|power|fan|temp|volt] [-2] | (-l login -x passwd [-X pass -L <authp>,<privp>])  [-p <port>] [-i] [-w treshold] [-c treshold] [-t <timeout>] [-V]\n";
}

sub isnnum { # Return true if arg is not a number
  my $num = shift;
  if ( $num =~ /^(\d+\.?\d*)|(^\.\d+)$/ ) { return 0 ;}
  return 1;
}

sub readOID {
        my($snmp,$oid) = @_;
	my $result = $snmp->get_request("$oid");
	return $$result{"$oid"};
}

sub help {
   print "\nSNMP xClarity Monitor\n";
   print_usage();
   print <<EOT;
-v, --verbose
   print extra debugging information
-h, --help
   print this help message
-H, --hostname=HOST
   name or IP address of host to check
-C, --community=COMMUNITY NAME
   community name for the host's SNMP agent (implies v1 protocol)
-2, --v2c
   Use snmp v2c
-l, --login=LOGIN ; -x, --passwd=PASSWD
   Login and auth password for snmpv3 authentication
   If no priv password exists, implies AuthNoPriv
-X, --privpass=PASSWD
   Priv password for snmpv3 (AuthPriv protocol)
-L, --protocols=<authproto>,<privproto>
   <authproto> : Authentication protocol (md5|sha : default md5)
   <privproto> : Priv protocole (des|aes : default des)
-P, --port=PORT
   SNMP port (Default 161)
-T, --type=cpu|mem|disk|power|fan|temp|volt
		cpu   : check CPU status health
		mem   : check Memory DIMM status health
		disk  : check disk status health
		power : check Power status health
		fan   : check fan status health and speed
		temp  : check temp sensors health and values
		volt  : check voltage sensors health and values
-i, --ignore
	to ignore unknown components
-w, --warning=INTEGER[,INTEGER]
	max warning treshold (for type : fan|temp|volt) or min,max warning treshold (type volt only)
-c, --critical=INTEGER[,INTEGER]
	max critical treshold (for type : fan|temp|volt) or min,max critical treshold (type volt only)
-t, --timeout=INTEGER
   timeout for SNMP in seconds (Default: 5)
-V, --version
   prints version number
EOT
}

# For verbose output
sub verb { my $t=shift; print $t,"\n" if defined($o_verb) ; }
sub check_options {
    Getopt::Long::Configure ("bundling");
    GetOptions(
		'v'		=> \$o_verb,		'verbose'		=> \$o_verb,
        'h'     => \$o_help,    	'help'        	=> \$o_help,
        'H:s'   => \$o_host,		'hostname:s'	=> \$o_host,
        'p:i'   => \$o_port,   		'port:i'		=> \$o_port,
        'C:s'   => \$o_community,	'community:s'	=> \$o_community,
		'l:s'	=> \$o_login,		'login:s'		=> \$o_login,
		'x:s'	=> \$o_passwd,		'passwd:s'		=> \$o_passwd,
		'X:s'	=> \$o_privpass,	'privpass:s'	=> \$o_privpass,
		'L:s'	=> \$v3protocols,	'protocols:s'	=> \$v3protocols,
        't:i'   => \$o_timeout,     'timeout:i'     => \$o_timeout,
		'i:s'   => \$o_ignore,      'ignore:s'     	=> \$o_ignore,
		'V'		=> \$o_version,		'version'		=> \$o_version,
		'2'     => \$o_version2,    'v2c'           => \$o_version2,
		'T:s'	=> \$o_check_type,	'type:s'		=> \$o_check_type
	);

    # Basic checks
	if (defined($o_ignore)) {
		$o_ignore = 1;
	}
	else{
		$o_ignore = 0;
	}
	if (defined($o_timeout) && (isnnum($o_timeout) || ($o_timeout < 2) || ($o_timeout > 60)))
	  { print "Timeout must be >1 and <60 !\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	if (!defined($o_timeout)) {$o_timeout=5;}
    if (defined ($o_help) ) { help(); exit $ERRORS{"UNKNOWN"}};
    if (defined($o_version)) { p_version(); exit $ERRORS{"UNKNOWN"}};
    if ( ! defined($o_host) ) # check host and filter
	{ print_usage(); exit $ERRORS{"UNKNOWN"}}
    # check snmp information
    if ( !defined($o_community) && (!defined($o_login)) )
	  { print "Put snmp login info!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	if ((defined($o_login) || defined($o_passwd)) && (defined($o_community) || defined($o_version2)) )
	  { print "Can't mix snmp v1,2c,3 protocols!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	if (defined ($v3protocols)) {
	  if (!defined($o_login)) { print "Put snmp V3 login info with protocols!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	  my @v3proto=split(/,/,$v3protocols);
	  if ((defined ($v3proto[0])) && ($v3proto[0] ne "")) {$o_authproto=$v3proto[0];	}	# Auth protocol
	  if (defined ($v3proto[1])) {$o_privproto=$v3proto[1];	}	# Priv  protocol
	  if ((defined ($v3proto[1])) && (!defined($o_privpass))) {
	    print "Put snmp V3 priv login info with priv protocols!\n"; print_usage(); exit $ERRORS{"UNKNOWN"}}
	}
}

########## MAIN #######

check_options();

# Check gobal timeout if snmp screws up
if (defined($TIMEOUT)) {
  verb("Alarm at $TIMEOUT + 5");
  alarm($TIMEOUT+5);
} else {
  verb("no global timeout defined : $o_timeout + 10");
  alarm ($o_timeout+10);
}

$SIG{'ALRM'} = sub {
 print "No answer from host\n";
 exit $ERRORS{"UNKNOWN"};
};

# Connect to host
my ($session,$error);
if ( defined($o_login) && ! defined($o_passwd))
{
    ($session, $error) = Net::SNMP->session(
      -hostname         => $o_host,
      -version          => '3',
      -username         => $o_login,
      -timeout          => $o_timeout
    );

}
elsif ( defined($o_login) && defined($o_passwd)) {
  # SNMPv3 login
  verb("SNMPv3 login");
    if (!defined ($o_privpass)) {
  verb("SNMPv3 AuthNoPriv login : $o_login, $o_authproto");
    ($session, $error) = Net::SNMP->session(
      -hostname   	=> $o_host,
      -version		=> '3',
      -username		=> $o_login,
      -authpassword	=> $o_passwd,
      -authprotocol	=> $o_authproto,
      -timeout          => $o_timeout
    );
  } else {
    verb("SNMPv3 AuthPriv login : $o_login, $o_authproto, $o_privproto");
    ($session, $error) = Net::SNMP->session(
      -hostname   	=> $o_host,
      -version		=> '3',
      -username		=> $o_login,
      -authpassword	=> $o_passwd,
      -authprotocol	=> $o_authproto,
      -privpassword	=> $o_privpass,
	  -privprotocol => $o_privproto,
      -timeout          => $o_timeout
    );
  }
} else {
	if (defined ($o_version2)) {
		# SNMPv2 Login
		verb("SNMP v2c login");
		  ($session, $error) = Net::SNMP->session(
		 -hostname  => $o_host,
		 -version   => 2,
		 -community => $o_community,
		 -port      => $o_port,
		 -timeout   => $o_timeout
		);
  	} else {
	  # SNMPV1 login
	  verb("SNMP v1 login");
	  ($session, $error) = Net::SNMP->session(
		-hostname  => $o_host,
		-community => $o_community,
		-port      => $o_port,
		-timeout   => $o_timeout
	  );
	}
}
if (!defined($session)) {
   printf("ERROR opening session: %s.\n", $error);
   exit $ERRORS{"UNKNOWN"};
}

sub search_idx{
	my ($table,$base_oid,$idx_col) = @_;
	my @idx;
	foreach my $key ( keys %$table) {
		if ( $key =~ /$base_oid\.$idx_col\./ ) {
			push @idx, $$table{$key};
			verb("OID : $key, Desc : $$table{$key}");
		}
	}
	return sort @idx;
}

sub calcul_health_status{
	my ($result_status) = @_;
	my $my_status = 3;
	if ( $result_status eq "Normal" ) {
		$my_status = 0;
	}
	elsif ( $result_status eq "Warning" ){
		$my_status = 1;
	}
	elsif ( $result_status eq "Unknown" ){
		if( $o_ignore ){
			$my_status = 0;
		}
		else{
			$my_status = 3;
		}
	}
	else{
		$my_status = 2;
	}
}

sub compare_status{
	my ($status1,$status2) = @_;
	if (( $status1 == 2 ) || ( $status2 == 2 )){
		return 2;
	}
	elsif (( $status1 == 1 ) || ( $status2 == 1 )){
		return 1;
	}
	elsif (( $status1 == 3 ) || ( $status2 == 3 )){
		return 3;
	}
	else{
		return 0;
	}
}

sub calcul_all_healthstatus{
	my ($table,$base_oid,$status_col,@idx_list) = @_;
	my $all_status = 0;
	foreach my $idx (@idx_list){
		my $result_status=$$table{"$base_oid.$status_col.$idx"};
		my $my_status = calcul_healthstatus($result_status);
		$all_status=compare_status($all_status,$my_status);
		verb("idx : $idx, ResultStatus : $result_status, EsiaStatus : $my_status, AllStatus = $all_status");
	}
	return $all_status;
}

sub str_status{
	my ($status) = @_;
	if( $status == 0 ){
		return "OK";
	}
	elsif( $status == 1 ){
		return "WARN";
	}
	elsif( $status == 2 ){
		return "CRIT";
	}
	else{
		return "UNKNOWN";
	}
}

sub cpu_health {
	my ($session) = @_;
	my $base_oid = $cpu_base_oid;
	my $idx_col = $cpu_id;
	my $health_col = $cpu_healthstatus;

	#Récupération des infos
	my $full_table = $session->get_table(Baseoid => $base_oid);
	my @idx_list = search_idx($full_table,$base_oid,$idx_col);

	my $all_status = 0;

	#Création de l'affichage
	my $output = '';
	foreach my $idx (@idx_list){
		my $status 	= $$full_table{"$base_oid.$cpu_healthstatus.$idx"};
		my $descr 	= $$full_table{"$base_oid.$cpu_desc.$idx"};

		my $model 	= $$full_table{"$base_oid.$cpu_model.$idx"};
		my $cores 	= $$full_table{"$base_oid.$cpu_cores.$idx"};
		my $threads = $$full_table{"$base_oid.$cpu_threads.$idx"};

		my $esia_status = calcul_health_status($status);
		my $esia_st_str = str_status($esia_status);
		$all_status = compare_status($all_status,$esia_status);

		$output .= "$esia_st_str - Status $status - $descr, $model ($cores cores/$threads threads)\n";
	}
	print $output;
	return $all_status;
}

sub mem_health {
	my ($session) = @_;
	my $base_oid = $mem_base_oid;
	my $idx_col = $mem_id;
	my $health_col = $mem_healthstatus;

	#Récupération des infos
	my $full_table = $session->get_table(Baseoid => $base_oid);
	my @idx_list = search_idx($full_table,$base_oid,$idx_col);

	#Calcul du status
	my $all_status = 0;

	#Création de l'affichage
	my $output = '';
	foreach my $idx (@idx_list){
		my $status 	= $$full_table{"$base_oid.$mem_healthstatus.$idx"};
		my $descr 	= $$full_table{"$base_oid.$mem_desc.$idx"};

		my $type	= $$full_table{"$base_oid.$mem_type.$idx"};
		my $model 	= $$full_table{"$base_oid.$mem_partnum.$idx"};
		my $sn 		= $$full_table{"$base_oid.$mem_FRU_SN.$idx"};

		my $conf_sp = $$full_table{"$base_oid.$mem_configspeed.$idx"};
		my $rt_sp	= $$full_table{"$base_oid.$mem_ratedspeed.$idx"};

		my $esia_status = calcul_health_status($status);
		my $esia_st_str = str_status($esia_status);
		$all_status = compare_status($all_status,$esia_status);

		$output .= "$esia_st_str - Status $status - $descr, Model : $model (SN : $sn speed : $conf_sp/$rt_sp)\n";
	}
	print $output;
	return $all_status;
}

sub disk_health {
	my ($session) = @_;
	my $base_oid = $disk_base_oid;
	my $idx_col = $disk_id;
	my $health_col = $disk_healthstatus;

	#Récupération des infos
	my $full_table = $session->get_table(Baseoid => $base_oid);
	my @idx_list = search_idx($full_table,$base_oid,$idx_col);

	#Calcul du status
	my $all_status = 0;

	#Création de l'affichage
	my $output = '';
	foreach my $idx (@idx_list){
		my $status 	= $$full_table{"$base_oid.$disk_healthstatus.$idx"};
		my $descr 	= $$full_table{"$base_oid.$disk_Name.$idx"};

		my $esia_status = calcul_health_status($status);
		my $esia_st_str = str_status($esia_status);
		$all_status = compare_status($all_status,$esia_status);

		$output .= "$esia_st_str - Status $status - $descr\n";
	}
	print $output;
	return $all_status;
}

sub pwr_health {
	my ($session) = @_;
	my $base_oid = $pwr_base_oid;
	my $idx_col = $pwr_id;
	my $health_col = $pwr_healthstatus;

	#Récupération des infos
	my $full_table = $session->get_table(Baseoid => $base_oid);
	my @idx_list = search_idx($full_table,$base_oid,$idx_col);

	#Calcul du status
	my $all_status = 0;

	#Création de l'affichage
	my $output = '';
	foreach my $idx (@idx_list){
		my $status 	= $$full_table{"$base_oid.$pwr_healthstatus.$idx"};
		my $descr 	= $$full_table{"$base_oid.$pwr_FRU_name.$idx"};

		my $model 	= $$full_table{"$base_oid.$pwr_FRU_num.$idx"};
		my $sn 		= $$full_table{"$base_oid.$pwr_FRU_SN.$idx"};

		my $esia_status = calcul_health_status($status);
		my $esia_st_str = str_status($esia_status);
		$all_status = compare_status($all_status,$esia_status);

		$output .= "$esia_st_str - Status $status - Power $idx, Model : $model (SN : $sn)\n";
	}
	print $output;
	return $all_status;
}

sub fan_health {
	my ($session) = @_;
	my $base_oid = $fan_base_oid;
	my $idx_col = $fan_id;
	my $health_col = $fan_healthstatus;

	#Récupération des infos
	my $full_table = $session->get_table(Baseoid => $base_oid);
	my @idx_list = search_idx($full_table,$base_oid,$idx_col);

	#Calcul du status
	my $all_status = 0;

	#Création de l'affichage
	my $output = '';
	foreach my $idx (@idx_list){
		my $status 	= $$full_table{"$base_oid.$fan_healthstatus.$idx"};
		my $descr 	= $$full_table{"$base_oid.$fan_Desc.$idx"};

		my $speed 	= $$full_table{"$base_oid.$fan_speed.$idx"};

		my $esia_status = calcul_health_status($status);
		my $esia_st_str = str_status($esia_status);
		$all_status = compare_status($all_status,$esia_status);

		$output .= "$esia_st_str - Status $status - $descr (speed : $speed)\n";
	}
	print $output;
	return $all_status;
}

sub temp_health {
	my ($session) = @_;
	my $base_oid = $temp_base_oid;
	my $idx_col = $temp_id;
	my $health_col = $temp_healthstatus;

	#Récupération des infos
	my $full_table = $session->get_table(Baseoid => $base_oid);
	my @idx_list = search_idx($full_table,$base_oid,$idx_col);

	#Calcul du status
	my $all_status = 0;

	#Création de l'affichage
	my $output = '';
	foreach my $idx (@idx_list){
		my $status 	= $$full_table{"$base_oid.$temp_healthstatus.$idx"};
		my $descr 	= $$full_table{"$base_oid.$temp_desc.$idx"};

		my $reading				= $$full_table{"$base_oid.$temp_reading.$idx"};
		my $nonCritLimitHigh 	= $$full_table{"$base_oid.$temp_NonCritLimitHigh.$idx"};
		my $critLimitHigh 		= $$full_table{"$base_oid.$temp_CritLimitHigh.$idx"};

		my $esia_status = calcul_health_status($status);
		my $esia_st_str = str_status($esia_status);
		$all_status = compare_status($all_status,$esia_status);

		$output .= "$esia_st_str - Status $status - $descr, value : $reading, Manufacturer limits : (Warning : $nonCritLimitHigh, Critic : $critLimitHigh)\n";
	}
	print $output;
	return $all_status;
}

sub volt_health {
	my ($session) = @_;
	my $base_oid = $volt_base_oid;
	my $idx_col = $volt_id;
	my $health_col = $volt_healthstatus;

	#Récupération des infos
	my $full_table = $session->get_table(Baseoid => $base_oid);
	my @idx_list = search_idx($full_table,$base_oid,$idx_col);

	#Calcul du status
	my $all_status = 0;

	#Création de l'affichage
	my $output = '';
	foreach my $idx (@idx_list){
		my $status 	= $$full_table{"$base_oid.$volt_healthstatus.$idx"};
		my $descr 	= $$full_table{"$base_oid.$volt_desc.$idx"};

		my $reading				= $$full_table{"$base_oid.$volt_reading.$idx"};

		my $nonCritLimitLow 	= $$full_table{"$base_oid.$volt_NonCritLimitLow.$idx"};
		my $critLimitLow 		= $$full_table{"$base_oid.$volt_CritLimitLow.$idx"};

		my $nonCritLimitHigh 	= $$full_table{"$base_oid.$volt_NonCritLimitHigh.$idx"};
		my $critLimitHigh 		= $$full_table{"$base_oid.$volt_CritLimitHigh.$idx"};

		my $esia_status = calcul_health_status($status);
		my $esia_st_str = str_status($esia_status);
		$all_status = compare_status($all_status,$esia_status);

		$output .= "$esia_st_str - Status $status - $descr, value : $reading, Manufacturer limits : (minCrit : $critLimitLow, minWarn : $nonCritLimitLow, maxWarn : $nonCritLimitHigh, maxCrit : $critLimitHigh)\n";
	}
	print $output;
	return $all_status;
}

my $i_status=3;
switch ($o_check_type) {
	case 'cpu'	{ $i_status = cpu_health($session);	 }
	case 'mem'	{ $i_status = mem_health($session);  }
	case 'disk'	{ $i_status = disk_health($session); }
	case 'power'{ $i_status = pwr_health($session);  }
	case 'fan' 	{ $i_status = fan_health($session);  }
	case 'temp' { $i_status = temp_health($session); }
	case 'volt' { $i_status = volt_health($session); }
	else { print 'ERROR, -T, --type parameter must be (cpu|mem|disk|power|fan|temp|volt)'."\n"; }
}
$session->close;

exit $i_status;
