#!/usr/bin/perl -w 
############################## check_snmp_zyxel_flash.pl ########
my $Version='1.0';
# Date : Mar 02 2017
# Author  : Faes Nathan
# Help : esia-sa.com
#################################################################

use strict;
use Net::SNMP;
use Getopt::Long;

my $PROGNAME='check_snmp_zyxel_flash.pl';
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my $return=0;
my ($opt_h,$opt_V);
my $community = "public";
my $hostname;
my $snmp_version = 2;
my $warning = 70;
my $critical = 80;

my $oid_sysDescr = "1.3.6.1.2.1.1.1";
my $oid_check = "1.3.6.1.4.1.890.1.6.22.1.7.0";

sub print_help() {
	printf "check l'utilisation flash d'un routeur Zyxel.\n";
	print "\n";
	print_usage();
	print "\n";
	print " -H (--hostname)     Hostname to query - (required)\n";
	print " -C (--community)    SNMP read community (defaults to public)\n";
	print " -v (--snmp_version) 1 for SNMP v1\n";
	print "                     2 for SNMP v2c (default)\n";
	print " -w                  warning (default: 80)\n";
	print " -c                  critical (default: 90)\n";
	print " -V (--version)      Plugin version\n";
	print " -h (--help)         usage help\n";
	print "\n";
}

sub print_usage() {
	print "Usage: \n";
	print "  $PROGNAME -H <HOSTNAME> [-C <community>] [-w warning] [-c critical]\n";
	print "  $PROGNAME [-h | --help]\n";
	print "  $PROGNAME [-V | --version]\n";
}
my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
} 

#
#MAIN
#
&Getopt::Long::config('bundling');
GetOptions(
	"V"   => \$opt_V,	"version"    => \$opt_V,
	"h"   => \$opt_h,	"help"       => \$opt_h,
	"C=s" => \$community,	"community=s" => \$community,
	"H=s" => \$hostname,	"hostname=s" => \$hostname,
	"v=i" => \$snmp_version,"snmp_version=i" => \$snmp_version,
	"w=i" => \$warning,"warning=i" => \$warning,
	"c=i" => \$critical,"critical=i" => \$critical,
);
# -h & --help print help
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }
if (!$hostname) { print_help(); exit $ERRORS{'UNKNOWN'}; }
my ($snmp, $snmperror);
if ($snmp_version == 2) 
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv2c',
		-community => $community,
		-timeout=>2
	);
}
else
{
	($snmp, $snmperror) = Net::SNMP->session(
		-hostname => $hostname,
		-version => 'snmpv1',
		-community => $community,
		-timeout=>2
	);
}
#on interroge en SNMP pour récupérer les informations 
my $result = $snmp->get_request("$oid_sysDescr");

#si l'host ne répond pas on blocke le script
if (!defined($result)) 
{
	if($snmp_version == 2)
	{
		($snmp, $snmperror) = Net::SNMP->session(
			-hostname => $hostname,
			-version => 'snmpv1',
			-community => $community
		);
		$result = $snmp->get_request("$oid_sysDescr");
		if (!defined($result)) 
		{
	   		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
	   		$snmp->close;
	  		 exit $ERRORS{"UNKNOWN"};
		}
	}
	else
	{
		printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
		$snmp->close;
		 exit $ERRORS{"UNKNOWN"};
	}
}

my $state;
my $compare_value;
my $snmpresult = $snmp->get_request("$oid_check");
my $value =  $$snmpresult{"$oid_check"};			
if($value =~ m/noSuch/i) 
{
    print "SNMP ERROR: bad value returned : $value\n";
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};  
}
if($value >= $critical)
{
	$return = priority_state($return,$ERRORS{CRITICAL});
	$state ="CRITICAL";
	$compare_value = " > $critical";
}
elsif($value >= $warning)
{
	$return = priority_state($return,$ERRORS{WARNING});
	$state ="WARNING";
	$compare_value = " > $warning";
}
else
{
	$return = priority_state($return,$ERRORS{OK});
	$compare_value = " < $warning";
	$state ="OK";
}
print "$state : FLASH : $value% $compare_value \n";

exit($return);
