#!/usr/bin/perl -w
############################## check_ssh_sanHP #################
my $Version='1.0';
# Date : Octob 23 2017 
#Author : Victor Marin 
#Help : esia-sa.com
#################################################################
use strict;
use Net::OpenSSH;
use Getopt::Long;
my $NAME='check_ssh_sanHP.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my($opt_h,$opt_V);
my $type;
my $key;
my ($hostname);
my $warning='80';
my $port='22';
my $critical='90';
my $result;
my $file = 'ssh_san.conf';
my $password;
my $username;
my $cmd;
my $pattern;
my @tab;
#my @tab_value_read;
my ($stdout,$exit);

 sub print_help() {
        print_usage();
        print "\t $NAME : Récupère l'utilisation de la mémoire via le protocole ssh\n";
        print "\tOptions :\n";
        print "\t\t-H | --host Adresse IP du device dont il faut récupérer les informations \n";
        print "\t\t-T | --type Valeur à vérifier (raid,fan,storage,disk ou cpu) \n";
        print "\t\t-K | --key dans le fichier snmp.conf la key pour les login-password\n";
        print "\t\t[-w] | --warning Seuil de warning (DEFAULT : 80)\n";
        print "\t\t[-c] | --critical Seuil critical (DEFAULT : 90)\n";
        print "\t\t[-f] | --config_file fichier de configuration login-password ssh (DEFAULT : snmp.conf)\n";
        print "\t\t[-p] | --port Port utilisé en SSH (DEFAULT : 22) \n";
        print "\t\t[-U] | --username Set le username dans le fichier de conf \n";
        print "\t\t[-P] | --password Set le password dans le fichier de conf \n";
        print "\t\t[-h] | --help affiche l'aide\n";
}
sub print_usage() {
        print "USAGE: $NAME -H -T -K [-w] [-c] [-f] [-p] [-h]\n";
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

########## MAIN ##########
&Getopt::Long::config('bundling'); GetOptions(
        "T=s" => \$type, "type" => \$type,
        "K=s" => \$key, "key=s" => \$key,
        "h" => \$opt_h, "help" => \$opt_h,
        "H=s" => \$hostname, "hostname=s" => \$hostname,
        "w=i" => \$warning, "warning=i" => \$warning,
        "c=i" => \$critical, "critical=i" => \$critical,
        "f=i" => \$file, "file=s" => \$file,
        "p=i" => \$port, "port=i" => \$port,
        "U=s" => \$username, "username=s" => \$username,
        "P=s" => \$password, "password=s" => \$password, );

# -h & --help print help
if(!$type) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }

my $return=$ERRORS{OK};
# Connection SSH
if(defined($key))
{
        if(!defined($username) || !defined($password))
        {
            open (FILE,"+</etc/esia/$file") or die("ERROR OPEN FILE : $! \n");
            while (my $line = <FILE>)
            {
                if(scalar(grep(/$key/,$line)))
                {
                   ($username,$password) = ($line =~ /$key:(\S+):(\S+)/);
                }
            }
            close FILE;
        }
}
else
{
        $username='monitor';
        $password='!monitor';
}

my $ssh = Net::OpenSSH->new($hostname, port => $port, user =>$username, password => $password, master_opts => [-o => "StrictHostKeyChecking=no"]);

if (!defined($ssh))
{
        print "[UNKNOWN] : Connection failed with $hostname\n";
        exit $ERRORS{'UNKNOWN'};
}
else
{
    #variables identiques mais avec valeurs différentes pour memory et load
    if($type eq 'raid')
    {
        #(pg 301)
        $cmd = 'show vdisks';
        @tab = $ssh->capture($cmd);

        my @name;my @size;my @free;my @raid;my @state;
        my $cmpt = 0;

        foreach my $v (@tab)
        {
            if($v =~ /name="name".*display-name="Name">(\S+)</)
            {
                my $name = ($v =~ /name="name".*display-name="Name">(\S+)</);
                push @name,$1;
                $cmpt++;
            }
            if($v =~ /units="[A-Z]B".*display-name="Size">(\S+)</)
            {
                my $size = ($v =~ /units="[A-Z]B".*display-name="Size">(\S+)</);
                push @size,$1;
            }
            if ($v =~ /units="[A-Z]B".*display-name="Free">(\S+)</)
            {
                my $free = ($v =~ /units="[A-Z]B".*display-name="Free">(\S+)</);
                push @free,$1;
            }
            if($v =~ /name="raidtype".*display-name="RAID">(\S+)</)
            {
                my $raid = ($v =~ /name="raidtype".*display-name="RAID">(\S+)</);
                push @raid,$1;
            }
            if($v =~ /name="health".*display-name="Health">(\S+)</)
            {
                my $state = ($v =~ /name="health".*display-name="Health">(\S+)</);
                push @state,$1;
            }
        }

        foreach (my $x=0 ;$x<$cmpt;$x++)
        {
            my $max = ($size[$x] =~ /(\S+)([A-Z]B)/);
            $max = $1;
            my $unitMax = $2;
            my $libre = ($free[$x] =~ /(\S+)([A-Z]B)/);
            $libre = $1;
            my $unitLibre = $2;
           
            my $space  = 0;            
            if($unitMax ne $unitLibre)	
            {
          		if($unitMax eq "TB" && $unitLibre eq "GB")
          		{
          			$libre = sprintf("%.2f",$libre/1024);	
          		}
          		elsif($unitMax eq "TB" && $unitLibre eq "MB")
          		{
          			$libre = sprintf("%.2f",$libre/(1024*1024));
          		}
          		elsif($unitMax eq "GB" && $unitLibre eq "MB")
          		{
          			$libre = sprintf("%.2f",$libre/1024);
          		}		
          	}
          	$space = sprintf("%.2f",$max - $libre);

            if($state[$x] eq "Degraded")
            {
                print "WARNING : $name[$x] ($space/$size[$x]) Raid : $raid[$x] : Health $state[$x] \n";
                $return = priority_state($return,$ERRORS{WARNING})
            }
                elsif($state[$x] eq "Fault")
            {
                print "CRITICAL : $name[$x] ($space/$size[$x]) Raid : $raid[$x] : Health $state[$x] \n";
                $return = priority_state($return,$ERRORS{CRITICAL})
            }
            elsif($state[$x] eq "Unknown")
            {
                print "UNKNOWN : $name[$x] ($space/$size[$x]) Raid : $raid[$x] : Health $state[$x] \n";
                $return = priority_state($return,$ERRORS{UNKNOWN})
            }
            else
            {
                print "OK : $name[$x] ($space/$size[$x]) Raid : $raid[$x] : Health $state[$x] \n";
            }
        }
    }
    elsif($type eq 'storage')
    {
        $cmd = 'show volumes';
        @tab = $ssh->capture($cmd);

        my @name;my @size;my @buzy;
        my $cmpt = 0;

        foreach my $v (@tab)
        {
            if($v =~ /name="volume-name".*display-name="Name">(\S+)</)
            {
                my $name = ($v =~ /name="volume-name".*display-name="Name">(\S+)</);
                push @name,$1;
                $cmpt++;
            }
            if($v =~ /units="[A-Z]B".*display-name="Size">(\S+)</)
            {
                my $size = ($v =~ /units="[A-Z]B".*display-name="Size">(\S+)</);
                push @size,$1;
            }
            if ($v =~ /units="[A-Z]B".*display-name="Allocated Size">(\S+)</)
            {
                my $buzy = ($v =~ /units="[A-Z]B".*display-name="Allocated Size">(\S+)</);
               push @buzy,$1;
            }
        }

        foreach (my $x=0 ;$x<$cmpt;$x++)
        {
			my $max = ($size[$x] =~ /(\S+)([A-Z]B)/);
            $max = $1;
            my $unitMax = $2;
            my $space = ($buzy[$x] =~ /(\S+)([A-Z]B)/);
            $space = $1;
            my $uniSpace = $2;
                              
            if($unitMax ne $uniSpace)	
            {
          		if($unitMax eq "TB" && $uniSpace eq "GB")
          		{
          			$space = sprintf("%.2f",$space/1024);	
          		}
          		elsif($unitMax eq "TB" && $uniSpace eq "MB")
          		{
          			$space = sprintf("%.2f",$space/(1024*1024));
          		}
          		elsif($unitMax eq "GB" && $uniSpace eq "MB")
          		{
          			$space = sprintf("%.2f",$space/1024);
          		}		
          	}
          	my $used = sprintf("%.2f",(($space/$max)*100));


            if($used < $warning)
            {
                print "OK : $name[$x] used : $used % ($space/$size[$x]) \n";
            }
            else
            {
                if($used > $critical)
                {
                    print "CRITICAL : $name[$x] used : $used % ($space/$size[$x]) \n";
                    $return = priority_state($return,$ERRORS{CRITICAL})
                }
                else
                {
                    print "WARNING : $name[$x] used : $used % ($space/$size[$x]) \n";
                    $return = priority_state($return,$ERRORS{WARNING})
                }
            }
        }
    }
    elsif($type eq 'fan')
    {
        #(pg 223)
        $cmd = 'show fans';
        @tab = $ssh->capture($cmd);

        my @name;my @speed;my @status;my @health;
        my $cmpt = 0;

        foreach my $v (@tab)
        {
            if($v =~ /name="name".*display-name="Name">(.*)</)
            {
                my $name = ($v =~ /name="name".*display-name="Name">(.*)</);
                push @name,$1;
                $cmpt++;
            }
            if($v =~ /name="status".*display-name="Status">(\S+)</)
            {
                my $status = ($v =~ /name="status".*display-name="Status">(\S+)</);
                push @status,$1;
            }
            if ($v =~ /name="speed".*display-name="Speed">(\S+)</)
            {
                my $speed = ($v =~ /name="speed".*display-name="Speed">(\S+)</);
                push @speed,$1;
	    }
	    if ($v =~ /name="health".*display-name="Health">(\S+)</)
            {
                my $health = ($v =~ /name="health".*display-name="Health">(\S+)</);
                push @health,$1;
            }
        }

        foreach (my $x=0 ;$x<$cmpt;$x++)
        {
            if($health[$x] eq "Degraded")
            {
                print "WARNING : Fan : $name[$x] $speed[$x] revoltions per minutes Status : $status[$x] : Health $health[$x] \n";
                $return = priority_state($return,$ERRORS{WARNING})
            }
            elsif($health[$x] eq "Fault")
            {
                print "CRITICAL : Fan : $name[$x] $speed[$x] revoltions per minutes Status : $status[$x] : Health $health[$x] \n";
                $return = priority_state($return,$ERRORS{CRITICAL})
            }
            elsif($health[$x] eq "Unknown")
            {
                print "UNKNOWN : Fan : $name[$x] $speed[$x] revoltions per minutes Status : $status[$x] : Health $health[$x] \n";
                $return = priority_state($return,$ERRORS{UNKNOWN})
            }
            else
            {
                print "OK : Fan : $name[$x] $speed[$x] revoltions per minutes Status : $status[$x] : Health $health[$x] \n";
            }
        }
    }
    elsif($type eq 'cpu')
    {
        $cmd = 'show controller-statistics';
        @tab = $ssh->capture($cmd);

        my @name;my @load;
        my $cmpt = 0;

        foreach my $v (@tab)
        {
            if($v =~ /name="durable-id".*display-name="Durable ID">(\S+)</)
            {
                my $name = ($v =~ /name="durable-id".*display-name="Durable ID">(\S+)</);
                push @name,$1;
                $cmpt++;
            }
            if($v =~ /name="cpu-load".*display-name="CPU Load">(\S+)</)
            {
                my $load = ($v =~ /name="cpu-load".*display-name="CPU Load">(\S+)</);
                push @load,$1;
            }
        }

        foreach (my $x=0 ;$x<$cmpt;$x++)
        {
            if($load[$x] < $warning)
            {
               print "OK : CPU $name[$x] used : $load[$x] % \n";
            }
            else
            {
                if($load[$x] > $critical)
                {
                    print "CRITICAL : CPU $name[$x] used : $load[$x] % \n";
                    $return = priority_state($return,$ERRORS{CRITICAL})
                }
                else
                {
                    print "WARNING : CPU $name[$x] used : $load[$x] % \n";
                    $return = priority_state($return,$ERRORS{WARNING})
                }
            }
        }
    }
    elsif($type eq 'disk')
    {
        $cmd = 'show disks';
        @tab = $ssh->capture($cmd);

        my @name;my @size;my @health;my @description;my @diskgroup;my @sn;my @temp;
        my $cmpt = 0;

        foreach my $v (@tab)
        {
            if($v =~ /name="durable-id".*display-name="Durable ID">(\S+)</)
            {
                my $name = ($v =~ /name="durable-id".*display-name="Durable ID">(\S+)</);
                push @name,$1;
                $cmpt++;
            }
            if($v =~ /name="size" units="[A-Z]B".*display-name="Size">(\S+)</)
            {
                my $size = ($v =~ /name="size" units="[A-Z]B".*display-name="Size">(\S+)</);
                push @size,$1;
            }
            if($v =~ /name="health".*display-name="Health">(\S+)</)
            {
                my $health = ($v =~ /name="health".*display-name="Health">(\S+)</);
                push @health,$1;
            }
            if($v =~ /name="description".*display-name="Description">(.+)</)
            {
                my $description = ($v =~ /name="description".*display-name="Description">(.+)</);
                push @description,$1;
            }
            if($v =~ /name="disk-group".*display-name="Disk Group">(\S+)</)
            {
                my $diskgroup = ($v =~ /name="disk-group".*display-name="Disk Group">(\S+)</);
                push @diskgroup,$1;
            }
            if($v =~ /name="serial-number".*display-name="Serial Number">(\S+)</)
            {
                my $sn = ($v =~ /name="cpu-load".*display-name="Serial Number">(\S+)</);
                push @sn,$1;
            }
            if($v =~ /name="temperature".*display-name="Temperature">(.+)</)
            {
                my $temp = ($v =~ /name="temperature".*display-name="Temperature">(.+)</);
                push @temp,$1;
            }
        }

        foreach (my $x=0 ;$x<$cmpt;$x++)
        {
            if($health[$x] eq "Degraded")
            {
                print "WARNING : $name[$x] : serial number : $sn[$x] : size : $size[$x] : temperature : $temp[$x] : disk group : $diskgroup[$x] : description : $description[$x] : Health : $health[$x] \n";
                $return = priority_state($return,$ERRORS{WARNING})
            }
            elsif($health[$x] eq "Fault")
            {
                print "CRITICAL : $name[$x] : serial number : $sn[$x] : size : $size[$x] : temperature : $temp[$x] : disk group : $diskgroup[$x] : description : $description[$x] : Health : $health[$x] \n";
                $return = priority_state($return,$ERRORS{CRITICAL})
            }
            elsif($health[$x] eq "Unknown" or $health[$x] eq "N/A")
            {
                print "UNKNOWN : $name[$x] : serial number : $sn[$x] : size : $size[$x] : temperature : $temp[$x] : disk group : $diskgroup[$x] : description : $description[$x] : Health : $health[$x] \n";
                $return = priority_state($return,$ERRORS{UNKNOWN})
            }
            else
            {
                print "OK : $name[$x] : serial number : $sn[$x] : size : $size[$x] : temperature : $temp[$x] : disk group : $diskgroup[$x] : description : $description[$x] : Health : $health[$x] \n";
            }
        }
    }
 	elsif($type eq 'cntl'){
		$cmd = 'show controllers';
		@tab = $ssh->capture($cmd);

		#Transform en string
		$result='';
		foreach my $v (@tab)
		{
			$result.="$v ";
		}

#Parsing du xml en une sorte de treeview d'objects
		my %obj_table;
		while ($result =~ /(<COMP[^<]+)(<OBJECT[^<]+(<PROPERTY[^<]*<\/PROPERTY>\s*)*<\/OBJECT>)/gm){
			my $entity_comp = $1;
			my $entity_object = $2;

			$entity_comp =~ /G="(\d+)" P="(\d+)"/;
			my $obj_parent = $1;
			my $obj_oid = $2;

			$entity_object =~ /<OBJECT.*basetype="(\S+)"/;
			my $obj_type = $1;
			if( $obj_type ne "sas-port" ){
				$obj_table{$obj_parent}{$obj_oid}{obj_oid} = $obj_oid;
				$obj_table{$obj_parent}{$obj_oid}{obj_parent} = $obj_parent;
				$obj_table{$obj_parent}{$obj_oid}{obj_type} = $obj_type;

				while ($entity_object =~ /<PROPERTY name="([^"]+)"[^>]+>([^>]+)</gm){
					$obj_table{$obj_parent}{$obj_oid}{$1} = $2;
				}
			}
		}

# Parcour de la  treeview des objects pour calcul de la valeur de retour et affichage
		my $global_status=-1; #-1 pour indiquer qu'il n'est pas setté
		my $status_oid = 0;
		my %controllers;
		my $last_parent_id = 0;
		my %uniq_table;
#Cette boucle ne peu fonctionner que si les controllers sont parcourus avant leurs enfants 
#   (sort du parent obligatoire car le parent du controller vaut 0)
		foreach my $parent ( sort { $a <=> $b } (keys %{obj_table}) ){
			foreach my $oid ( sort { $a <=> $b } (keys %{$obj_table{$parent}}) ){
				my $type = $obj_table{$parent}{$oid}{obj_type};

				my $durableid = '0';
				if( defined $obj_table{$parent}{$oid}{'durable-id'} ){
					$durableid = $obj_table{$parent}{$oid}{'durable-id'};
				}

				if( $durableid eq '0' || !defined $uniq_table{$durableid} ){ #Il y a des doublons dans le xml (sur les composants). On utilise ce if pour les ignorer
					$uniq_table{$durableid} = 1;

					if( $parent == 0 ){ #On sauvegarde les controlleurs et le status de la cmd pour plus tard
						if( $type eq 'status' ){
							$status_oid=$oid;
						}
						elsif( $type eq 'controllers' ){
							$controllers{$oid} = $obj_table{$parent}{$oid};
						}
					}
					else{ #On exécute l'affichage et le calcul
						if( $last_parent_id != $parent ){ #On affiche le controller des éléments qui vont suivre
							#controller_a (7CE517M253) [14 disks 4 vdisks 4 ports] - redundancy:Redundant status:Operational health:OK
							print "$controllers{$parent}{'durable-id'} ";
							print "($controllers{$parent}{'serial-number'}) ";
							print "[$controllers{$parent}{'disks'} disks ";
							print "$controllers{$parent}{'virtual-disks'} vdisks ";
							print "$controllers{$parent}{'host-ports'} ports] ";
							print "- redundancy:$controllers{$parent}{'redundancy-status'} ";
							print "status:$controllers{$parent}{'status'} ";
							print "health:$controllers{$parent}{'health'}";
							if (defined $controllers{$parent}{'health-reason'}){
								print " - ". $controllers{$parent}{'health-reason'};
							}
							print "\n";
							my $health = $controllers{$parent}{'health'};
							if( $global_status == 2 || $health =~ /Fault/i ) {
								$global_status = 2;
							}
							elsif( $global_status == 1 || $health =~ /Degraded/i ){
								$global_status = 1;
							}
							elsif( $global_status < 1 && ( $health =~ /OK/i  || $health =~ /N\/A/i )){
								$global_status = 0;      
							}
							else{
								$global_status = 3;
							}
						}
						#On affiche les sous-éléments du même controller (cela fonctionne grâce au sort sur les parents)
						if( $type eq 'network-parameters' ){
							# - [ Net] mgmtport_a (00:c0:ff:25:d0:53/1000mbps) [ip:192.168.7.21|255.255.255.0 gw:192.168.7.254 mode:Manual] - health:OK
							print " - [ net]";
							print "  ".$obj_table{$parent}{$oid}{'durable-id'};
							print " (".$obj_table{$parent}{$oid}{'mac-address'};
							print "/".$obj_table{$parent}{$oid}{'link-speed'}.")";

							print " [ip:".$obj_table{$parent}{$oid}{'ip-address'};
							print "|".$obj_table{$parent}{$oid}{'subnet-mask'};
							print " gw:".$obj_table{$parent}{$oid}{'gateway'};
							print " mode:".$obj_table{$parent}{$oid}{'addressing-mode'}."]";

							print " - health:".$obj_table{$parent}{$oid}{'health'};
						}
						elsif( $type eq 'port' ){
							# - [Port] hostport_A1 (500c0ff25dd2e000/12Gb) - status:Up health:OK
							print " - [port]";
							print " ".$obj_table{$parent}{$oid}{'durable-id'};
							print " (".$obj_table{$parent}{$oid}{'target-id'};
							print "/".$obj_table{$parent}{$oid}{'actual-speed'}.")";
							print " - status:".$obj_table{$parent}{$oid}{'status'};
							print " health:".$obj_table{$parent}{$oid}{'health'};
						}
						elsif( $type eq 'expander-ports' ){
							# - [expander-port] expport_egress_1.B1 (Out Port) - status:Disconnected health:N/A
							print " - [expander-port]";                    
							print " ".$obj_table{$parent}{$oid}{'durable-id'};
							print " (".$obj_table{$parent}{$oid}{'name'}.")";
							print " - status:".$obj_table{$parent}{$oid}{'status'};
							print " health:".$obj_table{$parent}{$oid}{'health'};
						}
						elsif( $type eq 'compact-flash' ){
							# - [compact-flash] Ctlr B CF (Controller B CompactFlash) - status:Installed health:OK
							print " - [compact-flash]";                    
							print " ".$obj_table{$parent}{$oid}{'durable-id'};
							print " (".$obj_table{$parent}{$oid}{'name'}.")";
							print " - status:".$obj_table{$parent}{$oid}{'status'};
							print " health:".$obj_table{$parent}{$oid}{'health'};
						}
						if (defined $obj_table{$parent}{$oid}{'health-reason'}){
							print " - ".$obj_table{$parent}{$oid}{'health-reason'};
						}
						print "\n";
						my $health = $obj_table{$parent}{$oid}{'health'};
						if( $global_status == 2 || $health =~ /Fault/i ) {
							$global_status = 2;
						}
						elsif( $global_status == 1 || $health =~ /Degraded/i ){
							$global_status = 1;
						}
						elsif( $global_status < 1 && ( $health =~ /OK/i  || $health =~ /N\/A/i )){
							$global_status = 0;      
						}
						else{
							$global_status = 3;
						}
					}
					$last_parent_id = $parent;
				}
			}
		}

		if( $obj_table{0}{$status_oid}{'response-type-numeric'} != 0 ){
			print "UNKNOWN - Response server status : ";
			print $obj_table{0}{$status_oid}{'response-type'}." - ";
			print $obj_table{0}{$status_oid}{'response'};
			print "\n";
			$global_status = 3;
		}
		elsif( $global_status == 0 ){
			print "GLOBAL STATE OK, all components health are in (OK or N/A) status\n";
		}
		elsif( $global_status == 1 ){
			print "GLOBAL STATE WARNING, any components health are in (Degraded) status\n";   
		}
		elsif( $global_status == 2 ){
			print "GLOBAL STATE CRITICAL, any components health are in (Fault) status\n";
		}
		else{
			print "GLOBAL STATE UNKNOWN, any components health are in unkown status\n";
		}
		exit($global_status);
	}
}
exit $return;
