#!/usr/bin/perl -w
############################## check_ssh_sanHP_3par #################
my $Version='1.0';
# Date : Octob 23 2017 
#Author : Victor Marin 
#Help : esia-sa.com
#################################################################
use strict;
use Switch;
use Net::OpenSSH;
use Getopt::Long;
my $NAME='check_ssh_sanHP_3par.pl';
my $TIMEOUT = 15;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
my($opt_h,$opt_V);
my $type;
my $key;
my ($hostname);
my $warning='80';
my $port='22';
my $critical='90';
my $result;
my $file = 'ssh_san.conf';
my $password;
my $username;
my $cmd;
my $pattern;
my @tab;
my ($stdout,$exit);

 sub print_help() {
        print_usage();
        print "\t $NAME : Récupère l'utilisation de la mémoire via le protocole ssh\n";
        print "\tOptions :\n";
        print "\t\t-H | --host Adresse IP du device dont il faut récupérer les informations \n";
        print "\t\t-T | --type Valeur à vérifier (raid,fan,storage,disk ou cpu) \n";
        print "\t\t-K | --key dans le fichier snmp.conf la key pour les login-password\n";
        print "\t\t[-w] | --warning Seuil de warning (DEFAULT : 80)\n";
        print "\t\t[-c] | --critical Seuil critical (DEFAULT : 90)\n";
        print "\t\t[-f] | --config_file fichier de configuration login-password ssh (DEFAULT : snmp.conf)\n";
        print "\t\t[-p] | --port Port utilisé en SSH (DEFAULT : 22) \n";
        print "\t\t[-U] | --username Set le username dans le fichier de conf \n";
        print "\t\t[-P] | --password Set le password dans le fichier de conf \n";
        print "\t\t[-h] | --help affiche l'aide\n";
}
sub print_usage() {
        print "USAGE: $NAME -H -T -K [-w] [-c] [-f] [-p] [-h]\n";
}

my @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state{
        my($a,$b) = @_;
        if( $a == $b ) { return $a; } #pas besoin d'aller dans la boucles si ils sont identiques

        foreach my $v (@priorities){
                if ( $v == $a || $v == $b ) { return $v; }
        }
}

########## MAIN ##########
&Getopt::Long::config('bundling'); GetOptions(
        "T=s" => \$type, "type" => \$type,
        "K=s" => \$key, "key=s" => \$key,
        "h" => \$opt_h, "help" => \$opt_h,
        "H=s" => \$hostname, "hostname=s" => \$hostname,
        "w=i" => \$warning, "warning=i" => \$warning,
        "c=i" => \$critical, "critical=i" => \$critical,
        "f=i" => \$file, "file=s" => \$file,
        "p=i" => \$port, "port=i" => \$port,
        "U=s" => \$username, "username=s" => \$username,
        "P=s" => \$password, "password=s" => \$password, );

# -h & --help print help
if(!$type) { print_help(); exit $ERRORS{'OK'}; }
if(!$hostname) {print_help(); exit $ERRORS{'OK'}; }

my $return=$ERRORS{OK};
# Connection SSH
if(defined($key))
{
        if(!defined($username) || !defined($password))
        {
            open (FILE,"+</etc/esia/$file") or die("ERROR OPEN FILE : $! \n");
            while (my $line = <FILE>)
            {
                if(scalar(grep(/$key/,$line)))
                {
                   ($username,$password) = ($line =~ /$key:(\S+):(\S+)/);
                }
            }
            close FILE;
        }
}
else
{
        $username='monitor';
        $password='!monitor';
}

my $ssh = Net::OpenSSH->new($hostname, port => $port, user =>$username, password => $password, master_opts => [-o => "StrictHostKeyChecking=no"]);

if (!defined($ssh))
{
        print "[UNKNOWN] : Connection failed with $hostname\n";
        exit $ERRORS{'UNKNOWN'};
}
else
{
    if($type eq 'raid')
    {
        #(pg 301)
        $cmd = 'showld';
        @tab = $ssh->capture($cmd);

        my ($id_disk,$namedisk,$raid,$status,$sizeMB,$usedMB);        
        my $state = "OK";
        my $cmpt = 0;
        my $cmptdisk = 0;
       
        foreach my $v (@tab)
        {
            ($id_disk,$namedisk,$raid,$status,$sizeMB,$usedMB) = ($v =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+\S+\s+(\S+)\s+(\S+)/);
            if(defined($namedisk) && $raid ne 'RAID')            
            { 
                my $percentUsed = ($usedMB/$sizeMB)*100;
                my $stateSpace = "OK";
                if($percentUsed>=$critical)
                {
                    $stateSpace = "CRITICAL";
                    $return = priority_state($return,$ERRORS{CRITICAL})
                }
                elsif($percentUsed>=$warning)
                {
                    $stateSpace = "WARNING";
                    $return = priority_state($return,$ERRORS{WARNING})
                }

                switch ($status) 
                {
                    case "normal"    { $state = "OK"; }
                    case "orphan"    { $state = "CRITICAL"; $cmpt++; print "$state : Logical disk : $id_disk,  Name : $namedisk ($status), Raid : $raid, Size ($stateSpace) : $usedMB /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{CRITICAL})}
                    case "preserved" { $state = "WARNING"; $cmpt++; print "$state : Logical disk : $id_disk,  Name : $namedisk ($status), Raid : $raid, Size ($stateSpace) : $usedMB /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{WARNING})}
                    case "stopped"   { $state = "CRITICAL"; $cmpt++; print "$state : Logical disk : $id_disk,  Name : $namedisk ($status), Raid : $raid, Size ($stateSpace) : $usedMB /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{CRITICAL})}
                    case "stopping"  { $state = "CRITICAL"; $cmpt++; print "$state : Logical disk : $id_disk,  Name : $namedisk ($status), Raid : $raid, Size ($stateSpace) : $usedMB /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{CRITICAL})}
                    case "removing"  { $state = "CRITICAL"; $cmpt++; print "$state : Logical disk : $id_disk,  Name : $namedisk ($status), Raid : $raid, Size ($stateSpace) : $usedMB /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{CRITICAL})}
                    case "auto_check"{ if($stateSpace ne "OK") { $cmpt++; print "$state : Logical disk : $id_disk,  Name : $namedisk ($status), Raid : $raid, Size ($stateSpace) : $usedMB /$sizeMB MB\n";}}
                    case "checking"  { $state = "WARNING"; $cmpt++; print "$state : Logical disk : $id_disk,  Name : $namedisk ($status), Raid : $raid, Size ($stateSpace) : $usedMB /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{WARNING})}
                    case "need_check"{ $state = "WARNING"; $cmpt++; print "$state : Logical disk : $id_disk,  Name : $namedisk ($status), Raid : $raid, Size ($stateSpace) : $usedMB /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{WARNING})}
                    case "need_pd"   { $state = "WARNING"; $cmpt++; print "$state : Logical disk : $id_disk,  Name : $namedisk ($status), Raid : $raid, Size ($stateSpace) : $usedMB /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{WARNING})}
                    else             { $state = "UNKNOWN"; $cmpt++; print "$state : Logical disk : $id_disk,  Name : $namedisk ($status), Raid : $raid, Size ($stateSpace) : $usedMB /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{UNKNOWN})}
                }                 
                $cmptdisk++;            
            }
        }
        if($cmpt==0)
        {
            print "All raid are OK \n";
        }
    }
    elsif($type eq 'disk')
    {
        $cmd = 'showpd';
        @tab = $ssh->capture($cmd);

        my ($id_disk,$positcage,$type,$rpm,$status,$sizeMB,$freeMB);
        my $state = "OK";
        my $cmpt = 0;
        my $cmptdisk = 0;
        my $cmptdiskssd = 0;
        my $cmptdiskfc = 0;
        my $cmptdisknl = 0;

        foreach my $v (@tab)
        {
            ($id_disk,$positcage,$type,$rpm,$status,$sizeMB,$freeMB) = ($v =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/);
            if(defined($type) && $type ne 'Type')
            {
                my $used = $sizeMB - $freeMB;
                my $percentUsed = ($used/$sizeMB)*100;
                my $stateSpace = "OK";
                if($percentUsed>=$critical)
                {
                    $stateSpace = "CRITICAL";                    
                    $return = priority_state($return,$ERRORS{CRITICAL})
                }
                elsif($percentUsed>=$warning)
                {
                    $stateSpace = "WARNING";
                    $return = priority_state($return,$ERRORS{WARNING})
                }

                switch ($status) 
                {
                    case "normal"   { if($stateSpace ne "OK") { $cmpt++; print "$state : Physical disk ($type) : $id_disk,  State : $status, Position : $positcage, Rpm : $rpm, Size ($stateSpace) : $used /$sizeMB MB\n"; }}
                    case "degraded" { $state = "CRITICAL"; $cmpt++; print "$state : Physical disk ($type) : $id_disk,  State : $status, Position : $positcage, Rpm : $rpm, Size ($stateSpace) : $used /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{CRITICAL}) }
                    case "new"      { $state = "WARNING"; $cmpt++; print "$state : Physical disk ($type) : $id_disk,  State : $status, Position : $positcage, Rpm : $rpm, Size ($stateSpace) : $used /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{WARNING}) }
                    case "failed"   { $state = "CRITICAL"; $cmpt++; print "$state : Physical disk ($type) : $id_disk,  State : $status, Position : $positcage, Rpm : $rpm, Size ($stateSpace) : $used /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{CRITICAL}) }
                    else            { $state = "UNKNOWN"; $cmpt++; print "$state : Physical disk ($type) : $id_disk,  State : $status, Position : $positcage, Rpm : $rpm, Size ($stateSpace) : $used /$sizeMB MB\n"; $return = priority_state($return,$ERRORS{UNKNOWN}) }
                } 


                switch ($type) 
                {
                    case "FC"  { $cmptdiskfc++;  }
                    case "NL"  { $cmptdisknl++; }
                    case "SSD" { $cmptdiskssd++; }   
                }
                $cmptdisk++;
                #OK: 42 0:0:0(FC), State:normal, Rpm:015, Size(OK):375/559MB
            }
        }
        if($cmpt==0)
        {
            print "All $cmptdisk physicals disks are OK (SSD : $cmptdiskssd, NL : $cmptdisknl, FC : $cmptdiskfc) \n";
        }
    }
    elsif($type eq 'power')
    {
        $cmd = 'showbattery';
        @tab = $ssh->capture($cmd);

        my ($node,$powersupplyNumber,$batteryId,$sn,$status,$chgLvl);
        my $state = "OK";

        foreach my $v (@tab)
        {
            ($node,$powersupplyNumber,$batteryId,$sn,$status,$chgLvl) = ($v =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/);
            if(defined($batteryId) && $batteryId ne 'Bat')
            {
                switch ($status) 
                {
                    case "OK"         { $state = "OK"; }
                    case "NotPresent" { $state = "WARNING"; $return = priority_state($return,$ERRORS{WARNING}) }                    
                    case "Failed"     { $state = "CRITICAL"; $return = priority_state($return,$ERRORS{CRITICAL}) }
                    case "MaxLifeLow" { $state = "CRITICAL"; $return = priority_state($return,$ERRORS{CRITICAL}) }
                    case "--"         { $state = "WARNING"; $return = priority_state($return,$ERRORS{WARNING}) }
                    else              { $state = "UNKNOWN"; $return = priority_state($return,$ERRORS{WARNING}) }
                }                
                print "$state Battery : serial number : $sn, State : $status, Charge : $chgLvl % \n";
            }
        }
    } 
    elsif($type eq 'interface')
    {
        $cmd = 'statport -iter 1';
        @tab = $ssh->capture($cmd);
        
        my ($port,$dataOrController,$typeIO,$IOsecCur,$IOsecAvg,$kbSecMax,$kbSecCur,$kbSecAvg);
        my $state = "OK";

        foreach my $v (@tab)
        {
            ($port,$dataOrController,$typeIO,$IOsecCur,$IOsecAvg,$kbSecMax,$kbSecCur,$kbSecAvg) = ($v =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/);                        
            if(defined($port) && $port =~ /\d\:\d\:\d/)
            {                            	
                if($kbSecCur<=$warning && $kbSecCur > 0)
                {
                    $state = "WARNING";
                    $return = priority_state($return,$ERRORS{WARNING})
                    
                }
                elsif($kbSecCur<=$critical && $kbSecCur > 0)
                {
                    $state = "CRITICAL";
                    $return = priority_state($return,$ERRORS{CRITICAL})
                }                
                print "$state : Port : $port, D/C : $dataOrController, I/O sec : cur : $IOsecCur, avg : $IOsecAvg, KB/sec : max : $kbSecMax, cur $kbSecCur, avg $kbSecAvg \n";
            }
        }
    }
    elsif($type eq 'volume')
    {
    	$cmd = 'showversion';
        @tab = $ssh->capture($cmd);

        my ($release,$idversion);       
    	
    	foreach my $v (@tab)
        {
        	($release) = ($v =~ /(\S+)\s+\S+\s+\S+/);
        	if(defined($release) && $release eq "Release")
        	{
        		($release,$idversion) = ($v =~ /(\S+)\s+\S+\s+(\S+)/);
        		
        	}
        }        
    	    	
        $cmd = 'showvv';
        @tab = $ssh->capture($cmd);

       	my ($id_disk,$namevolume,$type,$status,$admrsvd,$snprsvd,$userrsvd,$sizeMB);
        my $state = "OK";
        my $cmpt = 0;
        my $cmptvolume = 0;
        my $option_type;

		if($idversion eq "3.3.1")
		{
			foreach my $v (@tab)
        	{            
	            ($id_disk,$namevolume,$type,$status,$snprsvd,$userrsvd,$sizeMB) = ($v =~ /(\S+)\s+(\S+)\s+\S+\s+\S+\s+\S+\s+(\S+)\s+\S+\s+\S+\s+\S+\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/);
	            if(defined($type) && $type ne 'Type')
	            {              
	                switch ($type) 
	                {
	                    case "base"  { $option_type = "Base volume (not a copy)"; }
	                    case "pcopy" { $option_type = "Physical copy (full copy)"; }
	                    case "vcopy" { $option_type = "Snapshot copy (virtual copy)"; }
	                    else         { $state = "UNKNOWN"; $option_type = "UNKNOWN"; $return = priority_state($return,$ERRORS{UNKNOWN}) }
	                }
	
	                switch ($status) 
	                {
	                    case "normal"      { $state = "OK"; }
	                    case "preserved"   { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) }
	                    case "started"     { $state = "OK"; }
	                    case "started,stl" { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) }
	                    case "started,cpf "{ $state = "CRITICAL"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{CRITICAL}) }
	                    case "not_started" { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) }
	                    case "pmt"         { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) }
	                    case "auto_check"  { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) }
	                    case "checking"    { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) }
	                    case "need_check"  { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) } 
	                    case "need_ld"     { $state = "CRITICAL";$cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{CRITICAL}) }
	                    else               { $state = "UNKNOWN"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{UNKNOWN})}
	                }
	                $cmptvolume++;         
	            }           
	       	}
		}
		else
		{
			foreach my $v (@tab)
        	{            
	            ($id_disk,$namevolume,$type,$status,$admrsvd,$snprsvd,$userrsvd,$sizeMB) = ($v =~ /(\S+)\s+(\S+)\s+\S+\s+(\S+)\s+\S+\s+\S+\s+\S+\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/);
	            if(defined($type) && $type ne 'Type')
	            {              
	                switch ($type) 
	                {
	                    case "base"  { $option_type = "Base volume (not a copy)"; }
	                    case "pcopy" { $option_type = "Physical copy (full copy)"; }
	                    case "vcopy" { $option_type = "Snapshot copy (virtual copy)"; }
	                    else         { $state = "UNKNOWN"; $option_type = "UNKNOWN"; $return = priority_state($return,$ERRORS{UNKNOWN}) }
	                }
	
	                switch ($status) 
	                {
	                    case "normal"      { $state = "OK"; }
	                    case "preserved"   { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (adm:$admrsvd,snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) }
	                    case "started"     { $state = "OK"; }
	                    case "started,stl" { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (adm:$admrsvd,snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) }
	                    case "started,cpf "{ $state = "CRITICAL"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (adm:$admrsvd,snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{CRITICAL}) }
	                    case "not_started" { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (adm:$admrsvd,snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) }
	                    case "pmt"         { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (adm:$admrsvd,snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) }
	                    case "auto_check"  { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (adm:$admrsvd,snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) }
	                    case "checking"    { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (adm:$admrsvd,snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) }
	                    case "need_check"  { $state = "WARNING"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (adm:$admrsvd,snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{WARNING}) } 
	                    case "need_ld"     { $state = "CRITICAL";$cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (adm:$admrsvd,snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{CRITICAL}) }
	                    else               { $state = "UNKNOWN"; $cmpt++; print "$state : Volume : $id_disk, Name : $namevolume, Type ($type): $option_type, State : $status, Size : $sizeMB MB, Rsvd (adm:$admrsvd,snp:$snprsvd,user:$userrsvd)\n"; $return = priority_state($return,$ERRORS{UNKNOWN})}
	                }
	                $cmptvolume++;         
	            }           
	       	}
		}
				
        if($cmpt==0)
        {
            print "All $cmptvolume volumes are OK \n";
        }        
    }  
}
exit $return;
